@@ -1,4 +1,5 @@
 <?php
+declare(strict_types = 1);
 namespace TYPO3\CMS\Backend\Controller\ContentElement;
 
 /*
@@ -16,101 +17,121 @@ namespace TYPO3\CMS\Backend\Controller\ContentElement;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
-use TYPO3\CMS\Backend\Module\AbstractModule;
 use TYPO3\CMS\Backend\Template\DocumentTemplate;
+use TYPO3\CMS\Backend\Template\ModuleTemplate;
+use TYPO3\CMS\Backend\Tree\View\ContentCreationPagePositionMap;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Backend\View\BackendLayoutView;
 use TYPO3\CMS\Backend\Wizard\NewContentElementWizardHookInterface;
+use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Compatibility\PublicPropertyDeprecationTrait;
+use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Imaging\Icon;
-use TYPO3\CMS\Core\Imaging\IconProvider\BitmapIconProvider;
-use TYPO3\CMS\Core\Imaging\IconRegistry;
+use TYPO3\CMS\Core\Localization\LanguageService;
+use TYPO3\CMS\Core\Service\DependencyOrderingService;
+use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
+use TYPO3\CMS\Core\Utility\HttpUtility;
+use TYPO3\CMS\Fluid\View\StandaloneView;
 
 /**
  * Script Class for the New Content element wizard
+ * @internal This class is a specific Backend controller implementation and is not considered part of the Public TYPO3 API.
  */
-class NewContentElementController extends AbstractModule
+class NewContentElementController
 {
+    use PublicPropertyDeprecationTrait;
+
+    /**
+     * @var array
+     */
+    protected $deprecatedPublicProperties = [
+        'id' => 'Using $id of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'sys_language' => 'Using $sys_language of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'R_URI' => 'Using $R_URI of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'colPos' => 'Using $colPos of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'uid_pid' => 'Using $uid_pid of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'modTSconfig' => 'Using $modTSconfig of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'doc' => 'Using $doc of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'content' => 'Using $content of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'access' => 'Using $access of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+        'config' => 'Using $config of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
+    ];
+
     /**
      * Page id
      *
      * @var int
      */
-    public $id;
+    protected $id;
 
     /**
      * Sys language
      *
      * @var int
      */
-    public $sys_language = 0;
+    protected $sys_language = 0;
 
     /**
      * Return URL.
      *
      * @var string
      */
-    public $R_URI = '';
+    protected $R_URI = '';
 
     /**
      * If set, the content is destined for a specific column.
      *
      * @var int|null
      */
-    public $colPos;
+    protected $colPos;
 
     /**
      * @var int
      */
-    public $uid_pid;
+    protected $uid_pid;
 
     /**
      * Module TSconfig.
      *
      * @var array
      */
-    public $modTSconfig = [];
+    protected $modTSconfig = [];
 
     /**
      * Internal backend template object
      *
      * @var DocumentTemplate
      */
-    public $doc;
+    protected $doc;
 
     /**
      * Used to accumulate the content of the module.
      *
      * @var string
      */
-    public $content;
+    protected $content;
 
     /**
      * Access boolean.
      *
      * @var bool
      */
-    public $access;
+    protected $access;
 
     /**
      * config of the wizard
      *
      * @var array
      */
-    public $config;
+    protected $config;
 
     /**
      * @var array
      */
     protected $pageInfo;
 
-    /**
-     * @var array
-     */
-    protected $elementWrapper;
-
     /**
      * @var string
      */
@@ -121,49 +142,82 @@ class NewContentElementController extends AbstractModule
      */
     protected $MCONF;
 
+    /**
+     * @var StandaloneView
+     */
+    protected $view;
+
+    /**
+     * @var StandaloneView
+     */
+    protected $menuItemView;
+
+    /**
+     * ModuleTemplate object
+     *
+     * @var ModuleTemplate
+     */
+    protected $moduleTemplate;
+
     /**
      * Constructor
      */
     public function __construct()
     {
-        parent::__construct();
+        $this->moduleTemplate = GeneralUtility::makeInstance(ModuleTemplate::class);
         $GLOBALS['SOBE'] = $this;
-        $this->init();
+        $this->view = $this->getFluidTemplateObject();
+        $this->menuItemView = $this->getFluidTemplateObject('MenuItem.html');
+
+        // @deprecated since TYPO3 v9, will be obsolete in TYPO3 v10.0 with removal of init()
+        $request = $GLOBALS['TYPO3_REQUEST'];
+        // @deprecated since TYPO3 v9, will be moved out of __construct() in TYPO3 v10.0
+        $this->init($request);
     }
 
     /**
      * Constructor, initializing internal variables.
      *
-     * @return void
+     * @param ServerRequestInterface|null $request
      */
-    public function init()
+    public function init(ServerRequestInterface $request = null)
     {
+        if ($request === null) {
+            // Method signature in TYPO3 v10.0: protected function init(ServerRequestInterface $request)
+            trigger_error('NewContentElementController->init() will be set to protected in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+            $request = $GLOBALS['TYPO3_REQUEST'];
+        }
+
         $lang = $this->getLanguageService();
-        $lang->includeLLFile('EXT:lang/locallang_misc.xlf');
+        $lang->includeLLFile('EXT:core/Resources/Private/Language/locallang_misc.xlf');
         $LOCAL_LANG_orig = $GLOBALS['LOCAL_LANG'];
         $lang->includeLLFile('EXT:backend/Resources/Private/Language/locallang_db_new_content_el.xlf');
         ArrayUtility::mergeRecursiveWithOverrule($LOCAL_LANG_orig, $GLOBALS['LOCAL_LANG']);
         $GLOBALS['LOCAL_LANG'] = $LOCAL_LANG_orig;
 
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
+
         // Setting internal vars:
-        $this->id = (int)GeneralUtility::_GP('id');
-        $this->sys_language = (int)GeneralUtility::_GP('sys_language_uid');
-        $this->R_URI = GeneralUtility::sanitizeLocalUrl(GeneralUtility::_GP('returnUrl'));
-        $this->colPos = GeneralUtility::_GP('colPos') === null ? null : (int)GeneralUtility::_GP('colPos');
-        $this->uid_pid = (int)GeneralUtility::_GP('uid_pid');
+        $this->id = (int)($parsedBody['id'] ?? $queryParams['id'] ?? 0);
+        $this->sys_language = (int)($parsedBody['sys_language_uid'] ?? $queryParams['sys_language_uid'] ?? 0);
+        $this->R_URI = GeneralUtility::sanitizeLocalUrl($parsedBody['returnUrl'] ?? $queryParams['returnUrl'] ?? '');
+        $colPos = $parsedBody['colPos'] ?? $queryParams['colPos'] ?? null;
+        $this->colPos = $colPos === null ? null : (int)$colPos;
+        $this->uid_pid = (int)($parsedBody['uid_pid'] ?? $queryParams['uid_pid'] ?? 0);
         $this->MCONF['name'] = 'xMOD_db_new_content_el';
-        $this->modTSconfig = BackendUtility::getModTSconfig($this->id, 'mod.wizards.newContentElement');
+        $this->modTSconfig['properties'] = BackendUtility::getPagesTSconfig($this->id)['mod.']['wizards.']['newContentElement.'] ?? [];
         $config = BackendUtility::getPagesTSconfig($this->id);
         $this->config = $config['mod.']['wizards.']['newContentElement.'];
         // Starting the document template object:
         // We keep this here in case somebody relies on it in a hook or alike
         $this->doc = GeneralUtility::makeInstance(DocumentTemplate::class);
         // Setting up the context sensitive menu:
-        $this->moduleTemplate->getPageRenderer()->loadRequireJsModule('TYPO3/CMS/Backend/ClickMenu');
+        $this->moduleTemplate->getPageRenderer()->loadRequireJsModule('TYPO3/CMS/Backend/ContextMenu');
         // Getting the current page and receiving access information (used in main())
-        $perms_clause = $this->getBackendUser()->getPagePermsClause(1);
+        $perms_clause = $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW);
         $this->pageInfo = BackendUtility::readPageAccess($this->id, $perms_clause);
-        $this->access = is_array($this->pageInfo) ? 1 : 0;
+        $this->access = is_array($this->pageInfo);
     }
 
     /**
@@ -171,30 +225,118 @@ class NewContentElementController extends AbstractModule
      * As this controller goes only through the main() method, it is rather simple for now
      *
      * @param ServerRequestInterface $request the current request
-     * @param ResponseInterface $response
      * @return ResponseInterface the response with the content
      */
-    public function mainAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function mainAction(ServerRequestInterface $request): ResponseInterface
     {
-        $this->main();
+        $this->prepareContent('window');
         $this->moduleTemplate->setContent($this->content);
-        $response->getBody()->write($this->moduleTemplate->renderContent());
-        return $response;
+        return new HtmlResponse($this->moduleTemplate->renderContent());
+    }
+
+    /**
+     * Injects the request object for the current request or subrequest
+     * As this controller goes only through the main() method, it is rather simple for now
+     *
+     * @param ServerRequestInterface $request the current request
+     * @return ResponseInterface the response with the content
+     */
+    public function wizardAction(ServerRequestInterface $request): ResponseInterface
+    {
+        $this->prepareContent('list_frame');
+        return new HtmlResponse($this->content);
     }
 
     /**
      * Creating the module output.
      *
-     * @throws \UnexpectedValueException
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3v10 without substitute
      */
     public function main()
     {
-        $lang = $this->getLanguageService();
-        $this->content .= '<form action="" name="editForm" id="NewContentElementController"><input type="hidden" name="defValues" value="" />';
+        trigger_error('NewContentElementController->main() will be replaced by protected method prepareContent() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        $this->prepareContent('window');
+    }
+
+    /**
+     * Returns the array of elements in the wizard display.
+     * For the plugin section there is support for adding elements there from a global variable.
+     *
+     * @return array
+     */
+    public function wizardArray()
+    {
+        trigger_error('NewContentElementController->wizardArray() will be replaced by protected method getWizards() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        return $this->getWizards();
+    }
+
+    /**
+     * @param mixed $wizardElements
+     * @return array
+     */
+    public function wizard_appendWizards($wizardElements)
+    {
+        trigger_error('NewContentElementController->wizard_appendWizards() will be replaced by protected method getAppendWizards() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        return $this->getAppendWizards($wizardElements);
+    }
+
+    /**
+     * @param string $groupKey Not used
+     * @param string $itemKey Not used
+     * @param array $itemConf
+     * @return array
+     */
+    public function wizard_getItem($groupKey, $itemKey, $itemConf)
+    {
+        trigger_error('NewContentElementController->wizard_getItem() will be replaced by protected method getWizardItem() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        return $this->getWizardItem($groupKey, $itemKey, $itemConf);
+    }
+
+    /**
+     * @param string $groupKey Not used
+     * @param array $wizardGroup
+     * @return array
+     */
+    public function wizard_getGroupHeader($groupKey, $wizardGroup)
+    {
+        trigger_error('NewContentElementController->wizard_getGroupHeader() will be replaced by protected method getWizardGroupHeader() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        return $this->getWizardGroupHeader($wizardGroup);
+    }
+
+    /**
+     * Checks the array for elements which might contain unallowed default values and will unset them!
+     * Looks for the "tt_content_defValues" key in each element and if found it will traverse that array as fieldname /
+     * value pairs and check.
+     * The values will be added to the "params" key of the array (which should probably be unset or empty by default).
+     *
+     * @param array $wizardItems Wizard items, passed by reference
+     */
+    public function removeInvalidElements(&$wizardItems)
+    {
+        trigger_error('NewContentElementController->removeInvalidElements() will be replaced by protected method removeInvalidWizardItems() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        $this->removeInvalidWizardItems($wizardItems);
+    }
+
+    /**
+     * Creating the module output.
+     *
+     * @param string $clientContext JavaScript client context to be used
+     *        + 'window', legacy if rendered in current document
+     *        + 'list_frame', in case rendered in global modal
+     *
+     * @throws \UnexpectedValueException
+     */
+    protected function prepareContent(string $clientContext): void
+    {
+        $hasAccess = true;
         if ($this->id && $this->access) {
+
             // Init position map object:
-            $posMap = GeneralUtility::makeInstance(\TYPO3\CMS\Backend\Tree\View\ContentCreationPagePositionMap::class);
+            $posMap = GeneralUtility::makeInstance(
+                ContentCreationPagePositionMap::class,
+                null,
+                $clientContext
+            );
             $posMap->cur_sys_language = $this->sys_language;
             // If a column is pre-set:
             if (isset($this->colPos)) {
@@ -217,113 +359,71 @@ class NewContentElementController extends AbstractModule
             // ***************************
             // Creating content
             // ***************************
-            $this->content .= '<h1>' . $lang->getLL('newContentElement') . '</h1>';
             // Wizard
-            $wizardItems = $this->wizardArray();
+            $wizardItems = $this->getWizards();
             // Wrapper for wizards
-            $this->elementWrapper['section'] = ['', ''];
             // Hook for manipulating wizardItems, wrapper, onClickEvent etc.
-            if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['cms']['db_new_content_el']['wizardItemsHook'])) {
-                foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['cms']['db_new_content_el']['wizardItemsHook'] as $classData) {
-                    $hookObject = GeneralUtility::getUserObj($classData);
-                    if (!$hookObject instanceof NewContentElementWizardHookInterface) {
-                        throw new \UnexpectedValueException(
-                            $classData . ' must implement interface ' . NewContentElementWizardHookInterface::class,
-                            1227834741
-                        );
-                    }
-                    $hookObject->manipulateWizardItems($wizardItems, $this);
+            foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['cms']['db_new_content_el']['wizardItemsHook'] ?? [] as $className) {
+                $hookObject = GeneralUtility::makeInstance($className);
+                if (!$hookObject instanceof NewContentElementWizardHookInterface) {
+                    throw new \UnexpectedValueException(
+                        $className . ' must implement interface ' . NewContentElementWizardHookInterface::class,
+                        1227834741
+                    );
                 }
+                $hookObject->manipulateWizardItems($wizardItems, $this);
             }
-            // Add document inline javascript
-            $this->moduleTemplate->addJavaScriptCode(
-                'NewContentElementWizardInlineJavascript',
-                '
-				function goToalt_doc() {
-					' . $this->onClickEvent . '
-				}'
-            );
-
-            $iconRegistry = GeneralUtility::makeInstance(IconRegistry::class);
 
             // Traverse items for the wizard.
             // An item is either a header or an item rendered with a radio button and title/description and icon:
             $cc = ($key = 0);
             $menuItems = [];
-            foreach ($wizardItems as $k => $wInfo) {
-                if ($wInfo['header']) {
+
+            $this->view->assignMultiple([
+                'hasClickEvent' => $this->onClickEvent !== '',
+                'onClickEvent' => 'function goToalt_doc() { ' . $this->onClickEvent . '}',
+            ]);
+
+            foreach ($wizardItems as $wizardKey => $wInfo) {
+                $wizardOnClick = '';
+                if (isset($wInfo['header'])) {
                     $menuItems[] = [
-                        'label' => htmlspecialchars($wInfo['header']),
-                        'content' => $this->elementWrapper['section'][0]
+                        'label' => $wInfo['header'] ?: '-',
+                        'content' => ''
                     ];
                     $key = count($menuItems) - 1;
                 } else {
-                    $content = '';
-
                     if (!$this->onClickEvent) {
                         // Radio button:
-                        $oC = 'document.editForm.defValues.value=unescape(' . GeneralUtility::quoteJSvalue(rawurlencode($wInfo['params'])) . ');goToalt_doc();' . (!$this->onClickEvent ? 'window.location.hash=\'#sel2\';' : '');
-                        $content .= '<div class="media-left"><input type="radio" name="tempB" value="' . htmlspecialchars($k) . '" onclick="' . htmlspecialchars($oC) . '" /></div>';
+                        $wizardOnClick = 'document.editForm.defValues.value=unescape(' . GeneralUtility::quoteJSvalue(rawurlencode($wInfo['params'])) . '); window.location.hash=\'#sel2\';';
                         // Onclick action for icon/title:
-                        $aOnClick = 'document.getElementsByName(\'tempB\')[' . $cc . '].checked=1;' . $oC . 'return false;';
+                        $aOnClick = 'document.getElementsByName(\'tempB\')[' . $cc . '].checked=1;' . $wizardOnClick . 'return false;';
                     } else {
-                        $aOnClick = "document.editForm.defValues.value=unescape('" . rawurlencode($wInfo['params']) . "');goToalt_doc();" . (!$this->onClickEvent?"window.location.hash='#sel2';":'');
+                        $aOnClick = "document.editForm.defValues.value=unescape('" . rawurlencode($wInfo['params']) . "');goToalt_doc();";
                     }
 
-                    if (isset($wInfo['icon'])) {
-                        GeneralUtility::deprecationLog('The PageTS-Config: mod.wizards.newContentElement.wizardItems.*.elements.*.icon'
-                            . ' is deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8.'
-                            . ' Register your icon in IconRegistry::registerIcon and use the new setting:'
-                            . ' mod.wizards.newContentElement.wizardItems.*.elements.*.iconIdentifier');
-                        $wInfo['iconIdentifier'] = 'content-' . $k;
-                        $icon = $wInfo['icon'];
-                        if (StringUtility::beginsWith($icon, '../typo3conf/ext/')) {
-                            $icon = str_replace('../typo3conf/ext/', 'EXT:', $icon);
-                        }
-                        if (!StringUtility::beginsWith($icon, 'EXT:') && strpos($icon, '/') !== false) {
-                            $icon = TYPO3_mainDir . GeneralUtility::resolveBackPath($wInfo['icon']);
-                        }
-                        $iconRegistry->registerIcon($wInfo['iconIdentifier'], BitmapIconProvider::class, [
-                            'source' => $icon
-                        ]);
-                    }
                     $icon = $this->moduleTemplate->getIconFactory()->getIcon($wInfo['iconIdentifier'])->render();
-                    $menuItems[$key]['content'] .= '
-						<div class="media">
-							<a href="#" onclick="' . htmlspecialchars($aOnClick) . '">
-								' . $content . '
-								<div class="media-left">
-									' . $icon . '
-								</div>
-								<div class="media-body">
-									<strong>' . htmlspecialchars($wInfo['title']) . '</strong>' .
-                                    '<br />' .
-                                    nl2br(htmlspecialchars(trim($wInfo['description']))) .
-                                '</div>
-							</a>
-						</div>';
+
+                    $this->menuItemView->assignMultiple([
+                        'onClickEvent' => $this->onClickEvent,
+                        'aOnClick' => $aOnClick,
+                        'wizardInformation' => $wInfo,
+                        'icon' => $icon,
+                        'wizardOnClick' => $wizardOnClick,
+                        'wizardKey' => $wizardKey
+                    ]);
+                    $menuItems[$key]['content'] .= $this->menuItemView->render();
                     $cc++;
                 }
             }
-            // Add closing section-tag
-            foreach ($menuItems as $key => $val) {
-                $menuItems[$key]['content'] .= $this->elementWrapper['section'][1];
-            }
-            // Add the wizard table to the content, wrapped in tabs
-            $code = '<p>' . $lang->getLL('sel1', 1) . '</p>' . $this->moduleTemplate->getDynamicTabMenu(
+
+            $this->view->assign('renderedTabs', $this->moduleTemplate->getDynamicTabMenu(
                 $menuItems,
                 'new-content-element-wizard'
-            );
-
-            $this->content .= !$this->onClickEvent ? '<h2>' . $lang->getLL('1_selectType', true) . '</h2>' : '';
-            $this->content .= '<div>' . $code . '</div>';
+            ));
 
             // If the user must also select a column:
             if (!$this->onClickEvent) {
-                // Add anchor "sel2"
-                $this->content .= '<div><a name="sel2"></a></div>';
-                // Select position
-                $code = '<p>' . $lang->getLL('sel2', 1) . '</p>';
 
                 // Load SHARED page-TSconfig settings and retrieve column list from there, if applicable:
                 $colPosArray = GeneralUtility::callUserFunction(
@@ -335,15 +435,15 @@ class NewContentElementController extends AbstractModule
                 // Removing duplicates, if any
                 $colPosList = implode(',', array_unique(array_map('intval', $colPosIds)));
                 // Finally, add the content of the column selector to the content:
-                $code .= $posMap->printContentElementColumns($this->id, 0, $colPosList, 1, $this->R_URI);
-                $this->content .= '<h2>' . $lang->getLL('2_selectPosition', true) . '</h2><div>' . $code . '</div>';
+                $this->view->assign('posMap', $posMap->printContentElementColumns($this->id, 0, $colPosList, 1, $this->R_URI));
             }
         } else {
             // In case of no access:
-            $this->content = '';
-            $this->content .= '<h1>' . $lang->getLL('newContentElement') . '</h1>';
+            $hasAccess = false;
         }
-        $this->content .= '</form>';
+        $this->view->assign('hasAccess', $hasAccess);
+
+        $this->content = $this->view->render();
         // Setting up the buttons and markers for docheader
         $this->getButtons();
     }
@@ -368,40 +468,29 @@ class NewContentElementController extends AbstractModule
         $buttonBar->addButton($cshButton);
     }
 
-    /***************************
-     *
-     * OTHER FUNCTIONS:
-     *
-     ***************************/
-    /**
-     * Returns the content of wizardArray() function...
-     *
-     * @return array Returns the content of wizardArray() function...
-     * @deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8, use "wizardArray()" directly
-     */
-    public function getWizardItems()
-    {
-        GeneralUtility::logDeprecatedFunction();
-        return $this->wizardArray();
-    }
-
     /**
      * Returns the array of elements in the wizard display.
      * For the plugin section there is support for adding elements there from a global variable.
      *
      * @return array
      */
-    public function wizardArray()
+    protected function getWizards(): array
     {
         $wizardItems = [];
         if (is_array($this->config)) {
-            $wizards = $this->config['wizardItems.'];
-            $appendWizards = $this->wizard_appendWizards($wizards['elements.']);
+            $wizards = $this->config['wizardItems.'] ?? [];
+            $appendWizards = $this->getAppendWizards($wizards['elements.'] ?? []);
             if (is_array($wizards)) {
+                foreach ($wizards as $groupKey => $wizardGroup) {
+                    $this->prepareDependencyOrdering($wizards[$groupKey], 'before');
+                    $this->prepareDependencyOrdering($wizards[$groupKey], 'after');
+                }
+                $wizards = GeneralUtility::makeInstance(DependencyOrderingService::class)->orderByDependencies($wizards);
+
                 foreach ($wizards as $groupKey => $wizardGroup) {
                     $groupKey = rtrim($groupKey, '.');
                     $showItems = GeneralUtility::trimExplode(',', $wizardGroup['show'], true);
-                    $showAll = $wizardGroup['show'] === '*';
+                    $showAll = in_array('*', $showItems, true);
                     $groupItems = [];
                     if (is_array($appendWizards[$groupKey . '.']['elements.'])) {
                         $wizardElements = array_merge((array)$wizardGroup['elements.'], $appendWizards[$groupKey . '.']['elements.']);
@@ -412,7 +501,7 @@ class NewContentElementController extends AbstractModule
                         foreach ($wizardElements as $itemKey => $itemConf) {
                             $itemKey = rtrim($itemKey, '.');
                             if ($showAll || in_array($itemKey, $showItems)) {
-                                $tmpItem = $this->wizard_getItem($groupKey, $itemKey, $itemConf);
+                                $tmpItem = $this->getWizardItem($groupKey, $itemKey, $itemConf);
                                 if ($tmpItem) {
                                     $groupItems[$groupKey . '_' . $itemKey] = $tmpItem;
                                 }
@@ -420,31 +509,35 @@ class NewContentElementController extends AbstractModule
                         }
                     }
                     if (!empty($groupItems)) {
-                        $wizardItems[$groupKey] = $this->wizard_getGroupHeader($groupKey, $wizardGroup);
+                        $wizardItems[$groupKey] = $this->getWizardGroupHeader($wizardGroup);
                         $wizardItems = array_merge($wizardItems, $groupItems);
                     }
                 }
             }
         }
         // Remove elements where preset values are not allowed:
-        $this->removeInvalidElements($wizardItems);
+        $this->removeInvalidWizardItems($wizardItems);
         return $wizardItems;
     }
 
     /**
-     * @param mixed $wizardElements
+     * @param array $wizardElements
      * @return array
      */
-    public function wizard_appendWizards($wizardElements)
+    protected function getAppendWizards(array $wizardElements): array
     {
         if (!is_array($wizardElements)) {
             $wizardElements = [];
         }
         if (is_array($GLOBALS['TBE_MODULES_EXT']['xMOD_db_new_content_el']['addElClasses'])) {
             foreach ($GLOBALS['TBE_MODULES_EXT']['xMOD_db_new_content_el']['addElClasses'] as $class => $path) {
-                require_once $path;
+                if (!class_exists($class) && file_exists($path)) {
+                    require_once $path;
+                }
                 $modObj = GeneralUtility::makeInstance($class);
-                $wizardElements = $modObj->proc($wizardElements);
+                if (method_exists($modObj, 'proc')) {
+                    $wizardElements = $modObj->proc($wizardElements);
+                }
             }
         }
         $returnElements = [];
@@ -462,7 +555,7 @@ class NewContentElementController extends AbstractModule
      * @param array $itemConf
      * @return array
      */
-    public function wizard_getItem($groupKey, $itemKey, $itemConf)
+    protected function getWizardItem(string $groupKey, string $itemKey, array $itemConf): array
     {
         $itemConf['title'] = $this->getLanguageService()->sL($itemConf['title']);
         $itemConf['description'] = $this->getLanguageService()->sL($itemConf['description']);
@@ -472,11 +565,10 @@ class NewContentElementController extends AbstractModule
     }
 
     /**
-     * @param string $groupKey Not used
      * @param array $wizardGroup
      * @return array
      */
-    public function wizard_getGroupHeader($groupKey, $wizardGroup)
+    protected function getWizardGroupHeader(array $wizardGroup): array
     {
         return [
             'header' => $this->getLanguageService()->sL($wizardGroup['header'])
@@ -490,9 +582,8 @@ class NewContentElementController extends AbstractModule
      * The values will be added to the "params" key of the array (which should probably be unset or empty by default).
      *
      * @param array $wizardItems Wizard items, passed by reference
-     * @return void
      */
-    public function removeInvalidElements(&$wizardItems)
+    protected function removeInvalidWizardItems(array &$wizardItems): void
     {
         // Get TCEFORM from TSconfig of current page
         $row = ['pid' => $this->id];
@@ -503,7 +594,8 @@ class NewContentElementController extends AbstractModule
             // Exploding parameter string, if any (old style)
             if ($wizardItems[$key]['params']) {
                 // Explode GET vars recursively
-                $tempGetVars = GeneralUtility::explodeUrl2Array($wizardItems[$key]['params'], true);
+                $tempGetVars = [];
+                parse_str($wizardItems[$key]['params'], $tempGetVars);
                 // If tt_content values are set, merge them into the tt_content_defValues array,
                 // unset them from $tempGetVars and re-implode $tempGetVars into the param string
                 // (in case remaining parameters are around).
@@ -513,7 +605,7 @@ class NewContentElementController extends AbstractModule
                         $tempGetVars['defVals']['tt_content']
                     );
                     unset($tempGetVars['defVals']['tt_content']);
-                    $wizardItems[$key]['params'] = GeneralUtility::implodeArrayForUrl('', $tempGetVars);
+                    $wizardItems[$key]['params'] = HttpUtility::buildQueryString($tempGetVars, '&');
                 }
             }
             // If tt_content_defValues are defined...:
@@ -524,34 +616,33 @@ class NewContentElementController extends AbstractModule
                     if (is_array($GLOBALS['TCA']['tt_content']['columns'][$fN])) {
                         // Get information about if the field value is OK:
                         $config = &$GLOBALS['TCA']['tt_content']['columns'][$fN]['config'];
-                        $authModeDeny = $config['type'] == 'select' && $config['authMode']
+                        $authModeDeny = $config['type'] === 'select' && $config['authMode']
                             && !$backendUser->checkAuthMode('tt_content', $fN, $fV, $config['authMode']);
                         // explode TSconfig keys only as needed
-                        if (!isset($removeItems[$fN])) {
-                            $removeItems[$fN] = GeneralUtility::trimExplode(
+                        if (!isset($removeItems[$fN]) && isset($TCEFORM_TSconfig[$fN]['removeItems']) && $TCEFORM_TSconfig[$fN]['removeItems'] !== '') {
+                            $removeItems[$fN] = array_flip(GeneralUtility::trimExplode(
                                 ',',
                                 $TCEFORM_TSconfig[$fN]['removeItems'],
                                 true
-                            );
+                            ));
                         }
-                        if (!isset($keepItems[$fN])) {
-                            $keepItems[$fN] = GeneralUtility::trimExplode(
+                        if (!isset($keepItems[$fN]) && isset($TCEFORM_TSconfig[$fN]['keepItems']) && $TCEFORM_TSconfig[$fN]['keepItems'] !== '') {
+                            $keepItems[$fN] = array_flip(GeneralUtility::trimExplode(
                                 ',',
                                 $TCEFORM_TSconfig[$fN]['keepItems'],
                                 true
-                            );
+                            ));
                         }
-                        $isNotInKeepItems = !empty($keepItems[$fN]) && !in_array($fV, $keepItems[$fN]);
-                        if ($authModeDeny || $fN === 'CType' && (in_array($fV, $removeItems[$fN]) || $isNotInKeepItems)) {
+                        $isNotInKeepItems = !empty($keepItems[$fN]) && !isset($keepItems[$fN][$fV]);
+                        if ($authModeDeny || ($fN === 'CType' && (isset($removeItems[$fN][$fV]) || $isNotInKeepItems))) {
                             // Remove element all together:
                             unset($wizardItems[$key]);
                             break;
-                        } else {
-                            // Add the parameter:
-                            $wizardItems[$key]['params'] .= '&defVals[tt_content][' . $fN . ']=' . rawurlencode($fV);
-                            $tmp = explode('_', $key);
-                            $headersUsed[$tmp[0]] = $tmp[0];
                         }
+                        // Add the parameter:
+                        $wizardItems[$key]['params'] .= '&defVals[tt_content][' . $fN . ']=' . rawurlencode($this->getLanguageService()->sL($fV));
+                        $tmp = explode('_', $key);
+                        $headersUsed[$tmp[0]] = $tmp[0];
                     }
                 }
             }
@@ -566,22 +657,88 @@ class NewContentElementController extends AbstractModule
     }
 
     /**
-     * Returns LanguageService
+     * Prepare a wizard tab configuration for sorting.
      *
-     * @return \TYPO3\CMS\Lang\LanguageService
+     * @param array  $wizardGroup TypoScript wizard tab configuration
+     * @param string $key         Which array key should be prepared
      */
-    protected function getLanguageService()
+    protected function prepareDependencyOrdering(&$wizardGroup, $key)
+    {
+        if (isset($wizardGroup[$key])) {
+            $wizardGroup[$key] = GeneralUtility::trimExplode(',', $wizardGroup[$key]);
+            $wizardGroup[$key] = array_map(function ($s) {
+                return $s . '.';
+            }, $wizardGroup[$key]);
+        }
+    }
+
+    /**
+     * @return LanguageService
+     */
+    protected function getLanguageService(): LanguageService
     {
         return $GLOBALS['LANG'];
     }
 
     /**
-     * Returns the current BE user.
-     *
-     * @return \TYPO3\CMS\Core\Authentication\BackendUserAuthentication
+     * @return BackendUserAuthentication
      */
-    protected function getBackendUser()
+    protected function getBackendUser(): BackendUserAuthentication
     {
         return $GLOBALS['BE_USER'];
     }
+
+    /**
+     * @param string $filename
+     * @return StandaloneView
+     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidExtensionNameException
+     */
+    protected function getFluidTemplateObject(string $filename = 'Main.html'): StandaloneView
+    {
+        /** @var StandaloneView $view */
+        $view = GeneralUtility::makeInstance(StandaloneView::class);
+        $view->setTemplatePathAndFilename(GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Templates/NewContentElement/' . $filename));
+        $view->getRequest()->setControllerExtensionName('Backend');
+        return $view;
+    }
+
+    /**
+     * Provide information about the current page making use of the wizard
+     *
+     * @return array
+     */
+    public function getPageInfo(): array
+    {
+        return $this->pageInfo;
+    }
+
+    /**
+     * Provide information about the column position of the button that triggered the wizard
+     *
+     * @return int|null
+     */
+    public function getColPos(): ?int
+    {
+        return $this->colPos;
+    }
+
+    /**
+     * Provide information about the language used while triggering the wizard
+     *
+     * @return int
+     */
+    public function getSysLanguage(): int
+    {
+        return $this->sys_language;
+    }
+
+    /**
+     * Provide information about the element to position the new element after (uid) or into (pid)
+     *
+     * @return int
+     */
+    public function getUidPid(): int
+    {
+        return $this->uid_pid;
+    }
 }
