@@ -1,4 +1,5 @@
 <?php
+declare(strict_types = 1);
 namespace TYPO3\CMS\Backend\Controller;
 
 /*
@@ -17,13 +18,14 @@ namespace TYPO3\CMS\Backend\Controller;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Backend\Form\FormDataCompiler;
-use TYPO3\CMS\Backend\Form\FormDataGroup\InlineParentRecord;
 use TYPO3\CMS\Backend\Form\FormDataGroup\TcaDatabaseRecord;
 use TYPO3\CMS\Backend\Form\InlineStackProcessor;
 use TYPO3\CMS\Backend\Form\NodeFactory;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
-use TYPO3\CMS\Core\Localization\LocalizationFactory;
+use TYPO3\CMS\Core\Http\JsonResponse;
+use TYPO3\CMS\Core\Messaging\AbstractMessage;
+use TYPO3\CMS\Core\Messaging\FlashMessageService;
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
@@ -31,21 +33,27 @@ use TYPO3\CMS\Core\Utility\MathUtility;
 /**
  * Handle FormEngine inline ajax calls
  */
-class FormInlineAjaxController
+class FormInlineAjaxController extends AbstractFormEngineAjaxController
 {
     /**
      * Create a new inline child via AJAX.
      *
      * @param ServerRequestInterface $request
-     * @param ResponseInterface $response
      * @return ResponseInterface
      */
-    public function createAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function createAction(ServerRequestInterface $request): ResponseInterface
     {
-        $ajaxArguments = isset($request->getParsedBody()['ajax']) ? $request->getParsedBody()['ajax'] : $request->getQueryParams()['ajax'];
+        $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
+        $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
         $domObjectId = $ajaxArguments[0];
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
+        if (!MathUtility::canBeInterpretedAsInteger($inlineFirstPid) && strpos($inlineFirstPid, 'NEW') !== 0) {
+            throw new \RuntimeException(
+                'inlineFirstPid should either be an integer or a "NEW..." string',
+                1521220491
+            );
+        }
         $childChildUid = null;
         if (isset($ajaxArguments[1]) && MathUtility::canBeInterpretedAsInteger($ajaxArguments[1])) {
             $childChildUid = (int)$ajaxArguments[1];
@@ -55,45 +63,11 @@ class FormInlineAjaxController
         /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($ajaxArguments['context']);
+        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
+        $inlineTopMostParent = $inlineStackProcessor->getStructureLevel(0);
 
         // Parent, this table embeds the child table
         $parent = $inlineStackProcessor->getStructureLevel(-1);
-        $parentFieldName = $parent['field'];
-
-        if (MathUtility::canBeInterpretedAsInteger($parent['uid'])) {
-            $command = 'edit';
-            $vanillaUid = (int)$parent['uid'];
-            $databaseRow = [
-                // TcaInlineExpandCollapseState needs the record uid
-                'uid' => (int)$parent['uid'],
-            ];
-        } else {
-            $command = 'new';
-            $databaseRow = [];
-            $vanillaUid = (int)$inlineFirstPid;
-        }
-        $databaseRow = $this->addFlexFormDataStructurePointersFromAjaxContext($ajaxArguments, $databaseRow);
-
-        $formDataCompilerInputForParent = [
-            'vanillaUid' => $vanillaUid,
-            'command' => $command,
-            'tableName' => $parent['table'],
-            'databaseRow' => $databaseRow,
-            'inlineFirstPid' => $inlineFirstPid,
-            'columnsToProcess' => array_merge(
-                [$parentFieldName],
-                array_keys($databaseRow)
-            ),
-            // Do not resolve existing children, we don't need them now
-            'inlineResolveExistingChildren' => false,
-        ];
-        /** @var TcaDatabaseRecord $formDataGroup */
-        $formDataGroup = GeneralUtility::makeInstance(InlineParentRecord::class);
-        /** @var FormDataCompiler $formDataCompiler */
-        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
-        $parentData = $formDataCompiler->compile($formDataCompilerInputForParent);
-        $parentConfig = $parentData['processedTca']['columns'][$parentFieldName]['config'];
 
         // Child, a record from this table should be rendered
         $child = $inlineStackProcessor->getUnstableStructure();
@@ -102,12 +76,9 @@ class FormInlineAjaxController
             $childVanillaUid = -1 * abs((int)$child['uid']);
         } else {
             // Else inline first Pid is the storage pid of new inline records
-            $childVanillaUid = (int)$inlineFirstPid;
+            $childVanillaUid = $inlineFirstPid;
         }
 
-        if ($parentConfig['type'] === 'flex') {
-            $parentConfig = $this->getParentConfigFromFlexForm($parentConfig, $domObjectId);
-        }
         $childTableName = $parentConfig['foreign_table'];
 
         /** @var TcaDatabaseRecord $formDataGroup */
@@ -125,26 +96,15 @@ class FormInlineAjaxController
             'inlineParentTableName' => $parent['table'],
             'inlineParentFieldName' => $parent['field'],
             'inlineParentConfig' => $parentConfig,
+            'inlineTopMostParentUid' => $inlineTopMostParent['uid'],
+            'inlineTopMostParentTableName' => $inlineTopMostParent['table'],
+            'inlineTopMostParentFieldName' => $inlineTopMostParent['field'],
         ];
         if ($childChildUid) {
             $formDataCompilerInput['inlineChildChildUid'] = $childChildUid;
         }
         $childData = $formDataCompiler->compile($formDataCompilerInput);
 
-        // Set language of new child record to the language of the parent record:
-        // @todo: To my understanding, the below case can't happen: With localizationMode select, lang overlays
-        // @todo: of children are only created with the "synchronize" button that will trigger a different ajax action.
-        // @todo: The edge case of new page overlay together with localized media field, this code won't kick in either.
-        /**
-        if ($parent['localizationMode'] === 'select' && MathUtility::canBeInterpretedAsInteger($parent['uid'])) {
-            $parentRecord = $inlineRelatedRecordResolver->getRecord($parent['table'], $parent['uid']);
-            $parentLanguageField = $GLOBALS['TCA'][$parent['table']]['ctrl']['languageField'];
-            $childLanguageField = $GLOBALS['TCA'][$child['table']]['ctrl']['languageField'];
-            if ($parentRecord[$parentLanguageField] > 0) {
-                $record[$childLanguageField] = $parentRecord[$parentLanguageField];
-            }
-        }
-         */
         if ($parentConfig['foreign_selector'] && $parentConfig['appearance']['useCombination']) {
             // We have a foreign_selector. So, we just created a new record on an intermediate table in $childData.
             // Now, if a valid id is given as second ajax parameter, the intermediate row should be connected to an
@@ -164,17 +124,17 @@ class FormInlineAjaxController
                 $formDataCompilerInput = [
                     'command' => 'new',
                     'tableName' => $childData['processedTca']['columns'][$parentConfig['foreign_selector']]['config']['foreign_table'],
-                    'vanillaUid' => (int)$inlineFirstPid,
+                    'vanillaUid' => $inlineFirstPid,
                     'isInlineChild' => true,
                     'isInlineAjaxOpeningContext' => true,
                     'inlineStructure' => $inlineStackProcessor->getStructure(),
-                    'inlineFirstPid' => (int)$inlineFirstPid,
+                    'inlineFirstPid' => $inlineFirstPid,
                 ];
                 $childData['combinationChild'] = $formDataCompiler->compile($formDataCompilerInput);
             }
         }
 
-        $childData['inlineParentUid'] = (int)$parent['uid'];
+        $childData['inlineParentUid'] = $parent['uid'];
         $childData['renderType'] = 'inlineRecordContainer';
         $nodeFactory = GeneralUtility::makeInstance(NodeFactory::class);
         $childResult = $nodeFactory->create($childData)->render();
@@ -208,72 +168,53 @@ class FormInlineAjaxController
         // Fade out and fade in the new record in the browser view to catch the user's eye
         $jsonArray['scriptCall'][] = 'inline.fadeOutFadeIn(' . GeneralUtility::quoteJSvalue($objectId . '_div') . ');';
 
-        $response->getBody()->write(json_encode($jsonArray));
-
-        return $response;
+        return new JsonResponse($jsonArray);
     }
 
     /**
      * Show the details of a child record.
      *
      * @param ServerRequestInterface $request
-     * @param ResponseInterface $response
      * @return ResponseInterface
      */
-    public function detailsAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function detailsAction(ServerRequestInterface $request): ResponseInterface
     {
-        $ajaxArguments = isset($request->getParsedBody()['ajax']) ? $request->getParsedBody()['ajax'] : $request->getQueryParams()['ajax'];
+        $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
 
-        $domObjectId = $ajaxArguments[0];
+        $domObjectId = $ajaxArguments[0] ?? null;
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
+        $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
         // Parse the DOM identifier, add the levels to the structure stack
         /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($ajaxArguments['context']);
+        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
 
         // Parent, this table embeds the child table
         $parent = $inlineStackProcessor->getStructureLevel(-1);
         $parentFieldName = $parent['field'];
 
-        $databaseRow = [
-            // TcaInlineExpandCollapseState needs this
-            'uid' => (int)$parent['uid'],
-        ];
-
-        $databaseRow = $this->addFlexFormDataStructurePointersFromAjaxContext($ajaxArguments, $databaseRow);
-
-        $formDataCompilerInputForParent = [
-            'vanillaUid' => (int)$parent['uid'],
-            'command' => 'edit',
-            'tableName' => $parent['table'],
-            'databaseRow' => $databaseRow,
-            'inlineFirstPid' => $inlineFirstPid,
-            'columnsToProcess' => array_merge(
-                [$parentFieldName],
-                array_keys($databaseRow)
-            ),
-            // @todo: still needed?
-            'inlineStructure' => $inlineStackProcessor->getStructure(),
-            // Do not resolve existing children, we don't need them now
-            'inlineResolveExistingChildren' => false,
-        ];
-        /** @var TcaDatabaseRecord $formDataGroup */
-        $formDataGroup = GeneralUtility::makeInstance(InlineParentRecord::class);
-        /** @var FormDataCompiler $formDataCompiler */
-        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
-        $parentData = $formDataCompiler->compile($formDataCompilerInputForParent);
-        $parentConfig = $parentData['processedTca']['columns'][$parentFieldName]['config'];
-
-        if ($parentConfig['type'] === 'flex') {
-            $parentConfig = $this->getParentConfigFromFlexForm($parentConfig, $domObjectId);
-            $parentData['processedTca']['columns'][$parentFieldName]['config'] = $parentConfig;
-        }
-
         // Set flag in config so that only the fields are rendered
         // @todo: Solve differently / rename / whatever
-        $parentData['processedTca']['columns'][$parentFieldName]['config']['renderFieldsOnly'] = true;
+        $parentConfig['renderFieldsOnly'] = true;
+
+        $parentData = [
+            'processedTca' => [
+                'columns' => [
+                    $parentFieldName => [
+                        'config' => $parentConfig,
+                    ],
+                ],
+            ],
+            'tableName' => $parent['table'],
+            'inlineFirstPid' => $inlineFirstPid,
+            // Hand over given original return url to compile stack. Needed if inline children compile links to
+            // another view (eg. edit metadata in a nested inline situation like news with inline content element image),
+            // so the back link is still the link from the original request. See issue #82525. This is additionally
+            // given down in TcaInline data provider to compiled children data.
+            'returnUrl' => $parentConfig['originalReturnUrl'],
+        ];
 
         // Child, a record from this table should be rendered
         $child = $inlineStackProcessor->getUnstableStructure();
@@ -308,9 +249,7 @@ class FormInlineAjaxController
             $jsonArray['scriptCall'][] = 'inline.collapseAllRecords(' . GeneralUtility::quoteJSvalue($objectId) . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',\'' . (int)$child['uid'] . '\');';
         }
 
-        $response->getBody()->write(json_encode($jsonArray));
-
-        return $response;
+        return new JsonResponse($jsonArray);
     }
 
     /**
@@ -318,28 +257,35 @@ class FormInlineAjaxController
      * Handle AJAX calls to localize all records of a parent, localize a single record or to synchronize with the original language parent.
      *
      * @param ServerRequestInterface $request the incoming request
-     * @param ResponseInterface $response the empty response
      * @return ResponseInterface the filled response
      */
-    public function synchronizeLocalizeAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function synchronizeLocalizeAction(ServerRequestInterface $request): ResponseInterface
     {
-        $ajaxArguments = isset($request->getParsedBody()['ajax']) ? $request->getParsedBody()['ajax'] : $request->getQueryParams()['ajax'];
-        $domObjectId = $ajaxArguments[0];
-        $type = $ajaxArguments[1];
+        $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
+        $domObjectId = $ajaxArguments[0] ?? null;
+        $type = $ajaxArguments[1] ?? null;
+        $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
         /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         // Parse the DOM identifier (string), add the levels to the structure stack (array), load the TCA config:
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($ajaxArguments['context']);
+        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
 
-        $jsonArray = false;
+        $jsonArray = [
+            'data' => '',
+            'stylesheetFiles' => [],
+            'scriptCall' => [],
+        ];
         if ($type === 'localize' || $type === 'synchronize' || MathUtility::canBeInterpretedAsInteger($type)) {
             // Parent, this table embeds the child table
             $parent = $inlineStackProcessor->getStructureLevel(-1);
             $parentFieldName = $parent['field'];
 
+            $processedTca = $GLOBALS['TCA'][$parent['table']];
+            $processedTca['columns'][$parentFieldName]['config'] = $parentConfig;
+
             // Child, a record from this table should be rendered
             $child = $inlineStackProcessor->getUnstableStructure();
 
@@ -347,10 +293,7 @@ class FormInlineAjaxController
                 'vanillaUid' => (int)$parent['uid'],
                 'command' => 'edit',
                 'tableName' => $parent['table'],
-                'databaseRow' => [
-                    // TcaInlineExpandCollapseState needs this
-                    'uid' => (int)$parent['uid'],
-                ],
+                'processedTca' => $processedTca,
                 'inlineFirstPid' => $inlineFirstPid,
                 'columnsToProcess' => [
                     $parentFieldName
@@ -384,8 +327,8 @@ class FormInlineAjaxController
                     'language' => $parentLanguage,
                     'ids' => [$type],
                 ];
-            // Either localize or synchronize all child elements from default language of the parent element
             } else {
+                // Either localize or synchronize all child elements from default language of the parent element
                 $cmd[$parent['table']][$parent['uid']]['inlineLocalizeSynchronize'] = [
                     'field' => $parent['field'],
                     'language' => $parentLanguage,
@@ -393,25 +336,36 @@ class FormInlineAjaxController
                 ];
             }
 
-            /** @var $tce DataHandler */
+            /** @var DataHandler $tce */
             $tce = GeneralUtility::makeInstance(DataHandler::class);
-            $tce->stripslashes_values = false;
             $tce->start([], $cmd);
             $tce->process_cmdmap();
 
             $newItemList = $tce->registerDBList[$parent['table']][$parent['uid']][$parentFieldName];
 
-            $jsonArray = [
-                'data' => '',
-                'stylesheetFiles' => [],
-                'scriptCall' => [],
-            ];
             $nameObject = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid);
             $nameObjectForeignTable = $nameObject . '-' . $child['table'];
 
             $oldItems = $this->getInlineRelatedRecordsUidArray($oldItemList);
             $newItems = $this->getInlineRelatedRecordsUidArray($newItemList);
 
+            // Render error messages from DataHandler
+            $tce->printLogErrorMessages();
+            $flashMessageService = GeneralUtility::makeInstance(FlashMessageService::class);
+            $messages = $flashMessageService->getMessageQueueByIdentifier()->getAllMessagesAndFlush();
+            if (!empty($messages)) {
+                foreach ($messages as $message) {
+                    $jsonArray['messages'][] = [
+                        'title'    => $message->getTitle(),
+                        'message'  => $message->getMessage(),
+                        'severity' => $message->getSeverity()
+                    ];
+                    if ($message->getSeverity() === AbstractMessage::ERROR) {
+                        $jsonArray['hasErrors'] = true;
+                    }
+                }
+            }
+
             // Set the items that should be removed in the forms view:
             $removedItems = array_diff($oldItems, $newItems);
             foreach ($removedItems as $childUid) {
@@ -448,30 +402,23 @@ class FormInlineAjaxController
             }
             // Tell JS to add new HTML of one or multiple (localize all) records to DOM
             if (!empty($jsonArray['data'])) {
-                array_push(
-                    $jsonArray['scriptCall'],
-                    'inline.domAddNewRecord(\'bottom\', ' . GeneralUtility::quoteJSvalue($nameObject . '_records')
-                    . ', ' . GeneralUtility::quoteJSvalue($nameObjectForeignTable)
-                    . ', json.data);'
-                );
+                $jsonArray['scriptCall'][] = 'inline.domAddNewRecord(\'bottom\', ' . GeneralUtility::quoteJSvalue($nameObject . '_records')
+                . ', ' . GeneralUtility::quoteJSvalue($nameObjectForeignTable)
+                . ', json.data);';
             }
         }
-
-        $response->getBody()->write(json_encode($jsonArray));
-
-        return $response;
+        return new JsonResponse($jsonArray);
     }
 
     /**
-     * Adds localizations or synchronizes the locations of all child records.
+     * Store status of inline children expand / collapse state in backend user uC.
      *
      * @param ServerRequestInterface $request the incoming request
-     * @param ResponseInterface $response the empty response
      * @return ResponseInterface the filled response
      */
-    public function expandOrCollapseAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function expandOrCollapseAction(ServerRequestInterface $request): ResponseInterface
     {
-        $ajaxArguments = isset($request->getParsedBody()['ajax']) ? $request->getParsedBody()['ajax'] : $request->getQueryParams()['ajax'];
+        $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
         $domObjectId = $ajaxArguments[0];
 
         /** @var InlineStackProcessor $inlineStackProcessor */
@@ -509,9 +456,7 @@ class FormInlineAjaxController
                 $backendUser->writeUC();
             }
         }
-
-        $response->getBody()->write(json_encode([]));
-        return $response;
+        return (new JsonResponse())->setPayload([]);
     }
 
     /**
@@ -547,6 +492,7 @@ class FormInlineAjaxController
             'command' => 'edit',
             'tableName' => $childTableName,
             'vanillaUid' => (int)$childUid,
+            'returnUrl' => $parentData['returnUrl'],
             'isInlineChild' => true,
             'inlineStructure' => $inlineStructure,
             'inlineFirstPid' => $parentData['inlineFirstPid'],
@@ -559,10 +505,10 @@ class FormInlineAjaxController
             'inlineParentTableName' => $parentData['tableName'],
             'inlineParentFieldName' => $parentFieldName,
 
-             // values of the top most parent element set on first level and not overridden on following levels
-            'inlineTopMostParentUid' => $parentData['inlineTopMostParentUid'] ?: $inlineTopMostParent['uid'],
-            'inlineTopMostParentTableName' => $parentData['inlineTopMostParentTableName'] ?: $inlineTopMostParent['table'],
-            'inlineTopMostParentFieldName' => $parentData['inlineTopMostParentFieldName'] ?: $inlineTopMostParent['field'],
+            // values of the top most parent element set on first level and not overridden on following levels
+            'inlineTopMostParentUid' => $inlineTopMostParent['uid'],
+            'inlineTopMostParentTableName' => $inlineTopMostParent['table'],
+            'inlineTopMostParentFieldName' => $inlineTopMostParent['field'],
         ];
         // For foreign_selector with useCombination $mainChild is the mm record
         // and $combinationChild is the child-child. For "normal" relations, $mainChild
@@ -622,7 +568,10 @@ class FormInlineAjaxController
     protected function mergeChildResultIntoJsonResult(array $jsonResult, array $childResult)
     {
         $jsonResult['data'] .= $childResult['html'];
-        $jsonResult['stylesheetFiles'] = $childResult['stylesheetFiles'];
+        $jsonResult['stylesheetFiles'] = [];
+        foreach ($childResult['stylesheetFiles'] as $stylesheetFile) {
+            $jsonResult['stylesheetFiles'][] = $this->getRelativePathToStylesheetFile($stylesheetFile);
+        }
         if (!empty($childResult['inlineData'])) {
             $jsonResult['scriptCall'][] = 'inline.addToDataArray(' . json_encode($childResult['inlineData']) . ');';
         }
@@ -634,13 +583,12 @@ class FormInlineAjaxController
         foreach ($childResult['additionalJavaScriptPost'] as $singleAdditionalJavaScriptPost) {
             $jsonResult['scriptCall'][] = $singleAdditionalJavaScriptPost;
         }
-        $jsonResult['scriptCall'][] = $childResult['extJSCODE'];
         if (!empty($childResult['additionalInlineLanguageLabelFiles'])) {
             $labels = [];
             foreach ($childResult['additionalInlineLanguageLabelFiles'] as $additionalInlineLanguageLabelFile) {
                 ArrayUtility::mergeRecursiveWithOverrule(
                     $labels,
-                    $this->addInlineLanguageLabelFile($additionalInlineLanguageLabelFile)
+                    $this->getLabelsFromLocalizationFile($additionalInlineLanguageLabelFile)
                 );
             }
             $javaScriptCode = [];
@@ -656,72 +604,10 @@ class FormInlineAjaxController
 
             $jsonResult['scriptCall'][] = implode(LF, $javaScriptCode);
         }
-        if (!empty($childResult['requireJsModules'])) {
-            foreach ($childResult['requireJsModules'] as $module) {
-                $moduleName = null;
-                $callback = null;
-                if (is_string($module)) {
-                    // if $module is a string, no callback
-                    $moduleName = $module;
-                    $callback = null;
-                } elseif (is_array($module)) {
-                    // if $module is an array, callback is possible
-                    foreach ($module as $key => $value) {
-                        $moduleName = $key;
-                        $callback = $value;
-                        break;
-                    }
-                }
-                if ($moduleName !== null) {
-                    $inlineCodeKey = $moduleName;
-                    $javaScriptCode = 'require(["' . $moduleName . '"]';
-                    if ($callback !== null) {
-                        $inlineCodeKey .= sha1($callback);
-                        $javaScriptCode .= ', ' . $callback;
-                    }
-                    $javaScriptCode .= ');';
-                    $jsonResult['scriptCall'][] = '/*RequireJS-Module-' . $inlineCodeKey . '*/' . LF . $javaScriptCode;
-                }
-            }
-        }
-        return $jsonResult;
-    }
+        $requireJsModule = $this->createExecutableStringRepresentationOfRegisteredRequireJsModules($childResult);
+        $jsonResult['scriptCall'] = array_merge($requireJsModule, $jsonResult['scriptCall']);
 
-    /**
-     * @param string $file
-     *
-     * @return array
-     */
-    protected function addInlineLanguageLabelFile($file)
-    {
-        /** @var $languageFactory LocalizationFactory */
-        $languageFactory = GeneralUtility::makeInstance(LocalizationFactory::class);
-        $language = $GLOBALS['LANG']->lang;
-        $localizationArray = $languageFactory->getParsedData(
-            $file,
-            $language,
-            'utf-8',
-            1
-        );
-        if (is_array($localizationArray) && !empty($localizationArray)) {
-            if (!empty($localizationArray[$language])) {
-                $xlfLabelArray = $localizationArray['default'];
-                ArrayUtility::mergeRecursiveWithOverrule($xlfLabelArray, $localizationArray[$language], true, false);
-            } else {
-                $xlfLabelArray = $localizationArray['default'];
-            }
-        } else {
-            $xlfLabelArray = [];
-        }
-        $labelArray = [];
-        foreach ($xlfLabelArray as $key => $value) {
-            if (isset($value[0]['target'])) {
-                $labelArray[$key] = $value[0]['target'];
-            } else {
-                $labelArray[$key] = '';
-            }
-        }
-        return $labelArray;
+        return $jsonResult;
     }
 
     /**
@@ -744,29 +630,6 @@ class FormInlineAjaxController
         return $itemArray;
     }
 
-    /**
-     * Checks if a record selector may select a certain file type
-     *
-     * @param array $selectorConfiguration
-     * @param array $fileRecord
-     * @return bool
-     * @todo: check this ...
-     */
-    protected function checkInlineFileTypeAccessForField(array $selectorConfiguration, array $fileRecord)
-    {
-        if (!empty($selectorConfiguration['PA']['fieldConf']['config']['appearance']['elementBrowserAllowed'])) {
-            $allowedFileExtensions = GeneralUtility::trimExplode(
-                ',',
-                $selectorConfiguration['PA']['fieldConf']['config']['appearance']['elementBrowserAllowed'],
-                true
-            );
-            if (!in_array(strtolower($fileRecord['extension']), $allowedFileExtensions, true)) {
-                return false;
-            }
-        }
-        return true;
-    }
-
     /**
      * Return expand / collapse state array for a given table / uid combination
      *
@@ -823,10 +686,10 @@ class FormInlineAjaxController
      *
      * @param mixed $needle The element to be removed.
      * @param array $haystack The array the element should be removed from.
-     * @param mixed $strict Search elements strictly.
+     * @param bool $strict Search elements strictly.
      * @return array The array $haystack without the $needle
      */
-    protected function removeFromArray($needle, $haystack, $strict = null)
+    protected function removeFromArray($needle, $haystack, $strict = false)
     {
         $pos = array_search($needle, $haystack, $strict);
         if ($pos !== false) {
@@ -855,7 +718,7 @@ class FormInlineAjaxController
      * Get inlineFirstPid from a given objectId string
      *
      * @param string $domObjectId The id attribute of an element
-     * @return int|NULL Pid or null
+     * @return int|null Pid or null
      */
     protected function getInlineFirstPidFromDomObjectId($domObjectId)
     {
@@ -869,6 +732,29 @@ class FormInlineAjaxController
         return null;
     }
 
+    /**
+     * Validates the config that is transferred over the wire to provide the
+     * correct TCA config for the parent table
+     *
+     * @param string $contextString
+     * @throws \RuntimeException
+     * @return array
+     */
+    protected function extractSignedParentConfigFromRequest(string $contextString): array
+    {
+        if ($contextString === '') {
+            throw new \RuntimeException('Empty context string given', 1489751361);
+        }
+        $context = json_decode($contextString, true);
+        if (empty($context['config'])) {
+            throw new \RuntimeException('Empty context config section given', 1489751362);
+        }
+        if (!hash_equals(GeneralUtility::hmac(json_encode($context['config']), 'InlineContext'), $context['hmac'])) {
+            throw new \RuntimeException('Hash does not validate', 1489751363);
+        }
+        return $context['config'];
+    }
+
     /**
      * @return BackendUserAuthentication
      */
@@ -876,122 +762,4 @@ class FormInlineAjaxController
     {
         return $GLOBALS['BE_USER'];
     }
-
-    /**
-     * Extract the inline child table configuration from the flexform data structure
-     * using the the domObjectId to traverse the XML structure.
-     *
-     * domObjectId parsing has been copied from InlineStackProcessor::initializeByDomObjectId
-     *
-     * @param array $parentConfig
-     * @param string $domObjectId
-     * @return array
-     */
-    protected function getParentConfigFromFlexForm(array $parentConfig, $domObjectId)
-    {
-        list($flexFormPath, $foreignTableName) = $this->splitDomObjectId($domObjectId);
-
-        $childConfig = $parentConfig['ds']['sheets'];
-        $flexFormPath = explode(':', $flexFormPath);
-        foreach ($flexFormPath as $flexFormNode) {
-            // We are dealing with configuration information from a flexform,
-            // not value storage, identifiers that reference language or
-            // value nodes must be skipped.
-            if (!isset($childConfig[$flexFormNode]) && preg_match('/^[lv][[:alpha:]]+$/', $flexFormNode)) {
-                continue;
-            }
-            $childConfig = $childConfig[$flexFormNode];
-
-            // Skip to the field configuration of a sheet
-            if (isset($childConfig['ROOT']) && $childConfig['ROOT']['type'] == 'array') {
-                $childConfig = $childConfig['ROOT']['el'];
-            }
-        }
-
-        if (!isset($childConfig['config'])
-            || !is_array($childConfig['config'])
-            || $childConfig['config']['type'] !== 'inline'
-            || $childConfig['config']['foreign_table'] !== $foreignTableName
-        ) {
-            throw new \UnexpectedValueException(
-                'Configuration retrieved from FlexForm is incomplete or not of type "inline".',
-                1446996319
-            );
-        }
-        return $childConfig['config'];
-    }
-
-    /**
-     * Flexforms require additional database columns to be processed to determine the correct
-     * data structure to be used from a flexform. The required columns and their values are
-     * transmitted in the AJAX context of the request and need to be added to the fake database
-     * row for the inline parent.
-     *
-     * @param array $ajaxArguments The AJAX request arguments
-     * @param array $databaseRow The fake database row
-     * @return array The database row with the flexform data structure pointer columns added
-     */
-    protected function addFlexFormDataStructurePointersFromAjaxContext(array $ajaxArguments, array $databaseRow)
-    {
-        if (!isset($ajaxArguments['context'])) {
-            return $databaseRow;
-        }
-
-        $context = json_decode($ajaxArguments['context'], true);
-        if (GeneralUtility::hmac(serialize($context['config'])) !== $context['hmac']) {
-            return $databaseRow;
-        }
-
-        if (isset($context['config']['flexDataStructurePointers'])
-            && is_array($context['config']['flexDataStructurePointers'])
-        ) {
-            $databaseRow = array_merge($context['config']['flexDataStructurePointers'], $databaseRow);
-        }
-
-        return $databaseRow;
-    }
-
-    /**
-     * split the domObjectID and retrieve the needed parts
-     *
-     * @param string $domObjectId
-     *
-     * @return array
-     */
-    protected function splitDomObjectId($domObjectId)
-    {
-
-        // Substitute FlexForm addition and make parsing a bit easier
-        $domObjectId = str_replace('---', ':', $domObjectId);
-        $pattern = '/:data:(?<flexformPath>.*?)-(?<tableName>[^-]+)(?:-(?:NEW)?\w+)?$/';
-
-        /* EXPLANATION for the regex:
-         * according https://regex101.com/
-         *
-         * :data: matches the characters :data: literally (case sensitive)
-         * (?<flexformPath>.*?) Named capturing group flexformPath
-         * .*? matches any character (except newline)
-         * Quantifier: *? Between zero and unlimited times, as few times as possible, expanding as needed [lazy]
-         * - matches the character - literally
-         * (?<tableName>[^-]+) Named capturing group tableName
-         * [^-]+ match a single character not present in the list below
-         * Quantifier: + Between one and unlimited times, as many times as possible, giving back as needed [greedy]
-         * - the literal character -
-         * (?:-(?:NEW)?\w+)? Non-capturing group
-         * Quantifier: ? Between zero and one time, as many times as possible, giving back as needed [greedy]
-         * - matches the character - literally
-         * (?:NEW)? Non-capturing group
-         * Quantifier: ? Between zero and one time, as many times as possible, giving back as needed [greedy]
-         * NEW matches the characters NEW literally (case sensitive)
-         * \w+ match any word character [a-zA-Z0-9_]
-         * Quantifier: + Between one and unlimited times, as many times as possible, giving back as needed [greedy]
-         * $ assert position at end of a line
-         */
-
-        if (preg_match($pattern, $domObjectId, $match)) {
-            return [$match['flexformPath'], $match['tableName']];
-        }
-
-        return [];
-    }
 }
