@@ -1,4 +1,5 @@
 <?php
+declare(strict_types = 1);
 namespace TYPO3\CMS\Backend\Controller;
 
 /*
@@ -16,12 +17,23 @@ namespace TYPO3\CMS\Backend\Controller;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
+use Psr\Log\LoggerAwareInterface;
+use Psr\Log\LoggerAwareTrait;
+use Symfony\Component\HttpFoundation\Cookie;
 use TYPO3\CMS\Backend\Exception;
 use TYPO3\CMS\Backend\LoginProvider\LoginProviderInterface;
+use TYPO3\CMS\Backend\Routing\UriBuilder;
+use TYPO3\CMS\Backend\Template\DocumentTemplate;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
+use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\FormProtection\BackendFormProtection;
 use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
+use TYPO3\CMS\Core\Http\HtmlResponse;
+use TYPO3\CMS\Core\Http\NormalizedParams;
+use TYPO3\CMS\Core\Localization\LanguageService;
+use TYPO3\CMS\Core\Localization\Locales;
 use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\HttpUtility;
@@ -30,9 +42,12 @@ use TYPO3\CMS\Fluid\View\StandaloneView;
 
 /**
  * Script Class for rendering the login form
+ * @internal This class is a specific Backend controller implementation and is not considered part of the Public TYPO3 API.
  */
-class LoginController
+class LoginController implements LoggerAwareInterface
 {
+    use LoggerAwareTrait;
+
     /**
      * The URL to redirect to after login.
      *
@@ -52,7 +67,7 @@ class LoginController
      *
      * @var string
      */
-    protected $loginProviderIdentifier = null;
+    protected $loginProviderIdentifier;
 
     /**
      * List of registered and sorted login providers
@@ -87,20 +102,24 @@ class LoginController
      */
     public function __construct()
     {
+        // @deprecated since TYPO3 v9, will be obsolete in TYPO3 v10.0
+        $request = $GLOBALS['TYPO3_REQUEST'];
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
         $this->validateAndSortLoginProviders();
 
-        // We need a PHP session session for most login levels
-        session_start();
-        $this->redirectUrl = GeneralUtility::sanitizeLocalUrl(GeneralUtility::_GP('redirect_url'));
-        $this->loginProviderIdentifier = $this->detectLoginProvider();
+        $this->redirectUrl = GeneralUtility::sanitizeLocalUrl($parsedBody['redirect_url'] ?? $queryParams['redirect_url'] ?? null);
+        $this->loginProviderIdentifier = $this->detectLoginProvider($request);
 
-        $this->loginRefresh = (bool)GeneralUtility::_GP('loginRefresh');
+        $this->loginRefresh = (bool)($parsedBody['loginRefresh'] ?? $queryParams['loginRefresh'] ?? false);
         // Value of "Login" button. If set, the login button was pressed.
-        $this->submitValue = GeneralUtility::_GP('commandLI');
-
+        $this->submitValue = $parsedBody['commandLI'] ?? $queryParams['commandLI'] ?? null;
         // Try to get the preferred browser language
-        $preferredBrowserLanguage = $this->getLanguageService()->csConvObj
-            ->getPreferredClientLanguage(GeneralUtility::getIndpEnv('HTTP_ACCEPT_LANGUAGE'));
+        /** @var Locales $locales */
+        $locales = GeneralUtility::makeInstance(Locales::class);
+        $httpAcceptLanguage = $request->getServerParams()['HTTP_ACCEPT_LANGUAGE'];
+        $preferredBrowserLanguage = $locales
+            ->getPreferredClientLanguage($httpAcceptLanguage);
 
         // If we found a $preferredBrowserLanguage and it is not the default language and no be_user is logged in
         // initialize $this->getLanguageService() again with $preferredBrowserLanguage
@@ -109,15 +128,20 @@ class LoginController
             GeneralUtility::makeInstance(PageRenderer::class)->setLanguage($preferredBrowserLanguage);
         }
 
-        $this->getLanguageService()->includeLLFile('EXT:lang/locallang_login.xlf');
+        $this->getLanguageService()->includeLLFile('EXT:backend/Resources/Private/Language/locallang_login.xlf');
 
         // Setting the redirect URL to "index.php?M=main" if no alternative input is given
-        $this->redirectToURL = $this->redirectUrl ?: BackendUtility::getModuleUrl('main');
+        if ($this->redirectUrl) {
+            $this->redirectToURL = $this->redirectUrl;
+        } else {
+            $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
+            $this->redirectToURL = (string)$uriBuilder->buildUriFromRoute('main');
+        }
 
         // If "L" is "OUT", then any logged in is logged out. If redirect_url is given, we redirect to it
-        if (GeneralUtility::_GP('L') === 'OUT' && is_object($this->getBackendUserAuthentication())) {
+        if (($parsedBody['L'] ?? $queryParams['L'] ?? null) === 'OUT' && is_object($this->getBackendUserAuthentication())) {
             $this->getBackendUserAuthentication()->logoff();
-            HttpUtility::redirect($this->redirectUrl);
+            $this->redirectToUrl();
         }
 
         $this->view = $this->getFluidTemplateObject();
@@ -128,14 +152,23 @@ class LoginController
      * As this controller goes only through the main() method, it is rather simple for now
      *
      * @param ServerRequestInterface $request the current request
-     * @param ResponseInterface $response the current response
      * @return ResponseInterface the finished response with the content
      */
-    public function formAction(ServerRequestInterface $request, ResponseInterface $response)
+    public function formAction(ServerRequestInterface $request): ResponseInterface
     {
-        $content = $this->main();
-        $response->getBody()->write($content);
-        return $response;
+        return new HtmlResponse($this->createLoginLogoutForm($request));
+    }
+
+    /**
+     * Calls the main function but with loginRefresh enabled at any time
+     *
+     * @param ServerRequestInterface $request the current request
+     * @return ResponseInterface the finished response with the content
+     */
+    public function refreshAction(ServerRequestInterface $request): ResponseInterface
+    {
+        $this->loginRefresh = true;
+        return new HtmlResponse($this->createLoginLogoutForm($request));
     }
 
     /**
@@ -143,39 +176,57 @@ class LoginController
      *
      * @throws Exception
      * @return string The content to output
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
-    public function main()
+    public function main(): string
     {
-        /** @var $pageRenderer PageRenderer */
+        trigger_error('LoginController->main() will be replaced by protected method createLoginLogoutForm() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        return $this->createLoginLogoutForm($GLOBALS['TYPO3_REQUEST']);
+    }
+
+    /**
+     * Main function - creating the login/logout form
+     *
+     * @param ServerRequestInterface $request
+     * @return string $content
+     * @throws Exception
+     */
+    protected function createLoginLogoutForm(ServerRequestInterface $request): string
+    {
+        /** @var PageRenderer $pageRenderer */
         $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
         $pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Login');
 
-        // support placeholders for IE9 and lower
-        $clientInfo = GeneralUtility::clientInfo();
-        if ($clientInfo['BROWSER'] === 'msie' && $clientInfo['VERSION'] <= 9) {
-            $pageRenderer->addJsLibrary('placeholders', 'sysext/core/Resources/Public/JavaScript/Contrib/placeholders.min.js');
-        }
-
         // Checking, if we should make a redirect.
         // Might set JavaScript in the header to close window.
-        $this->checkRedirect();
+        $this->checkRedirect($request);
 
         // Extension Configuration
-        $extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['backend']);
+        $extConf = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('backend');
 
         // Background Image
         if (!empty($extConf['loginBackgroundImage'])) {
             $backgroundImage = $this->getUriForFileName($extConf['loginBackgroundImage']);
+            if ($backgroundImage === '') {
+                $this->logger->warning(
+                    'The configured TYPO3 backend login background image "' . htmlspecialchars($extConf['loginBackgroundImage']) .
+                    '" can\'t be resolved. Please check if the file exists and the extension is activated.'
+                );
+            }
             $this->getDocumentTemplate()->inDocStylesArray[] = '
-				@media (min-width: 768px){
-					.typo3-login-carousel-control.right,
-					.typo3-login-carousel-control.left,
-					.panel-login { border: 0; }
-					.typo3-login { background-image: url("' . $backgroundImage . '"); }
-				}
+				.typo3-login-carousel-control.right,
+				.typo3-login-carousel-control.left,
+				.panel-login { border: 0; }
+				.typo3-login { background-image: url("' . $backgroundImage . '"); }
+				.typo3-login-footnote { background-color: #000000; color: #ffffff; opacity: 0.5; }
 			';
         }
 
+        // Login Footnote
+        if (!empty($extConf['loginFootnote'])) {
+            $this->view->assign('loginFootnote', strip_tags(trim($extConf['loginFootnote'])));
+        }
+
         // Add additional css to use the highlight color in the login screen
         if (!empty($extConf['loginHighlightColor'])) {
             $this->getDocumentTemplate()->inDocStylesArray[] = '
@@ -194,11 +245,13 @@ class LoginController
 
         // Logo
         if (!empty($extConf['loginLogo'])) {
+            if ($this->getUriForFileName($extConf['loginLogo']) === '') {
+                $this->logger->warning(
+                    'The configured TYPO3 backend login logo "' . htmlspecialchars($extConf['loginLogo']) .
+                    '" can\'t be resolved. Please check if the file exists and the extension is activated.'
+                );
+            }
             $logo = $extConf['loginLogo'];
-        } elseif (!empty($GLOBALS['TBE_STYLES']['logo_login'])) {
-            // Fallback to old TBE_STYLES login logo
-            $logo = $GLOBALS['TBE_STYLES']['logo_login'];
-            GeneralUtility::deprecationLog('$GLOBALS["TBE_STYLES"]["logo_login"] is deprecated since TYPO3 CMS 7 and will be removed in TYPO3 CMS 8, please use the backend extension\'s configuration instead.');
         } else {
             // Use TYPO3 logo depending on highlight color
             if (!empty($extConf['loginHighlightColor'])) {
@@ -207,7 +260,7 @@ class LoginController
                 $logo = 'EXT:backend/Resources/Public/Images/typo3_orange.svg';
             }
             $this->getDocumentTemplate()->inDocStylesArray[] = '
-				.typo3-login-logo .typo3-login-image { max-width: 150px; }
+				.typo3-login-logo .typo3-login-image { max-width: 150px; height:100%;}
 			';
         }
         $logo = $this->getUriForFileName($logo);
@@ -216,7 +269,7 @@ class LoginController
         $formType = empty($this->getBackendUserAuthentication()->user['uid']) ? 'LoginForm' : 'LogoutForm';
         $this->view->assignMultiple([
             'backendUser' => $this->getBackendUserAuthentication()->user,
-            'hasLoginError' => $this->isLoginInProgress(),
+            'hasLoginError' => $this->isLoginInProgress($request),
             'formType' => $formType,
             'logo' => $logo,
             'images' => [
@@ -224,13 +277,15 @@ class LoginController
                 'typo3' => $this->getUriForFileName('EXT:backend/Resources/Public/Images/typo3_orange.svg'),
             ],
             'copyright' => BackendUtility::TYPO3_copyRightNotice(),
+            'redirectUrl' => $this->redirectUrl,
+            'loginRefresh' => $this->loginRefresh,
             'loginNewsItems' => $this->getSystemNews(),
             'loginProviderIdentifier' => $this->loginProviderIdentifier,
             'loginProviders' => $this->loginProviders
         ]);
 
         // Initialize interface selectors:
-        $this->makeInterfaceSelectorBox();
+        $this->makeInterfaceSelector($request);
 
         /** @var LoginProviderInterface $loginProvider */
         $loginProvider = GeneralUtility::makeInstance($this->loginProviders[$this->loginProviderIdentifier]['provider']);
@@ -246,21 +301,17 @@ class LoginController
     /**
      * Checking, if we should perform some sort of redirection OR closing of windows.
      *
-     * Do redirect:
-     *
-     * If a user is logged in AND
-     *   a) if either the login is just done (isLoginInProgress) or
-     *   b) a loginRefresh is done
+     * Do a redirect if a user is logged in
      *
+     * @param ServerRequestInterface $request
      * @throws \RuntimeException
      * @throws \UnexpectedValueException
+     * @throws \TYPO3\CMS\Backend\Routing\Exception\RouteNotFoundException
      */
-    protected function checkRedirect()
+    protected function checkRedirect(ServerRequestInterface $request): void
     {
-        if (
-            empty($this->getBackendUserAuthentication()->user['uid'])
-            && ($this->isLoginInProgress() || !$this->loginRefresh)
-        ) {
+        $backendUser = $this->getBackendUserAuthentication();
+        if (empty($backendUser->user['uid'])) {
             return;
         }
 
@@ -269,7 +320,7 @@ class LoginController
          * This assumes that a cookie-setting script (like this one) has been hit at
          * least once prior to this instance.
          */
-        if (!$_COOKIE[BackendUserAuthentication::getCookieName()]) {
+        if (!isset($_COOKIE[BackendUserAuthentication::getCookieName()])) {
             if ($this->submitValue === 'setCookie') {
                 /*
                  * we tried it a second time but still no cookie
@@ -278,33 +329,32 @@ class LoginController
                  */
                 throw new \RuntimeException('Login-error: Yeah, that\'s a classic. No cookies, no TYPO3. ' .
                     'Please accept cookies from TYPO3 - otherwise you\'ll not be able to use the system.', 1294586846);
-            } else {
-                // try it once again - that might be needed for auto login
-                $this->redirectToURL = 'index.php?commandLI=setCookie';
             }
+            // try it once again - that might be needed for auto login
+            $this->redirectToURL = 'index.php?commandLI=setCookie';
         }
-        $redirectToUrl = (string)$this->getBackendUserAuthentication()->getTSConfigVal('auth.BE.redirectToURL');
+        $redirectToUrl = (string)($backendUser->getTSConfig()['auth.']['BE.']['redirectToURL'] ?? '');
         if (empty($redirectToUrl)) {
             // Based on the interface we set the redirect script
-            switch (GeneralUtility::_GP('interface')) {
+            $parsedBody = $request->getParsedBody();
+            $queryParams = $request->getQueryParams();
+            $interface = $parsedBody['interface'] ?? $queryParams['interface'] ?? '';
+            switch ($interface) {
                 case 'frontend':
-                    $interface = 'frontend';
                     $this->redirectToURL = '../';
                     break;
                 case 'backend':
-                    $interface = 'backend';
-                    $this->redirectToURL = BackendUtility::getModuleUrl('main');
+                    $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
+                    $this->redirectToURL = (string)$uriBuilder->buildUriFromRoute('main');
                     break;
-                default:
-                    $interface = '';
             }
         } else {
             $this->redirectToURL = $redirectToUrl;
             $interface = '';
         }
         // store interface
-        $this->getBackendUserAuthentication()->uc['interfaceSetup'] = $interface;
-        $this->getBackendUserAuthentication()->writeUC();
+        $backendUser->uc['interfaceSetup'] = $interface;
+        $backendUser->writeUC();
 
         $formProtection = FormProtectionFactory::get();
         if (!$formProtection instanceof BackendFormProtection) {
@@ -313,34 +363,45 @@ class LoginController
         if ($this->loginRefresh) {
             $formProtection->setSessionTokenFromRegistry();
             $formProtection->persistSessionToken();
-            $this->getDocumentTemplate()->JScode .= $this->getDocumentTemplate()->wrapScriptTags('
-				if (parent.opener && parent.opener.TYPO3 && parent.opener.TYPO3.LoginRefresh) {
-					parent.opener.TYPO3.LoginRefresh.startTask();
-					parent.close();
+            $this->getDocumentTemplate()->JScode .= GeneralUtility::wrapJS('
+				if (window.opener && window.opener.TYPO3 && window.opener.TYPO3.LoginRefresh) {
+					window.opener.TYPO3.LoginRefresh.startTask();
+					window.close();
 				}
 			');
         } else {
             $formProtection->storeSessionTokenInRegistry();
-            HttpUtility::redirect($this->redirectToURL);
+            $this->redirectToUrl();
         }
     }
 
     /**
-     * Making interface selector:
+     * Making interface selector
      *
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
-    public function makeInterfaceSelectorBox()
+    public function makeInterfaceSelectorBox(): void
+    {
+        trigger_error('LoginController->makeInterfaceSelectorBox() will be replaced by protected method makeInterfaceSelector() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
+        $this->makeInterfaceSelector($GLOBALS['TYPO3_REQUEST']);
+    }
+
+    /**
+     * Making interface selector
+     * @param ServerRequestInterface $request
+     */
+    protected function makeInterfaceSelector(ServerRequestInterface $request): void
     {
         // If interfaces are defined AND no input redirect URL in GET vars:
-        if ($GLOBALS['TYPO3_CONF_VARS']['BE']['interfaces'] && ($this->isLoginInProgress() || !$this->redirectUrl)) {
+        if ($GLOBALS['TYPO3_CONF_VARS']['BE']['interfaces'] && ($this->isLoginInProgress($request) || !$this->redirectUrl)) {
             $parts = GeneralUtility::trimExplode(',', $GLOBALS['TYPO3_CONF_VARS']['BE']['interfaces']);
             if (count($parts) > 1) {
                 // Only if more than one interface is defined we will show the selector
+                $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
                 $interfaces = [
                     'backend' => [
                         'label' => $this->getLanguageService()->getLL('interface.backend'),
-                        'jumpScript' => BackendUtility::getModuleUrl('main'),
+                        'jumpScript' => (string)$uriBuilder->buildUriFromRoute('main'),
                         'interface' => 'backend'
                     ],
                     'frontend' => [
@@ -366,14 +427,21 @@ class LoginController
      *
      * @return array An array of login news.
      */
-    protected function getSystemNews()
+    protected function getSystemNews(): array
     {
         $systemNewsTable = 'sys_news';
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable($systemNewsTable);
         $systemNews = [];
-        $systemNewsRecords = $this->getDatabaseConnection()->exec_SELECTgetRows('title, content, crdate', $systemNewsTable, '1=1' . BackendUtility::BEenableFields($systemNewsTable) . BackendUtility::deleteClause($systemNewsTable), '', 'crdate DESC');
+        $systemNewsRecords = $queryBuilder
+            ->select('title', 'content', 'crdate')
+            ->from($systemNewsTable)
+            ->orderBy('crdate', 'DESC')
+            ->execute()
+            ->fetchAll();
         foreach ($systemNewsRecords as $systemNewsRecord) {
             $systemNews[] = [
-                'date' => date($GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'], $systemNewsRecord['crdate']),
+                'date' => date($GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'], (int)$systemNewsRecord['crdate']),
                 'header' => $systemNewsRecord['title'],
                 'content' => $systemNewsRecord['content']
             ];
@@ -384,38 +452,37 @@ class LoginController
     /**
      * Returns the uri of a relative reference, resolves the "EXT:" prefix
      * (way of referring to files inside extensions) and checks that the file is inside
-     * the PATH_site of the TYPO3 installation
+     * the project root of the TYPO3 installation
      *
      * @param string $filename The input filename/filepath to evaluate
      * @return string Returns the filename of $filename if valid, otherwise blank string.
      * @internal
      */
-    private function getUriForFileName($filename)
+    private function getUriForFileName($filename): string
     {
-        if (strpos($filename, '://')) {
+        // Check if it's already a URL
+        if (preg_match('/^(https?:)?\/\//', $filename)) {
             return $filename;
         }
-        $urlPrefix = '';
-        if (strpos($filename, 'EXT:') === 0) {
-            $absoluteFilename = GeneralUtility::getFileAbsFileName($filename);
-            $filename = '';
-            if ($absoluteFilename !== '') {
-                $filename = PathUtility::getAbsoluteWebPath($absoluteFilename);
-            }
-        } elseif (strpos($filename, '/') !== 0) {
-            $urlPrefix = GeneralUtility::getIndpEnv('TYPO3_SITE_PATH');
+        $absoluteFilename = GeneralUtility::getFileAbsFileName(ltrim($filename, '/'));
+        $filename = '';
+        if ($absoluteFilename !== '' && @is_file($absoluteFilename)) {
+            $filename = PathUtility::getAbsoluteWebPath($absoluteFilename);
         }
-        return $urlPrefix . $filename;
+        return $filename;
     }
 
     /**
      * Checks if login credentials are currently submitted
      *
+     * @param ServerRequestInterface $request
      * @return bool
      */
-    protected function isLoginInProgress()
+    protected function isLoginInProgress(ServerRequestInterface $request): bool
     {
-        $username = GeneralUtility::_GP('username');
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
+        $username = $parsedBody['username'] ?? $queryParams['username'] ?? null;
         return !empty($username) || !empty($this->submitValue);
     }
 
@@ -436,6 +503,14 @@ class LoginController
         return $view;
     }
 
+    /**
+     * Wrapper method to redirect to configured redirect URL
+     */
+    protected function redirectToUrl(): void
+    {
+        HttpUtility::redirect($this->redirectToURL);
+    }
+
     /**
      * Validates the registered login providers
      *
@@ -443,20 +518,16 @@ class LoginController
      */
     protected function validateAndSortLoginProviders()
     {
-        if (
-            !isset($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['backend']['loginProviders'])
-            || !is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['backend']['loginProviders'])
-            || empty($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['backend']['loginProviders'])
-        ) {
+        $providers = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['backend']['loginProviders'] ?? [];
+        if (empty($providers) || !is_array($providers)) {
             throw new \RuntimeException('No login providers are registered in $GLOBALS[\'TYPO3_CONF_VARS\'][\'EXTCONF\'][\'backend\'][\'loginProviders\'].', 1433417281);
         }
-        $providers = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['backend']['loginProviders'];
         foreach ($providers as $identifier => $configuration) {
             if (empty($configuration) || !is_array($configuration)) {
                 throw new \RuntimeException('Missing configuration for login provider "' . $identifier . '".', 1433416043);
             }
             if (!is_string($configuration['provider']) || empty($configuration['provider']) || !class_exists($configuration['provider']) || !is_subclass_of($configuration['provider'], LoginProviderInterface::class)) {
-                throw new \RuntimeException('The login provider "' . $identifier . '" defines an invalid provider. Ensure the class exists and implements the "' . LoginProviderInterface::class . '".', 1433416043);
+                throw new \RuntimeException('The login provider "' . $identifier . '" defines an invalid provider. Ensure the class exists and implements the "' . LoginProviderInterface::class . '".', 1460977275);
             }
             if (empty($configuration['label'])) {
                 throw new \RuntimeException('Missing label definition for login provider "' . $identifier . '".', 1433416044);
@@ -479,11 +550,14 @@ class LoginController
      * Detect the login provider, get from request or choose the
      * first one as default
      *
+     * @param ServerRequestInterface $request
      * @return string
      */
-    protected function detectLoginProvider()
+    protected function detectLoginProvider(ServerRequestInterface $request): string
     {
-        $loginProvider = GeneralUtility::_GP('loginProvider');
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
+        $loginProvider = $parsedBody['loginProvider'] ?? $queryParams['loginProvider'] ?? '';
         if ((empty($loginProvider) || !isset($this->loginProviders[$loginProvider])) && !empty($_COOKIE['be_lastLoginProvider'])) {
             $loginProvider = $_COOKIE['be_lastLoginProvider'];
         }
@@ -491,8 +565,25 @@ class LoginController
             reset($this->loginProviders);
             $loginProvider = key($this->loginProviders);
         }
-        setcookie('be_lastLoginProvider', $loginProvider, $GLOBALS['EXEC_TIME'] + 7776000); // 90 days
-        return $loginProvider;
+        // Use the secure option when the current request is served by a secure connection
+        /** @var NormalizedParams $normalizedParams */
+        $normalizedParams = $request->getAttribute('normalizedParams');
+        $isHttps = $normalizedParams->isHttps();
+        $cookieSecure = (bool)$GLOBALS['TYPO3_CONF_VARS']['SYS']['cookieSecure'] && $isHttps;
+        $cookie = new Cookie(
+            'be_lastLoginProvider',
+            (string)$loginProvider,
+            $GLOBALS['EXEC_TIME'] + 7776000, // 90 days
+            $normalizedParams->getSitePath() . TYPO3_mainDir,
+            '',
+            $cookieSecure,
+            true,
+            false,
+            Cookie::SAMESITE_STRICT
+        );
+        header('Set-Cookie: ' . $cookie->__toString(), false);
+
+        return (string)$loginProvider;
     }
 
     /**
@@ -506,9 +597,9 @@ class LoginController
     /**
      * Returns LanguageService
      *
-     * @return \TYPO3\CMS\Lang\LanguageService
+     * @return LanguageService
      */
-    protected function getLanguageService()
+    protected function getLanguageService(): LanguageService
     {
         return $GLOBALS['LANG'];
     }
@@ -516,27 +607,17 @@ class LoginController
     /**
      * @return BackendUserAuthentication
      */
-    protected function getBackendUserAuthentication()
+    protected function getBackendUserAuthentication(): BackendUserAuthentication
     {
         return $GLOBALS['BE_USER'];
     }
 
-    /**
-     * Returns the database connection
-     *
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
-
     /**
      * Returns an instance of DocumentTemplate
      *
-     * @return \TYPO3\CMS\Backend\Template\DocumentTemplate
+     * @return DocumentTemplate
      */
-    protected function getDocumentTemplate()
+    protected function getDocumentTemplate(): DocumentTemplate
     {
         return $GLOBALS['TBE_TEMPLATE'];
     }
