@@ -1,4 +1,5 @@
 <?php
+declare(strict_types = 1);
 namespace TYPO3\CMS\Backend\Controller\Wizard;
 
 /*
@@ -17,34 +18,57 @@ namespace TYPO3\CMS\Backend\Controller\Wizard;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Backend\Template\Components\ButtonBar;
+use TYPO3\CMS\Backend\Template\ModuleTemplate;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Core\Compatibility\PublicPropertyDeprecationTrait;
 use TYPO3\CMS\Core\Configuration\FlexForm\FlexFormTools;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
+use TYPO3\CMS\Core\Http\HtmlResponse;
+use TYPO3\CMS\Core\Http\RedirectResponse;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\HttpUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
 
 /**
  * Script Class for rendering the Table Wizard
+ * @internal This class is a specific Backend controller implementation and is not considered part of the Public TYPO3 API.
  */
 class TableController extends AbstractWizardController
 {
+    use PublicPropertyDeprecationTrait;
+
+    /**
+     * Properties which have been moved to protected status from public
+     *
+     * @var array
+     */
+    protected $deprecatedPublicProperties = [
+        'content' => 'Using $content of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'inputStyle' => 'Using $inputStyle of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'xmlStorage' => 'Using $xmlStorage of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'numNewRows' => 'Using $numNewRows of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'colsFieldName' => 'Using $colsFieldName of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'P' => 'Using $P of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'TABLECFG' => 'Using $TABLECFG of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'tableParsing_quote' => 'Using $tableParsing_quote of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+        'tableParsing_delimiter' => 'Using $tableParsing_delimiter of class TableController from the outside is discouraged, as this variable is only used for internal storage.',
+     ];
+
     /**
      * Content accumulation for the module.
      *
      * @var string
      */
-    public $content;
+    protected $content;
 
     /**
      * If TRUE, <input> fields are shown instead of textareas.
      *
      * @var bool
      */
-    public $inputStyle = false;
+    protected $inputStyle = false;
 
     /**
      * If set, the string version of the content is interpreted/written as XML
@@ -53,14 +77,14 @@ class TableController extends AbstractWizardController
      *
      * @var int
      */
-    public $xmlStorage = 0;
+    protected $xmlStorage = 0;
 
     /**
      * Number of new rows to add in bottom of wizard
      *
      * @var int
      */
-    public $numNewRows = 1;
+    protected $numNewRows = 1;
 
     /**
      * Name of field in parent record which MAY contain the number of columns for the table
@@ -68,21 +92,21 @@ class TableController extends AbstractWizardController
      *
      * @var string
      */
-    public $colsFieldName = 'cols';
+    protected $colsFieldName = 'cols';
 
     /**
      * Wizard parameters, coming from FormEngine linking to the wizard.
      *
      * @var array
      */
-    public $P;
+    protected $P;
 
     /**
      * The array which is constantly submitted by the multidimensional form of this wizard.
      *
      * @var array
      */
-    public $TABLECFG;
+    protected $TABLECFG;
 
     /**
      * Table parsing
@@ -90,42 +114,188 @@ class TableController extends AbstractWizardController
      *
      * @var string
      */
-    public $tableParsing_quote;
+    protected $tableParsing_quote;
 
     /**
      * delimiter between table cells
      *
      * @var string
      */
-    public $tableParsing_delimiter;
+    protected $tableParsing_delimiter;
 
     /**
      * @var IconFactory
      */
     protected $iconFactory;
 
+    /**
+     * ModuleTemplate object
+     *
+     * @var ModuleTemplate
+     */
+    protected $moduleTemplate;
+
     /**
      * Constructor
      */
     public function __construct()
     {
-        parent::__construct();
-        $this->getLanguageService()->includeLLFile('EXT:lang/locallang_wizards.xlf');
-        $GLOBALS['SOBE'] = $this;
+        $this->moduleTemplate = GeneralUtility::makeInstance(ModuleTemplate::class);
+        $this->getLanguageService()->includeLLFile('EXT:core/Resources/Private/Language/locallang_wizards.xlf');
 
-        $this->init();
+        // @deprecated since TYPO3 v9, will be moved out of __construct() in TYPO3 v10.0
+        $this->init($GLOBALS['TYPO3_REQUEST']);
+    }
+
+    /**
+     * Injects the request object for the current request or subrequest
+     * As this controller goes only through the main() method, it is rather simple for now
+     *
+     * @param ServerRequestInterface $request
+     * @return ResponseInterface
+     */
+    public function mainAction(ServerRequestInterface $request): ResponseInterface
+    {
+        $response = $this->renderContent($request);
+
+        if (empty($response)) {
+            $response = new HtmlResponse($this->moduleTemplate->renderContent());
+        }
+
+        return $response;
+    }
+
+    /**
+     * Main function, rendering the table wizard
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function main()
+    {
+        trigger_error('TableController->main() will be replaced by protected method renderContent() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+
+        $response = $this->renderContent($GLOBALS['TYPO3_REQUEST']);
+
+        if ($response instanceof RedirectResponse) {
+            HttpUtility::redirect($response->getHeaders()['location'][0]);
+        }
+    }
+
+    /**
+     * Draws the table wizard content
+     *
+     * @return string HTML content for the form.
+     * @throws \RuntimeException
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function tableWizard()
+    {
+        trigger_error('TableController->tableWizard() will be replaced by protected method renderTableWizard() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+
+        $result = $this->renderTableWizard($GLOBALS['TYPO3_REQUEST']);
+
+        if ($result instanceof RedirectResponse) {
+            HttpUtility::redirect($result->getHeaders()['location'][0]);
+        }
+
+        return $result;
+    }
+
+    /**
+     * Will get and return the configuration code string
+     * Will also save (and possibly redirect/exit) the content if a save button has been pressed
+     *
+     * @param array $row Current parent record row
+     * @return array Table config code in an array
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function getConfigCode($row)
+    {
+        trigger_error('TableController->getConfigCode() will be replaced by protected method getConfiguration() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+
+        $result = $this->getConfiguration($row, $GLOBALS['TYPO3_REQUEST']);
+
+        if ($result instanceof RedirectResponse) {
+            HttpUtility::redirect($result->getHeaders()['location'][0]);
+        }
+
+        return $result;
+    }
+
+    /**
+     * Creates the HTML for the Table Wizard:
+     *
+     * @param array $configuration Table config array
+     * @return string HTML for the table wizard
+     * @internal
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function getTableHTML($configuration)
+    {
+        trigger_error('TableController->getTableHTML() will be replaced by protected method getTableWizard() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+        return $this->getTableWizard($configuration);
+    }
+
+    /**
+     * Detects if a control button (up/down/around/delete) has been pressed for an item and accordingly it will
+     * manipulate the internal TABLECFG array
+     *
+     * @internal
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function changeFunc()
+    {
+        trigger_error('TableController->changeFunc() will be replaced by protected method manipulateTable() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+        $this->manipulateTable();
+    }
+
+    /**
+     * Converts the input array to a configuration code string
+     *
+     * @param array $cfgArr Array of table configuration (follows the input structure from the table wizard POST form)
+     * @return string The array converted into a string with line-based configuration.
+     * @see cfgString2CfgArray()
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function cfgArray2CfgString($cfgArr)
+    {
+        trigger_error('TableController->cfgArray2CfgString() will be replaced by protected method configurationArrayToString() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+        return $this->configurationArrayToString($cfgArr);
+    }
+
+    /**
+     * Converts the input configuration code string into an array
+     *
+     * @param string $configurationCode Configuration code
+     * @param int $columns Default number of columns
+     * @return array Configuration array
+     * @see cfgArray2CfgString()
+     *
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    public function cfgString2CfgArray($configurationCode, $columns)
+    {
+        trigger_error('TableController->cfgString2CfgArray() will be replaced by protected method configurationStringToArray() in TYPO3 v10.0. Do not call from other extensions.', E_USER_DEPRECATED);
+        return $this->configurationStringToArray($configurationCode, $columns);
     }
 
     /**
      * Initialization of the class
      *
-     * @return void
+     * @param ServerRequestInterface $request
      */
-    protected function init()
+    protected function init(ServerRequestInterface $request): void
     {
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
         // GPvars:
-        $this->P = GeneralUtility::_GP('P');
-        $this->TABLECFG = GeneralUtility::_GP('TABLE');
+        $this->P = $parsedBody['P'] ?? $queryParams['P'] ?? null;
+        $this->TABLECFG = $parsedBody['TABLE'] ?? $queryParams['TABLE'] ?? null;
         // Setting options:
         $this->xmlStorage = $this->P['params']['xmlOutput'];
         $this->numNewRows = MathUtility::forceIntegerInRange($this->P['params']['numNewRows'], 1, 50, 5);
@@ -135,60 +305,44 @@ class TableController extends AbstractWizardController
         $this->tableParsing_quote = '';
     }
 
-    /**
-     * Injects the request object for the current request or subrequest
-     * As this controller goes only through the main() method, it is rather simple for now
-     *
-     * @param ServerRequestInterface $request
-     * @param ResponseInterface $response
-     * @return ResponseInterface
-     */
-    public function mainAction(ServerRequestInterface $request, ResponseInterface $response)
-    {
-        $this->main();
-        $response->getBody()->write($this->moduleTemplate->renderContent());
-        return $response;
-    }
-
     /**
      * Main function, rendering the table wizard
      *
-     * @return void
+     * @param ServerRequestInterface $request
+     * @return ResponseInterface|null
      */
-    public function main()
+    protected function renderContent(ServerRequestInterface $request): ?ResponseInterface
     {
-        list($rUri) = explode('#', GeneralUtility::getIndpEnv('REQUEST_URI'));
+        $normalizedParams = $request->getAttribute('normalizedParams');
+        $requestUri = $normalizedParams->getRequestUri();
+        list($rUri) = explode('#', $requestUri);
         $this->content .= '<form action="' . htmlspecialchars($rUri) . '" method="post" id="TableController" name="wizardForm">';
         if ($this->P['table'] && $this->P['field'] && $this->P['uid']) {
-            $this->content .= '<h2>' . $this->getLanguageService()->getLL('table_title', true) . '</h2>'
-                . '<div>' . $this->tableWizard() . '</div>';
+            $tableWizard = $this->renderTableWizard($request);
+
+            if ($tableWizard instanceof RedirectResponse) {
+                return $tableWizard;
+            }
+
+            $this->content .= '<h2>' . htmlspecialchars($this->getLanguageService()->getLL('table_title')) . '</h2>'
+                . '<div>' . $tableWizard . '</div>';
         } else {
-            $this->content .= '<h2>' . $this->getLanguageService()->getLL('table_title', true) . '</h2>'
-                . '<div><span class="text-danger">' . $this->getLanguageService()->getLL('table_noData', true) . '</span></div>';
+            $this->content .= '<h2>' . htmlspecialchars($this->getLanguageService()->getLL('table_title')) . '</h2>'
+                . '<div><span class="text-danger">' . htmlspecialchars($this->getLanguageService()->getLL('table_noData')) . '</span></div>';
         }
         $this->content .= '</form>';
         // Setting up the buttons and markers for docHeader
         $this->getButtons();
         // Build the <body> for the module
         $this->moduleTemplate->setContent($this->content);
-    }
 
-    /**
-     * Outputting the accumulated content to screen
-     *
-     * @return void
-     * @deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8, use mainAction() instead
-     */
-    public function printContent()
-    {
-        GeneralUtility::logDeprecatedFunction();
-        echo $this->content;
+        return null;
     }
 
     /**
      * Create the panel of buttons for submitting the form or otherwise perform operations.
      */
-    protected function getButtons()
+    protected function getButtons(): void
     {
         $buttonBar = $this->moduleTemplate->getDocHeaderComponent()->getButtonBar();
         if ($this->P['table'] && $this->P['field'] && $this->P['uid']) {
@@ -200,49 +354,38 @@ class TableController extends AbstractWizardController
             // Close
             $closeButton = $buttonBar->makeLinkButton()
                 ->setHref($this->P['returnUrl'])
-                ->setTitle($this->getLanguageService()->sL('LLL:EXT:lang/locallang_core.xlf:rm.closeDoc'))
-                ->setIcon($this->moduleTemplate->getIconFactory()->getIcon('actions-document-close', Icon::SIZE_SMALL));
-            $buttonBar->addButton($closeButton);
+                ->setIcon($this->moduleTemplate->getIconFactory()->getIcon('actions-close', Icon::SIZE_SMALL))
+                ->setTitle($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:rm.closeDoc'))
+                ->setShowLabelText(true);
+            $buttonBar->addButton($closeButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
             // Save
             $saveButton = $buttonBar->makeInputButton()
                 ->setName('_savedok')
                 ->setValue('1')
                 ->setForm('TableController')
                 ->setIcon($this->moduleTemplate->getIconFactory()->getIcon('actions-document-save', Icon::SIZE_SMALL))
-                ->setTitle($this->getLanguageService()->sL('LLL:EXT:lang/locallang_core.xlf:rm.saveDoc'));
-            // Save & Close
-            $saveAndCloseButton = $buttonBar->makeInputButton()
-                ->setName('_saveandclosedok')
-                ->setValue('1')
-                ->setForm('TableController')
-                ->setTitle($this->getLanguageService()->sL('LLL:EXT:lang/locallang_core.xlf:rm.saveCloseDoc'))
-                ->setIcon($this->moduleTemplate->getIconFactory()->getIcon(
-                    'actions-document-save-close',
-                    Icon::SIZE_SMALL
-                ));
-            $splitButtonElement = $buttonBar->makeSplitButton()
-                ->addItem($saveButton)
-                ->addItem($saveAndCloseButton);
-
-            $buttonBar->addButton($splitButtonElement, ButtonBar::BUTTON_POSITION_LEFT, 3);
+                ->setTitle($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:rm.saveDoc'))
+                ->setShowLabelText(true);
+            $buttonBar->addButton($saveButton, ButtonBar::BUTTON_POSITION_LEFT, 2);
             // Reload
             $reloadButton = $buttonBar->makeInputButton()
                 ->setName('_refresh')
                 ->setValue('1')
                 ->setForm('TableController')
-                ->setTitle($this->getLanguageService()->getLL('forms_refresh'))
-                ->setIcon($this->moduleTemplate->getIconFactory()->getIcon('actions-refresh', Icon::SIZE_SMALL));
-            $buttonBar->addButton($reloadButton);
+                ->setIcon($this->moduleTemplate->getIconFactory()->getIcon('actions-refresh', Icon::SIZE_SMALL))
+                ->setTitle($this->getLanguageService()->getLL('forms_refresh'));
+            $buttonBar->addButton($reloadButton, ButtonBar::BUTTON_POSITION_RIGHT);
         }
     }
 
     /**
      * Draws the table wizard content
      *
-     * @return string HTML content for the form.
+     * @param ServerRequestInterface $request
+     * @return string|ResponseInterface HTML content for the form.
      * @throws \RuntimeException
      */
-    public function tableWizard()
+    protected function renderTableWizard(ServerRequestInterface $request)
     {
         if (!$this->checkEditAccess($this->P['table'], $this->P['uid'])) {
             throw new \RuntimeException('Wizard Error: No access', 1349692692);
@@ -254,58 +397,53 @@ class TableController extends AbstractWizardController
         }
         // This will get the content of the form configuration code field to us - possibly cleaned up,
         // saved to database etc. if the form has been submitted in the meantime.
-        $tableCfgArray = $this->getConfigCode($row);
+        $tableCfgArray = $this->getConfiguration($row, $request);
+
+        if ($tableCfgArray instanceof ResponseInterface) {
+            return $tableCfgArray;
+        }
+
         // Generation of the Table Wizards HTML code:
-        $content = $this->getTableHTML($tableCfgArray);
+        $content = $this->getTableWizard($tableCfgArray);
         // Return content:
         return $content;
     }
 
-    /*
-     *
-     * Helper functions
-     *
-     */
-
     /**
      * Will get and return the configuration code string
      * Will also save (and possibly redirect/exit) the content if a save button has been pressed
      *
      * @param array $row Current parent record row
-     * @return array Table config code in an array
-     * @internal
+     * @param ServerRequestInterface $request
+     * @return array|ResponseInterface Table config code in an array
      */
-    public function getConfigCode($row)
+    protected function getConfiguration(array $row, ServerRequestInterface $request)
     {
         // Get delimiter settings
-        $flexForm = GeneralUtility::xml2array($row['pi_flexform']);
-        if (is_array($flexForm)) {
-            $this->tableParsing_quote = $flexForm['data']['s_parsing']['lDEF']['tableparsing_quote']['vDEF'] ? chr((int)$flexForm['data']['s_parsing']['lDEF']['tableparsing_quote']['vDEF']) : '';
-            $this->tableParsing_delimiter = $flexForm['data']['s_parsing']['lDEF']['tableparsing_delimiter']['vDEF'] ? chr((int)$flexForm['data']['s_parsing']['lDEF']['tableparsing_delimiter']['vDEF']) : '|';
-        }
+        $this->tableParsing_quote = $row['table_enclosure'] ? chr((int)$row['table_enclosure']) : '';
+        $this->tableParsing_delimiter = $row['table_delimiter'] ? chr((int)$row['table_delimiter']) : '|';
         // If some data has been submitted, then construct
         if (isset($this->TABLECFG['c'])) {
             // Process incoming:
-            $this->changeFunc();
+            $this->manipulateTable();
             // Convert to string (either line based or XML):
             if ($this->xmlStorage) {
                 // Convert the input array to XML:
-                $bodyText = GeneralUtility::array2xml_cs($this->TABLECFG['c'], 'T3TableWizard');
+                $bodyText = '<?xml version="1.0" encoding="utf-8" standalone="yes" ?>' . LF . GeneralUtility::array2xml($this->TABLECFG['c'], '', 0, 'T3TableWizard');
                 // Setting cfgArr directly from the input:
                 $configuration = $this->TABLECFG['c'];
             } else {
                 // Convert the input array to a string of configuration code:
-                $bodyText = $this->cfgArray2CfgString($this->TABLECFG['c']);
+                $bodyText = $this->configurationArrayToString($this->TABLECFG['c']);
                 // Create cfgArr from the string based configuration - that way it is cleaned up
                 // and any incompatibilities will be removed!
-                $configuration = $this->cfgString2CfgArray($bodyText, $row[$this->colsFieldName]);
+                $configuration = $this->configurationStringToArray($bodyText, (int)$row[$this->colsFieldName]);
             }
             // If a save button has been pressed, then save the new field content:
             if ($_POST['_savedok'] || $_POST['_saveandclosedok']) {
                 // Get DataHandler object:
                 /** @var DataHandler $dataHandler */
                 $dataHandler = GeneralUtility::makeInstance(DataHandler::class);
-                $dataHandler->stripslashes_values = false;
                 // Put content into the data array:
                 $data = [];
                 if ($this->P['flexFormPath']) {
@@ -323,7 +461,7 @@ class TableController extends AbstractWizardController
                 $dataHandler->process_datamap();
                 // If the save/close button was pressed, then redirect the screen:
                 if ($_POST['_saveandclosedok']) {
-                    HttpUtility::redirect(GeneralUtility::sanitizeLocalUrl($this->P['returnUrl']));
+                    return new RedirectResponse(GeneralUtility::sanitizeLocalUrl($this->P['returnUrl']));
                 }
             }
         } else {
@@ -340,10 +478,11 @@ class TableController extends AbstractWizardController
                         $this->P['flexFormPath'],
                         $currentFlexFormData
                     );
-                    $configuration = $this->cfgString2CfgArray($configuration, 0);
+                    $configuration = $this->configurationStringToArray($configuration, 0);
                 } else {
                     // Regular line based table configuration:
-                    $configuration = $this->cfgString2CfgArray($row[$this->P['field']], $row[$this->colsFieldName]);
+                    $columns = $row[$this->colsFieldName] ?? 0;
+                    $configuration = $this->configurationStringToArray($row[$this->P['field']], (int)$columns);
                 }
             }
             $configuration = is_array($configuration) ? $configuration : [];
@@ -356,9 +495,8 @@ class TableController extends AbstractWizardController
      *
      * @param array $configuration Table config array
      * @return string HTML for the table wizard
-     * @internal
      */
-    public function getTableHTML($configuration)
+    protected function getTableWizard(array $configuration): string
     {
         // Traverse the rows:
         $tRows = [];
@@ -385,17 +523,17 @@ class TableController extends AbstractWizardController
                 $onClick = ' onclick="' . htmlspecialchars($onClick) . '"';
                 $ctrl = '';
                 if ($k !== 0) {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_up][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_up', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-up"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_up][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_up')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-up"></span></button>';
                 } else {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_bottom][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_bottom', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-double-down"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_bottom][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_bottom')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-double-down"></span></button>';
                 }
                 if ($k + 1 !== $countLines) {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_down][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_down', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-down"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_down][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_down')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-down"></span></button>';
                 } else {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_top][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_top', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-double-up"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[row_top][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_top')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-angle-double-up"></span></button>';
                 }
-                $ctrl .= '<button class="btn btn-default" name="TABLE[row_remove][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_removeRow', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-trash"></span></button>';
-                $ctrl .= '<button class="btn btn-default" name="TABLE[row_add][' . ($k + 1) * 2 . ']" title="' . $this->getLanguageService()->getLL('table_addRow', true) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-plus"></span></button>';
+                $ctrl .= '<button class="btn btn-default" name="TABLE[row_remove][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_removeRow')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-trash"></span></button>';
+                $ctrl .= '<button class="btn btn-default" name="TABLE[row_add][' . ($k + 1) * 2 . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_addRow')) . '"' . $onClick . '><span class="t3-icon fa fa-fw fa-plus"></span></button>';
                 $tRows[] = '
 					<tr>
 						<td>
@@ -420,17 +558,17 @@ class TableController extends AbstractWizardController
                 $b = $a * 2;
                 $ctrl = '';
                 if ($a !== 1) {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_left][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_left', true) . '"><span class="t3-icon fa fa-fw fa-angle-left"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_left][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_left')) . '"><span class="t3-icon fa fa-fw fa-angle-left"></span></button>';
                 } else {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_end][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_end', true) . '"><span class="t3-icon fa fa-fw fa-angle-double-right"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_end][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_end')) . '"><span class="t3-icon fa fa-fw fa-angle-double-right"></span></button>';
                 }
                 if ($a != $cols) {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_right][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_right', true) . '"><span class="t3-icon fa fa-fw fa-angle-right"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_right][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_right')) . '"><span class="t3-icon fa fa-fw fa-angle-right"></span></button>';
                 } else {
-                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_start][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_start', true) . '"><span class="t3-icon fa fa-fw fa-angle-double-left"></span></button>';
+                    $ctrl .= '<button class="btn btn-default" name="TABLE[col_start][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_start')) . '"><span class="t3-icon fa fa-fw fa-angle-double-left"></span></button>';
                 }
-                $ctrl .= '<button class="btn btn-default" name="TABLE[col_remove][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_removeColumn', true) . '"><span class="t3-icon fa fa-fw fa-trash"></span></button>';
-                $ctrl .= '<button class="btn btn-default" name="TABLE[col_add][' . $b . ']" title="' . $this->getLanguageService()->getLL('table_addColumn', true) . '"><span class="t3-icon fa fa-fw fa-plus"></span></button>';
+                $ctrl .= '<button class="btn btn-default" name="TABLE[col_remove][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_removeColumn')) . '"><span class="t3-icon fa fa-fw fa-trash"></span></button>';
+                $ctrl .= '<button class="btn btn-default" name="TABLE[col_add][' . $b . ']" title="' . htmlspecialchars($this->getLanguageService()->getLL('table_addColumn')) . '"><span class="t3-icon fa fa-fw fa-plus"></span></button>';
                 $cells[] = '<span class="btn-group">' . $ctrl . '</span>';
             }
             $tRows[] = '
@@ -469,11 +607,8 @@ class TableController extends AbstractWizardController
     /**
      * Detects if a control button (up/down/around/delete) has been pressed for an item and accordingly it will
      * manipulate the internal TABLECFG array
-     *
-     * @return void
-     * @internal
      */
-    public function changeFunc()
+    protected function manipulateTable(): void
     {
         if ($this->TABLECFG['col_remove']) {
             $kk = key($this->TABLECFG['col_remove']);
@@ -516,7 +651,7 @@ class TableController extends AbstractWizardController
             $cmd = '';
         }
         if ($cmd && MathUtility::canBeInterpretedAsInteger($kk)) {
-            if (StringUtility::beginsWith($cmd, 'row_')) {
+            if (strpos($cmd, 'row_') === 0) {
                 switch ($cmd) {
                     case 'row_remove':
                         unset($this->TABLECFG['c'][$kk]);
@@ -553,7 +688,7 @@ class TableController extends AbstractWizardController
                 }
                 ksort($this->TABLECFG['c']);
             }
-            if (StringUtility::beginsWith($cmd, 'col_')) {
+            if (strpos($cmd, 'col_') === 0) {
                 foreach ($this->TABLECFG['c'] as $cAK => $value) {
                     switch ($cmd) {
                         case 'col_remove':
@@ -587,9 +722,9 @@ class TableController extends AbstractWizardController
         foreach ($this->TABLECFG['c'] as $a => $value) {
             foreach ($this->TABLECFG['c'][$a] as $b => $value2) {
                 $this->TABLECFG['c'][$a][$b] = str_replace(
-                    LF,
-                    '<br />',
-                    str_replace(CR, '', $this->TABLECFG['c'][$a][$b])
+                    [CR, LF],
+                    ['', '<br />'],
+                    $this->TABLECFG['c'][$a][$b]
                 );
             }
         }
@@ -600,9 +735,9 @@ class TableController extends AbstractWizardController
      *
      * @param array $cfgArr Array of table configuration (follows the input structure from the table wizard POST form)
      * @return string The array converted into a string with line-based configuration.
-     * @see cfgString2CfgArray()
+     * @see configurationStringToArray()
      */
-    public function cfgArray2CfgString($cfgArr)
+    protected function configurationArrayToString(array $cfgArr): string
     {
         $inLines = [];
         // Traverse the elements of the table wizard and transform the settings into configuration code.
@@ -624,9 +759,9 @@ class TableController extends AbstractWizardController
      * @param string $configurationCode Configuration code
      * @param int $columns Default number of columns
      * @return array Configuration array
-     * @see cfgArray2CfgString()
+     * @see configurationArrayToString()
      */
-    public function cfgString2CfgArray($configurationCode, $columns)
+    protected function configurationStringToArray(string $configurationCode, int $columns): array
     {
         // Explode lines in the configuration code - each line is a table row.
         $tableLines = explode(LF, $configurationCode);
@@ -649,7 +784,7 @@ class TableController extends AbstractWizardController
                 ) {
                     $valueParts[$a] = substr(trim($valueParts[$a]), 1, -1);
                 }
-                $configurationArray[$key][$a] = $valueParts[$a];
+                $configurationArray[$key][$a] = (string)$valueParts[$a];
             }
         }
         return $configurationArray;
