@@ -15,14 +15,14 @@ namespace TYPO3\CMS\Backend\RecordList;
  */
 
 use TYPO3\CMS\Backend\Configuration\TranslationConfigurationProvider;
-use TYPO3\CMS\Backend\Routing\Router;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
-use TYPO3\CMS\Backend\Utility\BackendUtility;
-use TYPO3\CMS\Core\Database\DatabaseConnection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Query\Restriction\BackendWorkspaceRestriction;
+use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Lang\LanguageService;
 
 /**
  * Library with a single function addElement that returns table
@@ -30,13 +30,13 @@ use TYPO3\CMS\Lang\LanguageService;
  *
  * Base for class listing of database records and files in the
  * modules Web>List and File>Filelist
- * @see typo3/db_list.php
- * @see typo3/sysext/filelist/mod1/index.php
+ * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
  */
 abstract class AbstractRecordList
 {
     /**
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected $id = 0;
 
@@ -44,6 +44,7 @@ abstract class AbstractRecordList
      * default Max items shown
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $iLimit = 10;
 
@@ -51,16 +52,19 @@ abstract class AbstractRecordList
      * OBSOLETE - NOT USED ANYMORE. leftMargin
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $leftMargin = 0;
 
     /**
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $showIcon = 1;
 
     /**
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $no_noWrap = 0;
 
@@ -68,6 +72,7 @@ abstract class AbstractRecordList
      * If set this is <td> CSS-classname for odd columns in addElement. Used with db_layout / pages section
      *
      * @var string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $oddColumnsCssClass = '';
 
@@ -75,6 +80,7 @@ abstract class AbstractRecordList
      * Decides the columns shown. Filled with values that refers to the keys of the data-array. $this->fieldArray[0] is the title column.
      *
      * @var array
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $fieldArray = [];
 
@@ -82,6 +88,7 @@ abstract class AbstractRecordList
      * Keys are fieldnames and values are td-parameters to add in addElement(), please use $addElement_tdCSSClass for CSS-classes;
      *
      * @var array
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $addElement_tdParams = [];
 
@@ -89,6 +96,7 @@ abstract class AbstractRecordList
      * Keys are fieldnames and values are td-css-classes to add in addElement();
      *
      * @var array
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $addElement_tdCssClass = [];
 
@@ -97,6 +105,7 @@ abstract class AbstractRecordList
      * Max length of strings
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $fixedL = 30;
 
@@ -104,6 +113,7 @@ abstract class AbstractRecordList
      * Script URL
      *
      * @var string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $thisScript = '';
 
@@ -111,6 +121,7 @@ abstract class AbstractRecordList
      * Set to zero, if you don't want a left-margin with addElement function
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $setLMargin = 1;
 
@@ -118,6 +129,7 @@ abstract class AbstractRecordList
      * Counter increased for each element. Used to index elements for the JavaScript-code that transfers to the clipboard
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $counter = 0;
 
@@ -125,6 +137,7 @@ abstract class AbstractRecordList
      * This could be set to the total number of items. Used by the fwd_rew_navigation...
      *
      * @var string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $totalItems = '';
 
@@ -132,11 +145,13 @@ abstract class AbstractRecordList
      * Internal (used in this class.)
      *
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $firstElementNumber = 0;
 
     /**
      * @var int
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $eCounter = 0;
 
@@ -144,6 +159,7 @@ abstract class AbstractRecordList
      * String with accumulated HTML content
      *
      * @var string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $HTMLcode = '';
 
@@ -151,6 +167,7 @@ abstract class AbstractRecordList
      * Contains page translation languages
      *
      * @var array
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $pageOverlays = [];
 
@@ -158,24 +175,29 @@ abstract class AbstractRecordList
      * Contains sys language icons and titles
      *
      * @var array
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $languageIconTitles = [];
 
     /**
      * @var TranslationConfigurationProvider
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public $translateTools;
 
     /**
      * @var IconFactory
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected $iconFactory;
 
     /**
      * Constructor
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function __construct()
     {
+        trigger_error('The class AbstractRecordList will be removed in TYPO3 v10.0, as all logic was moved into specific classes that inherited from AbstractRecordList.', E_USER_DEPRECATED);
         if (isset($GLOBALS['BE_USER']->uc['titleLen']) && $GLOBALS['BE_USER']->uc['titleLen'] > 0) {
             $this->fixedL = $GLOBALS['BE_USER']->uc['titleLen'];
         }
@@ -186,16 +208,13 @@ abstract class AbstractRecordList
 
     /**
      * Sets the script url depending on being a module or script request
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected function determineScriptUrl()
     {
         if ($routePath = GeneralUtility::_GP('route')) {
-            $router = GeneralUtility::makeInstance(Router::class);
-            $route = $router->match($routePath);
             $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-            $this->thisScript = (string)$uriBuilder->buildUriFromRoute($route->getOption('_identifier'));
-        } elseif ($moduleName = GeneralUtility::_GP('M')) {
-            $this->thisScript = BackendUtility::getModuleUrl($moduleName);
+            $this->thisScript = (string)$uriBuilder->buildUriFromRoutePath($routePath);
         } else {
             $this->thisScript = GeneralUtility::getIndpEnv('SCRIPT_NAME');
         }
@@ -203,6 +222,7 @@ abstract class AbstractRecordList
 
     /**
      * @return string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected function getThisScript()
     {
@@ -218,23 +238,24 @@ abstract class AbstractRecordList
      * @param array $data Is the dataarray, record with the fields. Notice: These fields are (currently) NOT htmlspecialchar'ed before being wrapped in <td>-tags
      * @param string $rowParams Is insert in the <tr>-tags. Must carry a ' ' as first character
      * @param string $_ OBSOLETE - NOT USED ANYMORE. $lMargin is the leftMargin (int)
-     * @param string $_ OBSOLETE - NOT USED ANYMORE. Is the HTML <img>-tag for an alternative 'gfx/ol/line.gif'-icon (used in the top)
+     * @param string $_2 OBSOLETE - NOT USED ANYMORE. Is the HTML <img>-tag for an alternative 'gfx/ol/line.gif'-icon (used in the top)
      * @param string $colType Defines the tag being used for the columns. Default is td.
      * @return string HTML content for the table row
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function addElement($h, $icon, $data, $rowParams = '', $_ = '', $_2 = '', $colType = 'td')
     {
         $colType = ($colType === 'th') ? 'th' : 'td';
-        $noWrap = $this->no_noWrap ? '' : ' nowrap="nowrap"';
+        $noWrap = $this->no_noWrap ? '' : ' nowrap';
         // Start up:
-        $parent = isset($data['parent']) ? (int)$data['parent'] : 0;
+        $l10nParent = isset($data['_l10nparent_']) ? (int)$data['_l10nparent_'] : 0;
         $out = '
 		<!-- Element, begin: -->
-		<tr ' . $rowParams . ' data-uid="' . (int)$data['uid'] . '" data-l10nparent="' . $parent . '">';
+		<tr ' . $rowParams . ' data-uid="' . (int)$data['uid'] . '" data-l10nparent="' . $l10nParent . '">';
         // Show icon and lines
         if ($this->showIcon) {
             $out .= '
-			<' . $colType . ' nowrap="nowrap" class="col-icon">';
+			<' . $colType . ' class="col-icon nowrap">';
             if (!$h) {
                 $out .= '&nbsp;';
             } else {
@@ -243,7 +264,6 @@ abstract class AbstractRecordList
                         if ($icon) {
                             $out .= $icon;
                         }
-                    } else {
                     }
                 }
             }
@@ -270,7 +290,7 @@ abstract class AbstractRecordList
                         $cssClass = implode(' ', [$this->addElement_tdCssClass[$lastKey], $this->oddColumnsCssClass]);
                     }
                     $out .= '
-						<' . $colType . $noWrap . ' class="' . $cssClass . '"' . $colsp . $this->addElement_tdParams[$lastKey] . '>' . $data[$lastKey] . '</' . $colType . '>';
+						<' . $colType . ' class="' . $cssClass . $noWrap . '"' . $colsp . $this->addElement_tdParams[$lastKey] . '>' . $data[$lastKey] . '</' . $colType . '>';
                 }
                 $lastKey = $vKey;
                 $c = 1;
@@ -293,7 +313,7 @@ abstract class AbstractRecordList
                 $cssClass = implode(' ', [$this->addElement_tdCssClass[$lastKey], $this->oddColumnsCssClass]);
             }
             $out .= '
-				<' . $colType . $noWrap . ' class="' . $cssClass . '"' . $colsp . $this->addElement_tdParams[$lastKey] . '>' . $data[$lastKey] . '</' . $colType . '>';
+				<' . $colType . ' class="' . $cssClass . $noWrap . '"' . $colsp . $this->addElement_tdParams[$lastKey] . '>' . $data[$lastKey] . '</' . $colType . '>';
         }
         // End row
         $out .= '
@@ -304,8 +324,7 @@ abstract class AbstractRecordList
 
     /**
      * Dummy function, used to write the top of a table listing.
-     *
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function writeTop()
     {
@@ -316,6 +335,7 @@ abstract class AbstractRecordList
      *
      * @param string $table Table name
      * @return array array([boolean], [HTML]) where [boolean] is 1 for reverse element, [HTML] is the table-row code for the element
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function fwd_rwd_nav($table = '')
     {
@@ -329,16 +349,15 @@ abstract class AbstractRecordList
                 $code = $this->addElement(1, '', $theData, 'class="fwd_rwd_nav"');
             }
             return [1, $code];
-        } else {
-            if ($this->eCounter == $this->firstElementNumber + $this->iLimit) {
-                // 	Forward
-                $theData = [];
-                $titleCol = $this->fieldArray[0];
-                $theData[$titleCol] = $this->fwd_rwd_HTML('rwd', $this->eCounter, $table);
-                $code = $this->addElement(1, '', $theData, 'class="fwd_rwd_nav"');
-            }
-            return [0, $code];
         }
+        if ($this->eCounter == $this->firstElementNumber + $this->iLimit) {
+            // 	Forward
+            $theData = [];
+            $titleCol = $this->fieldArray[0];
+            $theData[$titleCol] = $this->fwd_rwd_HTML('rwd', $this->eCounter, $table);
+            $code = $this->addElement(1, '', $theData, 'class="fwd_rwd_nav"');
+        }
+        return [0, $code];
     }
 
     /**
@@ -348,7 +367,8 @@ abstract class AbstractRecordList
      * @param int $pointer Pointer
      * @param string $table Table name
      * @return string
-     * @access private
+     * @internal
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function fwd_rwd_HTML($type, $pointer, $table = '')
     {
@@ -357,7 +377,7 @@ abstract class AbstractRecordList
         switch ($type) {
             case 'fwd':
                 $href = $this->listURL() . '&pointer=' . ($pointer - $this->iLimit) . $tParam;
-                $content = '<a href="' . htmlspecialchars($href) . '">' . $this->iconFactory->getIcon('actions-move-up', Icon::SIZE_SMALL)->render() . '</a> <i>[1 - ' . $pointer . ']</i>';
+                $content = '<a href="' . htmlspecialchars($href) . '">' . $this->iconFactory->getIcon('actions-move-up', Icon::SIZE_SMALL)->render() . '</a> <i>[' . (max(0, $pointer - $this->iLimit) + 1) . ' - ' . $pointer . ']</i>';
                 break;
             case 'rwd':
                 $href = $this->listURL() . '&pointer=' . $pointer . $tParam;
@@ -374,6 +394,7 @@ abstract class AbstractRecordList
      * @param string $table Table name to display. Enter "-1" for the current table.
      * @param string $exclList Comma separated list of fields NOT to include ("sortField", "sortRev" or "firstElementNumber")
      * @return string URL
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function listURL($altId = '', $table = '-1', $exclList = '')
     {
@@ -384,6 +405,7 @@ abstract class AbstractRecordList
      * Returning JavaScript for ClipBoard functionality.
      *
      * @return string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function CBfunctions()
     {
@@ -421,13 +443,35 @@ abstract class AbstractRecordList
 
     /**
      * Initializes page languages and icons
-     *
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function initializeLanguages()
     {
         // Look up page overlays:
-        $this->pageOverlays = $this->getDatabaseConnection()->exec_SELECTgetRows('*', 'pages_language_overlay', 'pid=' . (int)$this->id . BackendUtility::deleteClause('pages_language_overlay') . BackendUtility::versioningPlaceholderClause('pages_language_overlay'), '', '', '', 'sys_language_uid');
+        $localizationParentField = $GLOBALS['TCA']['pages']['ctrl']['transOrigPointerField'];
+        $languageField = $GLOBALS['TCA']['pages']['ctrl']['languageField'];
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable('pages');
+        $queryBuilder->getRestrictions()
+            ->removeAll()
+            ->add(GeneralUtility::makeInstance(DeletedRestriction::class))
+            ->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
+        $result = $queryBuilder
+            ->select('*')
+            ->from('pages')
+            ->where(
+                $queryBuilder->expr()->andX(
+                    $queryBuilder->expr()->eq($localizationParentField, $queryBuilder->createNamedParameter($this->id, \PDO::PARAM_INT)),
+                    $queryBuilder->expr()->gt($languageField, $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+                )
+            )
+            ->execute();
+
+        $this->pageOverlays = [];
+        while ($row = $result->fetch()) {
+            $this->pageOverlays[$row[$languageField]] = $row;
+        }
+
         $this->languageIconTitles = $this->getTranslateTools()->getSystemLanguages($this->id);
     }
 
@@ -437,6 +481,7 @@ abstract class AbstractRecordList
      * @param int $sys_language_uid Sys language uid
      * @param bool $addAsAdditionalText If set to true, only the flag is returned
      * @return string Language icon
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     public function languageFlag($sys_language_uid, $addAsAdditionalText = true)
     {
@@ -457,6 +502,7 @@ abstract class AbstractRecordList
      * Gets an instance of TranslationConfigurationProvider
      *
      * @return TranslationConfigurationProvider
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected function getTranslateTools()
     {
@@ -471,8 +517,9 @@ abstract class AbstractRecordList
      * sys_refindex records you hand over
      *
      * @param int $references number of records from sys_refindex table
-     * @param string $launchViewParameter JavaScript String, which will be passed as parameters to top.launchView
+     * @param string $launchViewParameter JavaScript String, which will be passed as parameters to top.TYPO3.InfoWindow.showItem
      * @return string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected function generateReferenceToolTip($references, $launchViewParameter = '')
     {
@@ -481,9 +528,9 @@ abstract class AbstractRecordList
         } else {
             $htmlCode = '<a href="#"';
             if ($launchViewParameter !== '') {
-                $htmlCode .= ' onclick="' . htmlspecialchars(('top.launchView(' . $launchViewParameter . '); return false;')) . '"';
+                $htmlCode .= ' onclick="' . htmlspecialchars('top.TYPO3.InfoWindow.showItem(' . $launchViewParameter . '); return false;') . '"';
             }
-            $htmlCode .= ' title="' . htmlspecialchars($this->getLanguageService()->sl('LLL:EXT:backend/Resources/Private/Language/locallang.xlf:show_references') . ' (' .  $references . ')') . '">';
+            $htmlCode .= ' title="' . htmlspecialchars($this->getLanguageService()->sL('LLL:EXT:backend/Resources/Private/Language/locallang.xlf:show_references') . ' (' . $references . ')') . '">';
             $htmlCode .= $references;
             $htmlCode .= '</a>';
         }
@@ -493,19 +540,10 @@ abstract class AbstractRecordList
     /**
      * Returns the language service
      * @return LanguageService
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
      */
     protected function getLanguageService()
     {
         return $GLOBALS['LANG'];
     }
-
-    /**
-     * Returns the database connection
-     *
-     * @return DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
 }
