@@ -14,15 +14,17 @@ namespace TYPO3\CMS\Backend\Tree\View;
  * The TYPO3 project - inspiring people to share!
  */
 
-use TYPO3\CMS\Backend\Routing\Router;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Database\DatabaseConnection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Query\QueryHelper;
+use TYPO3\CMS\Core\Database\Query\Restriction\BackendWorkspaceRestriction;
+use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Lang\LanguageService;
 
 /**
  * Base class for creating a browsable array/page/folder tree in HTML
@@ -32,15 +34,15 @@ abstract class AbstractTreeView
     // EXTERNAL, static:
     // If set, the first element in the tree is always expanded.
     /**
-     * @var int
+     * @var bool
      */
-    public $expandFirst = 0;
+    public $expandFirst = false;
 
     // If set, then ALL items will be expanded, regardless of stored settings.
     /**
-     * @var int
+     * @var bool
      */
-    public $expandAll = 0;
+    public $expandAll = false;
 
     // Holds the current script to reload to.
     /**
@@ -61,6 +63,11 @@ abstract class AbstractTreeView
      */
     public $ext_IconMode = false;
 
+    /**
+     * @var bool
+     */
+    public $ext_showPathAboveMounts = false;
+
     // If set, the id of the mounts will be added to the internal ids array
     /**
      * @var int
@@ -96,9 +103,9 @@ abstract class AbstractTreeView
      * values are the ID of the root element (COULD be zero or anything else.
      * For pages that would be the uid of the page, zero for the pagetree root.)
      *
-     * @var array|NULL
+     * @var array|null
      */
-    public $MOUNTS = null;
+    public $MOUNTS;
 
     /**
      * Database table to get the tree data from.
@@ -140,7 +147,7 @@ abstract class AbstractTreeView
      * @see addField()
      * @var array
      */
-    public $fieldArray = ['uid', 'pid', 'title'];
+    public $fieldArray = ['uid', 'pid', 'title', 'is_siteroot'];
 
     /**
      * List of other fields which are ALLOWED to set (here, based on the "pages" table!)
@@ -172,8 +179,7 @@ abstract class AbstractTreeView
     public $domIdPrefix = 'row';
 
     /**
-     * If TRUE, HTML code is also accumulated in ->tree array during rendering of the tree.
-     * If 2, then also the icon prefix code (depthData) is stored
+     * If 1, HTML code is also accumulated in ->tree array during rendering of the tree
      *
      * @var int
      */
@@ -191,6 +197,7 @@ abstract class AbstractTreeView
      * This value has formerly been "subLevel" and "--sublevel--"
      *
      * @var string
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
      */
     public $subLevelID = '_SUB_LEVEL';
 
@@ -229,16 +236,19 @@ abstract class AbstractTreeView
      */
     public $specUIDmap = [];
 
-    // For arrays:
-    // Holds the input data array
     /**
+     * For arrays, holds the input data array
+     *
      * @var bool
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
      */
     public $data = false;
 
-    // Holds an index with references to the data array.
     /**
+     * For arrays, holds an index with references to the data array.
+     *
      * @var bool
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
      */
     public $dataLookup = false;
 
@@ -267,6 +277,12 @@ abstract class AbstractTreeView
      */
     public $recs = [];
 
+    /**
+     * @var bool
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     */
+    private $setDataFromArrayDeprecationThrown = false;
+
     /**
      * Constructor
      */
@@ -281,12 +297,8 @@ abstract class AbstractTreeView
     protected function determineScriptUrl()
     {
         if ($routePath = GeneralUtility::_GP('route')) {
-            $router = GeneralUtility::makeInstance(Router::class);
-            $route = $router->match($routePath);
             $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-            $this->thisScript = (string)$uriBuilder->buildUriFromRoute($route->getOption('_identifier'));
-        } elseif ($moduleName = GeneralUtility::_GP('M')) {
-            $this->thisScript = BackendUtility::getModuleUrl($moduleName);
+            $this->thisScript = (string)$uriBuilder->buildUriFromRoutePath($routePath);
         } else {
             $this->thisScript = GeneralUtility::getIndpEnv('SCRIPT_NAME');
         }
@@ -305,7 +317,6 @@ abstract class AbstractTreeView
      *
      * @param string $clause Record WHERE clause
      * @param string $orderByFields Record ORDER BY field
-     * @return void
      */
     public function init($clause = '', $orderByFields = '')
     {
@@ -324,6 +335,8 @@ abstract class AbstractTreeView
         }
         // Sets the tree name which is used to identify the tree, used for JavaScript and other things
         $this->treeName = str_replace('_', '', $this->treeName ?: $this->table);
+
+        // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove with $this->data and friends.
         // Setting this to FALSE disables the use of array-trees by default
         $this->data = false;
         $this->dataLookup = false;
@@ -334,7 +347,6 @@ abstract class AbstractTreeView
      *
      * @param string $field Field name to
      * @param bool $noCheck If set, the fieldname will be set no matter what. Otherwise the field name must either be found as key in $GLOBALS['TCA'][$table]['columns'] or in the list ->defaultList
-     * @return void
      */
     public function addField($field, $noCheck = false)
     {
@@ -345,8 +357,6 @@ abstract class AbstractTreeView
 
     /**
      * Resets the tree, recs, ids, ids_hierarchy and orig_ids_hierarchy internal variables. Use it if you need it.
-     *
-     * @return void
      */
     public function reset()
     {
@@ -373,6 +383,7 @@ abstract class AbstractTreeView
         // Get stored tree structure AND updating it if needed according to incoming PM GET var.
         $this->initializePositionSaving();
         // Init done:
+        $lastMountPointPid = 0;
         $treeArr = [];
         // Traverse mounts:
         foreach ($this->MOUNTS as $idx => $uid) {
@@ -393,6 +404,15 @@ abstract class AbstractTreeView
                 if (is_array($rootRec)) {
                     $firstHtml .= $this->getIcon($rootRec);
                 }
+
+                if ($this->ext_showPathAboveMounts) {
+                    $mountPointPid = $rootRec['pid'];
+                    if ($lastMountPointPid !== $mountPointPid) {
+                        $title = $this->getMountPointPath((int)$mountPointPid);
+                        $this->tree[] = ['isMountPointPath' => true, 'title' => $title];
+                    }
+                    $lastMountPointPid = $mountPointPid;
+                }
             } else {
                 // Artificial record for the tree root, id=0
                 $rootRec = $this->getRootRecord();
@@ -487,7 +507,7 @@ abstract class AbstractTreeView
      * @param int $nextCount The number of sub-elements to the current element.
      * @param bool $isOpen The element was expanded to render subelements if this flag is set.
      * @return string Image tag with the plus/minus icon.
-     * @access private
+     * @internal
      * @see \TYPO3\CMS\Backend\Tree\View\PageTreeView::PMicon()
      */
     public function PMicon($row, $a, $c, $nextCount, $isOpen)
@@ -496,9 +516,8 @@ abstract class AbstractTreeView
             $cmd = $this->bank . '_' . ($isOpen ? '0_' : '1_') . $row['uid'] . '_' . $this->treeName;
             $bMark = $this->bank . '_' . $row['uid'];
             return $this->PM_ATagWrap('', $cmd, $bMark, $isOpen);
-        } else {
-            return '';
         }
+        return '';
     }
 
     /**
@@ -509,7 +528,7 @@ abstract class AbstractTreeView
      * @param string $bMark If set, the link will have an anchor point (=$bMark) and a name attribute (=$bMark)
      * @param bool $isOpen
      * @return string Link-wrapped input string
-     * @access private
+     * @internal
      */
     public function PM_ATagWrap($icon, $cmd, $bMark = '', $isOpen = false)
     {
@@ -518,19 +537,18 @@ abstract class AbstractTreeView
             $name = $bMark ? ' name="' . $bMark . '"' : '';
             $aUrl = $this->getThisScript() . 'PM=' . $cmd . $anchor;
             return '<a class="list-tree-control ' . ($isOpen ? 'list-tree-control-open' : 'list-tree-control-closed') . '" href="' . htmlspecialchars($aUrl) . '"' . $name . '><i class="fa"></i></a>';
-        } else {
-            return $icon;
         }
+        return $icon;
     }
 
     /**
      * Wrapping $title in a-tags.
      *
      * @param string $title Title string
-     * @param string $row Item record
+     * @param array $row Item record
      * @param int $bank Bank pointer (which mount point number)
      * @return string
-     * @access private
+     * @internal
      */
     public function wrapTitle($title, $row, $bank = 0)
     {
@@ -544,7 +562,7 @@ abstract class AbstractTreeView
      * @param string $icon The image tag for the icon
      * @param array $row The row for the current element
      * @return string The processed icon input value.
-     * @access private
+     * @internal
      */
     public function wrapIcon($icon, $row)
     {
@@ -569,7 +587,7 @@ abstract class AbstractTreeView
      * @param string $str Input string, like a page title for the tree
      * @param array $row record row with "php_tree_stop" field
      * @return string Modified string
-     * @access private
+     * @internal
      */
     public function wrapStop($str, $row)
     {
@@ -591,19 +609,18 @@ abstract class AbstractTreeView
      *
      * @param int $id Record id/key
      * @return bool
-     * @access private
+     * @internal
      * @see \TYPO3\CMS\Backend\Tree\View\PageTreeView::expandNext()
      */
     public function expandNext($id)
     {
-        return $this->stored[$this->bank][$id] || $this->expandAll ? 1 : 0;
+        return !empty($this->stored[$this->bank][$id]) || $this->expandAll;
     }
 
     /**
      * Get stored tree structure AND updating it if needed according to incoming PM GET var.
      *
-     * @return void
-     * @access private
+     * @internal
      */
     public function initializePositionSaving()
     {
@@ -632,8 +649,7 @@ abstract class AbstractTreeView
      * Saves the content of ->stored (keeps track of expanded positions in the tree)
      * $this->treeName will be used as key for BE_USER->uc[] to store it in
      *
-     * @return void
-     * @access private
+     * @internal
      */
     public function savePosition()
     {
@@ -671,7 +687,8 @@ abstract class AbstractTreeView
         }
         $title = $this->showDefaultTitleAttribute ? htmlspecialchars('UID: ' . $row['uid']) : $this->getTitleAttrib($row);
         $iconFactory = GeneralUtility::makeInstance(IconFactory::class);
-        $icon = '<span title="' . $title . '">' . $iconFactory->getIconForRecord($this->table, $row, Icon::SIZE_SMALL)->render() . '</span>';
+        $icon = $row['is_siteroot'] ? $iconFactory->getIcon('apps-pagetree-folder-root', Icon::SIZE_SMALL) : $iconFactory->getIconForRecord($this->table, $row, Icon::SIZE_SMALL);
+        $icon = '<span title="' . $title . '">' . $icon->render() . '</span>';
         return $this->wrapIcon($icon, $row);
     }
 
@@ -686,7 +703,7 @@ abstract class AbstractTreeView
     public function getTitleStr($row, $titleLen = 30)
     {
         $title = htmlspecialchars(GeneralUtility::fixed_lgd_cs($row['title'], $titleLen));
-        $title = trim($row['title']) === '' ? '<em>[' . $this->getLanguageService()->sL('LLL:EXT:lang/locallang_core.xlf:labels.no_title', true) . ']</em>' : $title;
+        $title = trim($row['title']) === '' ? '<em>[' . htmlspecialchars($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.no_title')) . ']</em>' : $title;
         return $title;
     }
 
@@ -735,7 +752,6 @@ abstract class AbstractTreeView
      * @param int $uid item id for which to select subitems (parent id)
      * @param int $depth Max depth (recursivity limit)
      * @param string $depthData HTML-code prefix for recursive calls.
-
      * @return int The count of items on the level
      */
     public function getTree($uid, $depth = 999, $depthData = '')
@@ -752,8 +768,7 @@ abstract class AbstractTreeView
         $idH = [];
         // Traverse the records:
         while ($crazyRecursionLimiter > 0 && ($row = $this->getDataNext($res))) {
-            $pageUid = ($this->table === 'pages') ? $row['uid'] : $row['pid'];
-            if (!$this->getBackendUser()->isInWebMount($pageUid)) {
+            if (!$this->getBackendUser()->isInWebMount($this->table === 'pages' ? $row : $row['pid'])) {
                 // Current record is not within web mount => skip it
                 continue;
             }
@@ -826,18 +841,35 @@ abstract class AbstractTreeView
      *
      * @param int $uid Id to count subitems for
      * @return int
-     * @access private
+     * @internal
      */
     public function getCount($uid)
     {
         if (is_array($this->data)) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove the "if" along with $this->data and friends.
+            trigger_error('Handling array data in AbstractTreeView will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
             $res = $this->getDataInit($uid);
             return $this->getDataCount($res);
-        } else {
-            $db = $this->getDatabaseConnection();
-            $where = $this->parentField . '=' . $db->fullQuoteStr($uid, $this->table) . BackendUtility::deleteClause($this->table) . BackendUtility::versioningPlaceholderClause($this->table) . $this->clause;
-            return $db->exec_SELECTcountRows('uid', $this->table, $where);
         }
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($this->table);
+        $queryBuilder->getRestrictions()
+                ->removeAll()
+                ->add(GeneralUtility::makeInstance(DeletedRestriction::class))
+                ->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
+        $count = $queryBuilder
+                ->count('uid')
+                ->from($this->table)
+                ->where(
+                    $queryBuilder->expr()->eq(
+                        $this->parentField,
+                        $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)
+                    ),
+                    QueryHelper::stripLogicalOperatorPrefix($this->clause)
+                )
+                ->execute()
+                ->fetchColumn();
+
+        return (int)$count;
     }
 
     /**
@@ -861,10 +893,10 @@ abstract class AbstractTreeView
     public function getRecord($uid)
     {
         if (is_array($this->data)) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove "if" with $this->data and friends.
             return $this->dataLookup[$uid];
-        } else {
-            return BackendUtility::getRecordWSOL($this->table, $uid);
         }
+        return BackendUtility::getRecordWSOL($this->table, $uid);
     }
 
     /**
@@ -875,22 +907,41 @@ abstract class AbstractTreeView
      * @param int $parentId parent item id
      *
      * @return mixed Data handle (Tables: An sql-resource, arrays: A parentId integer. -1 is returned if there were NO subLevel.)
-     * @access private
+     * @internal
      */
     public function getDataInit($parentId)
     {
         if (is_array($this->data)) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove "if" with $this->data and friends.
             if (!is_array($this->dataLookup[$parentId][$this->subLevelID])) {
                 $parentId = -1;
             } else {
                 reset($this->dataLookup[$parentId][$this->subLevelID]);
             }
             return $parentId;
-        } else {
-            $db = $this->getDatabaseConnection();
-            $where = $this->parentField . '=' . $db->fullQuoteStr($parentId, $this->table) . BackendUtility::deleteClause($this->table) . BackendUtility::versioningPlaceholderClause($this->table) . $this->clause;
-            return $db->exec_SELECTquery(implode(',', $this->fieldArray), $this->table, $where, '', $this->orderByFields);
         }
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($this->table);
+        $queryBuilder->getRestrictions()
+                ->removeAll()
+                ->add(GeneralUtility::makeInstance(DeletedRestriction::class))
+                ->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
+        $queryBuilder
+                ->select(...$this->fieldArray)
+                ->from($this->table)
+                ->where(
+                    $queryBuilder->expr()->eq(
+                        $this->parentField,
+                        $queryBuilder->createNamedParameter($parentId, \PDO::PARAM_INT)
+                    ),
+                    QueryHelper::stripLogicalOperatorPrefix($this->clause)
+                );
+
+        foreach (QueryHelper::parseOrderBy($this->orderByFields) as $orderPair) {
+            list($fieldName, $order) = $orderPair;
+            $queryBuilder->addOrderBy($fieldName, $order);
+        }
+
+        return $queryBuilder->execute();
     }
 
     /**
@@ -898,16 +949,16 @@ abstract class AbstractTreeView
      *
      * @param mixed $res Data handle
      * @return int number of items
-     * @access private
+     * @internal
      * @see getDataInit()
      */
     public function getDataCount(&$res)
     {
         if (is_array($this->data)) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove "if" with $this->data and friends.
             return count($this->dataLookup[$res][$this->subLevelID]);
-        } else {
-            return $this->getDatabaseConnection()->sql_num_rows($res);
         }
+        return $res->rowCount();
     }
 
     /**
@@ -915,41 +966,43 @@ abstract class AbstractTreeView
      *
      * @param mixed $res Data handle
      *
-     * @return array item data array OR FALSE if end of elements.
-     * @access private
+     * @return array|bool item data array OR FALSE if end of elements.
+     * @internal
      * @see getDataInit()
      */
     public function getDataNext(&$res)
     {
         if (is_array($this->data)) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove the "if" along with $this->data and friends.
             if ($res < 0) {
                 $row = false;
             } else {
-                list(, $row) = each($this->dataLookup[$res][$this->subLevelID]);
-            }
-            return $row;
-        } else {
-            while ($row = @$this->getDatabaseConnection()->sql_fetch_assoc($res)) {
-                BackendUtility::workspaceOL($this->table, $row, $this->BE_USER->workspace, true);
-                if (is_array($row)) {
-                    break;
-                }
+                $key = key($this->dataLookup[$res][$this->subLevelID]);
+                next($this->dataLookup[$res][$this->subLevelID]);
+                $row = $this->dataLookup[$res][$this->subLevelID][$key];
             }
             return $row;
         }
+        while ($row = $res->fetch()) {
+            BackendUtility::workspaceOL($this->table, $row, $this->BE_USER->workspace, true);
+            if (is_array($row)) {
+                break;
+            }
+        }
+        return $row;
     }
 
     /**
      * Getting the tree data: frees data handle
      *
      * @param mixed $res Data handle
-     * @return void
-     * @access private
+     * @internal
      */
     public function getDataFree(&$res)
     {
+        // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Remove "if" with $this->data and friends. Keep $res->closeCursor().
         if (!is_array($this->data)) {
-            $this->getDatabaseConnection()->sql_free_result($res);
+            $res->closeCursor();
         }
     }
 
@@ -963,11 +1016,18 @@ abstract class AbstractTreeView
      * @param array $dataArr The input array, see examples below in this script.
      * @param bool $traverse Internal, for recursion.
      * @param int $pid Internal, for recursion.
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
      */
     public function setDataFromArray(&$dataArr, $traverse = false, $pid = 0)
     {
+        if (!$this->setDataFromArrayDeprecationThrown) {
+            // Throw deprecation only once for this recursive method
+            $this->setDataFromArrayDeprecationThrown = true;
+            trigger_error('AbstractTreeView->setDataFromArray() will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
+        }
+
         if (!$traverse) {
+            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
             $this->data = &$dataArr;
             $this->dataLookup = [];
             // Add root
@@ -989,14 +1049,42 @@ abstract class AbstractTreeView
      *
      * @param array $treeArr Content for $this->data
      * @param array $treeLookupArr Content for $this->dataLookup
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
      */
     public function setDataFromTreeArray(&$treeArr, &$treeLookupArr)
     {
+        trigger_error('AbstractTreeView->setDataFromTreeArray() will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
         $this->data = &$treeArr;
         $this->dataLookup = &$treeLookupArr;
     }
 
+    /**
+     * Returns the mount point path for a temporary mount or the given id
+     *
+     * @param int $uid
+     * @return string
+     */
+    protected function getMountPointPath(int $uid): string
+    {
+        if ($uid <= 0) {
+            return '';
+        }
+        $rootline = array_reverse(BackendUtility::BEgetRootLine($uid));
+        array_shift($rootline);
+        $path = [];
+        foreach ($rootline as $rootlineElement) {
+            $record = BackendUtility::getRecordWSOL('pages', $rootlineElement['uid'], 'title, nav_title', '', true, true);
+            $text = $record['title'];
+            if ((bool)($this->getBackendUser()->getTSConfig()['options.']['pageTree.']['showNavTitle'] ?? false)
+                && trim($record['nav_title'] ?? '') !== ''
+            ) {
+                $text = $record['nav_title'];
+            }
+            $path[] = htmlspecialchars($text);
+        }
+        return '/' . implode('/', $path);
+    }
+
     /**
      * @return LanguageService
      */
@@ -1012,12 +1100,4 @@ abstract class AbstractTreeView
     {
         return $GLOBALS['BE_USER'];
     }
-
-    /**
-     * @return DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
 }
