@@ -16,7 +16,9 @@ namespace TYPO3\CMS\Backend\Tree\View;
 
 /**
  * Extension for the tree class that generates the tree of pages in the page-wizard mode
- * see typo3/sysext/backend/Classes/Tree/View/NewRecordPageTreeView.php
+ *
+ * @see \TYPO3\CMS\Backend\Controller\NewRecordController
+ * @internal This class is a TYPO3 Backend implementation and is not considered part of the Public TYPO3 API.
  */
 class NewRecordPageTreeView extends PageTreeView
 {
@@ -29,6 +31,6 @@ class NewRecordPageTreeView extends PageTreeView
      */
     public function expandNext($id)
     {
-        return $id == $GLOBALS['SOBE']->id ? 1 : 0;
+        return $id == $GLOBALS['SOBE']->id;
     }
 }
