@@ -14,6 +14,8 @@ namespace TYPO3\CMS\Core\Cache\Backend;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Cache\Exception;
+use TYPO3\CMS\Core\Cache\Exception\InvalidDataException;
 use TYPO3\CMS\Core\Utility\StringUtility;
 
 /**
@@ -21,9 +23,8 @@ use TYPO3\CMS\Core\Utility\StringUtility;
  * PHP module. Redis is a noSQL database with very good scaling characteristics
  * in proportion to the amount of entries and data size.
  *
- * @see http://code.google.com/p/redis/
- * @see http://github.com/owlient/phpredis
- * @api
+ * @see https://redis.io/
+ * @see https://github.com/phpredis/phpredis
  */
 class RedisBackend extends AbstractBackend implements TaggableBackendInterface
 {
@@ -37,7 +38,7 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * To save these additional calls on every set(),
      * we just make every entry volatile and treat a high number as "unlimited"
      *
-     * @see http://code.google.com/p/redis/wiki/ExpireCommand
+     * @see https://redis.io/commands/expire
      * @var int Faked unlimited lifetime
      */
     const FAKED_UNLIMITED_LIFETIME = 31536000;
@@ -73,6 +74,13 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      */
     protected $connected = false;
 
+    /**
+     * Persistent connection
+     *
+     * @var bool
+     */
+    protected $persistentConnection = false;
+
     /**
      * Hostname / IP of the Redis server, defaults to 127.0.0.1.
      *
@@ -115,17 +123,24 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      */
     protected $compressionLevel = -1;
 
+    /**
+     * limit in seconds (default is 0 meaning unlimited)
+     *
+     * @var int
+     */
+    protected $connectionTimeout = 0;
+
     /**
      * Construct this backend
      *
-     * @param string $context FLOW3's application context
+     * @param string $context Unused, for backward compatibility only
      * @param array $options Configuration options
-     * @throws \TYPO3\CMS\Core\Cache\Exception if php redis module is not loaded
+     * @throws Exception if php redis module is not loaded
      */
     public function __construct($context, array $options = [])
     {
         if (!extension_loaded('redis')) {
-            throw new \TYPO3\CMS\Core\Cache\Exception('The PHP extension "redis" must be installed and loaded in order to use the redis backend.', 1279462933);
+            throw new Exception('The PHP extension "redis" must be installed and loaded in order to use the redis backend.', 1279462933);
         }
         parent::__construct($context, $options);
     }
@@ -133,39 +148,50 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
     /**
      * Initializes the redis backend
      *
-     * @return void
-     * @throws \TYPO3\CMS\Core\Cache\Exception if access to redis with password is denied or if database selection fails
+     * @throws Exception if access to redis with password is denied or if database selection fails
      */
     public function initializeObject()
     {
         $this->redis = new \Redis();
         try {
-            $this->connected = $this->redis->connect($this->hostname, $this->port);
+            if ($this->persistentConnection) {
+                $this->connected = $this->redis->pconnect($this->hostname, $this->port, $this->connectionTimeout, (string)$this->database);
+            } else {
+                $this->connected = $this->redis->connect($this->hostname, $this->port, $this->connectionTimeout);
+            }
         } catch (\Exception $e) {
-            \TYPO3\CMS\Core\Utility\GeneralUtility::sysLog('Could not connect to redis server.', 'core', \TYPO3\CMS\Core\Utility\GeneralUtility::SYSLOG_SEVERITY_ERROR);
+            $this->logger->alert('Could not connect to redis server.', ['exception' => $e]);
         }
         if ($this->connected) {
             if ($this->password !== '') {
                 $success = $this->redis->auth($this->password);
                 if (!$success) {
-                    throw new \TYPO3\CMS\Core\Cache\Exception('The given password was not accepted by the redis server.', 1279765134);
+                    throw new Exception('The given password was not accepted by the redis server.', 1279765134);
                 }
             }
-            if ($this->database > 0) {
+            if ($this->database >= 0) {
                 $success = $this->redis->select($this->database);
                 if (!$success) {
-                    throw new \TYPO3\CMS\Core\Cache\Exception('The given database "' . $this->database . '" could not be selected.', 1279765144);
+                    throw new Exception('The given database "' . $this->database . '" could not be selected.', 1279765144);
                 }
             }
         }
     }
 
+    /**
+     * Setter for persistent connection
+     *
+     * @param bool $persistentConnection
+     */
+    public function setPersistentConnection($persistentConnection)
+    {
+        $this->persistentConnection = $persistentConnection;
+    }
+
     /**
      * Setter for server hostname
      *
      * @param string $hostname Hostname
-     * @return void
-     * @api
      */
     public function setHostname($hostname)
     {
@@ -176,8 +202,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Setter for server port
      *
      * @param int $port Port
-     * @return void
-     * @api
      */
     public function setPort($port)
     {
@@ -188,13 +212,11 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Setter for database number
      *
      * @param int $database Database
-     * @return void
      * @throws \InvalidArgumentException if database number is not valid
-     * @api
      */
     public function setDatabase($database)
     {
-        if (!is_integer($database)) {
+        if (!is_int($database)) {
             throw new \InvalidArgumentException('The specified database number is of type "' . gettype($database) . '" but an integer is expected.', 1279763057);
         }
         if ($database < 0) {
@@ -207,8 +229,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Setter for authentication password
      *
      * @param string $password Password
-     * @return void
-     * @api
      */
     public function setPassword($password)
     {
@@ -219,9 +239,7 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Enable data compression
      *
      * @param bool $compression TRUE to enable compression
-     * @return void
      * @throws \InvalidArgumentException if compression parameter is not of type boolean
-     * @api
      */
     public function setCompression($compression)
     {
@@ -237,13 +255,11 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * gzcompress default level will be used.
      *
      * @param int $compressionLevel -1 to 9: Compression level
-     * @return void
      * @throws \InvalidArgumentException if compressionLevel parameter is not within allowed bounds
-     * @api
      */
     public function setCompressionLevel($compressionLevel)
     {
-        if (!is_integer($compressionLevel)) {
+        if (!is_int($compressionLevel)) {
             throw new \InvalidArgumentException('The specified compression of type "' . gettype($compressionLevel) . '" but an integer is expected.', 1289679154);
         }
         if ($compressionLevel >= -1 && $compressionLevel <= 9) {
@@ -253,6 +269,27 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
         }
     }
 
+    /**
+     * Set connection timeout.
+     * This value in seconds is used as a maximum number
+     * of seconds to wait if a connection can be established.
+     *
+     * @param int $connectionTimeout limit in seconds, a value greater or equal than 0
+     * @throws \InvalidArgumentException if compressionLevel parameter is not within allowed bounds
+     */
+    public function setConnectionTimeout($connectionTimeout)
+    {
+        if (!is_int($connectionTimeout)) {
+            throw new \InvalidArgumentException('The specified connection timeout is of type "' . gettype($connectionTimeout) . '" but an integer is expected.', 1487849315);
+        }
+
+        if ($connectionTimeout < 0) {
+            throw new \InvalidArgumentException('The specified connection timeout "' . $connectionTimeout . '" must be greater or equal than zero.', 1487849326);
+        }
+
+        $this->connectionTimeout = $connectionTimeout;
+    }
+
     /**
      * Save data in the cache
      *
@@ -263,10 +300,8 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * @param string $data Data to be stored
      * @param array $tags Tags to associate with this cache entry
      * @param int $lifetime Lifetime of this cache entry in seconds. If NULL is specified, default lifetime is used. "0" means unlimited lifetime.
-     * @return void
      * @throws \InvalidArgumentException if identifier is not valid
-     * @throws \TYPO3\CMS\Core\Cache\Exception\InvalidDataException if data is not a string
-     * @api
+     * @throws InvalidDataException if data is not a string
      */
     public function set($entryIdentifier, $data, array $tags = [], $lifetime = null)
     {
@@ -274,10 +309,10 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
             throw new \InvalidArgumentException('The specified identifier is of type "' . gettype($entryIdentifier) . '" which can\'t be converted to string.', 1377006651);
         }
         if (!is_string($data)) {
-            throw new \TYPO3\CMS\Core\Cache\Exception\InvalidDataException('The specified data is of type "' . gettype($data) . '" but a string is expected.', 1279469941);
+            throw new InvalidDataException('The specified data is of type "' . gettype($data) . '" but a string is expected.', 1279469941);
         }
-        $lifetime = $lifetime === null ? $this->defaultLifetime : $lifetime;
-        if (!is_integer($lifetime)) {
+        $lifetime = $lifetime ?? $this->defaultLifetime;
+        if (!is_int($lifetime)) {
             throw new \InvalidArgumentException('The specified lifetime is of type "' . gettype($lifetime) . '" but an integer or NULL is expected.', 1279488008);
         }
         if ($lifetime < 0) {
@@ -299,8 +334,8 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
             if (!empty($removeTags) || !empty($addTags)) {
                 $queue = $this->redis->multi(\Redis::PIPELINE);
                 foreach ($removeTags as $tag) {
-                    $queue->sRemove(self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier, $tag);
-                    $queue->sRemove(self::TAG_IDENTIFIERS_PREFIX . $tag, $entryIdentifier);
+                    $queue->sRem(self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier, $tag);
+                    $queue->sRem(self::TAG_IDENTIFIERS_PREFIX . $tag, $entryIdentifier);
                 }
                 foreach ($addTags as $tag) {
                     $queue->sAdd(self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier, $tag);
@@ -319,7 +354,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * @param string $entryIdentifier An identifier which describes the cache entry to load
      * @return mixed The cache entry's content as a string or FALSE if the cache entry could not be loaded
      * @throws \InvalidArgumentException if identifier is not a string
-     * @api
      */
     public function get($entryIdentifier)
     {
@@ -344,7 +378,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * @param string $entryIdentifier Identifier specifying the cache entry
      * @return bool TRUE if such an entry exists, FALSE if not
      * @throws \InvalidArgumentException if identifier is not a string
-     * @api
      */
     public function has($entryIdentifier)
     {
@@ -363,7 +396,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * @param string $entryIdentifier Specifies the cache entry to remove
      * @return bool TRUE if (at least) an entry could be removed or FALSE if no entry was found
      * @throws \InvalidArgumentException if identifier is not a string
-     * @api
      */
     public function remove($entryIdentifier)
     {
@@ -376,9 +408,9 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
                 $assignedTags = $this->redis->sMembers(self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier);
                 $queue = $this->redis->multi(\Redis::PIPELINE);
                 foreach ($assignedTags as $tag) {
-                    $queue->sRemove(self::TAG_IDENTIFIERS_PREFIX . $tag, $entryIdentifier);
+                    $queue->sRem(self::TAG_IDENTIFIERS_PREFIX . $tag, $entryIdentifier);
                 }
-                $queue->delete(self::IDENTIFIER_DATA_PREFIX . $entryIdentifier, self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier);
+                $queue->del(self::IDENTIFIER_DATA_PREFIX . $entryIdentifier, self::IDENTIFIER_TAGS_PREFIX . $entryIdentifier);
                 $queue->exec();
                 $elementsDeleted = true;
             }
@@ -396,7 +428,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * @param string $tag The tag to search for
      * @return array An array of entries with all matching entries. An empty array if no entries matched
      * @throws \InvalidArgumentException if tag is not a string
-     * @api
      */
     public function findIdentifiersByTag($tag)
     {
@@ -414,14 +445,11 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Removes all cache entries of this cache.
      *
      * Scales O(1) with number of cache entries
-     *
-     * @return void
-     * @api
      */
     public function flush()
     {
         if ($this->connected) {
-            $this->redis->flushdb();
+            $this->redis->flushDB();
         }
     }
 
@@ -432,9 +460,7 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * Scales O(n^2) with number of tag entries
      *
      * @param string $tag Tag the entries must have
-     * @return void
      * @throws \InvalidArgumentException if identifier is not a string
-     * @api
      */
     public function flushByTag($tag)
     {
@@ -456,22 +482,19 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      * This methods finds those entries and cleans them up.
      *
      * Scales O(n*m) with number of cache entries (n) and number of tags (m)
-     *
-     * @return void
-     * @api
      */
     public function collectGarbage()
     {
-        $identifierToTagsKeys = $this->redis->getKeys(self::IDENTIFIER_TAGS_PREFIX . '*');
+        $identifierToTagsKeys = $this->redis->keys(self::IDENTIFIER_TAGS_PREFIX . '*');
         foreach ($identifierToTagsKeys as $identifierToTagsKey) {
             list(, $identifier) = explode(':', $identifierToTagsKey);
             // Check if the data entry still exists
-            if (!$this->redis->exists((self::IDENTIFIER_DATA_PREFIX . $identifier))) {
+            if (!$this->redis->exists(self::IDENTIFIER_DATA_PREFIX . $identifier)) {
                 $tagsToRemoveIdentifierFrom = $this->redis->sMembers($identifierToTagsKey);
                 $queue = $this->redis->multi(\Redis::PIPELINE);
-                $queue->delete($identifierToTagsKey);
+                $queue->del($identifierToTagsKey);
                 foreach ($tagsToRemoveIdentifierFrom as $tag) {
-                    $queue->sRemove(self::TAG_IDENTIFIERS_PREFIX . $tag, $identifier);
+                    $queue->sRem(self::TAG_IDENTIFIERS_PREFIX . $tag, $identifier);
                 }
                 $queue->exec();
             }
@@ -487,7 +510,6 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
      *
      * @param array $identifiers List of identifiers to remove
      * @param array $tags List of tags to be handled
-     * @return void
      */
     protected function removeIdentifierEntriesAndRelations(array $identifiers, array $tags)
     {
@@ -515,7 +537,7 @@ class RedisBackend extends AbstractBackend implements TaggableBackendInterface
         foreach ($tagToIdentifiersSetsToRemoveIdentifiersFrom as $tagToIdentifiersSet) {
             $queue->sDiffStore(self::TAG_IDENTIFIERS_PREFIX . $tagToIdentifiersSet, self::TAG_IDENTIFIERS_PREFIX . $tagToIdentifiersSet, $uniqueTempKey);
         }
-        $queue->delete(array_merge($prefixedKeysToDelete, $prefixedIdentifierToTagsKeysToDelete));
+        $queue->del(array_merge($prefixedKeysToDelete, $prefixedIdentifierToTagsKeysToDelete));
         $queue->exec();
     }
 
