@@ -14,34 +14,14 @@ namespace TYPO3\CMS\Core\Cache\Frontend;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Cache\Backend\TransientBackendInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
  * A cache frontend for any kinds of PHP variables
- *
- * This file is a backport from FLOW3
- * @api
  */
 class VariableFrontend extends AbstractFrontend
 {
-    /**
-     * If the extension "igbinary" is installed, use it for increased performance.
-     * Caching the result of extension_loaded() here is faster than calling extension_loaded() multiple times.
-     *
-     * @var bool
-     */
-    protected $useIgBinary = false;
-
-    /**
-     * Initializes this cache frontend
-     *
-     * @return void
-     */
-    public function initializeObject()
-    {
-        $this->useIgBinary = extension_loaded('igbinary');
-    }
-
     /**
      * Saves the value of a PHP variable in the cache. Note that the variable
      * will be serialized if necessary.
@@ -49,70 +29,73 @@ class VariableFrontend extends AbstractFrontend
      * @param string $entryIdentifier An identifier used for this cache entry
      * @param mixed $variable The variable to cache
      * @param array $tags Tags to associate with this cache entry
-     * @param int $lifetime Lifetime of this cache entry in seconds. If NULL is specified, the default lifetime is used. "0" means unlimited liftime.
-     * @return void
+     * @param int $lifetime Lifetime of this cache entry in seconds. If NULL is specified, the default lifetime is used. "0" means unlimited lifetime.
      * @throws \InvalidArgumentException if the identifier or tag is not valid
-     * @api
      */
     public function set($entryIdentifier, $variable, array $tags = [], $lifetime = null)
     {
         if (!$this->isValidEntryIdentifier($entryIdentifier)) {
-            throw new \InvalidArgumentException('"' . $entryIdentifier . '" is not a valid cache entry identifier.', 1233058264);
+            throw new \InvalidArgumentException(
+                '"' . $entryIdentifier . '" is not a valid cache entry identifier.',
+                1233058264
+            );
         }
         foreach ($tags as $tag) {
             if (!$this->isValidTag($tag)) {
                 throw new \InvalidArgumentException('"' . $tag . '" is not a valid tag for a cache entry.', 1233058269);
             }
         }
-        if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/cache/frontend/class.t3lib_cache_frontend_variablefrontend.php']['set'])) {
-            foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/cache/frontend/class.t3lib_cache_frontend_variablefrontend.php']['set'] as $_funcRef) {
-                $params = [
-                    'entryIdentifier' => &$entryIdentifier,
-                    'variable' => &$variable,
-                    'tags' => &$tags,
-                    'lifetime' => &$lifetime
-                ];
-                GeneralUtility::callUserFunction($_funcRef, $params, $this);
-            }
+
+        foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/cache/frontend/class.t3lib_cache_frontend_variablefrontend.php']['set'] ?? [] as $_funcRef) {
+            $params = [
+                'entryIdentifier' => &$entryIdentifier,
+                'variable' => &$variable,
+                'tags' => &$tags,
+                'lifetime' => &$lifetime
+            ];
+            GeneralUtility::callUserFunction($_funcRef, $params, $this);
         }
-        if ($this->useIgBinary === true) {
-            $this->backend->set($entryIdentifier, igbinary_serialize($variable), $tags, $lifetime);
-        } else {
-            $this->backend->set($entryIdentifier, serialize($variable), $tags, $lifetime);
+        if (!$this->backend instanceof TransientBackendInterface) {
+            $variable = serialize($variable);
         }
+        $this->backend->set($entryIdentifier, $variable, $tags, $lifetime);
     }
 
     /**
      * Finds and returns a variable value from the cache.
      *
      * @param string $entryIdentifier Identifier of the cache entry to fetch
+     *
      * @return mixed The value
      * @throws \InvalidArgumentException if the identifier is not valid
-     * @api
      */
     public function get($entryIdentifier)
     {
         if (!$this->isValidEntryIdentifier($entryIdentifier)) {
-            throw new \InvalidArgumentException('"' . $entryIdentifier . '" is not a valid cache entry identifier.', 1233058294);
+            throw new \InvalidArgumentException(
+                '"' . $entryIdentifier . '" is not a valid cache entry identifier.',
+                1233058294
+            );
         }
         $rawResult = $this->backend->get($entryIdentifier);
         if ($rawResult === false) {
             return false;
-        } else {
-            return $this->useIgBinary === true ? igbinary_unserialize($rawResult) : unserialize($rawResult);
         }
+        return $this->backend instanceof TransientBackendInterface ? $rawResult : unserialize($rawResult);
     }
 
     /**
      * Finds and returns all cache entries which are tagged by the specified tag.
      *
      * @param string $tag The tag to search for
+     *
      * @return array An array with the content of all matching entries. An empty array if no entries matched
      * @throws \InvalidArgumentException if the tag is not valid
-     * @api
+     * @deprecated since TYPO3 v9, Avoid using this method since it is not compliant to PSR-6
      */
     public function getByTag($tag)
     {
+        trigger_error('VariableFrontend->getByTag() will be removed in TYPO3 v10.0. Avoid using this method since it is not compliant to PSR-6.', E_USER_DEPRECATED);
         if (!$this->isValidTag($tag)) {
             throw new \InvalidArgumentException('"' . $tag . '" is not a valid tag for a cache entry.', 1233058312);
         }
@@ -121,7 +104,7 @@ class VariableFrontend extends AbstractFrontend
         foreach ($identifiers as $identifier) {
             $rawResult = $this->backend->get($identifier);
             if ($rawResult !== false) {
-                $entries[] = $this->useIgBinary === true ? igbinary_unserialize($rawResult) : unserialize($rawResult);
+                $entries[] = $this->backend instanceof TransientBackendInterface ? $rawResult : unserialize($rawResult);
             }
         }
         return $entries;
