@@ -14,11 +14,12 @@ namespace TYPO3\CMS\Core\DataHandling;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Type\Enumeration;
+
 /**
  * Enumeration object for tca type
- *
  */
-class TableColumnType extends \TYPO3\CMS\Core\Type\Enumeration
+final class TableColumnType extends Enumeration
 {
     const __default = self::INPUT;
 
@@ -37,6 +38,7 @@ class TableColumnType extends \TYPO3\CMS\Core\Type\Enumeration
     const FLEX = 'FLEX';
     const INLINE = 'INLINE';
     const IMAGEMANIPULATION = 'IMAGEMANIPULATION';
+    const SLUG = 'SLUG';
 
     /**
      * @param mixed $type
