@@ -14,8 +14,6 @@ namespace TYPO3\CMS\Core\Imaging;
  * The TYPO3 project - inspiring people to share!
  */
 
-use Psr\Http\Message\ResponseInterface;
-use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Core\Resource\File;
 use TYPO3\CMS\Core\Resource\FolderInterface;
 use TYPO3\CMS\Core\Resource\InaccessibleFolder;
@@ -52,6 +50,13 @@ class IconFactory
      */
     protected $overlayPriorities = [];
 
+    /**
+     * Runtime icon cache
+     *
+     * @var array
+     */
+    protected static $iconCache = [];
+
     /**
      * @param IconRegistry $iconRegistry
      */
@@ -62,33 +67,6 @@ class IconFactory
         $this->overlayPriorities = $GLOBALS['TYPO3_CONF_VARS']['SYS']['IconFactory']['overlayPriorities'];
     }
 
-    /**
-     * @param ServerRequestInterface $request
-     * @param ResponseInterface $response
-     * @return string
-     * @internal
-     */
-    public function processAjaxRequest(ServerRequestInterface $request, ResponseInterface $response)
-    {
-        $parsedBody = $request->getParsedBody();
-        $queryParams = $request->getQueryParams();
-        $requestedIcon = json_decode(
-            isset($parsedBody['icon']) ? $parsedBody['icon'] : $queryParams['icon'],
-            true
-        );
-
-        list($identifier, $size, $overlayIdentifier, $iconState) = $requestedIcon;
-        if (empty($overlayIdentifier)) {
-            $overlayIdentifier = null;
-        }
-        $iconState = IconState::cast($iconState);
-        $response->getBody()->write(
-            $this->getIcon($identifier, $size, $overlayIdentifier, $iconState)->render()
-        );
-        $response = $response->withHeader('Content-Type', 'text/html; charset=utf-8');
-        return $response;
-    }
-
     /**
      * @param string $identifier
      * @param string $size "large", "small" or "default", see the constants of the Icon class
@@ -98,14 +76,16 @@ class IconFactory
      */
     public function getIcon($identifier, $size = Icon::SIZE_DEFAULT, $overlayIdentifier = null, IconState $state = null)
     {
-        if ($this->iconRegistry->isDeprecated($identifier)) {
-            $deprecationSettings = $this->iconRegistry->getDeprecationSettings($identifier);
-            GeneralUtility::deprecationLog(sprintf($deprecationSettings['message'], $identifier));
-            if (!empty($deprecationSettings['replacement'])) {
-                $identifier = $deprecationSettings['replacement'];
-            }
+        $cacheIdentifier = md5($identifier . $size . $overlayIdentifier . (string)$state);
+        if (!empty(static::$iconCache[$cacheIdentifier])) {
+            return static::$iconCache[$cacheIdentifier];
         }
-        if (!$this->iconRegistry->isRegistered($identifier)) {
+
+        if (
+            !$this->iconRegistry->isDeprecated($identifier)
+            && !$this->iconRegistry->isRegistered($identifier)
+        ) {
+            // in case icon identifier is neither deprecated nor registered
             $identifier = $this->iconRegistry->getDefaultIconIdentifier();
         }
 
@@ -117,6 +97,8 @@ class IconFactory
         $iconProvider = GeneralUtility::makeInstance($iconConfiguration['provider']);
         $iconProvider->prepareIconMarkup($icon, $iconConfiguration['options']);
 
+        static::$iconCache[$cacheIdentifier] = $icon;
+
         return $icon;
     }
 
@@ -132,9 +114,6 @@ class IconFactory
     {
         $iconIdentifier = $this->mapRecordTypeToIconIdentifier($table, $row);
         $overlayIdentifier = $this->mapRecordTypeToOverlayIdentifier($table, $row);
-        if (empty($overlayIdentifier)) {
-            $overlayIdentifier = null;
-        }
         return $this->getIcon($iconIdentifier, $size, $overlayIdentifier);
     }
 
@@ -195,7 +174,9 @@ class IconFactory
                     }
                 }
             }
-            if (is_array($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])) {
+            if (isset($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])
+                && is_array($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])
+            ) {
                 foreach ($recordType as $key => $type) {
                     if (isset($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'][$type])) {
                         $recordType[$key] = $GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'][$type];
@@ -226,7 +207,9 @@ class IconFactory
                 unset($type);
                 $recordType[0] = 'tcarecords-' . $table . '-default';
             }
-        } elseif (is_array($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])) {
+        } elseif (isset($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])
+            && is_array($GLOBALS['TCA'][$table]['ctrl']['typeicon_classes'])
+        ) {
             $recordType[0] = $GLOBALS['TCA'][$table]['ctrl']['typeicon_classes']['default'];
         } else {
             $recordType[0] = 'tcarecords-' . $table . '-default';
@@ -263,10 +246,10 @@ class IconFactory
             'fe_group' => false,
             'deleted' => false,
             'protectedSection' => false,
-            'nav_hide' => (bool)$row['nav_hide']
+            'nav_hide' => !empty($row['nav_hide']),
         ];
         // Icon state based on "enableFields":
-        if (is_array($tcaCtrl['enablecolumns'])) {
+        if (isset($tcaCtrl['enablecolumns']) && is_array($tcaCtrl['enablecolumns'])) {
             $enableColumns = $tcaCtrl['enablecolumns'];
             // If "hidden" is enabled:
             if (isset($enableColumns['disabled']) && !empty($row[$enableColumns['disabled']])) {
@@ -316,13 +299,10 @@ class IconFactory
         }
 
         // Hook to define an alternative iconName
-        if (!empty($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS'][IconFactory::class]['overrideIconOverlay'])) {
-            $hookObjects = $GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS'][IconFactory::class]['overrideIconOverlay'];
-            foreach ($hookObjects as $classRef) {
-                $hookObject = GeneralUtility::getUserObj($classRef);
-                if (method_exists($hookObject, 'postOverlayPriorityLookup')) {
-                    $iconName = $hookObject->postOverlayPriorityLookup($table, $row, $status, $iconName);
-                }
+        foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS'][self::class]['overrideIconOverlay'] ?? [] as $className) {
+            $hookObject = GeneralUtility::makeInstance($className);
+            if (method_exists($hookObject, 'postOverlayPriorityLookup')) {
+                $iconName = $hookObject->postOverlayPriorityLookup($table, $row, $status, $iconName);
             }
         }
 
@@ -415,8 +395,6 @@ class IconFactory
                     $overlayIdentifier = 'overlay-locked';
                 }
             }
-
-        // File
         } elseif ($resource instanceof File) {
             $mimeTypeIcon = $this->iconRegistry->getIconIdentifierForMimeType($resource->getMimeType());
 
@@ -467,7 +445,7 @@ class IconFactory
         $icon->setIdentifier($identifier);
         $icon->setSize($size);
         $icon->setState($iconConfiguration['state'] ?: new IconState());
-        if ($overlayIdentifier !== null) {
+        if (!empty($overlayIdentifier)) {
             $icon->setOverlayIcon($this->getIcon($overlayIdentifier, Icon::SIZE_OVERLAY));
         }
         if (!empty($iconConfiguration['options']['spinning'])) {
@@ -497,7 +475,7 @@ class IconFactory
         $overlayIdentifier
     ) {
         $result = $this->getSignalSlotDispatcher()->dispatch(
-            IconFactory::class,
+            self::class,
             'buildIconForResourceSignal',
             [$resource, $size, $options, $iconIdentifier, $overlayIdentifier]
         );
@@ -515,4 +493,12 @@ class IconFactory
     {
         return GeneralUtility::makeInstance(Dispatcher::class);
     }
+
+    /**
+     * clear icon cache
+     */
+    public function clearIconCache()
+    {
+        static::$iconCache = [];
+    }
 }
