@@ -14,6 +14,11 @@ namespace TYPO3\CMS\Core\Resource\Driver;
  * The TYPO3 project - inspiring people to share!
  */
 
+use Psr\Http\Message\ResponseInterface;
+use TYPO3\CMS\Core\Charset\CharsetConverter;
+use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Core\Http\Response;
+use TYPO3\CMS\Core\Http\SelfEmittableLazyOpenStream;
 use TYPO3\CMS\Core\Resource\Exception;
 use TYPO3\CMS\Core\Resource\FolderInterface;
 use TYPO3\CMS\Core\Resource\ResourceStorage;
@@ -23,9 +28,8 @@ use TYPO3\CMS\Core\Utility\PathUtility;
 
 /**
  * Driver for the local file system
- *
  */
-class LocalDriver extends AbstractHierarchicalFilesystemDriver
+class LocalDriver extends AbstractHierarchicalFilesystemDriver implements StreamableDriverInterface
 {
     /**
      * @var string
@@ -52,12 +56,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      *
      * @var string
      */
-    protected $baseUri = null;
-
-    /**
-     * @var \TYPO3\CMS\Core\Charset\CharsetConverter
-     */
-    protected $charsetConversion;
+    protected $baseUri;
 
     /** @var array */
     protected $mappingFolderNameToRole = [
@@ -76,7 +75,8 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         $this->capabilities =
             ResourceStorage::CAPABILITY_BROWSABLE
             | ResourceStorage::CAPABILITY_PUBLIC
-            | ResourceStorage::CAPABILITY_WRITABLE;
+            | ResourceStorage::CAPABILITY_WRITABLE
+            | ResourceStorage::CAPABILITY_HIERARCHICAL_IDENTIFIERS;
     }
 
     /**
@@ -90,13 +90,12 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     public function mergeConfigurationCapabilities($capabilities)
     {
         $this->capabilities &= $capabilities;
+
         return $this->capabilities;
     }
 
     /**
      * Processes the configuration for this driver.
-     *
-     * @return void
      */
     public function processConfiguration()
     {
@@ -111,8 +110,6 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     /**
      * Initializes this object. This is called by the storage after the driver
      * has been attached.
-     *
-     * @return void
      */
     public function initialize()
     {
@@ -121,14 +118,12 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     /**
      * Determines the base URL for this driver, from the configuration or
      * the TypoScript frontend object
-     *
-     * @return void
      */
     protected function determineBaseUrl()
     {
         // only calculate baseURI if the storage does not enforce jumpUrl Script
         if ($this->hasCapability(ResourceStorage::CAPABILITY_PUBLIC)) {
-            if (GeneralUtility::isFirstPartOfStr($this->absoluteBasePath, PATH_site)) {
+            if (GeneralUtility::isFirstPartOfStr($this->absoluteBasePath, Environment::getPublicPath())) {
                 // use site-relative URLs
                 $temporaryBaseUri = rtrim(PathUtility::stripPathSitePrefix($this->absoluteBasePath), '/');
                 if ($temporaryBaseUri !== '') {
@@ -149,6 +144,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      * @throws Exception\InvalidConfigurationException
      * @param array $configuration
      * @return string
+     * @throws Exception\InvalidPathException
      */
     protected function calculateBasePath(array $configuration)
     {
@@ -159,9 +155,9 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             );
         }
 
-        if ($configuration['pathType'] === 'relative') {
+        if (!empty($configuration['pathType']) && $configuration['pathType'] === 'relative') {
             $relativeBasePath = $configuration['basePath'];
-            $absoluteBasePath = PATH_site . $relativeBasePath;
+            $absoluteBasePath = Environment::getPublicPath() . '/' . $relativeBasePath;
         } else {
             $absoluteBasePath = $configuration['basePath'];
         }
@@ -178,10 +174,10 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
 
     /**
      * Returns the public URL to a file.
-     * For the local driver, this will always return a path relative to PATH_site.
+     * For the local driver, this will always return a path relative to public web path.
      *
      * @param string $identifier
-     * @return string
+     * @return string|null NULL if file is missing or deleted, the generated url otherwise
      * @throws \TYPO3\CMS\Core\Resource\Exception
      */
     public function getPublicUrl($identifier)
@@ -234,16 +230,18 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     {
         $parentFolderIdentifier = $this->canonicalizeAndCheckFolderIdentifier($parentFolderIdentifier);
         $newFolderName = trim($newFolderName, '/');
-        if ($recursive == false) {
+        if ($recursive === false) {
             $newFolderName = $this->sanitizeFileName($newFolderName);
-            $newIdentifier = $parentFolderIdentifier . $newFolderName . '/';
+            $newIdentifier = $this->canonicalizeAndCheckFolderIdentifier($parentFolderIdentifier . $newFolderName . '/');
             GeneralUtility::mkdir($this->getAbsolutePath($newIdentifier));
         } else {
             $parts = GeneralUtility::trimExplode('/', $newFolderName);
             $parts = array_map([$this, 'sanitizeFileName'], $parts);
             $newFolderName = implode('/', $parts);
-            $newIdentifier = $parentFolderIdentifier . $newFolderName . '/';
-            GeneralUtility::mkdir_deep($this->getAbsolutePath($parentFolderIdentifier) . '/', $newFolderName);
+            $newIdentifier = $this->canonicalizeAndCheckFolderIdentifier(
+                $parentFolderIdentifier . $newFolderName . '/'
+            );
+            GeneralUtility::mkdir_deep($this->getAbsolutePath($newIdentifier));
         }
         return $newIdentifier;
     }
@@ -287,9 +285,12 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
                 1314516810
             );
         }
+        $absolutePath = $this->getAbsolutePath($folderIdentifier);
         return [
             'identifier' => $folderIdentifier,
             'name' => PathUtility::basename($folderIdentifier),
+            'mtime' => filemtime($absolutePath),
+            'ctime' => filectime($absolutePath),
             'storage' => $this->storageUid
         ];
     }
@@ -302,31 +303,19 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      * Previously in \TYPO3\CMS\Core\Utility\File\BasicFileUtility::cleanFileName()
      *
      * @param string $fileName Input string, typically the body of a fileName
-     * @param string $charset Charset of the a fileName (defaults to current charset; depending on context)
+     * @param string $charset Charset of the a fileName (defaults to utf-8)
      * @return string Output string with any characters not matching [.a-zA-Z0-9_-] is substituted by '_' and trailing dots removed
      * @throws Exception\InvalidFileNameException
      */
-    public function sanitizeFileName($fileName, $charset = '')
+    public function sanitizeFileName($fileName, $charset = 'utf-8')
     {
         // Handle UTF-8 characters
         if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['UTF8filesystem']) {
             // Allow ".", "-", 0-9, a-z, A-Z and everything beyond U+C0 (latin capital letter a with grave)
             $cleanFileName = preg_replace('/[' . self::UNSAFE_FILENAME_CHARACTER_EXPRESSION . ']/u', '_', trim($fileName));
         } else {
-            // Define character set
-            if (!$charset) {
-                if (TYPO3_MODE === 'FE') {
-                    $charset = $GLOBALS['TSFE']->renderCharset;
-                } else {
-                    // default for Backend
-                    $charset = 'utf-8';
-                }
-            }
-            // If a charset was found, convert fileName
-            if ($charset) {
-                $fileName = $this->getCharsetConversion()->specCharsToASCII($charset, $fileName);
-            }
-            // Replace unwanted characters by underscores
+            $fileName = GeneralUtility::makeInstance(CharsetConverter::class)->specCharsToASCII($charset, $fileName);
+            // Replace unwanted characters with underscores
             $cleanFileName = preg_replace('/[' . self::UNSAFE_FILENAME_CHARACTER_EXPRESSION . '\\xC0-\\xFF]/', '_', trim($fileName));
         }
         // Strip trailing dots and return
@@ -375,7 +364,6 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         if ($iterator->count() === 0) {
             return [];
         }
-        $iterator->seek($start);
 
         // $c is the counter for how many items we still have to fetch (-1 is unlimited)
         $c = $numberOfItems > 0 ? $numberOfItems : - 1;
@@ -386,22 +374,28 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             // go on to the next iterator item now as we might skip this one early
             $iterator->next();
 
-            if (
+            try {
+                if (
                 !$this->applyFilterMethodsToDirectoryItem(
                     $filterMethods,
                     $iteratorItem['name'],
                     $iteratorItem['identifier'],
                     $this->getParentFolderIdentifierOfIdentifier($iteratorItem['identifier'])
                 )
-            ) {
-                continue;
+                ) {
+                    continue;
+                }
+                if ($start > 0) {
+                    $start--;
+                } else {
+                    $items[$iteratorItem['identifier']] = $iteratorItem['identifier'];
+                    // Decrement item counter to make sure we only return $numberOfItems
+                    // we cannot do this earlier in the method (unlike moving the iterator forward) because we only add the
+                    // item here
+                    --$c;
+                }
+            } catch (Exception\InvalidPathException $e) {
             }
-
-            $items[$iteratorItem['identifier']] = $iteratorItem['identifier'];
-            // Decrement item counter to make sure we only return $numberOfItems
-            // we cannot do this earlier in the method (unlike moving the iterator forward) because we only add the
-            // item here
-            --$c;
         }
         return $items;
     }
@@ -426,8 +420,12 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
                 // If calling the method succeeded and thus we can't use that as a return value.
                 if ($result === -1) {
                     return false;
-                } elseif ($result === false) {
-                    throw new \RuntimeException('Could not apply file/folder name filter ' . $filter[0] . '::' . $filter[1]);
+                }
+                if ($result === false) {
+                    throw new \RuntimeException(
+                        'Could not apply file/folder name filter ' . $filter[0] . '::' . $filter[1],
+                        1476046425
+                    );
                 }
             }
         }
@@ -536,7 +534,8 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         if ($recursive) {
             $iterator = new \RecursiveIteratorIterator(
                 new \RecursiveDirectoryIterator($path, $iteratorMode),
-                \RecursiveIteratorIterator::SELF_FIRST
+                \RecursiveIteratorIterator::SELF_FIRST,
+                \RecursiveIteratorIterator::CATCH_GET_CHILD
             );
         } else {
             $iterator = new \RecursiveDirectoryIterator($path, $iteratorMode);
@@ -544,23 +543,28 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
 
         $directoryEntries = [];
         while ($iterator->valid()) {
-            /** @var $entry \SplFileInfo */
+            /** @var \SplFileInfo $entry */
             $entry = $iterator->current();
-            // skip non-files/non-folders, and empty entries
-            if ((!$entry->isFile() && !$entry->isDir()) || $entry->getFilename() == '' ||
-                ($entry->isFile() && !$includeFiles) || ($entry->isDir() && !$includeDirs)) {
+            $isFile = $entry->isFile();
+            $isDirectory = $isFile ? false : $entry->isDir();
+            if (
+                (!$isFile && !$isDirectory) // skip non-files/non-folders
+                || ($isFile && !$includeFiles) // skip files if they are excluded
+                || ($isDirectory && !$includeDirs) // skip directories if they are excluded
+                || $entry->getFilename() === '' // skip empty entries
+            ) {
                 $iterator->next();
                 continue;
             }
             $entryIdentifier = '/' . substr($entry->getPathname(), $pathLength);
             $entryName = PathUtility::basename($entryIdentifier);
-            if ($entry->isDir()) {
+            if ($isDirectory) {
                 $entryIdentifier .= '/';
             }
             $entryArray = [
                 'identifier' => $entryIdentifier,
                 'name' => $entryName,
-                'type' => $entry->isDir() ? 'dir' : 'file'
+                'type' => $isDirectory ? 'dir' : 'file'
             ];
             $directoryEntries[$entryIdentifier] = $entryArray;
             $iterator->next();
@@ -644,7 +648,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     {
         if (empty($propertiesToExtract)) {
             $propertiesToExtract = [
-                'size', 'atime', 'atime', 'mtime', 'ctime', 'mimetype', 'name',
+                'size', 'atime', 'mtime', 'ctime', 'mimetype', 'name', 'extension',
                 'identifier', 'identifier_hash', 'storage', 'folder_hash'
             ];
         }
@@ -682,6 +686,8 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
                 return $fileInfo->getCTime();
             case 'name':
                 return PathUtility::basename($fileIdentifier);
+            case 'extension':
+                return PathUtility::pathinfo($fileIdentifier, PATHINFO_EXTENSION);
             case 'mimetype':
                 return (string)$fileInfo->getMimeType();
             case 'identifier':
@@ -693,7 +699,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             case 'folder_hash':
                 return $this->hashIdentifier($this->getParentFolderIdentifierOfIdentifier($identifier));
             default:
-                throw new \InvalidArgumentException(sprintf('The information "%s" is not available.', $property));
+                throw new \InvalidArgumentException(sprintf('The information "%s" is not available.', $property), 1476047422);
         }
     }
 
@@ -753,7 +759,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      * This assumes that the local file exists, so no further check is done here!
      * After a successful the original file must not exist anymore.
      *
-     * @param string $localFilePath (within PATH_site)
+     * @param string $localFilePath within public web path
      * @param string $targetFolderIdentifier
      * @param string $newFileName optional, if not given original name is used
      * @param bool $removeOriginal if set the original file will be removed after successful operation
@@ -764,7 +770,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     public function addFile($localFilePath, $targetFolderIdentifier, $newFileName = '', $removeOriginal = true)
     {
         $localFilePath = $this->canonicalizeAndCheckFilePath($localFilePath);
-        // as for the "virtual storage" for backwards-compatibility, this check always fails, as the file probably lies under PATH_site
+        // as for the "virtual storage" for backwards-compatibility, this check always fails, as the file probably lies under public web path
         // thus, it is not checked here
         // @todo is check in storage
         if (GeneralUtility::isFirstPartOfStr($localFilePath, $this->absoluteBasePath) && $this->storageUid > 0) {
@@ -784,7 +790,10 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             $result = copy($localFilePath, $targetPath);
         }
         if ($result === false || !file_exists($targetPath)) {
-            throw new \RuntimeException('Adding file ' . $localFilePath . ' at ' . $newFileIdentifier . ' failed.');
+            throw new \RuntimeException(
+                'Adding file ' . $localFilePath . ' at ' . $newFileIdentifier . ' failed.',
+                1476046453
+            );
         }
         clearstatcache();
         // Change the permissions of the file
@@ -965,6 +974,11 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             $destinationFile = $recycleDirectory . '/' . $timeStamp . '_' . PathUtility::basename($filePath);
         }
         $result = rename($filePath, $destinationFile);
+        // Update the mtime for the file, so the recycler garbage collection task knows which files to delete
+        // Using ctime() is not possible there since this is not supported on Windows
+        if ($result) {
+            touch($destinationFile);
+        }
         return $result;
     }
 
@@ -984,7 +998,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         $identifierMap = [];
         $identifierMap[$sourceFolderIdentifier] = $targetFolderIdentifier;
         foreach ($filesAndFolders as $oldItem) {
-            if ($oldItem['type'] == 'dir') {
+            if ($oldItem['type'] === 'dir') {
                 $oldIdentifier = $oldItem['identifier'];
                 $newIdentifier = $this->canonicalizeAndCheckFolderIdentifier(
                     str_replace($sourceFolderIdentifier, $targetFolderIdentifier, $oldItem['identifier'])
@@ -1051,27 +1065,30 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         $targetFolderPath = $this->getAbsolutePath($newFolderIdentifier);
 
         mkdir($targetFolderPath);
-        /** @var $iterator \RecursiveDirectoryIterator */
+        /** @var \RecursiveDirectoryIterator $iterator */
         $iterator = new \RecursiveIteratorIterator(
             new \RecursiveDirectoryIterator($sourceFolderPath),
-            \RecursiveIteratorIterator::SELF_FIRST
+            \RecursiveIteratorIterator::SELF_FIRST,
+            \RecursiveIteratorIterator::CATCH_GET_CHILD
         );
         // Rewind the iterator as this is important for some systems e.g. Windows
         $iterator->rewind();
         while ($iterator->valid()) {
-            /** @var $current \RecursiveDirectoryIterator */
+            /** @var \RecursiveDirectoryIterator $current */
             $current = $iterator->current();
             $fileName = $current->getFilename();
             $itemSubPath = GeneralUtility::fixWindowsFilePath($iterator->getSubPathname());
             if ($current->isDir() && !($fileName === '..' || $fileName === '.')) {
                 GeneralUtility::mkdir($targetFolderPath . '/' . $itemSubPath);
             } elseif ($current->isFile()) {
-                $result = copy($sourceFolderPath . '/' . $itemSubPath, $targetFolderPath . '/' . $itemSubPath);
+                $copySourcePath = $sourceFolderPath . '/' . $itemSubPath;
+                $copyTargetPath = $targetFolderPath . '/' . $itemSubPath;
+                $result = copy($copySourcePath, $copyTargetPath);
                 if ($result === false) {
                     // rollback
                     GeneralUtility::rmdir($targetFolderIdentifier, true);
                     throw new Exception\FileOperationErrorException(
-                        'Copying file "' . $sourceFolderPath . $itemSubPath . '" to "' . $targetFolderPath . $itemSubPath . '" failed.',
+                        'Copying resource "' . $copySourcePath . '" to "' . $copyTargetPath . '" failed.',
                         1330119452
                     );
                 }
@@ -1145,7 +1162,9 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             throw new \RuntimeException(
                 sprintf(
                     'Creating filename mapping after renaming "%1$s" to "%2$s" failed. Reverted rename operation.\\n\\nOriginal error: %3$s"',
-                    $sourcePath, $targetPath, $e->getMessage()
+                    $sourcePath,
+                    $targetPath,
+                    $e->getMessage()
                 ),
                 1334160746
             );
@@ -1165,12 +1184,8 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     public function deleteFile($fileIdentifier)
     {
         $filePath = $this->getAbsolutePath($fileIdentifier);
-        $recycleDirectory = $this->getRecycleDirectory($filePath);
-        if (!empty($recycleDirectory)) {
-            $result = $this->recycleFileOrFolder($filePath, $recycleDirectory);
-        } else {
-            $result = unlink($filePath);
-        }
+        $result = unlink($filePath);
+
         if ($result === false) {
             throw new \RuntimeException('Deletion of file ' . $fileIdentifier . ' failed.', 1320855304);
         }
@@ -1238,9 +1253,8 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     {
         if ($writable === false) {
             return $this->getAbsolutePath($fileIdentifier);
-        } else {
-            return $this->copyFileToTemporaryPath($fileIdentifier);
         }
+        return $this->copyFileToTemporaryPath($fileIdentifier);
     }
 
     /**
@@ -1293,20 +1307,14 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      * @param string $fileName
      * @param string $parentFolderIdentifier
      * @return string
-     * @throws Exception\InvalidFileNameException
      * @throws \RuntimeException
      */
     public function createFile($fileName, $parentFolderIdentifier)
     {
-        if (!$this->isValidFilename($fileName)) {
-            throw new Exception\InvalidFileNameException(
-                'Invalid characters in fileName "' . $fileName . '"',
-                1320572272
-            );
-        }
+        $fileName = $this->sanitizeFileName(ltrim($fileName, '/'));
         $parentFolderIdentifier = $this->canonicalizeAndCheckFolderIdentifier($parentFolderIdentifier);
-        $fileIdentifier =  $this->canonicalizeAndCheckFileIdentifier(
-            $parentFolderIdentifier . $this->sanitizeFileName(ltrim($fileName, '/'))
+        $fileIdentifier = $this->canonicalizeAndCheckFileIdentifier(
+            $parentFolderIdentifier . $fileName
         );
         $absoluteFilePath = $this->getAbsolutePath($fileIdentifier);
         $result = touch($absoluteFilePath);
@@ -1355,27 +1363,6 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
         return $result;
     }
 
-    /**
-     * Gets the charset conversion object.
-     *
-     * @return \TYPO3\CMS\Core\Charset\CharsetConverter
-     */
-    protected function getCharsetConversion()
-    {
-        if (!isset($this->charsetConversion)) {
-            if (TYPO3_MODE === 'FE') {
-                $this->charsetConversion = $GLOBALS['TSFE']->csConvObj;
-            } elseif (is_object($GLOBALS['LANG'])) {
-                // BE assumed:
-                $this->charsetConversion = $GLOBALS['LANG']->csConvObj;
-            } else {
-                // The object may not exist yet, so we need to create it now. Happens in the Install Tool for example.
-                $this->charsetConversion = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Charset\CharsetConverter::class);
-            }
-        }
-        return $this->charsetConversion;
-    }
-
     /**
      * Returns the role of an item (currently only folders; can later be extended for files as well)
      *
@@ -1385,10 +1372,7 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
     public function getRole($folderIdentifier)
     {
         $name = PathUtility::basename($folderIdentifier);
-        $role = $this->mappingFolderNameToRole[$name];
-        if (empty($role)) {
-            $role = FolderInterface::ROLE_DEFAULT;
-        }
+        $role = $this->mappingFolderNameToRole[$name] ?? FolderInterface::ROLE_DEFAULT;
         return $role;
     }
 
@@ -1398,13 +1382,43 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
      * buffer before. Will be taken care of by the Storage.
      *
      * @param string $identifier
-     * @return void
      */
     public function dumpFileContents($identifier)
     {
         readfile($this->getAbsolutePath($this->canonicalizeAndCheckFileIdentifier($identifier)), 0);
     }
 
+    /**
+     * Stream file using a PSR-7 Response object.
+     *
+     * @param string $identifier
+     * @param array $properties
+     * @return ResponseInterface
+     */
+    public function streamFile(string $identifier, array $properties): ResponseInterface
+    {
+        $fileInfo = $this->getFileInfoByIdentifier($identifier, ['name', 'mimetype', 'mtime', 'size']);
+        $downloadName = $properties['filename_overwrite'] ?? $fileInfo['name'] ?? '';
+        $mimeType = $properties['mimetype_overwrite'] ?? $fileInfo['mimetype'] ?? '';
+        $contentDisposition = ($properties['as_download'] ?? false) ? 'attachment' : 'inline';
+
+        $filePath = $this->getAbsolutePath($this->canonicalizeAndCheckFileIdentifier($identifier));
+
+        return new Response(
+            new SelfEmittableLazyOpenStream($filePath),
+            200,
+            [
+                'Content-Disposition' => $contentDisposition . '; filename="' . $downloadName . '"',
+                'Content-Type' => $mimeType,
+                'Content-Length' => (string)$fileInfo['size'],
+                'Last-Modified' => gmdate('D, d M Y H:i:s', $fileInfo['mtime']) . ' GMT',
+                // Cache-Control header is needed here to solve an issue with browser IE8 and lower
+                // See for more information: http://support.microsoft.com/kb/323308
+                'Cache-Control' => '',
+            ]
+        );
+    }
+
     /**
      * Get the path of the nearest recycler folder of a given $path.
      * Return an empty string if there is no recycler folder available.
@@ -1434,11 +1448,11 @@ class LocalDriver extends AbstractHierarchicalFilesystemDriver
             $recycleDirectory = $searchDirectory . '/' . $recyclerSubdirectory;
             if (is_dir($recycleDirectory)) {
                 return $recycleDirectory;
-            } elseif ($searchDirectory === $rootDirectory) {
-                return '';
-            } else {
-                $searchDirectory = PathUtility::dirname($searchDirectory);
             }
+            if ($searchDirectory === $rootDirectory) {
+                return '';
+            }
+            $searchDirectory = PathUtility::dirname($searchDirectory);
         }
 
         return '';
