@@ -14,6 +14,9 @@ namespace TYPO3\CMS\Core\Resource;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Utility\MathUtility;
+
 /**
  * Representation of a specific processed version of a file. These are created by the FileProcessingService,
  * which in turn uses helper classes for doing the actual file processing. See there for a detailed description.
@@ -119,14 +122,13 @@ class ProcessedFile extends AbstractFile
         if (is_array($databaseRow)) {
             $this->reconstituteFromDatabaseRecord($databaseRow);
         }
-        $this->taskTypeRegistry = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Processing\TaskTypeRegistry::class);
+        $this->taskTypeRegistry = GeneralUtility::makeInstance(Processing\TaskTypeRegistry::class);
     }
 
     /**
      * Creates a ProcessedFile object from a database record.
      *
      * @param array $databaseRow
-     * @return ProcessedFile
      */
     protected function reconstituteFromDatabaseRecord(array $databaseRow)
     {
@@ -137,6 +139,10 @@ class ProcessedFile extends AbstractFile
         $this->identifier = $databaseRow['identifier'];
         $this->name = $databaseRow['name'];
         $this->properties = $databaseRow;
+
+        if (!empty($databaseRow['storage']) && (int)$this->storage->getUid() !== (int)$databaseRow['storage']) {
+            $this->storage = ResourceFactory::getInstance()->getStorageObject($databaseRow['storage']);
+        }
     }
 
     /********************************
@@ -161,7 +167,6 @@ class ProcessedFile extends AbstractFile
      * Replace the current file contents with the given string
      *
      * @param string $contents The contents to write to the file.
-     * @return File The file object (allows chaining).
      * @throws \BadMethodCallException
      */
     public function setContents($contents)
@@ -173,7 +178,6 @@ class ProcessedFile extends AbstractFile
      * Injects a local file, which is a processing result into the object.
      *
      * @param string $filePath
-     * @return void
      * @throws \RuntimeException
      */
     public function updateWithLocalFile($filePath)
@@ -187,7 +191,9 @@ class ProcessedFile extends AbstractFile
         // Update some related properties
         $this->identifier = $addedFile->getIdentifier();
         $this->originalFileSha1 = $this->originalFile->getSha1();
-        $this->updateProperties($addedFile->getProperties());
+        if ($addedFile instanceof AbstractFile) {
+            $this->updateProperties($addedFile->getProperties());
+        }
         $this->deleted = false;
         $this->updated = true;
     }
@@ -244,8 +250,8 @@ class ProcessedFile extends AbstractFile
      */
     public function setName($name)
     {
-        // Remove the existing file
-        if ($this->name !== $name && $this->name !== '' && $this->exists()) {
+        // Remove the existing file, but only we actually have a name or the name has changed
+        if (!empty($this->name) && $this->name !== $name && $this->exists()) {
             $this->delete();
         }
 
@@ -256,6 +262,22 @@ class ProcessedFile extends AbstractFile
         $this->updated = true;
     }
 
+    /**
+     * Checks if this file exists.
+     * Since the original file may reside in a different storage
+     * we ask the original file if it exists in case the processed is representing it
+     *
+     * @return bool TRUE if this file physically exists
+     */
+    public function exists()
+    {
+        if ($this->usesOriginalFile()) {
+            return $this->originalFile->exists();
+        }
+
+        return parent::exists();
+    }
+
     /******************
      * SPECIAL METHODS
      ******************/
@@ -307,9 +329,8 @@ class ProcessedFile extends AbstractFile
     {
         if ($this->usesOriginalFile()) {
             return $this->originalFile->getName();
-        } else {
-            return $this->name;
         }
+        return $this->name;
     }
 
     /**
@@ -324,7 +345,7 @@ class ProcessedFile extends AbstractFile
             $this->properties = [];
         }
 
-        if (array_key_exists('uid', $properties) && \TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($properties['uid'])) {
+        if (array_key_exists('uid', $properties) && MathUtility::canBeInterpretedAsInteger($properties['uid'])) {
             $this->properties['uid'] = $properties['uid'];
         }
 
@@ -383,7 +404,7 @@ class ProcessedFile extends AbstractFile
     }
 
     /**
-     * @return void
+     * Defines that the original file should be used.
      */
     public function setUsesOriginalFile()
     {
@@ -398,7 +419,7 @@ class ProcessedFile extends AbstractFile
      */
     public function usesOriginalFile()
     {
-        return $this->identifier == null || $this->identifier === $this->originalFile->getIdentifier();
+        return $this->identifier === null || $this->identifier === $this->originalFile->getIdentifier();
     }
 
     /**
@@ -425,9 +446,8 @@ class ProcessedFile extends AbstractFile
         // Only delete file when original isn't used
         if (!$this->usesOriginalFile()) {
             return parent::delete();
-        } else {
-            return true;
         }
+        return true;
     }
 
     /**
@@ -442,9 +462,8 @@ class ProcessedFile extends AbstractFile
         // The uid always (!) has to come from this file and never the original file (see getOriginalFile() to get this)
         if ($this->isUnchanged() && $key !== 'uid') {
             return $this->originalFile->getProperty($key);
-        } else {
-            return $this->properties[$key];
         }
+        return $this->properties[$key];
     }
 
     /**
@@ -550,16 +569,16 @@ class ProcessedFile extends AbstractFile
      * Returns a publicly accessible URL for this file
      *
      * @param bool $relativeToCurrentScript Determines whether the URL returned should be relative to the current script, in case it is relative at all
-     * @return NULL|string NULL if file is deleted, the generated URL otherwise
+     * @return string|null NULL if file is deleted, the generated URL otherwise
      */
     public function getPublicUrl($relativeToCurrentScript = false)
     {
         if ($this->deleted) {
             return null;
-        } elseif ($this->usesOriginalFile()) {
-            return $this->getOriginalFile()->getPublicUrl($relativeToCurrentScript);
-        } else {
-            return $this->getStorage()->getPublicUrl($this, $relativeToCurrentScript);
         }
+        if ($this->usesOriginalFile()) {
+            return $this->getOriginalFile()->getPublicUrl($relativeToCurrentScript);
+        }
+        return $this->getStorage()->getPublicUrl($this, $relativeToCurrentScript);
     }
 }
