@@ -14,6 +14,7 @@ namespace TYPO3\CMS\Core\Resource\Processing;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Imaging\GraphicalFunctions;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
@@ -21,21 +22,6 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  */
 class LocalImageProcessor implements ProcessorInterface
 {
-    /**
-     * @var \TYPO3\CMS\Core\Log\Logger
-     */
-    protected $logger;
-
-    /**
-     * Constructor
-     */
-    public function __construct()
-    {
-        /** @var $logManager \TYPO3\CMS\Core\Log\LogManager */
-        $logManager = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Log\LogManager::class);
-        $this->logger = $logManager->getLogger(__CLASS__);
-    }
-
     /**
      * Returns TRUE if this processor can process the given task.
      *
@@ -77,17 +63,15 @@ class LocalImageProcessor implements ProcessorInterface
                     ['width' => $imageDimensions[0], 'height' => $imageDimensions[1], 'size' => filesize($result['filePath']), 'checksum' => $task->getConfigurationChecksum()]
                 );
                 $task->getTargetFile()->updateWithLocalFile($result['filePath']);
-
-            // New dimensions + no new file (for instance svg)
             } elseif (!empty($result['width']) && !empty($result['height']) && empty($result['filePath'])) {
+                // New dimensions + no new file (for instance svg)
                 $task->setExecuted(true);
                 $task->getTargetFile()->setUsesOriginalFile();
                 $task->getTargetFile()->updateProperties(
                     ['width' => $result['width'], 'height' => $result['height'], 'size' => $task->getSourceFile()->getSize(), 'checksum' => $task->getConfigurationChecksum()]
                 );
-
-            // Seems we have no valid processing result
             } else {
+                // Seems we have no valid processing result
                 $task->setExecuted(false);
             }
         } catch (\Exception $e) {
@@ -128,9 +112,8 @@ class LocalImageProcessor implements ProcessorInterface
             $task->getTargetFile()->updateProperties($properties);
 
             return true;
-        } else {
-            return false;
         }
+        return false;
     }
 
     /**
@@ -142,10 +125,10 @@ class LocalImageProcessor implements ProcessorInterface
     {
         switch ($taskName) {
             case 'Preview':
-                $helper = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Processing\LocalPreviewHelper::class, $this);
+                $helper = GeneralUtility::makeInstance(LocalPreviewHelper::class, $this);
             break;
             case 'CropScaleMask':
-                $helper = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Processing\LocalCropScaleMaskHelper::class, $this);
+                $helper = GeneralUtility::makeInstance(LocalCropScaleMaskHelper::class, $this);
             break;
             default:
                 throw new \InvalidArgumentException('Cannot find helper for task name: "' . $taskName . '"', 1353401352);
@@ -155,37 +138,10 @@ class LocalImageProcessor implements ProcessorInterface
     }
 
     /**
-     * Creates error image based on gfx/notfound_thumb.png
-     * Requires GD lib enabled, otherwise it will exit with the three
-     * textstrings outputted as text. Outputs the image stream to browser and exits!
-     *
-     * @param string $filename Name of the file
-     * @param string $textline1 Text line 1
-     * @param string $textline2 Text line 2
-     * @param string $textline3 Text line 3
-     * @return void
-     * @throws \RuntimeException
-     *
-     * @deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8. Use \TYPO3\CMS\Core\Imaging\GraphicalFunctions::getTemporaryImageWithText() instead.
+     * @return GraphicalFunctions
      */
-    public function getTemporaryImageWithText($filename, $textline1, $textline2, $textline3)
+    protected function getGraphicalFunctionsObject(): GraphicalFunctions
     {
-        GeneralUtility::logDeprecatedFunction();
-        $graphicalFunctions = $this->getGraphicalFunctionsObject();
-        $graphicalFunctions->getTemporaryImageWithText($filename, $textline1, $textline2, $textline3);
-    }
-
-    /**
-     * @return \TYPO3\CMS\Core\Imaging\GraphicalFunctions
-     */
-    protected function getGraphicalFunctionsObject()
-    {
-        static $graphicalFunctionsObject = null;
-
-        if ($graphicalFunctionsObject === null) {
-            $graphicalFunctionsObject = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Imaging\GraphicalFunctions::class);
-        }
-
-        return $graphicalFunctionsObject;
+        return GeneralUtility::makeInstance(GraphicalFunctions::class);
     }
 }
