@@ -14,10 +14,10 @@ namespace TYPO3\CMS\Core\Resource;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Core\Utility\PathUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
 
 /**
  * Compressor
@@ -28,23 +28,13 @@ class ResourceCompressor
     /**
      * @var string
      */
-    protected $targetDirectory = 'typo3temp/compressor/';
-
-    /**
-     * @var string
-     */
-    protected $relativePath = '';
+    protected $targetDirectory = 'typo3temp/assets/compressed/';
 
     /**
      * @var string
      */
     protected $rootPath = '';
 
-    /**
-     * @var string
-     */
-    protected $backPath = '';
-
     /**
      * gzipped versions are only created if $TYPO3_CONF_VARS[TYPO3_MODE]['compressionLevel'] is set
      *
@@ -57,6 +47,9 @@ class ResourceCompressor
      */
     protected $gzipCompressionLevel = -1;
 
+    /**
+     * @var string
+     */
     protected $htaccessTemplate = '<FilesMatch "\\.(js|css)(\\.gzip)?$">
 	<IfModule mod_expires.c>
 		ExpiresActive on
@@ -71,13 +64,13 @@ class ResourceCompressor
     public function __construct()
     {
         // we check for existence of our targetDirectory
-        if (!is_dir(PATH_site . $this->targetDirectory)) {
-            GeneralUtility::mkdir(PATH_site . $this->targetDirectory);
+        if (!is_dir(Environment::getPublicPath() . '/' . $this->targetDirectory)) {
+            GeneralUtility::mkdir_deep(Environment::getPublicPath() . '/' . $this->targetDirectory);
         }
         // if enabled, we check whether we should auto-create the .htaccess file
         if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['generateApacheHtaccess']) {
             // check whether .htaccess exists
-            $htaccessPath = PATH_site . $this->targetDirectory . '.htaccess';
+            $htaccessPath = Environment::getPublicPath() . '/' . $this->targetDirectory . '.htaccess';
             if (!file_exists($htaccessPath)) {
                 GeneralUtility::writeFile($htaccessPath, $this->htaccessTemplate);
             }
@@ -92,72 +85,13 @@ class ResourceCompressor
                 $this->gzipCompressionLevel = (int)$compressionLevel;
             }
         }
-        $this->setInitialPaths();
-    }
-
-    /**
-     * Sets initial values for paths.
-     *
-     * @return void
-     */
-    public function setInitialPaths()
-    {
-        $this->setInitialRelativePath();
-        $this->setInitialRootPath();
-        $this->setInitialBackPath();
-    }
-
-    /**
-     * Sets relative back path
-     *
-     * @return void
-     */
-    protected function setInitialBackPath()
-    {
-        $backPath = TYPO3_MODE === 'BE' ? $GLOBALS['BACK_PATH'] : '';
-        $this->setBackPath($backPath);
-    }
-
-    /**
-     * Sets absolute path to working directory
-     *
-     * @return void
-     */
-    protected function setInitialRootPath()
-    {
-        $rootPath = TYPO3_MODE === 'BE' ? PATH_typo3 : PATH_site;
-        $this->setRootPath($rootPath);
-    }
-
-    /**
-     * Sets relative path to PATH_site
-     *
-     * @return void
-     */
-    protected function setInitialRelativePath()
-    {
-        $relativePath = TYPO3_MODE === 'BE' ? $GLOBALS['BACK_PATH'] . '../' : '';
-        $this->setRelativePath($relativePath);
-    }
-
-    /**
-     * Sets relative path to PATH_site
-     *
-     * @param string $relativePath Relative path to site root
-     * @return void
-     */
-    public function setRelativePath($relativePath)
-    {
-        if (is_string($relativePath)) {
-            $this->relativePath = $relativePath;
-        }
+        $this->setRootPath(TYPO3_MODE === 'BE' ? Environment::getBackendPath() . '/' : Environment::getPublicPath() . '/');
     }
 
     /**
      * Sets absolute path to working directory
      *
      * @param string $rootPath Absolute path
-     * @return void
      */
     public function setRootPath($rootPath)
     {
@@ -166,19 +100,6 @@ class ResourceCompressor
         }
     }
 
-    /**
-     * Sets relative back path
-     *
-     * @param string $backPath Back path
-     * @return void
-     */
-    public function setBackPath($backPath)
-    {
-        if (is_string($backPath)) {
-            $this->backPath = $backPath;
-        }
-    }
-
     /**
      * Concatenates the Stylesheet files
      *
@@ -197,20 +118,20 @@ class ResourceCompressor
             if (!empty($fileOptions['excludeFromConcatenation'])) {
                 continue;
             }
-            // we remove BACK_PATH from $filename, so make it relative to root path
             $filenameFromMainDir = $this->getFilenameFromMainDir($fileOptions['file']);
             // if $options['baseDirectories'] set, we only include files below these directories
             if (
                 !isset($options['baseDirectories'])
                 || $this->checkBaseDirectory(
-                    $filenameFromMainDir, array_merge($options['baseDirectories'], [$this->targetDirectory])
+                    $filenameFromMainDir,
+                    array_merge($options['baseDirectories'], [$this->targetDirectory])
                 )
             ) {
                 $type = isset($fileOptions['media']) ? strtolower($fileOptions['media']) : 'all';
                 if (!isset($filesToIncludeByType[$type])) {
                     $filesToIncludeByType[$type] = [];
                 }
-                if ($fileOptions['forceOnTop']) {
+                if (!empty($fileOptions['forceOnTop'])) {
                     array_unshift($filesToIncludeByType[$type], $filenameFromMainDir);
                 } else {
                     $filesToIncludeByType[$type][] = $filenameFromMainDir;
@@ -225,9 +146,8 @@ class ResourceCompressor
                     continue;
                 }
                 $targetFile = $this->createMergedCssFile($filesToInclude);
-                $targetFileRelative = $this->relativePath . $targetFile;
                 $concatenatedOptions = [
-                    'file' => $targetFileRelative,
+                    'file' => $targetFile,
                     'rel' => 'stylesheet',
                     'media' => $mediaOption,
                     'compress' => true,
@@ -236,7 +156,7 @@ class ResourceCompressor
                     'allWrap' => ''
                 ];
                 // place the merged stylesheet on top of the stylesheets
-                $cssFiles = array_merge($cssFiles, [$targetFileRelative => $concatenatedOptions]);
+                $cssFiles = array_merge($cssFiles, [$targetFile => $concatenatedOptions]);
             }
         }
         return $cssFiles;
@@ -250,6 +170,8 @@ class ResourceCompressor
      */
     public function concatenateJsFiles(array $jsFiles)
     {
+        $concatenatedJsFileIsAsync = false;
+        $allFilesToConcatenateAreAsync = true;
         $filesToInclude = [];
         foreach ($jsFiles as $key => $fileOptions) {
             // invalid section found or no concatenation allowed, so continue
@@ -259,31 +181,35 @@ class ResourceCompressor
             if (!isset($filesToInclude[$fileOptions['section']])) {
                 $filesToInclude[$fileOptions['section']] = [];
             }
-            // we remove BACK_PATH from $filename, so make it relative to root path
             $filenameFromMainDir = $this->getFilenameFromMainDir($fileOptions['file']);
-            if ($fileOptions['forceOnTop']) {
+            if (!empty($fileOptions['forceOnTop'])) {
                 array_unshift($filesToInclude[$fileOptions['section']], $filenameFromMainDir);
             } else {
                 $filesToInclude[$fileOptions['section']][] = $filenameFromMainDir;
             }
+            if (!empty($fileOptions['async']) && (bool)$fileOptions['async']) {
+                $concatenatedJsFileIsAsync = true;
+            } else {
+                $allFilesToConcatenateAreAsync = false;
+            }
             // remove the file from the incoming file array
             unset($jsFiles[$key]);
         }
         if (!empty($filesToInclude)) {
             foreach ($filesToInclude as $section => $files) {
                 $targetFile = $this->createMergedJsFile($files);
-                $targetFileRelative = $this->relativePath . $targetFile;
                 $concatenatedOptions = [
-                    'file' => $targetFileRelative,
+                    'file' => $targetFile,
                     'type' => 'text/javascript',
                     'section' => $section,
                     'compress' => true,
                     'excludeFromConcatenation' => true,
                     'forceOnTop' => false,
-                    'allWrap' => ''
+                    'allWrap' => '',
+                    'async' => $concatenatedJsFileIsAsync && $allFilesToConcatenateAreAsync,
                 ];
                 // place the merged javascript on top of the JS files
-                $jsFiles = array_merge([$targetFileRelative => $concatenatedOptions], $jsFiles);
+                $jsFiles = array_merge([$targetFile => $concatenatedOptions], $jsFiles);
             }
         }
         return $jsFiles;
@@ -361,13 +287,15 @@ class ResourceCompressor
         }
         $targetFile = $this->targetDirectory . 'merged-' . md5($unique) . '.' . $type;
         // if the file doesn't already exist, we create it
-        if (!file_exists((PATH_site . $targetFile))) {
+        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile)) {
             $concatenated = '';
             // concatenate all the files together
             foreach ($filesToInclude as $filename) {
-                $contents = GeneralUtility::getUrl(GeneralUtility::resolveBackPath($this->rootPath . $filename));
+                $filenameAbsolute = GeneralUtility::resolveBackPath($this->rootPath . $filename);
+                $filename = PathUtility::stripPathSitePrefix($filenameAbsolute);
+                $contents = file_get_contents($filenameAbsolute);
                 // remove any UTF-8 byte order mark (BOM) from files
-                if (StringUtility::beginsWith($contents, "\xEF\xBB\xBF")) {
+                if (strpos($contents, "\xEF\xBB\xBF") === 0) {
                     $contents = substr($contents, 3);
                 }
                 // only fix paths if files aren't already in typo3temp (already processed)
@@ -380,7 +308,7 @@ class ResourceCompressor
             if ($type === 'css') {
                 $concatenated = $this->cssFixStatements($concatenated);
             }
-            GeneralUtility::writeFile(PATH_site . $targetFile, $concatenated);
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $targetFile, $concatenated);
         }
         return $targetFile;
     }
@@ -430,82 +358,20 @@ class ResourceCompressor
         } else {
             $unique = $filenameAbsolute;
         }
+        // make sure it is again the full filename
+        $filename = PathUtility::stripPathSitePrefix($filenameAbsolute);
 
-        $pathinfo = PathUtility::pathinfo($filename);
+        $pathinfo = PathUtility::pathinfo($filenameAbsolute);
         $targetFile = $this->targetDirectory . $pathinfo['filename'] . '-' . md5($unique) . '.css';
         // only create it, if it doesn't exist, yet
-        if (!file_exists((PATH_site . $targetFile)) || $this->createGzipped && !file_exists((PATH_site . $targetFile . '.gzip'))) {
-            $contents = $this->compressCssString(GeneralUtility::getUrl($filenameAbsolute));
+        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . '.gzip')) {
+            $contents = $this->compressCssString(file_get_contents($filenameAbsolute));
             if (strpos($filename, $this->targetDirectory) === false) {
                 $contents = $this->cssFixRelativeUrlPaths($contents, PathUtility::dirname($filename) . '/');
             }
             $this->writeFileAndCompressed($targetFile, $contents);
         }
-        return $this->relativePath . $this->returnFileReference($targetFile);
-    }
-
-    /**
-     * Callback function for preg_replace
-     *
-     * @see compressCssFile
-     * @param array $matches
-     * @return string the compressed string
-     * @deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8, not in use anymore
-     */
-    public static function compressCssPregCallback($matches)
-    {
-        GeneralUtility::logDeprecatedFunction();
-        if ($matches[1]) {
-            // Group 1: Double quoted string.
-            return $matches[1];
-        } elseif ($matches[2]) {
-            // Group 2: Single quoted string.
-            return $matches[2];
-        } elseif ($matches[3]) {
-            // Group 3: Regular non-MacIE5-hack comment.
-            return '
-';
-        } elseif ($matches[4]) {
-            // Group 4: MacIE5-hack-type-1 comment.
-            return '
-/*\\T1*/
-';
-        } elseif ($matches[5]) {
-            // Group 5,6,7: MacIE5-hack-type-2 comment
-            $matches[6] = preg_replace('/\\s++([+>{};,)])/S', '$1', $matches[6]);
-            // Clean pre-punctuation.
-            $matches[6] = preg_replace('/([+>{}:;,(])\\s++/S', '$1', $matches[6]);
-            // Clean post-punctuation.
-            $matches[6] = preg_replace('/;?\\}/S', '}
-', $matches[6]);
-            // Add a touch of formatting.
-            return '
-/*T2\\*/' . $matches[6] . '
-/*T2E*/
-';
-        } elseif ($matches[8]) {
-            // Group 8: calc function (see http://www.w3.org/TR/2006/WD-css3-values-20060919/#calc)
-            return 'calc' . $matches[8];
-        } elseif (isset($matches[9])) {
-            // Group 9: Non-string, non-comment. Safe to clean whitespace here.
-            $matches[9] = preg_replace('/^\\s++/', '', $matches[9]);
-            // Strip all leading whitespace.
-            $matches[9] = preg_replace('/\\s++$/', '', $matches[9]);
-            // Strip all trailing whitespace.
-            $matches[9] = preg_replace('/\\s{2,}+/', ' ', $matches[9]);
-            // Consolidate multiple whitespace.
-            $matches[9] = preg_replace('/\\s++([+>{};,)])/S', '$1', $matches[9]);
-            // Clean pre-punctuation.
-            $matches[9] = preg_replace('/([+>{}:;,(])\\s++/S', '$1', $matches[9]);
-            // Clean post-punctuation.
-            $matches[9] = preg_replace('/;?\\}/S', '}
-', $matches[9]);
-            // Add a touch of formatting.
-            return $matches[9];
-        }
-        return $matches[0] . '
-/* ERROR! Unexpected _proccess_css_minify() parameter */
-';
+        return $this->returnFileReference($targetFile);
     }
 
     /**
@@ -550,51 +416,70 @@ class ResourceCompressor
         $pathinfo = PathUtility::pathinfo($filename);
         $targetFile = $this->targetDirectory . $pathinfo['filename'] . '-' . md5($unique) . '.js';
         // only create it, if it doesn't exist, yet
-        if (!file_exists((PATH_site . $targetFile)) || $this->createGzipped && !file_exists((PATH_site . $targetFile . '.gzip'))) {
-            $contents = GeneralUtility::getUrl($filenameAbsolute);
+        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . '.gzip')) {
+            $contents = file_get_contents($filenameAbsolute);
             $this->writeFileAndCompressed($targetFile, $contents);
         }
-        return $this->relativePath . $this->returnFileReference($targetFile);
+        return $this->returnFileReference($targetFile);
     }
 
     /**
      * Finds the relative path to a file, relative to the root path.
      *
      * @param string $filename the name of the file
-     * @return string the path to the file relative to the root path
+     * @return string the path to the file relative to the root path ($this->rootPath)
      */
     protected function getFilenameFromMainDir($filename)
     {
-        // if BACK_PATH is empty return $filename
-        if (empty($this->backPath)) {
-            return $filename;
+        /*
+         * The various paths may have those values (e.g. if TYPO3 is installed in a subdir)
+         * - docRoot = /var/www/html/
+         * - Environment::getPublicPath() = /var/www/html/sites/site1/
+         * - $this->rootPath = /var/www/html/sites/site1/typo3
+         *
+         * The file names passed into this function may be either:
+         * - relative to $this->rootPath
+         * - relative to Environment::getPublicPath()
+         * - relative to docRoot
+         */
+        $docRoot = GeneralUtility::getIndpEnv('TYPO3_DOCUMENT_ROOT');
+        $fileNameWithoutSlash = ltrim($filename, '/');
+
+        // if the file is an absolute reference within the docRoot
+        $absolutePath = $docRoot . '/' . $fileNameWithoutSlash;
+        // Calling is_file without @ for a path starting with '../' causes a PHP Warning when using open_basedir restriction
+        if (@is_file($absolutePath)) {
+            if (strpos($absolutePath, $this->rootPath) === 0) {
+                // the path is within the current root path, simply strip rootPath off
+                return substr($absolutePath, strlen($this->rootPath));
+            }
+            // the path is not within the root path, strip off the site path, the remaining logic below
+            // takes care about adjusting the path correctly.
+            $filename = substr($absolutePath, strlen(Environment::getPublicPath() . '/'));
         }
         // if the file exists in the root path, just return the $filename
-        if (strpos($filename, $this->backPath) === 0) {
-            $file = str_replace($this->backPath, '', $filename);
-            if (is_file(GeneralUtility::resolveBackPath($this->rootPath . $file))) {
-                return $file;
-            }
+        if (is_file($this->rootPath . $fileNameWithoutSlash)) {
+            return $fileNameWithoutSlash;
         }
         // if the file is from a special TYPO3 internal directory, add the missing typo3/ prefix
-        if (is_file(realpath(PATH_site . TYPO3_mainDir . $filename))) {
-            $filename = TYPO3_mainDir . $filename;
+        if (is_file(realpath(Environment::getBackendPath() . '/' . $filename))) {
+            $filename = 'typo3/' . $filename;
         }
-        // build the file path relatively to the PATH_site
-        $backPath = str_replace(TYPO3_mainDir, '', $this->backPath);
-        $file = str_replace($backPath, '', $filename);
-        if (substr($file, 0, 3) === '../') {
-            $file = GeneralUtility::resolveBackPath(PATH_typo3 . $file);
+        // build the file path relative to the public web path
+        if (strpos($filename, 'EXT:') === 0) {
+            $file = GeneralUtility::getFileAbsFileName($filename);
+        } elseif (strpos($filename, '../') === 0) {
+            $file = GeneralUtility::resolveBackPath(Environment::getBackendPath() . '/' . $filename);
         } else {
-            $file = PATH_site . $file;
+            $file = Environment::getPublicPath() . '/' . $filename;
         }
-        // check if the file exists, and if so, return the path relative to TYPO3_mainDir
+
+        // check if the file exists, and if so, return the path relative to PATH_thisScript
         if (is_file($file)) {
-            $mainDirDepth = substr_count(TYPO3_mainDir, '/');
-            return str_repeat('../', $mainDirDepth) . str_replace(PATH_site, '', $file);
+            return rtrim(PathUtility::getRelativePathTo($file), '/');
         }
         // none of above conditions were met, fallback to default behaviour
-        return substr($filename, strlen($this->backPath));
+        return $filename;
     }
 
     /**
@@ -624,8 +509,7 @@ class ResourceCompressor
      */
     protected function cssFixRelativeUrlPaths($contents, $oldDir)
     {
-        $mainDir = TYPO3_MODE === 'BE' ? TYPO3_mainDir : '';
-        $newDir = '../../' . $mainDir . $oldDir;
+        $newDir = '../../../' . $oldDir;
         // Replace "url()" paths
         if (stripos($contents, 'url') !== false) {
             $regex = '/url(\\(\\s*["\']?(?!\\/)([^"\']+)["\']?\\s*\\))/iU';
@@ -728,15 +612,14 @@ class ResourceCompressor
      *
      * @param string $filename Target filename
      * @param string $contents File contents
-     * @return void
      */
     protected function writeFileAndCompressed($filename, $contents)
     {
         // write uncompressed file
-        GeneralUtility::writeFile(PATH_site . $filename, $contents);
+        GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $contents);
         if ($this->createGzipped) {
             // create compressed version
-            GeneralUtility::writeFile(PATH_site . $filename . '.gzip', gzencode($contents, $this->gzipCompressionLevel));
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename . '.gzip', gzencode($contents, $this->gzipCompressionLevel));
         }
     }
 
@@ -751,10 +634,9 @@ class ResourceCompressor
     {
         // if the client accepts gzip and we can create gzipped files, we give him compressed versions
         if ($this->createGzipped && strpos(GeneralUtility::getIndpEnv('HTTP_ACCEPT_ENCODING'), 'gzip') !== false) {
-            return $filename . '.gzip';
-        } else {
-            return $filename;
+            $filename .= '.gzip';
         }
+        return PathUtility::getRelativePath($this->rootPath, Environment::getPublicPath() . '/') . $filename;
     }
 
     /**
@@ -767,11 +649,11 @@ class ResourceCompressor
     {
         $externalContent = GeneralUtility::getUrl($url);
         $filename = $this->targetDirectory . 'external-' . md5($url);
-        // write only if file does not exist and md5 of the content is not the same as fetched one
-        if (!file_exists(PATH_site . $filename)
-            && (md5($externalContent) !== md5(GeneralUtility::getUrl(PATH_site . $filename)))
+        // Write only if file does not exist OR md5 of the content is not the same as fetched one
+        if (!file_exists(Environment::getPublicPath() . '/' . $filename)
+            || (md5($externalContent) !== md5(file_get_contents(Environment::getPublicPath() . '/' . $filename)))
         ) {
-            GeneralUtility::writeFile(PATH_site . $filename, $externalContent);
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $externalContent);
         }
         return $filename;
     }
@@ -801,7 +683,8 @@ class ResourceCompressor
         // There are different conditions for removing leading and trailing
         // whitespace.
         // @see http://php.net/manual/regexp.reference.subpatterns.php
-        $contents = preg_replace('<
+        $contents = preg_replace(
+            '<
 				# Strip leading and trailing whitespace.
 				\s*([@{};,])\s*
 				# Strip only leading whitespace from:
