@@ -14,15 +14,20 @@ namespace TYPO3\CMS\Core\Resource;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
 use TYPO3\CMS\Core\Resource\Index\FileIndexRepository;
+use TYPO3\CMS\Core\Service\FlexFormService;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Core\Utility\PathUtility;
-use TYPO3\CMS\Extbase\Service\FlexFormService;
+use TYPO3\CMS\Extbase\SignalSlot\Dispatcher;
 
-// @todo implement constructor-level caching
 /**
  * Factory class for FAL objects
+ * @todo implement constructor-level caching
  */
 class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\SingletonInterface
 {
@@ -61,19 +66,21 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      *
      * @var array
      */
-    protected $localDriverStorageCache = null;
+    protected $localDriverStorageCache;
 
     /**
-     * @var \TYPO3\CMS\Extbase\SignalSlot\Dispatcher
+     * @var Dispatcher
      */
     protected $signalSlotDispatcher;
 
     /**
      * Inject signal slot dispatcher
+     *
+     * @param Dispatcher $signalSlotDispatcher an instance of the signal slot dispatcher
      */
-    public function __construct(\TYPO3\CMS\Extbase\SignalSlot\Dispatcher $signalSlotDispatcher = null)
+    public function __construct(Dispatcher $signalSlotDispatcher = null)
     {
-        $this->signalSlotDispatcher = $signalSlotDispatcher ?: GeneralUtility::makeInstance(\TYPO3\CMS\Extbase\SignalSlot\Dispatcher::class);
+        $this->signalSlotDispatcher = $signalSlotDispatcher ?: GeneralUtility::makeInstance(Dispatcher::class);
     }
 
     /**
@@ -86,8 +93,8 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     public function getDriverObject($driverIdentificationString, array $driverConfiguration)
     {
-        /** @var $driverRegistry Driver\DriverRegistry */
-        $driverRegistry = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Driver\DriverRegistry::class);
+        /** @var Driver\DriverRegistry $driverRegistry */
+        $driverRegistry = GeneralUtility::makeInstance(Driver\DriverRegistry::class);
         $driverClass = $driverRegistry->getDriverClass($driverIdentificationString);
         $driverObject = GeneralUtility::makeInstance($driverClass, $driverConfiguration);
         return $driverObject;
@@ -101,12 +108,12 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      * getDefaultStorage->getDefaultFolder() will get you fileadmin/user_upload/ in a standard
      * TYPO3 installation.
      *
-     * @return null|ResourceStorage
+     * @return ResourceStorage|null
      */
     public function getDefaultStorage()
     {
-        /** @var $storageRepository StorageRepository */
-        $storageRepository = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\StorageRepository::class);
+        /** @var StorageRepository $storageRepository */
+        $storageRepository = GeneralUtility::makeInstance(StorageRepository::class);
 
         $allStorages = $storageRepository->findAll();
         foreach ($allStorages as $storage) {
@@ -136,9 +143,10 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         if ($uid === 0 && $fileIdentifier !== null) {
             $uid = $this->findBestMatchingStorageByLocalPath($fileIdentifier);
         }
-        if (!$this->storageInstances[$uid]) {
+        if (empty($this->storageInstances[$uid])) {
             $storageConfiguration = null;
             $storageObject = null;
+            list($_, $uid, $recordData, $fileIdentifier) = $this->emitPreProcessStorageSignal($uid, $recordData, $fileIdentifier);
             // If the built-in storage with UID=0 is requested:
             if ($uid === 0) {
                 $recordData = [
@@ -147,7 +155,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
                     'name' => 'Fallback Storage',
                     'description' => 'Internal storage, mounting the main TYPO3_site directory.',
                     'driver' => 'Local',
-                    'processingfolder' => 'typo3temp/_processed_/',
+                    'processingfolder' => 'typo3temp/assets/_processed_/',
                     // legacy code
                     'configuration' => '',
                     'is_online' => true,
@@ -161,9 +169,9 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
                     'pathType' => 'relative'
                 ];
             } elseif (count($recordData) === 0 || (int)$recordData['uid'] !== $uid) {
-                /** @var $storageRepository StorageRepository */
-                $storageRepository = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\StorageRepository::class);
-                /** @var $storage ResourceStorage */
+                /** @var StorageRepository $storageRepository */
+                $storageRepository = GeneralUtility::makeInstance(StorageRepository::class);
+                /** @var ResourceStorage $storage */
                 $storageObject = $storageRepository->findByUid($uid);
             }
             if (!$storageObject instanceof ResourceStorage) {
@@ -175,6 +183,19 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         return $this->storageInstances[$uid];
     }
 
+    /**
+     * Emits a signal before a resource storage was initialized
+     *
+     * @param int $uid
+     * @param array $recordData
+     * @param string $fileIdentifier
+     * @return mixed
+     */
+    protected function emitPreProcessStorageSignal($uid, $recordData, $fileIdentifier)
+    {
+        return $this->signalSlotDispatcher->dispatch(\TYPO3\CMS\Core\Resource\ResourceFactory::class, self::SIGNAL_PreProcessStorage, [$this, $uid, $recordData, $fileIdentifier]);
+    }
+
     /**
      * Emits a signal after a resource storage was initialized
      *
@@ -182,12 +203,12 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     protected function emitPostProcessStorageSignal(ResourceStorage $storageObject)
     {
-        $this->signalSlotDispatcher->dispatch(\TYPO3\CMS\Core\Resource\ResourceFactory::class, self::SIGNAL_PostProcessStorage, [$this, $storageObject]);
+        $this->signalSlotDispatcher->dispatch(self::class, self::SIGNAL_PostProcessStorage, [$this, $storageObject]);
     }
 
     /**
      * Checks whether a file resides within a real storage in local file system.
-     * If no match is found, uid 0 is returned which is a fallback storage pointing to PATH_site.
+     * If no match is found, uid 0 is returned which is a fallback storage pointing to fileadmin in public web path.
      *
      * The file identifier is adapted accordingly to match the new storage's base path.
      *
@@ -220,14 +241,12 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
 
     /**
      * Creates an array mapping all uids to the basePath of storages using the "local" driver.
-     *
-     * @return void
      */
     protected function initializeLocalStorageCache()
     {
-        /** @var $storageRepository StorageRepository */
-        $storageRepository = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\StorageRepository::class);
-        /** @var $storageObjects ResourceStorage[] */
+        /** @var StorageRepository $storageRepository */
+        $storageRepository = GeneralUtility::makeInstance(StorageRepository::class);
+        /** @var ResourceStorage[] $storageObjects */
         $storageObjects = $storageRepository->findByStorageType('Local');
 
         $storageCache = [];
@@ -271,9 +290,19 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         if (!$this->collectionInstances[$uid]) {
             // Get mount data if not already supplied as argument to this function
             if (empty($recordData) || $recordData['uid'] !== $uid) {
-                /** @var $GLOBALS['TYPO3_DB'] \TYPO3\CMS\Core\Database\DatabaseConnection */
-                $recordData = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('*', 'sys_file_collection', 'uid=' . (int)$uid . ' AND deleted=0');
-                if (!is_array($recordData)) {
+                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_collection');
+                $queryBuilder->getRestrictions()->removeAll()->add(GeneralUtility::makeInstance(DeletedRestriction::class));
+                $recordData = $queryBuilder->select('*')
+                    ->from('sys_file_collection')
+                    ->where(
+                        $queryBuilder->expr()->eq(
+                            'uid',
+                            $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)
+                        )
+                    )
+                    ->execute()
+                    ->fetch();
+                if (empty($recordData)) {
                     throw new \InvalidArgumentException('No collection found for given UID: "' . $uid . '"', 1314085992);
                 }
             }
@@ -291,8 +320,10 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     public function createCollectionObject(array $collectionData)
     {
-        /** @var $registry Collection\FileCollectionRegistry */
-        $registry = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Collection\FileCollectionRegistry::class);
+        /** @var Collection\FileCollectionRegistry $registry */
+        $registry = GeneralUtility::makeInstance(Collection\FileCollectionRegistry::class);
+
+        /** @var \TYPO3\CMS\Core\Collection\AbstractRecordCollection $class */
         $class = $registry->getFileCollectionClass($collectionData['type']);
 
         return $class::create($collectionData);
@@ -307,13 +338,12 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     public function createStorageObject(array $storageRecord, array $storageConfiguration = null)
     {
-        $className = \TYPO3\CMS\Core\Resource\ResourceStorage::class;
         if (!$storageConfiguration) {
             $storageConfiguration = $this->convertFlexFormDataToConfigurationArray($storageRecord['configuration']);
         }
         $driverType = $storageRecord['driver'];
         $driverObject = $this->getDriverObject($driverType, $storageConfiguration);
-        return GeneralUtility::makeInstance($className, $driverObject, $storageRecord);
+        return GeneralUtility::makeInstance(ResourceStorage::class, $driverObject, $storageRecord);
     }
 
     /**
@@ -326,7 +356,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     public function createFolderObject(ResourceStorage $storage, $identifier, $name)
     {
-        return GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Folder::class, $storage, $identifier, $name);
+        return GeneralUtility::makeInstance(Folder::class, $storage, $identifier, $name);
     }
 
     /**
@@ -337,7 +367,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      * @param array $fileData The record row from database.
      *
      * @throws \InvalidArgumentException
-     * @throws \TYPO3\CMS\Core\Resource\Exception\FileDoesNotExistException
+     * @throws Exception\FileDoesNotExistException
      * @return File
      */
     public function getFileObject($uid, array $fileData = [])
@@ -345,12 +375,12 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         if (!is_numeric($uid)) {
             throw new \InvalidArgumentException('The UID of file has to be numeric. UID given: "' . $uid . '"', 1300096564);
         }
-        if (!$this->fileInstances[$uid]) {
+        if (empty($this->fileInstances[$uid])) {
             // Fetches data in case $fileData is empty
             if (empty($fileData)) {
                 $fileData = $this->getFileIndexRepository()->findOneByUid($uid);
                 if ($fileData === false) {
-                    throw new \TYPO3\CMS\Core\Resource\Exception\FileDoesNotExistException('No file found for given UID: ' . $uid, 1317178604);
+                    throw new Exception\FileDoesNotExistException('No file found for given UID: ' . $uid, 1317178604);
                 }
             }
             $this->fileInstances[$uid] = $this->createFileObject($fileData);
@@ -362,7 +392,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      * Gets an file object from an identifier [storage]:[fileId]
      *
      * @param string $identifier
-     * @return File
+     * @return File|ProcessedFile|null
      * @throws \InvalidArgumentException
      */
     public function getFileObjectFromCombinedIdentifier($identifier)
@@ -393,7 +423,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      *
      * @param int $storageUid
      * @param string $fileIdentifier
-     * @return NULL|File|ProcessedFile
+     * @return File|ProcessedFile|null
      */
     public function getFileObjectByStorageAndIdentifier($storageUid, &$fileIdentifier)
     {
@@ -430,47 +460,47 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      * - "file:23"
      *
      * @param string $input
-     * @return File|Folder
+     * @return File|Folder|null
      */
     public function retrieveFileOrFolderObject($input)
     {
-        // Remove PATH_site because absolute paths under Windows systems contain ':'
+        // Remove Environment::getPublicPath() because absolute paths under Windows systems contain ':'
         // This is done in all considered sub functions anyway
-        $input = str_replace(PATH_site, '', $input);
+        $input = str_replace(Environment::getPublicPath() . '/', '', $input);
 
         if (GeneralUtility::isFirstPartOfStr($input, 'file:')) {
             $input = substr($input, 5);
             return $this->retrieveFileOrFolderObject($input);
-        } elseif (\TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($input)) {
+        }
+        if (MathUtility::canBeInterpretedAsInteger($input)) {
             return $this->getFileObject($input);
-        } elseif (strpos($input, ':') > 0) {
-            list($prefix, $folderIdentifier) = explode(':', $input);
-            if (\TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($prefix)) {
+        }
+        if (strpos($input, ':') > 0) {
+            list($prefix) = explode(':', $input);
+            if (MathUtility::canBeInterpretedAsInteger($prefix)) {
                 // path or folder in a valid storageUID
                 return $this->getObjectFromCombinedIdentifier($input);
-            } elseif ($prefix == 'EXT') {
+            }
+            if ($prefix === 'EXT') {
                 $input = GeneralUtility::getFileAbsFileName($input);
                 if (empty($input)) {
                     return null;
                 }
 
-                $input = PathUtility::getRelativePath(PATH_site, PathUtility::dirname($input)) . PathUtility::basename($input);
+                $input = PathUtility::getRelativePath(Environment::getPublicPath() . '/', PathUtility::dirname($input)) . PathUtility::basename($input);
                 return $this->getFileObjectFromCombinedIdentifier($input);
-            } else {
-                return null;
-            }
-        } else {
-            // this is a backwards-compatible way to access "0-storage" files or folders
-            // eliminate double slashes, /./ and /../
-            $input = \TYPO3\CMS\Core\Utility\PathUtility::getCanonicalPath(ltrim($input, '/'));
-            if (@is_file(PATH_site . $input)) {
-                // only the local file
-                return $this->getFileObjectFromCombinedIdentifier($input);
-            } else {
-                // only the local path
-                return $this->getFolderObjectFromCombinedIdentifier($input);
             }
+            return null;
         }
+        // this is a backwards-compatible way to access "0-storage" files or folders
+        // eliminate double slashes, /./ and /../
+        $input = PathUtility::getCanonicalPath(ltrim($input, '/'));
+        if (@is_file(Environment::getPublicPath() . '/' . $input)) {
+            // only the local file
+            return $this->getFileObjectFromCombinedIdentifier($input);
+        }
+        // only the local path
+        return $this->getFolderObjectFromCombinedIdentifier($input);
     }
 
     /**
@@ -494,9 +524,9 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
             // please note that getStorageObject() might modify $folderIdentifier when
             // auto-detecting the best-matching storage to use
             $folderIdentifier = $parts[0];
-            // make sure to not use an absolute path, and remove PATH_site if it is prepended
-            if (GeneralUtility::isFirstPartOfStr($folderIdentifier, PATH_site)) {
-                $folderIdentifier = \TYPO3\CMS\Core\Utility\PathUtility::stripPathSitePrefix($parts[0]);
+            // make sure to not use an absolute path, and remove Environment::getPublicPath if it is prepended
+            if (GeneralUtility::isFirstPartOfStr($folderIdentifier, Environment::getPublicPath() . '/')) {
+                $folderIdentifier = PathUtility::stripPathSitePrefix($parts[0]);
             }
         }
         return $this->getStorageObject($storageUid, [], $folderIdentifier)->getFolder($folderIdentifier);
@@ -520,7 +550,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      *
      * @param string $identifier
      *
-     * @throws \TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException
+     * @throws Exception\ResourceDoesNotExistException
      * @return FileInterface|Folder
      */
     public function getObjectFromCombinedIdentifier($identifier)
@@ -529,11 +559,11 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         $storage = $this->getStorageObject($storageId);
         if ($storage->hasFile($objectIdentifier)) {
             return $storage->getFile($objectIdentifier);
-        } elseif ($storage->hasFolder($objectIdentifier)) {
-            return $storage->getFolder($objectIdentifier);
-        } else {
-            throw new \TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException('Object with identifier "' . $identifier . '" does not exist in storage', 1329647780);
         }
+        if ($storage->hasFolder($objectIdentifier)) {
+            return $storage->getFolder($objectIdentifier);
+        }
+        throw new Exception\ResourceDoesNotExistException('Object with identifier "' . $identifier . '" does not exist in storage', 1329647780);
     }
 
     /**
@@ -555,7 +585,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
         } else {
             throw new \RuntimeException('A file needs to reside in a Storage', 1381570997);
         }
-        $fileObject = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\File::class, $fileData, $storageObject);
+        $fileObject = GeneralUtility::makeInstance(File::class, $fileData, $storageObject);
         return $fileObject;
     }
 
@@ -568,7 +598,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      * @param bool $raw Whether to get raw results without performing overlays
      * @return FileReference
      * @throws \InvalidArgumentException
-     * @throws \TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException
+     * @throws Exception\ResourceDoesNotExistException
      */
     public function getFileReferenceObject($uid, array $fileReferenceData = [], $raw = false)
     {
@@ -583,7 +613,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
             if (empty($fileReferenceData)) {
                 $fileReferenceData = $this->getFileReferenceData($uid, $raw);
                 if (!is_array($fileReferenceData)) {
-                    throw new \TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException(
+                    throw new Exception\ResourceDoesNotExistException(
                         'No file reference (sys_file_reference) was found for given UID: "' . $uid . '"',
                         1317178794
                     );
@@ -605,7 +635,7 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
     public function createFileReferenceObject(array $fileReferenceData)
     {
         /** @var FileReference $fileReferenceObject */
-        $fileReferenceObject = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\FileReference::class, $fileReferenceData);
+        $fileReferenceObject = GeneralUtility::makeInstance(FileReference::class, $fileReferenceData);
         return $fileReferenceObject;
     }
 
@@ -614,17 +644,27 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      *
      * @param int $uid The uid of the file usage (sys_file_reference) to be fetched
      * @param bool $raw Whether to get raw results without performing overlays
-     * @return NULL|array
+     * @return array|null
      */
     protected function getFileReferenceData($uid, $raw = false)
     {
         if (!$raw && TYPO3_MODE === 'BE') {
-            $fileReferenceData = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordWSOL('sys_file_reference', $uid);
+            $fileReferenceData = BackendUtility::getRecordWSOL('sys_file_reference', $uid);
         } elseif (!$raw && is_object($GLOBALS['TSFE'])) {
             $fileReferenceData = $GLOBALS['TSFE']->sys_page->checkRecord('sys_file_reference', $uid);
         } else {
-            /** @var $GLOBALS['TYPO3_DB'] \TYPO3\CMS\Core\Database\DatabaseConnection */
-            $fileReferenceData = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('*', 'sys_file_reference', 'uid=' . (int)$uid . ' AND deleted=0');
+            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
+            $queryBuilder->getRestrictions()->removeAll()->add(GeneralUtility::makeInstance(DeletedRestriction::class));
+            $fileReferenceData = $queryBuilder->select('*')
+                ->from('sys_file_reference')
+                ->where(
+                    $queryBuilder->expr()->eq(
+                        'uid',
+                        $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)
+                    )
+                )
+                ->execute()
+                ->fetch();
         }
         return $fileReferenceData;
     }
@@ -646,16 +686,17 @@ class ResourceFactory implements ResourceFactoryInterface, \TYPO3\CMS\Core\Singl
      */
     protected function getProcessedFileRepository()
     {
-        return GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\ProcessedFileRepository::class);
+        return GeneralUtility::makeInstance(ProcessedFileRepository::class);
     }
 
     /**
      * Returns an instance of the Indexer
      *
-     * @return \TYPO3\CMS\Core\Resource\Index\Indexer
+     * @param ResourceStorage $storage
+     * @return Index\Indexer
      */
     protected function getIndexer(ResourceStorage $storage)
     {
-        return GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\Index\Indexer::class, $storage);
+        return GeneralUtility::makeInstance(Index\Indexer::class, $storage);
     }
 }
