@@ -15,6 +15,7 @@ namespace TYPO3\CMS\Core\TimeTracker;
  */
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\SingletonInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
@@ -22,8 +23,14 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  *
  * Is used to register how much time is used with operations in TypoScript
  */
-class TimeTracker
+class TimeTracker implements SingletonInterface
 {
+    /**
+     * If set to true (see constructor) then then the timetracking is enabled
+     * @var bool
+     */
+    protected $isEnabled = false;
+
     /**
      * Is loaded with the millisecond time when this object is created
      *
@@ -37,7 +44,7 @@ class TimeTracker
      *
      * @var bool
      */
-    public $LR = 1;
+    public $LR = true;
 
     /**
      * @var array
@@ -50,7 +57,6 @@ class TimeTracker
         // Determines max length of displayed content FROM FILE cObjects before it gets cropped. Reason is that most FILE cObjects are huge and often used as template-code.
         'flag_tree' => 1,
         'flag_messages' => 1,
-        'flag_queries' => 0,
         'flag_content' => 0,
         'allTime' => 0,
         'keyLgd' => 40
@@ -59,12 +65,22 @@ class TimeTracker
     /**
      * @var array
      */
-    public $wrapError = [];
+    public $wrapError = [
+        0 => ['', ''],
+        1 => ['<strong>', '</strong>'],
+        2 => ['<strong style="color:#ff6600;">', '</strong>'],
+        3 => ['<strong style="color:#ff0000;">', '</strong>']
+    ];
 
     /**
      * @var array
      */
-    public $wrapIcon = [];
+    public $wrapIcon = [
+        0 => '',
+        1 => 'actions-document-info',
+        2 => 'status-dialog-warning',
+        3 => 'status-dialog-error'
+    ];
 
     /**
      * @var int
@@ -113,26 +129,25 @@ class TimeTracker
      * Logging parsing times in the scripts
      *
      *******************************************/
+
     /**
-     * Constructor
-     * Sets the starting time
+     * TimeTracker constructor.
      *
-     * @return void
+     * @param bool $isEnabled
+     */
+    public function __construct($isEnabled = true)
+    {
+        $this->isEnabled = $isEnabled;
+    }
+
+    /**
+     * Sets the starting time
      */
     public function start()
     {
-        $this->wrapError = [
-            0 => ['', ''],
-            1 => ['<strong>', '</strong>'],
-            2 => ['<strong style="color:#ff6600;">', '</strong>'],
-            3 => ['<strong style="color:#ff0000;">', '</strong>']
-        ];
-        $this->wrapIcon = [
-            0 => '',
-            1 => 'actions-document-info',
-            2 => 'status-dialog-warning',
-            3 => 'status-dialog-error'
-        ];
+        if (!$this->isEnabled) {
+            return;
+        }
         $this->starttime = $this->getMilliseconds();
     }
 
@@ -141,13 +156,15 @@ class TimeTracker
      *
      * @param string $tslabel Label string for the entry, eg. TypoScript property name
      * @param string $value Additional value(?)
-     * @return void
      * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::cObjGetSingle(), pull()
      */
     public function push($tslabel, $value = '')
     {
-        array_push($this->tsStack[$this->tsStackPointer], $tslabel);
-        array_push($this->currentHashPointer, 'timetracker_' . $this->uniqueCounter++);
+        if (!$this->isEnabled) {
+            return;
+        }
+        $this->tsStack[$this->tsStackPointer][] = $tslabel;
+        $this->currentHashPointer[] = 'timetracker_' . $this->uniqueCounter++;
         $this->tsStackLevel++;
         $this->tsStackLevelMax[] = $this->tsStackLevel;
         // setTSlog
@@ -165,11 +182,13 @@ class TimeTracker
      * Pulls an element from the TypoScript tracking array
      *
      * @param string $content The content string generated within the push/pull part.
-     * @return void
      * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::cObjGetSingle(), push()
      */
     public function pull($content = '')
     {
+        if (!$this->isEnabled) {
+            return;
+        }
         $k = end($this->currentHashPointer);
         $this->tsStackLog[$k]['endtime'] = microtime(true);
         $this->tsStackLog[$k]['content'] = $content;
@@ -183,13 +202,16 @@ class TimeTracker
      *
      * @param string $content The message string
      * @param int $num Message type: 0: information, 1: message, 2: warning, 3: error
-     * @return void
      * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::CONTENT()
      */
     public function setTSlogMessage($content, $num = 0)
     {
+        if (!$this->isEnabled) {
+            return;
+        }
         end($this->currentHashPointer);
         $k = current($this->currentHashPointer);
+        $placeholder = '';
         // Enlarge the "details" column by adding a span
         if (strlen($content) > 30) {
             $placeholder = '<br /><span style="width: 300px; height: 1px; display: inline-block;"></span>';
@@ -203,10 +225,12 @@ class TimeTracker
      *
      * @param array $data Query array
      * @param string $msg Message/Label to attach
-     * @return void
      */
     public function setTSselectQuery(array $data, $msg = '')
     {
+        if (!$this->isEnabled) {
+            return;
+        }
         end($this->currentHashPointer);
         $k = current($this->currentHashPointer);
         if ($msg !== '') {
@@ -218,11 +242,13 @@ class TimeTracker
     /**
      * Increases the stack pointer
      *
-     * @return void
      * @see decStackPointer(), \TYPO3\CMS\Frontend\Page\PageGenerator::renderContent(), \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::cObjGetSingle()
      */
     public function incStackPointer()
     {
+        if (!$this->isEnabled) {
+            return;
+        }
         $this->tsStackPointer++;
         $this->tsStack[$this->tsStackPointer] = [];
     }
@@ -230,11 +256,13 @@ class TimeTracker
     /**
      * Decreases the stack pointer
      *
-     * @return void
      * @see incStackPointer(), \TYPO3\CMS\Frontend\Page\PageGenerator::renderContent(), \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::cObjGetSingle()
      */
     public function decStackPointer()
     {
+        if (!$this->isEnabled) {
+            return;
+        }
         unset($this->tsStack[$this->tsStackPointer]);
         $this->tsStackPointer--;
     }
@@ -247,6 +275,9 @@ class TimeTracker
      */
     public function getMilliseconds($microtime = null)
     {
+        if (!$this->isEnabled) {
+            return 0;
+        }
         if (!isset($microtime)) {
             $microtime = microtime(true);
         }
@@ -264,6 +295,30 @@ class TimeTracker
         return $this->getMilliseconds($microtime) - $this->starttime;
     }
 
+    /**
+     * Get total parse time in milliseconds(without backend user initialization)
+     *
+     * @return int
+     */
+    public function getParseTime(): int
+    {
+        // Compensates for the time consumed with Back end user initialization.
+        $processStart = $this->getMilliseconds($GLOBALS['TYPO3_MISC']['microtime_start'] ?? null);
+
+        $beUserInitializationStart = $this->getMilliseconds($GLOBALS['TYPO3_MISC']['microtime_BE_USER_start'] ?? null);
+        $beUserInitializationEnd = $this->getMilliseconds($GLOBALS['TYPO3_MISC']['microtime_BE_USER_end'] ?? null);
+        $beUserInitialization = $beUserInitializationEnd - $beUserInitializationStart;
+
+        $processEnd = $this->getMilliseconds($GLOBALS['TYPO3_MISC']['microtime_end'] ?? null);
+        $totalParseTime = $processEnd - $processStart;
+
+        if ($beUserInitialization > 0) {
+            $totalParseTime -= $beUserInitialization;
+        }
+
+        return $totalParseTime;
+    }
+
     /*******************************************
      *
      * Printing the parsing time information (for Admin Panel)
@@ -276,6 +331,9 @@ class TimeTracker
      */
     public function printTSlog()
     {
+        if (!$this->isEnabled) {
+            return '';
+        }
         // Calculate times and keys for the tsStackLog
         foreach ($this->tsStackLog as $uniqueId => &$data) {
             $data['endtime'] = $this->getDifferenceToStarttime($data['endtime']);
@@ -292,7 +350,7 @@ class TimeTracker
             $this->createHierarchyArray($arr, $data['level'], $uniqueId);
         }
         // Parsing the registeret content and create icon-html for the tree
-        $this->tsStackLog[$arr['0.'][0]]['content'] = $this->fixContent($arr['0.'], $this->tsStackLog[$arr['0.'][0]]['content'], '', 0, $arr['0.'][0]);
+        $this->tsStackLog[$arr['0.'][0]]['content'] = $this->fixContent($arr['0.'], $this->tsStackLog[$arr['0.'][0]]['content'], '', $arr['0.'][0]);
         // Displaying the tree:
         $outputArr = [];
         $outputArr[] = $this->fw('TypoScript Key');
@@ -308,26 +366,19 @@ class TimeTracker
         $outputArr[] = $this->fw('Details');
         $out = '';
         foreach ($outputArr as $row) {
-            $out .= '
-				<th><strong>' . $row . '</strong></th>';
+            $out .= '<th>' . $row . '</th>';
         }
-        $out = '<tr class="typo3-adminPanel-itemRow">' . $out . '</tr>';
+        $out = '<thead><tr>' . $out . '</tr></thead>';
         $flag_tree = $this->printConf['flag_tree'];
         $flag_messages = $this->printConf['flag_messages'];
         $flag_content = $this->printConf['flag_content'];
-        $flag_queries = $this->printConf['flag_queries'];
         $keyLgd = $this->printConf['keyLgd'];
-        $factor = $this->printConf['factor'];
-        $col = $this->printConf['col'];
-        $highlight_col = $this->printConf['highlight_col'];
         $c = 0;
         foreach ($this->tsStackLog as $uniqueId => $data) {
+            $logRowClass = '';
             if ($this->highlightLongerThan && (int)$data['owntime'] > (int)$this->highlightLongerThan) {
                 $logRowClass = 'typo3-adminPanel-logRow-highlight';
-            } else {
-                $logRowClass = $c % 2 ? 'line-odd' : 'line-even';
             }
-            $logRowClass .= ' typo3-adminPanel-section-content-title';
             $item = '';
             // If first...
             if (!$c) {
@@ -340,13 +391,13 @@ class TimeTracker
             if (!$flag_tree && $data['stackPointer']) {
                 $temp = [];
                 foreach ($data['tsStack'] as $k => $v) {
-                    $temp[] = GeneralUtility::fixed_lgd_cs(implode($v, $k ? '.' : '/'), -$keyLgd);
+                    $temp[] = GeneralUtility::fixed_lgd_cs(implode($k ? '.' : '/', $v), -$keyLgd);
                 }
                 array_pop($temp);
                 $temp = array_reverse($temp);
                 array_pop($temp);
                 if (!empty($temp)) {
-                    $keyLabel = '<br /><span style="color:#999999;">' . implode($temp, '<br />') . '</span>';
+                    $keyLabel = '<br /><span style="color:#999999;">' . implode('<br />', $temp) . '</span>';
                 }
             }
             if ($flag_tree) {
@@ -358,7 +409,7 @@ class TimeTracker
             $theLabel = GeneralUtility::fixed_lgd_cs($theLabel, -$keyLgd);
             $theLabel = $data['stackPointer'] ? '<span class="stackPointer">' . $theLabel . '</span>' : $theLabel;
             $keyLabel = $theLabel . $keyLabel;
-            $item .= '<td class="' . $logRowClass . '">' . ($flag_tree ? $data['icons'] : '') . $this->fw($keyLabel) . '</td>';
+            $item .= '<th scope="row" class="typo3-adminPanel-table-cell-key ' . $logRowClass . '">' . ($flag_tree ? $data['icons'] : '') . $this->fw($keyLabel) . '</th>';
             // Key value:
             $keyValue = $data['value'];
             $item .= '<td class="' . $logRowClass . ' typo3-adminPanel-tsLogTime">' . $this->fw(htmlspecialchars($keyValue)) . '</td>';
@@ -378,9 +429,6 @@ class TimeTracker
                     $msgArr[] = nl2br($v);
                 }
             }
-            if ($flag_queries && is_array($data['selectQuery'])) {
-                $msgArr[] = \TYPO3\CMS\Core\Utility\DebugUtility::viewArray($data['selectQuery']);
-            }
             if ($flag_content && (string)$data['content'] !== '') {
                 $maxlen = 120;
                 // Break lines which are too longer than $maxlen chars (can happen if content contains long paths...)
@@ -390,16 +438,16 @@ class TimeTracker
                         $data['content'] = str_replace($reg[0][$key], $match, $data['content']);
                     }
                 }
-                $msgArr[] = '<span style="color:#000066;">' . nl2br($data['content']) . '</span>';
+                $msgArr[] = nl2br($data['content']);
             }
             if (!empty($msgArr)) {
-                $msg = implode($msgArr, '<hr />');
+                $msg = implode('<hr />', $msgArr);
             }
-            $item .= '<td valign="top" class="' . $logRowClass . '" style="text-align:left;">' . $this->fw($msg) . '</td>';
-            $out .= '<tr class="typo3-adminPanel-itemRow">' . $item . '</tr>';
+            $item .= '<td class="typo3-adminPanel-table-cell-content">' . $this->fw($msg) . '</td>';
+            $out .= '<tr>' . $item . '</tr>';
             $c++;
         }
-        $out = '<table class="typo3-adminPanel-table typo3-adminPanel-tsLog">' . $out . '</table>';
+        $out = '<div class="typo3-adminPanel-table-overflow"><table class="typo3-adminPanel-table typo3-adminPanel-table-debug">' . $out . '</table></div>';
         return $out;
     }
 
@@ -409,11 +457,10 @@ class TimeTracker
      * @param array $arr Array which is modified with content. Reference
      * @param string $content Current content string for the level
      * @param string $depthData Prefixed icons for new PM icons
-     * @param bool $first Set this for the first call from outside.
      * @param string $vKey Seems to be the previous tsStackLog key
      * @return string Returns the $content string generated/modified. Also the $arr array is modified!
      */
-    protected function fixContent(&$arr, $content, $depthData = '', $first = 0, $vKey = '')
+    protected function fixContent(&$arr, $content, $depthData = '', $vKey = '')
     {
         $ac = 0;
         $c = 0;
@@ -428,18 +475,18 @@ class TimeTracker
         foreach ($arr as $k => $v) {
             if (\TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($k)) {
                 $c++;
-                $deeper = is_array($arr[$k . '.']) ? 1 : 0;
+                $deeper = isset($arr[$k . '.']);
                 $LN = $ac == $c ? 'blank' : 'line';
 
                 $BTM = $ac == $c ? 'bottom' : '';
-                $PM = is_array($arr[$k . '.']) ? '<i class="fa fa-' . ($deeper ? 'minus' : 'plus') . '-square-o"></i>' : '<span class="treeline-icon treeline-icon-join' . ($BTM ? 'bottom' : '') . '"></span>';
+                $PM = $deeper ? '<i class="fa fa-' . ($deeper ? 'minus' : 'plus') . '-square-o"></i>' : '<span class="treeline-icon treeline-icon-join' . ($BTM ? 'bottom' : '') . '"></span>';
 
-                $this->tsStackLog[$v]['icons'] = $depthData . ($first ? '' : $PM);
+                $this->tsStackLog[$v]['icons'] = $depthData . $PM;
                 if ($this->tsStackLog[$v]['content'] !== '') {
                     $content = str_replace($this->tsStackLog[$v]['content'], $v, $content);
                 }
-                if (is_array($arr[$k . '.'])) {
-                    $this->tsStackLog[$v]['content'] = $this->fixContent($arr[$k . '.'], $this->tsStackLog[$v]['content'], $depthData . ($first ? '' : '<span class="treeline-icon treeline-icon-' . $LN . '"></span>'), 0, $v);
+                if ($deeper) {
+                    $this->tsStackLog[$v]['content'] = $this->fixContent($arr[$k . '.'], $this->tsStackLog[$v]['content'], $depthData . '<span class="treeline-icon treeline-icon-' . $LN . '"></span>', $v);
                 } else {
                     $this->tsStackLog[$v]['content'] = $this->fixCLen($this->tsStackLog[$v]['content'], $this->tsStackLog[$v]['value']);
                     $this->tsStackLog[$v]['subtime'] = '';
@@ -475,7 +522,7 @@ class TimeTracker
      */
     protected function fixCLen($c, $v)
     {
-        $len = $v == 'FILE' ? $this->printConf['contentLength_FILE'] : $this->printConf['contentLength'];
+        $len = $v === 'FILE' ? $this->printConf['contentLength_FILE'] : $this->printConf['contentLength'];
         if (strlen($c) > $len) {
             $c = '<span style="color:green;">' . htmlspecialchars(GeneralUtility::fixed_lgd_cs($c, $len)) . '</span>';
         } else {
@@ -501,8 +548,7 @@ class TimeTracker
      * @param array $arr Array (passed by reference) and modified
      * @param int $pointer Pointer value
      * @param string $uniqueId Unique ID string
-     * @return void
-     * @access private
+     * @internal
      * @see printTSlog()
      */
     protected function createHierarchyArray(&$arr, $pointer, $uniqueId)
