@@ -14,11 +14,59 @@ namespace TYPO3\CMS\Core\Utility;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Utility\Exception\MissingArrayPathException;
+
 /**
  * Class with helper functions for array handling
  */
 class ArrayUtility
 {
+    /**
+     * Validates the given $arrayToTest by checking if an element is not in $allowedArrayKeys.
+     *
+     * @param array $arrayToTest
+     * @param array $allowedArrayKeys
+     * @throws \InvalidArgumentException if an element in $arrayToTest is not in $allowedArrayKeys
+     * @internal
+     */
+    public static function assertAllArrayKeysAreValid(array $arrayToTest, array $allowedArrayKeys)
+    {
+        $notAllowedArrayKeys = array_keys(array_diff_key($arrayToTest, array_flip($allowedArrayKeys)));
+        if (count($notAllowedArrayKeys) !== 0) {
+            throw new \InvalidArgumentException(
+                sprintf(
+                    'The options "%s" were not allowed (allowed were: "%s")',
+                    implode(', ', $notAllowedArrayKeys),
+                    implode(', ', $allowedArrayKeys)
+                ),
+                1325697085
+            );
+        }
+    }
+
+    /**
+     * Recursively convert 'true' and 'false' strings to boolean values.
+     *
+     * @param array $array
+     * @return array the modified array
+     */
+    public static function convertBooleanStringsToBooleanRecursive(array $array): array
+    {
+        $result = $array;
+        foreach ($result as $key => $value) {
+            if (is_array($value)) {
+                $result[$key] = self::convertBooleanStringsToBooleanRecursive($value);
+            } else {
+                if ($value === 'true') {
+                    $result[$key] = true;
+                } elseif ($value === 'false') {
+                    $result[$key] = false;
+                }
+            }
+        }
+        return $result;
+    }
+
     /**
      * Reduce an array by a search value and keep the array structure.
      *
@@ -60,11 +108,11 @@ class ArrayUtility
         // Write to $resultArray (by reference!) if types and value match
         $callback = function (&$value, $key) use ($needle, &$resultArray) {
             if ($value === $needle) {
-                ($resultArray[$key] = $value);
+                $resultArray[$key] = $value;
             } elseif (is_array($value)) {
-                ($subArrayMatches = static::filterByValueRecursive($needle, $value));
+                $subArrayMatches = static::filterByValueRecursive($needle, $value);
                 if (!empty($subArrayMatches)) {
-                    ($resultArray[$key] = $subArrayMatches);
+                    $resultArray[$key] = $subArrayMatches;
                 }
             }
         };
@@ -96,9 +144,8 @@ class ArrayUtility
     {
         $isValid = true;
         try {
-            // Use late static binding to enable mocking of this call in unit tests
             static::getValueByPath($array, $path, $delimiter);
-        } catch (\RuntimeException $e) {
+        } catch (MissingArrayPathException $e) {
             $isValid = false;
         }
         return $isValid;
@@ -123,32 +170,78 @@ class ArrayUtility
      * must be enclosed by " (double quote), see unit tests for details
      *
      * @param array $array Input array
-     * @param string $path Path within the array
+     * @param array|string $path Path within the array
      * @param string $delimiter Defined path delimiter, default /
      * @return mixed
-     * @throws \RuntimeException
+     * @throws \RuntimeException if the path is empty, or if the path does not exist
+     * @throws \InvalidArgumentException if the path is neither array nor string
      */
     public static function getValueByPath(array $array, $path, $delimiter = '/')
     {
-        if (empty($path)) {
-            throw new \RuntimeException('Path must not be empty', 1341397767);
-        }
         // Extract parts of the path
-        $path = str_getcsv($path, $delimiter);
+        if (is_string($path)) {
+            if ($path === '') {
+                // Programming error has to be sanitized before calling the method -> global exception
+                throw new \RuntimeException('Path must not be empty', 1341397767);
+            }
+            $path = str_getcsv($path, $delimiter);
+        } elseif (!is_array($path)) {
+            // Programming error has to be sanitized before calling the method -> global exception
+            throw new \InvalidArgumentException('getValueByPath() expects $path to be string or array, "' . gettype($path) . '" given.', 1476557628);
+        }
         // Loop through each part and extract its value
         $value = $array;
         foreach ($path as $segment) {
-            if (array_key_exists($segment, $value)) {
+            if (is_array($value) && array_key_exists($segment, $value)) {
                 // Replace current value with child
                 $value = $value[$segment];
             } else {
-                // Fail if key does not exist
-                throw new \RuntimeException('Path does not exist in array', 1341397869);
+                // Throw specific exception if there is no such path
+                throw new MissingArrayPathException('Segment ' . $segment . ' of path ' . implode($delimiter, $path) . ' does not exist in array', 1341397869);
             }
         }
         return $value;
     }
 
+    /**
+     * Reindex keys from the current nesting level if all keys within
+     * the current nesting level are integers.
+     *
+     * @param array $array
+     * @return array
+     */
+    public static function reIndexNumericArrayKeysRecursive(array $array): array
+    {
+        if (count(array_filter(array_keys($array), 'is_string')) === 0) {
+            $array = array_values($array);
+        }
+        foreach ($array as $key => $value) {
+            if (is_array($value) && !empty($value)) {
+                $array[$key] = self::reIndexNumericArrayKeysRecursive($value);
+            }
+        }
+        return $array;
+    }
+
+    /**
+     * Recursively remove keys if their value are NULL.
+     *
+     * @param array $array
+     * @return array the modified array
+     */
+    public static function removeNullValuesRecursive(array $array): array
+    {
+        $result = $array;
+        foreach ($result as $key => $value) {
+            if (is_array($value)) {
+                $result[$key] = self::removeNullValuesRecursive($value);
+            } elseif ($value === null) {
+                unset($result[$key]);
+            }
+        }
+        return $result;
+    }
+
     /**
      * Modifies or sets a new value in an array by given path
      *
@@ -169,7 +262,7 @@ class ArrayUtility
      * );
      *
      * @param array $array Input array to manipulate
-     * @param string $path Path in array to search for
+     * @param string|array $path Path in array to search for
      * @param mixed $value Value to set at path location in array
      * @param string $delimiter Path delimiter
      * @return array Modified array
@@ -177,20 +270,21 @@ class ArrayUtility
      */
     public static function setValueByPath(array $array, $path, $value, $delimiter = '/')
     {
-        if (empty($path)) {
-            throw new \RuntimeException('Path must not be empty', 1341406194);
+        if (is_string($path)) {
+            if ($path === '') {
+                throw new \RuntimeException('Path must not be empty', 1341406194);
+            }
+            // Extract parts of the path
+            $path = str_getcsv($path, $delimiter);
+        } elseif (!is_array($path) && !$path instanceof \ArrayAccess) {
+            throw new \InvalidArgumentException('setValueByPath() expects $path to be string, array or an object implementing \\ArrayAccess, "' . (is_object($path) ? get_class($path) : gettype($path)) . '" given.', 1478781081);
         }
-        if (!is_string($path)) {
-            throw new \RuntimeException('Path must be a string', 1341406402);
-        }
-        // Extract parts of the path
-        $path = str_getcsv($path, $delimiter);
         // Point to the root of the array
         $pointer = &$array;
         // Find path in given array
         foreach ($path as $segment) {
             // Fail if the part is empty
-            if (empty($segment)) {
+            if ($segment === '') {
                 throw new \RuntimeException('Invalid path segment specified', 1341406846);
             }
             // Create cell if it doesn't exist
@@ -216,12 +310,12 @@ class ArrayUtility
      */
     public static function removeByPath(array $array, $path, $delimiter = '/')
     {
-        if (empty($path)) {
-            throw new \RuntimeException('Path must not be empty', 1371757718);
-        }
         if (!is_string($path)) {
             throw new \RuntimeException('Path must be a string', 1371757719);
         }
+        if ($path === '') {
+            throw new \RuntimeException('Path must not be empty', 1371757718);
+        }
         // Extract parts of the path
         $path = str_getcsv($path, $delimiter);
         $pathDepth = count($path);
@@ -231,11 +325,11 @@ class ArrayUtility
         foreach ($path as $segment) {
             $currentDepth++;
             // Fail if the part is empty
-            if (empty($segment)) {
+            if ($segment === '') {
                 throw new \RuntimeException('Invalid path segment specified', 1371757720);
             }
             if (!array_key_exists($segment, $pointer)) {
-                throw new \RuntimeException('Path segment ' . $segment . ' does not exist in array', 1371758436);
+                throw new MissingArrayPathException('Segment ' . $segment . ' of path ' . implode($delimiter, $path) . ' does not exist in array', 1371758436);
             }
             if ($currentDepth === $pathDepth) {
                 unset($pointer[$segment]);
@@ -249,7 +343,7 @@ class ArrayUtility
     /**
      * Sorts an array recursively by key
      *
-     * @param $array Array to sort recursively by key
+     * @param array $array Array to sort recursively by key
      * @return array Sorted array
      */
     public static function sortByKeyRecursive(array $array)
@@ -281,7 +375,7 @@ class ArrayUtility
             if (!isset($a[$key]) || !isset($b[$key])) {
                 throw new \RuntimeException('The specified sorting key "' . $key . '" is not available in the given array.', 1373727309);
             }
-            return ($ascending) ? strcasecmp($a[$key], $b[$key]) : strcasecmp($b[$key], $a[$key]);
+            return $ascending ? strcasecmp($a[$key], $b[$key]) : strcasecmp($b[$key], $a[$key]);
         });
         if (!$sortResult) {
             throw new \RuntimeException('The function uasort() failed for unknown reasons.', 1373727329);
@@ -302,7 +396,7 @@ class ArrayUtility
      */
     public static function arrayExport(array $array = [], $level = 0)
     {
-        $lines = '[' . LF;
+        $lines = "[\n";
         $level++;
         $writeKeyIndex = false;
         $expectedKeyIndex = 0;
@@ -326,26 +420,25 @@ class ArrayUtility
                 if (!empty($value)) {
                     $lines .= self::arrayExport($value, $level);
                 } else {
-                    $lines .= '[],' . LF;
+                    $lines .= "[],\n";
                 }
             } elseif (is_int($value) || is_float($value)) {
-                $lines .= $value . ',' . LF;
-            } elseif (is_null($value)) {
-                $lines .= 'null' . ',' . LF;
+                $lines .= $value . ",\n";
+            } elseif ($value === null) {
+                $lines .= "null,\n";
             } elseif (is_bool($value)) {
                 $lines .= $value ? 'true' : 'false';
-                $lines .= ',' . LF;
+                $lines .= ",\n";
             } elseif (is_string($value)) {
                 // Quote \ to \\
-                $stringContent = str_replace('\\', '\\\\', $value);
                 // Quote ' to \'
-                $stringContent = str_replace('\'', '\\\'', $stringContent);
-                $lines .= '\'' . $stringContent . '\'' . ',' . LF;
+                $stringContent = str_replace(['\\', '\''], ['\\\\', '\\\''], $value);
+                $lines .= '\'' . $stringContent . "',\n";
             } else {
                 throw new \RuntimeException('Objects are not supported', 1342294987);
             }
         }
-        $lines .= str_repeat('    ', ($level - 1)) . ']' . ($level - 1 == 0 ? '' : ',' . LF);
+        $lines .= str_repeat('    ', $level - 1) . ']' . ($level - 1 == 0 ? '' : ",\n");
         return $lines;
     }
 
@@ -380,13 +473,14 @@ class ArrayUtility
      *
      * @param array $array The (relative) array to be converted
      * @param string $prefix The (relative) prefix to be used (e.g. 'section.')
+     * @param bool $keepDots
      * @return array
      */
     public static function flatten(array $array, $prefix = '')
     {
         $flatArray = [];
         foreach ($array as $key => $value) {
-            // Ensure there is no trailling dot:
+            // Ensure there is no trailing dot:
             $key = rtrim($key, '.');
             if (!is_array($value)) {
                 $flatArray[$prefix . $key] = $value;
@@ -481,7 +575,7 @@ class ArrayUtility
         $level++;
         $allKeysAreNumeric = true;
         foreach ($array as $key => $_) {
-            if (is_numeric($key) === false) {
+            if (is_int($key) === false) {
                 $allKeysAreNumeric = false;
                 break;
             }
@@ -516,7 +610,6 @@ class ArrayUtility
      * @param bool $addKeys If set to FALSE, keys that are NOT found in $original will not be set. Thus only existing value can/will be overruled from overrule array.
      * @param bool $includeEmptyValues If set, values from $overrule will overrule if they are empty or zero.
      * @param bool $enableUnsetFeature If set, special values "__UNSET" can be used in the overrule array in order to unset array keys in the original array.
-     * @return void
      */
     public static function mergeRecursiveWithOverrule(array &$original, array $overrule, $addKeys = true, $includeEmptyValues = true, $enableUnsetFeature = true)
     {
@@ -540,38 +633,6 @@ class ArrayUtility
         reset($original);
     }
 
-    /**
-     * Check if an string item exists in an array.
-     * Please note that the order of function parameters is reverse compared to the PHP function in_array()!!!
-     *
-     * Comparison to PHP in_array():
-     * -> $array = array(0, 1, 2, 3);
-     * -> variant_a := \TYPO3\CMS\Core\Utility\ArrayUtility::inArray($array, $needle)
-     * -> variant_b := in_array($needle, $array)
-     * -> variant_c := in_array($needle, $array, TRUE)
-     * +---------+-----------+-----------+-----------+
-     * | $needle | variant_a | variant_b | variant_c |
-     * +---------+-----------+-----------+-----------+
-     * | '1a'    | FALSE     | TRUE      | FALSE     |
-     * | ''      | FALSE     | TRUE      | FALSE     |
-     * | '0'     | TRUE      | TRUE      | FALSE     |
-     * | 0       | TRUE      | TRUE      | TRUE      |
-     * +---------+-----------+-----------+-----------+
-     *
-     * @param array $in_array One-dimensional array of items
-     * @param string $item Item to check for
-     * @return bool TRUE if $item is in the one-dimensional array $in_array
-     */
-    public static function inArray(array $in_array, $item)
-    {
-        foreach ($in_array as $val) {
-            if (!is_array($val) && (string)$val === (string)$item) {
-                return true;
-            }
-        }
-        return false;
-    }
-
     /**
      * Removes the value $cmpValue from the $array if found there. Returns the modified array
      *
@@ -627,10 +688,11 @@ class ArrayUtility
             }
             // Do the filtering:
             if (is_array($keepItems) && !empty($keepItems)) {
+                $keepItems = array_flip($keepItems);
                 foreach ($array as $key => $value) {
                     // Get the value to compare by using the callback function:
                     $keepValue = isset($getValueFunc) ? call_user_func($getValueFunc, $value) : $value;
-                    if (!in_array($keepValue, $keepItems)) {
+                    if (!isset($keepItems[$keepValue])) {
                         unset($array[$key]);
                     }
                 }
@@ -657,7 +719,7 @@ class ArrayUtility
 
     /**
      * Filters keys off from first array that also exist in second array. Comparison is done by keys.
-     * This method is a recursive version of php array_diff_assoc()
+     * This method is a recursive version of php array_diff_key()
      *
      * @param array $array1 Source array
      * @param array $array2 Reduce source array by this array
@@ -671,7 +733,10 @@ class ArrayUtility
                 $differenceArray[$key] = $value;
             } elseif (is_array($value)) {
                 if (is_array($array2[$key])) {
-                    $differenceArray[$key] = self::arrayDiffAssocRecursive($value, $array2[$key]);
+                    $recursiveResult = self::arrayDiffAssocRecursive($value, $array2[$key]);
+                    if (!empty($recursiveResult)) {
+                        $differenceArray[$key] = $recursiveResult;
+                    }
                 }
             }
         }
@@ -695,4 +760,105 @@ class ArrayUtility
 
         return true;
     }
+
+    /**
+     * Takes a TypoScript array as input and returns an array which contains all integer properties found which had a value (not only properties). The output array will be sorted numerically.
+     *
+     * @param array $setupArr TypoScript array with numerical array in
+     * @param bool $acceptAnyKeys If set, then a value is not required - the properties alone will be enough.
+     * @return array An array with all integer properties listed in numeric order.
+     * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::cObjGet(), \TYPO3\CMS\Frontend\Imaging\GifBuilder, \TYPO3\CMS\Frontend\ContentObject\Menu\ImageMenuContentObject::makeImageMap()
+     */
+    public static function filterAndSortByNumericKeys($setupArr, $acceptAnyKeys = false)
+    {
+        $filteredKeys = [];
+        $keys = array_keys($setupArr);
+        foreach ($keys as $key) {
+            if ($acceptAnyKeys || MathUtility::canBeInterpretedAsInteger($key)) {
+                $filteredKeys[] = (int)$key;
+            }
+        }
+        $filteredKeys = array_unique($filteredKeys);
+        sort($filteredKeys);
+        return $filteredKeys;
+    }
+
+    /**
+     * If the array contains numerical keys only, sort it in ascending order
+     *
+     * @param array $array
+     *
+     * @return array
+     */
+    public static function sortArrayWithIntegerKeys(array $array)
+    {
+        if (count(array_filter(array_keys($array), 'is_string')) === 0) {
+            ksort($array);
+        }
+        return $array;
+    }
+
+    /**
+     * Sort keys from the current nesting level if all keys within the
+     * current nesting level are integers.
+     *
+     * @param array $array
+     * @return array
+     */
+    public static function sortArrayWithIntegerKeysRecursive(array $array): array
+    {
+        $array = static::sortArrayWithIntegerKeys($array);
+        foreach ($array as $key => $value) {
+            if (is_array($value) && !empty($value)) {
+                $array[$key] = self::sortArrayWithIntegerKeysRecursive($value);
+            }
+        }
+        return $array;
+    }
+
+    /**
+     * Recursively translate values.
+     *
+     * @param array $array
+     * @return array the modified array
+     */
+    public static function stripTagsFromValuesRecursive(array $array): array
+    {
+        $result = $array;
+        foreach ($result as $key => $value) {
+            if (is_array($value)) {
+                $result[$key] = self::stripTagsFromValuesRecursive($value);
+            } elseif (is_string($value) || (is_object($value) && method_exists($value, '__toString'))) {
+                $result[$key] = strip_tags($value);
+            }
+        }
+        return $result;
+    }
+
+    /**
+     * Recursively filter an array
+     *
+     * @param array $array
+     * @param callable|null $callback
+     * @return array the filtered array
+     * @see https://secure.php.net/manual/en/function.array-filter.php
+     */
+    public static function filterRecursive(array $array, callable $callback = null): array
+    {
+        $callback = $callback ?: function ($value) {
+            return (bool)$value;
+        };
+
+        foreach ($array as $key => $value) {
+            if (is_array($value)) {
+                $array[$key] = self::filterRecursive($value, $callback);
+            }
+
+            if (!call_user_func($callback, $value)) {
+                unset($array[$key]);
+            }
+        }
+
+        return $array;
+    }
 }
