@@ -1,7 +1,7 @@
 <?php
 return [
     'ctrl' => [
-        'title' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference',
+        'title' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference',
         'label' => 'uid_local',
         'formattedLabel_userFunc' => 'TYPO3\\CMS\\Core\\Resource\\Service\\UserFileInlineLabelService->getInlineLabel',
         'formattedLabel_userFunc_options' => [
@@ -15,7 +15,6 @@ return [
         'cruser_id' => 'cruser_id',
         'type' => 'uid_local:type',
         'hideTable' => true,
-        'sortby' => 'sorting',
         'delete' => 'deleted',
         'versioningWS' => true,
         'languageField' => 'sys_language_uid',
@@ -41,34 +40,35 @@ return [
     ],
     'columns' => [
         't3ver_label' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_general.xlf:LGL.versionLabel',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.versionLabel',
             'config' => [
                 'type' => 'input',
-                'size' => '30',
-                'max' => '30'
+                'size' => 30,
+                'max' => 30
             ]
         ],
         'sys_language_uid' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_general.xlf:LGL.language',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.language',
             'config' => [
                 'type' => 'select',
                 'renderType' => 'selectSingle',
                 'foreign_table' => 'sys_language',
                 'foreign_table_where' => 'ORDER BY sys_language.title',
                 'items' => [
-                    ['LLL:EXT:lang/locallang_general.xlf:LGL.allLanguages', -1],
-                    ['LLL:EXT:lang/locallang_general.xlf:LGL.default_value', 0]
+                    ['LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.allLanguages', -1],
+                    ['LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.default_value', 0]
                 ],
                 'default' => 0,
-                'showIconTable' => true,
+                'fieldWizard' => [
+                    'selectIcons' => [
+                        'disabled' => false,
+                    ],
+                ],
             ]
         ],
         'l10n_parent' => [
             'displayCond' => 'FIELD:sys_language_uid:>:0',
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_general.xlf:LGL.l18n_parent',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.l18n_parent',
             'config' => [
                 'type' => 'select',
                 'renderType' => 'selectSingle',
@@ -81,23 +81,20 @@ return [
             ]
         ],
         'l10n_diffsource' => [
-            'exclude' => 0,
             'config' => [
                 'type' => 'passthrough',
                 'default' => ''
             ]
         ],
         'hidden' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_general.xlf:LGL.hidden',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.hidden',
             'config' => [
                 'type' => 'check',
-                'default' => '0'
+                'default' => 0
             ]
         ],
         'uid_local' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.uid_local',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.uid_local',
             'config' => [
                 'type' => 'group',
                 'internal_type' => 'db',
@@ -105,66 +102,59 @@ return [
                 'eval' => 'int',
                 'maxitems' => 1,
                 'minitems' => 0,
-                'allowed' => 'sys_file'
+                'allowed' => 'sys_file',
+                'hideSuggest' => true,
             ]
         ],
         'uid_foreign' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.uid_foreign',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.uid_foreign',
             'config' => [
                 'type' => 'input',
-                'size' => '10',
+                'size' => 10,
                 'eval' => 'int'
             ]
         ],
         'tablenames' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.tablenames',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.tablenames',
             'config' => [
                 'type' => 'input',
-                'size' => '30',
+                'size' => 30,
                 'eval' => 'trim'
             ]
         ],
         'fieldname' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.fieldname',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.fieldname',
             'config' => [
                 'type' => 'input',
-                'size' => '30'
+                'size' => 30
             ]
         ],
         'sorting_foreign' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.sorting_foreign',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.sorting_foreign',
             'config' => [
                 'type' => 'input',
-                'size' => '4',
-                'max' => '4',
+                'size' => 4,
+                'max' => 4,
                 'eval' => 'int',
-                'range' => [
-                    'upper' => '1000',
-                    'lower' => '10'
-                ],
                 'default' => 0
             ]
         ],
         'table_local' => [
-            'exclude' => 0,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.table_local',
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.table_local',
             'config' => [
                 'type' => 'input',
-                'size' => '20',
+                'size' => 20,
                 'default' => 'sys_file'
             ]
         ],
         'title' => [
             'l10n_mode' => 'prefixLangTitle',
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.title',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.title',
             'config' => [
                 'type' => 'input',
-                'size' => '20',
+                'size' => 20,
+                'max' => 255,
                 'eval' => 'null',
                 'placeholder' => '__row|uid_local|metadata|title',
                 'mode' => 'useOrOverridePlaceholder',
@@ -172,22 +162,19 @@ return [
             ]
         ],
         'link' => [
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.link',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.link',
             'config' => [
                 'type' => 'input',
-                'size' => '20',
+                'renderType' => 'inputLink',
+                'size' => 20,
                 'max' => 1024,
-                'wizards' => [
-                    'link' => [
-                        'type' => 'popup',
-                        'title' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.link',
-                        'icon' => 'EXT:backend/Resources/Public/Images/FormFieldWizard/wizard_link.gif',
-                        'module' => [
-                            'name' => 'wizard_link',
+                'fieldControl' => [
+                    'linkPopup' => [
+                        'options' => [
+                            'title' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.link',
                         ],
-                        'JSopenParams' => 'width=800,height=600,status=0,menubar=0,scrollbars=1'
-                    ]
+                    ],
                 ],
                 'softref' => 'typolink'
             ]
@@ -195,12 +182,12 @@ return [
         'description' => [
             // This is used for captions in the frontend
             'l10n_mode' => 'prefixLangTitle',
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.description',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.description',
             'config' => [
                 'type' => 'text',
-                'cols' => '20',
-                'rows' => '5',
+                'cols' => 20,
+                'rows' => 5,
                 'eval' => 'null',
                 'placeholder' => '__row|uid_local|metadata|description',
                 'mode' => 'useOrOverridePlaceholder',
@@ -209,11 +196,11 @@ return [
         ],
         'alternative' => [
             'l10n_mode' => 'prefixLangTitle',
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.alternative',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.alternative',
             'config' => [
                 'type' => 'input',
-                'size' => '20',
+                'size' => 20,
                 'eval' => 'null',
                 'placeholder' => '__row|uid_local|metadata|alternative',
                 'mode' => 'useOrOverridePlaceholder',
@@ -221,18 +208,25 @@ return [
             ],
         ],
         'crop' => [
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.crop',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.crop',
             'config' => [
                 'type' => 'imageManipulation'
             ]
         ],
         'autoplay' => [
-            'exclude' => 1,
-            'label' => 'LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.autoplay',
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.autoplay',
             'config' => [
                 'type' => 'check',
-                'default' => 0
+                'renderType' => 'checkboxToggle',
+                'default' => 0,
+                'items' => [
+                    [
+                        0 => '',
+                        1 => '',
+                    ]
+                ],
             ]
         ],
     ],
@@ -241,42 +235,44 @@ return [
         // We leave the extensive definition of each type here anyway, to make clear that you can use it to differentiate between the types.
         '0' => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ],
         \TYPO3\CMS\Core\Resource\File::FILETYPE_TEXT => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ],
         \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ],
         \TYPO3\CMS\Core\Resource\File::FILETYPE_AUDIO => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ],
         \TYPO3\CMS\Core\Resource\File::FILETYPE_VIDEO => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ],
         \TYPO3\CMS\Core\Resource\File::FILETYPE_APPLICATION => [
             'showitem' => '
-				--palette--;LLL:EXT:lang/locallang_tca.xlf:sys_file_reference.basicoverlayPalette;basicoverlayPalette,
+				--palette--;;basicoverlayPalette,
 				--palette--;;filePalette'
         ]
     ],
     'palettes' => [
         // Used for basic overlays: having a filelist etc
         'basicoverlayPalette' => [
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.basicoverlayPalette',
             'showitem' => 'title,description',
         ],
         // Used for everything that is an image (because it has a link and an alternative text)
         'imageoverlayPalette' => [
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.imageoverlayPalette',
             'showitem' => '
 				title,alternative,--linebreak--,
 				link,description,--linebreak--,crop
@@ -284,12 +280,14 @@ return [
         ],
         // Used for everything that is a video
         'videoOverlayPalette' => [
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.videoOverlayPalette',
             'showitem' => '
 				title,description,--linebreak--,autoplay
 				',
         ],
         // Used for everything that is a audio file
         'audioOverlayPalette' => [
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_tca.xlf:sys_file_reference.audioOverlayPalette',
             'showitem' => '
 				title,description,--linebreak--,autoplay
 				',
