@@ -19,6 +19,7 @@ namespace TYPO3\CMS\Extbase\Configuration;
  * implementation of the configuration manager and provides an unified acccess point.
  *
  * Use the shutdown() method to drop the concrete implementation.
+ * @internal only to be used within Extbase, not part of TYPO3 Core API.
  */
 class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
 {
@@ -55,8 +56,6 @@ class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\Configura
 
     /**
      * Initializes the object
-     *
-     * @return void
      */
     public function initializeObject()
     {
@@ -64,7 +63,6 @@ class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\Configura
     }
 
     /**
-     * @return void
      */
     protected function initializeConcreteConfigurationManager()
     {
@@ -77,7 +75,6 @@ class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\Configura
 
     /**
      * @param \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer $contentObject
-     * @return void
      */
     public function setContentObject(\TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer $contentObject = null)
     {
@@ -97,7 +94,6 @@ class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\Configura
      * Note that this is a low level method and only makes sense to be used by Extbase internally.
      *
      * @param array $configuration The new configuration
-     * @return void
      */
     public function setConfiguration(array $configuration = [])
     {
@@ -126,7 +122,7 @@ class ConfigurationManager implements \TYPO3\CMS\Extbase\Configuration\Configura
         switch ($configurationType) {
             case self::CONFIGURATION_TYPE_SETTINGS:
                 $configuration = $this->concreteConfigurationManager->getConfiguration($extensionName, $pluginName);
-                return $configuration['settings'];
+                return $configuration['settings'] ?? [];
             case self::CONFIGURATION_TYPE_FRAMEWORK:
                 return $this->concreteConfigurationManager->getConfiguration($extensionName, $pluginName);
             case self::CONFIGURATION_TYPE_FULL_TYPOSCRIPT:
