@@ -1,7 +1,7 @@
 <?php
 namespace TYPO3\CMS\Extbase\DomainObject;
 
-/**
+/*
  * This file is part of the TYPO3 CMS project.
  *
  * It is free software; you can redistribute it and/or modify it under
@@ -13,18 +13,21 @@ namespace TYPO3\CMS\Extbase\DomainObject;
  *
  * The TYPO3 project - inspiring people to share!
  */
-/**
- * A abstract Value Object. A Value Object is an object that describes some characteristic
- * or attribute (e.g. a color) but carries no concept of identity.
- */
-abstract class AbstractValueObject extends \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject {
 
-	/**
-	 * Returns the value of the Value Object. Must be overwritten by a concrete value object.
-	 *
-	 * @return string
-	 */
-	public function getValue() {
-		return $this->__toString();
-	}
+/**
+ * An abstract Value Object. A Value Object is an object that describes some characteristic
+ * or attribute (e.g. a color) but carries no concept of identity.
+ * @internal only to be used within Extbase, not part of TYPO3 Core API.
+ */
+abstract class AbstractValueObject extends \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject
+{
+    /**
+     * Returns the value of the Value Object. Must be overwritten by a concrete value object.
+     *
+     * @return string
+     */
+    public function getValue()
+    {
+        return $this->__toString();
+    }
 }
