@@ -1,25 +1,19 @@
 <?php
 namespace TYPO3\CMS\Extbase\Property;
 
-/*                                                                        *
- * This script belongs to the Extbase framework                           *
- *                                                                        *
- * It is free software; you can redistribute it and/or modify it under    *
- * the terms of the GNU Lesser General Public License as published by the *
- * Free Software Foundation, either version 3 of the License, or (at your *
- * option) any later version.                                             *
- *                                                                        *
- * This script is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHAN-    *
- * TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser       *
- * General Public License for more details.                               *
- *                                                                        *
- * You should have received a copy of the GNU Lesser General Public       *
- * License along with the script.                                         *
- * If not, see http://www.gnu.org/licenses/lgpl.html                      *
- *                                                                        *
- * The TYPO3 project - inspiring people to share!                         *
- *                                                                        */
+/*
+ * This file is part of the TYPO3 CMS project.
+ *
+ * It is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, either version 2
+ * of the License, or any later version.
+ *
+ * For the full copyright and license information, please read the
+ * LICENSE.txt file that was distributed with this source code.
+ *
+ * The TYPO3 project - inspiring people to share!
+ */
+
 use TYPO3\CMS\Extbase\Property\Exception\TargetNotFoundException;
 use TYPO3\CMS\Extbase\Property\TypeConverter\AbstractTypeConverter;
 use TYPO3\CMS\Extbase\Utility\TypeHandlingUtility;
@@ -27,9 +21,6 @@ use TYPO3\CMS\Extbase\Utility\TypeHandlingUtility;
 /**
  * The Property Mapper transforms simple types (arrays, strings, integers, floats, booleans) to objects or other simple types.
  * It is used most prominently to map incoming HTTP arguments to objects.
- *
- * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
- * @api
  */
 class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
 {
@@ -64,6 +55,7 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
 
     /**
      * @param \TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function injectObjectManager(\TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager)
     {
@@ -72,6 +64,7 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
 
     /**
      * @param \TYPO3\CMS\Extbase\Property\PropertyMappingConfigurationBuilder $configurationBuilder
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function injectConfigurationBuilder(\TYPO3\CMS\Extbase\Property\PropertyMappingConfigurationBuilder $configurationBuilder)
     {
@@ -83,7 +76,7 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
      * Here, the typeConverter array gets initialized.
      *
      * @throws Exception\DuplicateTypeConverterException
-     * @return void
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function initializeObject()
     {
@@ -106,7 +99,6 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
      * @param PropertyMappingConfigurationInterface $configuration Configuration for the property mapping. If NULL, the PropertyMappingConfigurationBuilder will create a default configuration.
      * @throws Exception
      * @return mixed an instance of $targetType
-     * @api
      */
     public function convert($source, $targetType, PropertyMappingConfigurationInterface $configuration = null)
     {
@@ -133,7 +125,6 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
      * Get the messages of the last Property Mapping
      *
      * @return \TYPO3\CMS\Extbase\Error\Result
-     * @api
      */
     public function getMessages()
     {
@@ -168,7 +159,10 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
         $targetType = $typeConverter->getTargetTypeForSource($source, $targetType, $configuration);
 
         if (!is_object($typeConverter) || !$typeConverter instanceof \TYPO3\CMS\Extbase\Property\TypeConverterInterface) {
-            throw new Exception\TypeConverterException('Type converter for "' . $source . '" -> "' . $targetType . '" not found.');
+            throw new Exception\TypeConverterException(
+                'Type converter for "' . $source . '" -> "' . $targetType . '" not found.',
+                1476045062
+            );
         }
 
         $convertedChildProperties = [];
@@ -245,7 +239,10 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
         }
 
         if ($converter === null) {
-            throw new Exception\TypeConverterException('No converter found which can be used to convert from "' . $sourceType . '" to "' . $targetType . '".');
+            throw new Exception\TypeConverterException(
+                'No converter found which can be used to convert from "' . $sourceType . '" to "' . $targetType . '".',
+                1476044883
+            );
         }
 
         return $converter;
@@ -297,9 +294,8 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
         }
         if (isset($convertersForSource['object'])) {
             return $this->findEligibleConverterWithHighestPriority($convertersForSource['object'], $source, $targetClass);
-        } else {
-            return null;
         }
+        return null;
     }
 
     /**
@@ -313,7 +309,7 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
         if (!is_array($converters)) {
             return null;
         }
-        krsort($converters);
+        krsort($converters, SORT_NUMERIC);
         reset($converters);
         /** @var AbstractTypeConverter $converter */
         foreach ($converters as $converter) {
@@ -357,17 +353,20 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
     {
         if (is_string($source)) {
             return 'string';
-        } elseif (is_array($source)) {
-            return 'array';
-        } elseif (is_float($source)) {
-            return 'float';
-        } elseif (is_integer($source)) {
-            return 'integer';
-        } elseif (is_bool($source)) {
-            return 'boolean';
-        } else {
-            throw new Exception\InvalidSourceException('The source is not of type string, array, float, integer or boolean, but of type "' . gettype($source) . '"', 1297773150);
         }
+        if (is_array($source)) {
+            return 'array';
+        }
+        if (is_float($source)) {
+            return 'float';
+        }
+        if (is_int($source)) {
+            return 'integer';
+        }
+        if (is_bool($source)) {
+            return 'boolean';
+        }
+        throw new Exception\InvalidSourceException('The source is not of type string, array, float, integer or boolean, but of type "' . gettype($source) . '"', 1297773150);
     }
 
     /**
@@ -376,6 +375,7 @@ class PropertyMapper implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $compositeType
      * @return string
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function parseCompositeType($compositeType)
     {
