@@ -1,34 +1,24 @@
 <?php
 namespace TYPO3\CMS\Extbase\Utility;
 
-/*                                                                        *
- * This script belongs to the Extbase framework                           *
- *                                                                        *
- * It is free software; you can redistribute it and/or modify it under    *
- * the terms of the GNU Lesser General Public License as published by the *
- * Free Software Foundation, either version 3 of the License, or (at your *
- * option) any later version.                                             *
- *                                                                        *
- * This script is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHAN-    *
- * TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser       *
- * General Public License for more details.                               *
- *                                                                        *
- * You should have received a copy of the GNU Lesser General Public       *
- * License along with the script.                                         *
- * If not, see http://www.gnu.org/licenses/lgpl.html                      *
- *                                                                        *
- * The TYPO3 project - inspiring people to share!                         *
- *                                                                        */
+/*
+ * This file is part of the TYPO3 CMS project.
+ *
+ * It is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, either version 2
+ * of the License, or any later version.
+ *
+ * For the full copyright and license information, please read the
+ * LICENSE.txt file that was distributed with this source code.
+ *
+ * The TYPO3 project - inspiring people to share!
+ */
+
 /**
  * This class is a backport of the corresponding class of TYPO3 Flow.
  * All credits go to the TYPO3 Flow team.
- */
-/**
- * A debugging utility class
  *
- * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
- * @api
+ * A debugging utility class
  */
 class DebuggerUtility
 {
@@ -78,8 +68,6 @@ class DebuggerUtility
 
     /**
      * Clear the state of the debugger
-     *
-     * @return void
      */
     protected static function clearState()
     {
@@ -101,20 +89,26 @@ class DebuggerUtility
         if (is_string($value)) {
             $croppedValue = strlen($value) > 2000 ? substr($value, 0, 2000) . '...' : $value;
             if ($plainText) {
-                $dump = self::ansiEscapeWrap(('"' . implode((PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, ($level + 1))), str_split($croppedValue, 76)) . '"'), '33', $ansiColors) . ' (' . strlen($value) . ' chars)';
+                $dump = self::ansiEscapeWrap('"' . implode(PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level + 1), str_split($croppedValue, 76)) . '"', '33', $ansiColors) . ' (' . strlen($value) . ' chars)';
             } else {
-                $dump = sprintf('\'<span class="extbase-debug-string">%s</span>\' (%s chars)', implode('<br />' . str_repeat(self::HTML_INDENT, ($level + 1)), str_split(htmlspecialchars($croppedValue), 76)), strlen($value));
+                $lines = str_split($croppedValue, 76);
+                $lines = array_map('htmlspecialchars', $lines);
+                $dump = sprintf('\'<span class="extbase-debug-string">%s</span>\' (%s chars)', implode('<br />' . str_repeat(self::HTML_INDENT, $level + 1), $lines), strlen($value));
             }
         } elseif (is_numeric($value)) {
             $dump = sprintf('%s (%s)', self::ansiEscapeWrap($value, '35', $ansiColors), gettype($value));
         } elseif (is_bool($value)) {
             $dump = $value ? self::ansiEscapeWrap('TRUE', '32', $ansiColors) : self::ansiEscapeWrap('FALSE', '32', $ansiColors);
-        } elseif (is_null($value) || is_resource($value)) {
+        } elseif ($value === null || is_resource($value)) {
             $dump = gettype($value);
         } elseif (is_array($value)) {
             $dump = self::renderArray($value, $level + 1, $plainText, $ansiColors);
         } elseif (is_object($value)) {
-            $dump = self::renderObject($value, $level + 1, $plainText, $ansiColors);
+            if ($value instanceof \Closure) {
+                $dump = self::renderClosure($value, $level + 1, $plainText, $ansiColors);
+            } else {
+                $dump = self::renderObject($value, $level + 1, $plainText, $ansiColors);
+            }
         }
         return $dump;
     }
@@ -184,9 +178,31 @@ class DebuggerUtility
         }
         if ($plainText) {
             return $header . $content;
-        } else {
-            return '<span class="extbase-debugger-tree">' . $header . '<span class="extbase-debug-content">' . $content . '</span></span>';
         }
+        return '<span class="extbase-debugger-tree">' . $header . '<span class="extbase-debug-content">' . $content . '</span></span>';
+    }
+
+    /**
+     * Renders a dump of the given closure
+     *
+     * @param \Closure $object
+     * @param int $level
+     * @param bool $plainText
+     * @param bool $ansiColors
+     * @return string
+     */
+    protected static function renderClosure($object, $level, $plainText = false, $ansiColors = false)
+    {
+        $header = self::renderHeader($object, $level, $plainText, $ansiColors);
+        if ($level < self::$maxDepth && (!self::isAlreadyRendered($object) || $plainText)) {
+            $content = self::renderContent($object, $level, $plainText, $ansiColors);
+        } else {
+            $content = '';
+        }
+        if ($plainText) {
+            return $header . $content;
+        }
+        return '<span class="extbase-debugger-tree"><input type="checkbox" /><span class="extbase-debug-header">' . $header . '</span><span class="extbase-debug-content">' . $content . '</span></span>';
     }
 
     /**
@@ -235,41 +251,43 @@ class DebuggerUtility
         if ($plainText) {
             $dump .= self::ansiEscapeWrap($className, '36', $ansiColors);
         } else {
-            $dump .= '<span class="extbase-debug-type">' . $className . '</span>';
+            $dump .= '<span class="extbase-debug-type">' . htmlspecialchars($className) . '</span>';
         }
-        if ($object instanceof \TYPO3\CMS\Core\SingletonInterface) {
-            $scope = 'singleton';
-        } else {
-            $scope = 'prototype';
-        }
-        if ($plainText) {
-            $dump .= ' ' . self::ansiEscapeWrap($scope, '44;37', $ansiColors);
-        } else {
-            $dump .= $scope ? '<span class="extbase-debug-scope">' . $scope . '</span>' : '';
-        }
-        if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject) {
-            if ($object->_isDirty()) {
-                $persistenceType = 'modified';
-            } elseif ($object->_isNew()) {
-                $persistenceType = 'transient';
+        if (!$object instanceof \Closure) {
+            if ($object instanceof \TYPO3\CMS\Core\SingletonInterface) {
+                $scope = 'singleton';
             } else {
-                $persistenceType = 'persistent';
+                $scope = 'prototype';
+            }
+            if ($plainText) {
+                $dump .= ' ' . self::ansiEscapeWrap($scope, '44;37', $ansiColors);
+            } else {
+                $dump .= $scope ? '<span class="extbase-debug-scope">' . $scope . '</span>' : '';
+            }
+            if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject) {
+                if ($object->_isDirty()) {
+                    $persistenceType = 'modified';
+                } elseif ($object->_isNew()) {
+                    $persistenceType = 'transient';
+                } else {
+                    $persistenceType = 'persistent';
+                }
+            }
+            if ($object instanceof \TYPO3\CMS\Extbase\Persistence\ObjectStorage && $object->_isDirty()) {
+                $persistenceType = 'modified';
+            }
+            if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractEntity) {
+                $domainObjectType = 'entity';
+            } elseif ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractValueObject) {
+                $domainObjectType = 'valueobject';
+            } else {
+                $domainObjectType = 'object';
+            }
+            if ($plainText) {
+                $dump .= ' ' . self::ansiEscapeWrap(($persistenceType ? $persistenceType . ' ' : '') . $domainObjectType, '42;30', $ansiColors);
+            } else {
+                $dump .= '<span class="extbase-debug-ptype">' . ($persistenceType ? $persistenceType . ' ' : '') . $domainObjectType . '</span>';
             }
-        }
-        if ($object instanceof \TYPO3\CMS\Extbase\Persistence\ObjectStorage && $object->_isDirty()) {
-            $persistenceType = 'modified';
-        }
-        if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractEntity) {
-            $domainObjectType = 'entity';
-        } elseif ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractValueObject) {
-            $domainObjectType = 'valueobject';
-        } else {
-            $domainObjectType = 'object';
-        }
-        if ($plainText) {
-            $dump .= ' ' . self::ansiEscapeWrap(($persistenceType ? $persistenceType . ' ' : '') . $domainObjectType, '42;30', $ansiColors);
-        } else {
-            $dump .= '<span class="extbase-debug-ptype">' . ($persistenceType ? $persistenceType . ' ' : '') . $domainObjectType . '</span>';
         }
         if (strpos(implode('|', self::$blacklistedClassNames), get_class($object)) > 0) {
             if ($plainText) {
@@ -279,13 +297,13 @@ class DebuggerUtility
             }
         } elseif (self::$renderedObjects->contains($object) && !$plainText) {
             $dump = '<a href="javascript:;" onclick="document.location.hash=\'#' . spl_object_hash($object) . '\';" class="extbase-debug-seeabove">' . $dump . '<span class="extbase-debug-filtered">see above</span></a>';
-        } elseif ($level >= self::$maxDepth && !$object instanceof \DateTime) {
+        } elseif ($level >= self::$maxDepth && !$object instanceof \DateTimeInterface) {
             if ($plainText) {
                 $dump .= ' ' . self::ansiEscapeWrap('max depth', '47;30', $ansiColors);
             } else {
                 $dump .= '<span class="extbase-debug-filtered">max depth</span>';
             }
-        } elseif ($level > 1 && !$object instanceof \DateTime && !$plainText) {
+        } elseif ($level > 1 && !$object instanceof \DateTimeInterface && !$plainText) {
             if (($object instanceof \Countable && empty($object)) || empty($classReflection->getProperties())) {
                 $dump = '<span>' . $dump . '</span>';
             } else {
@@ -296,8 +314,8 @@ class DebuggerUtility
             $objectCount = count($object);
             $dump .= $objectCount > 0 ? ' (' . $objectCount . ' items)' : ' (empty)';
         }
-        if ($object instanceof \DateTime) {
-            $dump .= ' (' . $object->format(\DateTime::RFC3339) . ', ' . $object->getTimestamp() . ')';
+        if ($object instanceof \DateTimeInterface) {
+            $dump .= ' (' . $object->format(\DateTimeInterface::RFC3339) . ', ' . $object->getTimestamp() . ')';
         }
         if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\DomainObjectInterface && !$object->_isNew()) {
             $dump .= ' (uid=' . $object->getUid() . ', pid=' . $object->getPid() . ')';
@@ -322,25 +340,102 @@ class DebuggerUtility
             if (!$plainText) {
                 $dump .= '<a name="' . spl_object_hash($object) . '" id="' . spl_object_hash($object) . '"></a>';
             }
-            if (get_class($object) === 'stdClass') {
-                $objReflection = new \ReflectionObject($object);
-                $properties = $objReflection->getProperties();
-            } else {
-                $classReflection = new \ReflectionClass(get_class($object));
-                $properties = $classReflection->getProperties();
-            }
-            foreach ($properties as $property) {
-                if (self::isBlacklisted($property)) {
-                    continue;
-                }
-                $dump .= PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level) . ($plainText ? '' : '<span class="extbase-debug-property">') . self::ansiEscapeWrap($property->getName(), '37', $ansiColors) . ($plainText ? '' : '</span>') . ' => ';
-                $property->setAccessible(true);
-                $dump .= self::renderDump($property->getValue($object), $level, $plainText, $ansiColors);
-                if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject && !$object->_isNew() && $object->_isDirty($property->getName())) {
+            if ($object instanceof \Closure) {
+                $dump .= PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level)
+                    . ($plainText ? '' : '<span class="extbase-debug-closure">')
+                    . self::ansiEscapeWrap('function (', '33', $ansiColors) . ($plainText ? '' : '</span>');
+
+                $reflectionFunction = new \ReflectionFunction($object);
+                $params = [];
+                foreach ($reflectionFunction->getParameters() as $parameter) {
+                    $parameterDump = '';
+                    if ($parameter->isArray()) {
+                        if ($plainText) {
+                            $parameterDump .= self::ansiEscapeWrap('array ', '36', $ansiColors);
+                        } else {
+                            $parameterDump .= '<span class="extbase-debug-type">array </span>';
+                        }
+                    } elseif ($parameter->getClass()) {
+                        if ($plainText) {
+                            $parameterDump .= self::ansiEscapeWrap($parameter->getClass()->name . ' ', '36', $ansiColors);
+                        } else {
+                            $parameterDump .= '<span class="extbase-debug-type">'
+                                . htmlspecialchars($parameter->getClass()->name) . '</span>';
+                        }
+                    }
+                    if ($parameter->isPassedByReference()) {
+                        $parameterDump .= '&';
+                    }
+                    if ($parameter->isVariadic()) {
+                        $parameterDump .= '...';
+                    }
                     if ($plainText) {
-                        $dump .= ' ' . self::ansiEscapeWrap('modified', '43;30', $ansiColors);
+                        $parameterDump .= self::ansiEscapeWrap('$' . $parameter->name, '37', $ansiColors);
                     } else {
-                        $dump .= '<span class="extbase-debug-dirty">modified</span>';
+                        $parameterDump .= '<span class="extbase-debug-property">'
+                            . htmlspecialchars('$' . $parameter->name) . '</span>';
+                    }
+                    if ($parameter->isDefaultValueAvailable()) {
+                        $parameterDump .= ' = ';
+                        if ($plainText) {
+                            $parameterDump .= self::ansiEscapeWrap(var_export($parameter->getDefaultValue(), true), '33', $ansiColors);
+                        } else {
+                            $parameterDump .= '<span class="extbase-debug-string">'
+                                . htmlspecialchars(var_export($parameter->getDefaultValue(), true)) . '</span>';
+                        }
+                    }
+                    $params[] = $parameterDump;
+                }
+                $dump .= implode(', ', $params);
+                if ($plainText) {
+                    $dump .= self::ansiEscapeWrap(') {' . PHP_EOL, '33', $ansiColors);
+                } else {
+                    $dump .= '<span class="extbase-debug-closure">) {' . PHP_EOL . '</span>';
+                }
+                $lines = file($reflectionFunction->getFileName());
+                for ($l = $reflectionFunction->getStartLine(); $l < $reflectionFunction->getEndLine() - 1; ++$l) {
+                    $dump .= $plainText ? $lines[$l] : htmlspecialchars($lines[$l]);
+                }
+                $dump .= str_repeat(self::PLAINTEXT_INDENT, $level);
+                if ($plainText) {
+                    $dump .= self::ansiEscapeWrap('}' . PHP_EOL, '33', $ansiColors);
+                } else {
+                    $dump .= '<span class="extbase-debug-closure">}</span>';
+                }
+            } else {
+                if (get_class($object) === 'stdClass') {
+                    $objReflection = new \ReflectionObject($object);
+                    $properties = $objReflection->getProperties();
+                } else {
+                    $classReflection = new \ReflectionClass(get_class($object));
+                    $properties = $classReflection->getProperties();
+                }
+                foreach ($properties as $property) {
+                    if (self::isBlacklisted($property)) {
+                        continue;
+                    }
+                    $dump .= PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level);
+                    if ($plainText) {
+                        $dump .= self::ansiEscapeWrap($property->getName(), '37', $ansiColors);
+                    } else {
+                        $dump .= '<span class="extbase-debug-property">'
+                            . htmlspecialchars($property->getName()) . '</span>';
+                    }
+                    $dump .= ' => ';
+                    $property->setAccessible(true);
+                    $visibility = ($property->isProtected() ? 'protected' : ($property->isPrivate() ? 'private' : 'public'));
+                    if ($plainText) {
+                        $dump .= self::ansiEscapeWrap($visibility, '42;30', $ansiColors) . ' ';
+                    } else {
+                        $dump .= '<span class="extbase-debug-visibility">' . $visibility . '</span>';
+                    }
+                    $dump .= self::renderDump($property->getValue($object), $level, $plainText, $ansiColors);
+                    if ($object instanceof \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject && !$object->_isNew() && $object->_isDirty($property->getName())) {
+                        if ($plainText) {
+                            $dump .= ' ' . self::ansiEscapeWrap('modified', '43;30', $ansiColors);
+                        } else {
+                            $dump .= '<span class="extbase-debug-dirty">modified</span>';
+                        }
                     }
                 }
             }
@@ -359,10 +454,16 @@ class DebuggerUtility
     {
         $dump = '';
         foreach ($collection as $key => $value) {
-            $dump .= PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level) . ($plainText ? '' : '<span class="extbase-debug-property">') . self::ansiEscapeWrap($key, '37', $ansiColors) . ($plainText ? '' : '</span>') . ' => ';
+            $dump .= PHP_EOL . str_repeat(self::PLAINTEXT_INDENT, $level);
+            if ($plainText) {
+                $dump .= self::ansiEscapeWrap($key, '37', $ansiColors);
+            } else {
+                $dump .= '<span class="extbase-debug-property">' . htmlspecialchars($key) . '</span>';
+            }
+            $dump .= ' => ';
             $dump .= self::renderDump($value, $level, $plainText, $ansiColors);
         }
-        if ($collection instanceof \Iterator) {
+        if ($collection instanceof \Iterator && !$collection instanceof \Generator) {
             $collection->rewind();
         }
         return $dump;
@@ -380,9 +481,8 @@ class DebuggerUtility
     {
         if ($enable) {
             return '[' . $ansiColors . 'm' . $string . '[0m';
-        } else {
-            return $string;
         }
+        return $string;
     }
 
     /**
@@ -397,7 +497,6 @@ class DebuggerUtility
      * @param array $blacklistedClassNames An array of class names (RegEx) to be filtered. Default is an array of some common class names.
      * @param array $blacklistedPropertyNames An array of property names and/or array keys (RegEx) to be filtered. Default is an array of some common property names.
      * @return string if $return is TRUE, the dump is returned. By default, the dump is directly displayed, and nothing is returned.
-     * @api
      */
     public static function var_dump($variable, $title = null, $maxDepth = 8, $plainText = false, $ansiColors = true, $return = false, $blacklistedClassNames = null, $blacklistedPropertyNames = null)
     {
@@ -409,9 +508,11 @@ class DebuggerUtility
         if ($ansiColors === true) {
             $title = '[1m' . $title . '[0m';
         }
+        $backupBlacklistedClassNames = self::$blacklistedClassNames;
         if (is_array($blacklistedClassNames)) {
             self::$blacklistedClassNames = $blacklistedClassNames;
         }
+        $backupBlacklistedPropertyNames = self::$blacklistedPropertyNames;
         if (is_array($blacklistedPropertyNames)) {
             self::$blacklistedPropertyNames = $blacklistedPropertyNames;
         }
@@ -435,13 +536,15 @@ class DebuggerUtility
 					.extbase-debugger-center .extbase-debug-string{color:#ce9178;white-space:normal}
 					.extbase-debugger-center .extbase-debug-type{color:#569CD6;padding-right:4px}
 					.extbase-debugger-center .extbase-debug-unregistered{background-color:#dce1e8}
-					.extbase-debugger-center .extbase-debug-filtered,.extbase-debugger-center .extbase-debug-proxy,.extbase-debugger-center .extbase-debug-ptype,.extbase-debugger-center .extbase-debug-scope{color:#fff;font-size:10px;line-height:12px;padding:2px 4px;margin-right:2px;position:relative;top:-1px}
+					.extbase-debugger-center .extbase-debug-filtered,.extbase-debugger-center .extbase-debug-proxy,.extbase-debugger-center .extbase-debug-ptype,.extbase-debugger-center .extbase-debug-visibility,.extbase-debugger-center .extbase-debug-scope{color:#fff;font-size:10px;line-height:12px;padding:2px 4px;margin-right:2px;position:relative;top:-1px}
 					.extbase-debugger-center .extbase-debug-scope{background-color:#497AA2}
 					.extbase-debugger-center .extbase-debug-ptype{background-color:#698747}
+					.extbase-debugger-center .extbase-debug-visibility{background-color:#698747}
 					.extbase-debugger-center .extbase-debug-dirty{background-color:#FFFFB6}
 					.extbase-debugger-center .extbase-debug-filtered{background-color:#4F4F4F}
 					.extbase-debugger-center .extbase-debug-seeabove{text-decoration:none;font-style:italic}
 					.extbase-debugger-center .extbase-debug-property{color:#f1f1f1}
+					.extbase-debugger-center .extbase-debug-closure{color:#9BA223;}
 				</style>';
             self::$stylesheetEchoed = true;
         }
@@ -457,11 +560,13 @@ class DebuggerUtility
 			</div>
 			';
         }
+        self::$blacklistedClassNames = $backupBlacklistedClassNames;
+        self::$blacklistedPropertyNames = $backupBlacklistedPropertyNames;
         if ($return === true) {
             return $css . $output;
-        } else {
-            echo $css . $output;
         }
+        echo $css . $output;
+
         return '';
     }
 }
