@@ -16,8 +16,6 @@ namespace TYPO3\CMS\Extbase\Validation\Validator;
 
 /**
  * Validator for email addresses
- *
- * @api
  */
 class EmailAddressValidator extends AbstractValidator
 {
@@ -25,8 +23,6 @@ class EmailAddressValidator extends AbstractValidator
      * Checks if the given value is a valid email address.
      *
      * @param mixed $value The value that should be validated
-     * @return void
-     * @api
      */
     public function isValid($value)
     {
@@ -35,7 +31,9 @@ class EmailAddressValidator extends AbstractValidator
                 $this->translateErrorMessage(
                     'validator.emailaddress.notvalid',
                     'extbase'
-                ), 1221559976);
+                ),
+                1221559976
+            );
         }
     }
 
