@@ -14,9 +14,13 @@ namespace TYPO3\CMS\Extensionmanager\Controller;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Registry;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+
 /**
  * Controller for handling extension related actions like
  * installing, removing, downloading of data or files
+ * @internal This class is a specific controller implementation and is not considered part of the Public TYPO3 API.
  */
 class ActionController extends AbstractController
 {
@@ -87,7 +91,7 @@ class ActionController extends AbstractController
             } else {
                 // install
                 $extension = $this->extensionModelUtility->mapExtensionArrayToModel(
-                    $this->installUtility->enrichExtensionWithDetails($extensionKey)
+                    $this->installUtility->enrichExtensionWithDetails($extensionKey, false)
                 );
                 if ($this->managementService->installExtension($extension) === false) {
                     $this->redirect('unresolvedDependencies', 'List', null, ['extensionKey' => $extensionKey]);
@@ -98,14 +102,16 @@ class ActionController extends AbstractController
         } catch (\TYPO3\CMS\Core\Package\Exception\PackageStatesFileNotWritableException $e) {
             $this->addFlashMessage($e->getMessage(), '', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR);
         }
-        $this->redirect('index', 'List', null, [self::TRIGGER_RefreshModuleMenu => true]);
+        $this->redirect('index', 'List', null, [
+            self::TRIGGER_RefreshModuleMenu => true,
+            self::TRIGGER_RefreshTopbar => true
+        ]);
     }
 
     /**
      * Install an extension and omit dependency checking
      *
      * @param string $extensionKey
-     * @return void
      */
     public function installExtensionWithoutSystemDependencyCheckAction($extensionKey)
     {
@@ -153,22 +159,20 @@ class ActionController extends AbstractController
     }
 
     /**
-     * Download data of an extension as sql statements
+     * Reloads the static SQL data of an extension
      *
      * @param string $extension
-     * @throws \TYPO3\CMS\Extensionmanager\Exception\ExtensionManagerException
      */
-    protected function downloadExtensionDataAction($extension)
+    protected function reloadExtensionDataAction($extension)
     {
-        $error = null;
-        $sqlData = $this->installUtility->getExtensionSqlDataDump($extension);
-        $dump = $sqlData['extTables'] . $sqlData['staticSql'];
-        $fileName = $extension . '_sqlDump.sql';
-        $filePath = PATH_site . 'typo3temp/ExtensionManager/' . $fileName;
-        $error = \TYPO3\CMS\Core\Utility\GeneralUtility::writeFileToTypo3tempDir($filePath, $dump);
-        if (is_string($error)) {
-            throw new \TYPO3\CMS\Extensionmanager\Exception\ExtensionManagerException($error, 1343048718);
-        }
-        $this->fileHandlingUtility->sendSqlDumpFileToBrowserAndDelete($filePath, $fileName);
+        $extension = $this->installUtility->enrichExtensionWithDetails($extension, false);
+        $registryKey = $extension['siteRelPath'] . 'ext_tables_static+adt.sql';
+
+        $registry = GeneralUtility::makeInstance(Registry::class);
+        $registry->remove('extensionDataImport', $registryKey);
+
+        $this->installUtility->processExtensionSetup($extension['key']);
+
+        $this->redirect('index', 'List');
     }
 }
