@@ -14,11 +14,15 @@ namespace TYPO3\CMS\Extensionmanager\Controller;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
 use TYPO3\CMS\Core\Messaging\FlashMessage;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Extbase\Mvc\View\JsonView;
 use TYPO3\CMS\Extensionmanager\Domain\Model\Extension;
 
 /**
  * Controller for actions related to the TER download of an extension
+ * @internal This class is a specific controller implementation and is not considered part of the Public TYPO3 API.
  */
 class DownloadController extends AbstractController
 {
@@ -48,9 +52,14 @@ class DownloadController extends AbstractController
     protected $downloadUtility;
 
     /**
-     * @var \TYPO3\CMS\Extensionmanager\Utility\ConfigurationUtility
+     * @var JsonView
      */
-    protected $configurationUtility;
+    protected $defaultViewObjectName = JsonView::class;
+
+    /**
+     * @var JsonView
+     */
+    protected $view;
 
     /**
      * @param \TYPO3\CMS\Extensionmanager\Domain\Repository\ExtensionRepository $extensionRepository
@@ -93,11 +102,11 @@ class DownloadController extends AbstractController
     }
 
     /**
-     * @param \TYPO3\CMS\Extensionmanager\Utility\ConfigurationUtility $configurationUtility
+     * Defines which view object should be used for the installFromTer action
      */
-    public function injectConfigurationUtility(\TYPO3\CMS\Extensionmanager\Utility\ConfigurationUtility $configurationUtility)
+    protected function initializeInstallFromTerAction()
     {
-        $this->configurationUtility = $configurationUtility;
+        $this->defaultViewObjectName = \TYPO3\CMS\Fluid\View\TemplateView::class;
     }
 
     /**
@@ -112,7 +121,17 @@ class DownloadController extends AbstractController
         $title = '';
         $hasDependencies = false;
         $hasErrors = false;
-        if ($this->configurationUtility->getCurrentConfiguration('extensionmanager')['automaticInstallation']['value']) {
+        $dependencyTypes = null;
+        $configuration = [
+            'value' => [
+                'dependencies' => [],
+            ],
+        ];
+        $isAutomaticInstallationEnabled = (bool)GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('extensionmanager', 'automaticInstallation');
+        if (!$isAutomaticInstallationEnabled) {
+            // if automatic installation is deactivated, no dependency check is needed (download only)
+            $action = 'installExtensionWithoutSystemDependencyCheck';
+        } else {
             $action = 'installFromTer';
             try {
                 $dependencyTypes = $this->managementService->getAndResolveDependencies($extension);
@@ -122,11 +141,23 @@ class DownloadController extends AbstractController
                     foreach ($dependencyTypes as $dependencyType => $dependencies) {
                         $extensions = '';
                         foreach ($dependencies as $extensionKey => $dependency) {
-                            $extensions .= $this->translate('downloadExtension.dependencies.extensionWithVersion', [
+                            if (!isset($configuration['value']['dependencies'][$dependencyType])) {
+                                $configuration['value']['dependencies'][$dependencyType] = [];
+                            }
+                            $configuration['value']['dependencies'][$dependencyType][$extensionKey] = [
+                                '_exclude' => [
+                                    'categoryIndexFromStringOrNumber',
+                                ],
+                            ];
+                            $extensions .= $this->translate(
+                                'downloadExtension.dependencies.extensionWithVersion',
+                                [
                                     $extensionKey, $dependency->getVersion()
-                                ]) . '<br />';
+                                ]
+                            ) . '<br />';
                         }
-                        $message .= $this->translate('downloadExtension.dependencies.typeHeadline',
+                        $message .= $this->translate(
+                            'downloadExtension.dependencies.typeHeadline',
                             [
                                 $this->translate('downloadExtension.dependencyType.' . $dependencyType),
                                 $extensions
@@ -135,22 +166,27 @@ class DownloadController extends AbstractController
                     }
                     $title = $this->translate('downloadExtension.dependencies.resolveAutomatically');
                 }
-                $this->view->assign('dependencies', $dependencyTypes);
             } catch (\Exception $e) {
                 $hasErrors = true;
                 $title = $this->translate('downloadExtension.dependencies.errorTitle');
                 $message = $e->getMessage();
             }
-        } else {
-            // if automatic installation is deactivated, no dependency check is needed (download only)
-            $action = 'installExtensionWithoutSystemDependencyCheck';
         }
-        $this->view->assign('extension', $extension)
-            ->assign('action', $action)
-            ->assign('hasDependencies', $hasDependencies)
-            ->assign('hasErrors', $hasErrors)
-            ->assign('message', $message)
-            ->assign('title', $title);
+
+        $url = $this->uriBuilder->uriFor(
+            $action,
+            ['extension' => $extension->getUid(), 'format' => 'json'],
+            'Download'
+        );
+        $this->view->setConfiguration($configuration);
+        $this->view->assign('value', [
+            'dependencies' => $dependencyTypes,
+            'url' => $url,
+            'message' => $message,
+            'hasErrors' => $hasErrors,
+            'hasDependencies' => $hasDependencies,
+            'title' => $title
+        ]);
     }
 
     /**
@@ -159,14 +195,14 @@ class DownloadController extends AbstractController
      * @param \TYPO3\CMS\Extensionmanager\Domain\Model\Extension $extension
      * @param string $downloadPath
      */
-    public function installFromTerAction(\TYPO3\CMS\Extensionmanager\Domain\Model\Extension $extension, $downloadPath)
+    public function installFromTerAction(\TYPO3\CMS\Extensionmanager\Domain\Model\Extension $extension, $downloadPath = 'Local')
     {
         list($result, $errorMessages) = $this->installFromTer($extension, $downloadPath);
-        $emConfiguration = $this->configurationUtility->getCurrentConfiguration('extensionmanager');
+        $isAutomaticInstallationEnabled = (bool)GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('extensionmanager', 'automaticInstallation');
         $this->view
             ->assign('result', $result)
             ->assign('extension', $extension)
-            ->assign('installationTypeLanguageKey', (bool)$emConfiguration['automaticInstallation']['value'] ? '' : '.downloadOnly')
+            ->assign('installationTypeLanguageKey', $isAutomaticInstallationEnabled ? '' : '.downloadOnly')
             ->assign('unresolvedDependencies', $errorMessages);
     }
 
@@ -187,7 +223,6 @@ class DownloadController extends AbstractController
      * redirects directly to configuration after installing
      *
      * @param \TYPO3\CMS\Extensionmanager\Domain\Model\Extension $extension
-     * @return void
      */
     public function installDistributionAction(\TYPO3\CMS\Extensionmanager\Domain\Model\Extension $extension)
     {
@@ -218,8 +253,11 @@ class DownloadController extends AbstractController
         } else {
             // FlashMessage that extension is installed
             $this->addFlashMessage(
-                \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('distribution.welcome.message', 'extensionmanager')
-                    . $extension->getExtensionKey(),
+                \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate(
+                    'distribution.welcome.message',
+                    'extensionmanager',
+                    [$extension->getExtensionKey()]
+                ),
                 \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('distribution.welcome.headline', 'extensionmanager')
             );
 
@@ -272,8 +310,6 @@ class DownloadController extends AbstractController
      * Show update comments for extensions that can be updated.
      * Fetches update comments for all versions between the current
      * installed and the highest version.
-     *
-     * @return void
      */
     protected function updateCommentForUpdatableVersionsAction()
     {
@@ -296,9 +332,14 @@ class DownloadController extends AbstractController
             }
             $updateComments[$updatableVersion->getVersion()] = $updatableVersion->getUpdateComment();
         }
-        $this->view->assign('updateComments', $updateComments)
-            ->assign('extensionKey', $extensionKey)
-            ->assign('version', $highestPossibleVersion);
+
+        $this->view->assign('value', [
+            'updateComments' => $updateComments,
+            'url' => $this->uriBuilder->uriFor(
+                'updateExtension',
+                ['extension' => $extensionKey, 'version' => $highestPossibleVersion]
+            )
+        ]);
     }
 
     /**
@@ -315,7 +356,8 @@ class DownloadController extends AbstractController
         $errorMessages = [];
         try {
             $this->downloadUtility->setDownloadPath($downloadPath);
-            $this->managementService->setAutomaticInstallationEnabled($this->configurationUtility->getCurrentConfiguration('extensionmanager')['automaticInstallation']['value']);
+            $isAutomaticInstallationEnabled = (bool)GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('extensionmanager', 'automaticInstallation');
+            $this->managementService->setAutomaticInstallationEnabled($isAutomaticInstallationEnabled);
             if (($result = $this->managementService->installExtension($extension)) === false) {
                 $errorMessages = $this->managementService->getDependencyErrors();
             }
