@@ -14,8 +14,14 @@ namespace TYPO3\CMS\Extensionmanager\Domain\Repository;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Database\Connection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Platform\PlatformInformation;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+
 /**
  * A repository for extensions
+ * @internal This class is a specific domain repository implementation and is not part of the Public TYPO3 API.
  */
 class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
 {
@@ -24,36 +30,15 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
      */
     const TABLE_NAME = 'tx_extensionmanager_domain_model_extension';
 
-    /**
-     * @var \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected $databaseConnection;
-
-    /**
-     * @var \TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper
-     */
-    protected $dataMapper;
-
-    /**
-     * @param \TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper $dataMapper
-     */
-    public function injectDataMapper(\TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper $dataMapper)
-    {
-        $this->dataMapper = $dataMapper;
-    }
-
     /**
      * Do not include pid in queries
-     *
-     * @return void
      */
     public function initializeObject()
     {
-        /** @var $defaultQuerySettings \TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface */
+        /** @var \TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface $defaultQuerySettings */
         $defaultQuerySettings = $this->objectManager->get(\TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface::class);
         $defaultQuerySettings->setRespectStoragePage(false);
         $this->setDefaultQuerySettings($defaultQuerySettings);
-        $this->databaseConnection = $GLOBALS['TYPO3_DB'];
     }
 
     /**
@@ -95,7 +80,7 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
     {
         $query = $this->createQuery();
         $query->matching($query->logicalAnd($query->equals('extensionKey', $extensionKey), $query->greaterThanOrEqual('reviewState', 0)));
-        $query->setOrderings(['version' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING]);
+        $query->setOrderings(['integerVersion' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING]);
         return $query->execute();
     }
 
@@ -149,29 +134,57 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
      */
     public function findByTitleOrAuthorNameOrExtensionKey($searchString)
     {
-        $quotedSearchString = $this->databaseConnection->escapeStrForLike($this->databaseConnection->quoteStr($searchString, 'tx_extensionmanager_domain_model_extension'), 'tx_extensionmanager_domain_model_extension');
-        $quotedSearchStringForLike = '\'%' . $quotedSearchString . '%\'';
-        $quotedSearchString = '\'' . $quotedSearchString . '\'';
-        $select =
-            self::TABLE_NAME . '.*, ' .
-            'CASE ' .
-                'WHEN extension_key = ' . $quotedSearchString . ' THEN 16 ' .
-                'WHEN extension_key LIKE ' . $quotedSearchStringForLike . ' THEN 8 ' .
-                'WHEN title LIKE ' . $quotedSearchStringForLike . ' THEN 4 ' .
-                'WHEN description LIKE ' . $quotedSearchStringForLike . ' THEN 2 ' .
-                'WHEN author_name LIKE ' . $quotedSearchStringForLike . ' THEN 1 ' .
-            'END AS position';
-        $where = '(
-					extension_key = ' . $quotedSearchString . ' OR
-					extension_key LIKE ' . $quotedSearchStringForLike . ' OR
-					title LIKE ' . $quotedSearchStringForLike . ' OR
-					description LIKE ' . $quotedSearchStringForLike . ' OR
-					author_name LIKE ' . $quotedSearchStringForLike . '
-				)
-				AND current_version = 1 AND review_state >= 0';
-        $order = 'position DESC';
-        $result = $this->databaseConnection->exec_SELECTgetRows($select, self::TABLE_NAME, $where, '', $order);
-        return $this->dataMapper->map(\TYPO3\CMS\Extensionmanager\Domain\Model\Extension::class, $result);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable(self::TABLE_NAME);
+
+        $searchPlaceholderForLike = '%' . $queryBuilder->escapeLikeWildcards($searchString) . '%';
+
+        $searchConstraints = [
+            'extension_key' => $queryBuilder->expr()->eq(
+                'extension_key',
+                $queryBuilder->createNamedParameter($searchString, \PDO::PARAM_STR)
+            ),
+            'extension_key_like' => $queryBuilder->expr()->like(
+                'extension_key',
+                $queryBuilder->createNamedParameter($searchPlaceholderForLike, \PDO::PARAM_STR)
+            ),
+            'title' => $queryBuilder->expr()->like(
+                'title',
+                $queryBuilder->createNamedParameter($searchPlaceholderForLike, \PDO::PARAM_STR)
+            ),
+            'description' => $queryBuilder->expr()->like(
+                'description',
+                $queryBuilder->createNamedParameter($searchPlaceholderForLike, \PDO::PARAM_STR)
+            ),
+            'author_name' => $queryBuilder->expr()->like(
+                'author_name',
+                $queryBuilder->createNamedParameter($searchPlaceholderForLike, \PDO::PARAM_STR)
+            ),
+        ];
+
+        $caseStatement = 'CASE ' .
+            'WHEN ' . $searchConstraints['extension_key'] . ' THEN 16 ' .
+            'WHEN ' . $searchConstraints['extension_key_like'] . ' THEN 8 ' .
+            'WHEN ' . $searchConstraints['title'] . ' THEN 4 ' .
+            'WHEN ' . $searchConstraints['description'] . ' THEN 2 ' .
+            'WHEN ' . $searchConstraints['author_name'] . ' THEN 1 ' .
+            'END AS ' . $queryBuilder->quoteIdentifier('position');
+
+        $result = $queryBuilder
+            ->select('*')
+            ->addSelectLiteral($caseStatement)
+            ->from(self::TABLE_NAME)
+            ->where(
+                $queryBuilder->expr()->orX(...array_values($searchConstraints)),
+                $queryBuilder->expr()->eq('current_version', $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->gte('review_state', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            )
+            ->orderBy('position', 'DESC')
+            ->execute()
+            ->fetchAll();
+
+        $dataMapper = $this->objectManager->get(\TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper::class);
+        return $dataMapper->map(\TYPO3\CMS\Extensionmanager\Domain\Model\Extension::class, $result);
     }
 
     /**
@@ -228,7 +241,6 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
         $query->matching(
             $query->logicalAnd(
                 $query->equals('category', \TYPO3\CMS\Extensionmanager\Domain\Model\Extension::DISTRIBUTION_CATEGORY),
-                $query->equals('currentVersion', 1),
                 $query->logicalNot($query->equals('ownerusername', 'typo3v4'))
             )
         );
@@ -251,7 +263,6 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
         $query->matching(
             $query->logicalAnd(
                 $query->equals('category', \TYPO3\CMS\Extensionmanager\Domain\Model\Extension::DISTRIBUTION_CATEGORY),
-                $query->equals('currentVersion', 1),
                 $query->equals('ownerusername', 'typo3v4')
             )
         );
@@ -308,22 +319,33 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
     /**
      * Sets current_version = 1 for all extensions where the extension version is maximal.
      *
-     * For performance reasons, the "native" TYPO3_DB is used here directly.
+     * For performance reasons, the "native" database connection is used here directly.
      *
      * @param int $repositoryUid
-     * @return void
      */
     protected function markExtensionWithMaximumVersionAsCurrent($repositoryUid)
     {
         $uidsOfCurrentVersion = $this->fetchMaximalVersionsForAllExtensions($repositoryUid);
-
-        $this->databaseConnection->exec_UPDATEquery(
-            self::TABLE_NAME,
-            'uid IN (' . implode(',', $uidsOfCurrentVersion) . ')',
-            [
-                'current_version' => 1,
-            ]
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable(self::TABLE_NAME);
+        $connection = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getConnectionForTable(self::TABLE_NAME);
+        $maxBindParameters = PlatformInformation::getMaxBindParameters(
+            $connection->getDatabasePlatform()
         );
+
+        foreach (array_chunk($uidsOfCurrentVersion, $maxBindParameters - 10) as $chunk) {
+            $queryBuilder
+                ->update(self::TABLE_NAME)
+                ->where(
+                    $queryBuilder->expr()->in(
+                        'uid',
+                        $queryBuilder->createNamedParameter($chunk, Connection::PARAM_INT_ARRAY)
+                    )
+                )
+                ->set('current_version', 1)
+                ->execute();
+        }
     }
 
     /**
@@ -336,19 +358,37 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
      */
     protected function fetchMaximalVersionsForAllExtensions($repositoryUid)
     {
-        $queryResult = $this->databaseConnection->sql_query(
-            'SELECT a.uid AS uid ' .
-            'FROM ' . self::TABLE_NAME . ' a ' .
-            'LEFT JOIN ' . self::TABLE_NAME . ' b ON a.repository = b.repository AND a.extension_key = b.extension_key AND a.integer_version < b.integer_version ' .
-            'WHERE a.repository = ' . (int)$repositoryUid . ' AND b.extension_key IS NULL ' .
-            'ORDER BY a.uid'
-        );
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable(self::TABLE_NAME);
+
+        $queryResult = $queryBuilder
+            ->select('a.uid AS uid')
+            ->from(self::TABLE_NAME, 'a')
+            ->leftJoin(
+                'a',
+                self::TABLE_NAME,
+                'b',
+                $queryBuilder->expr()->andX(
+                    $queryBuilder->expr()->eq('a.repository', $queryBuilder->quoteIdentifier('b.repository')),
+                    $queryBuilder->expr()->eq('a.extension_key', $queryBuilder->quoteIdentifier('b.extension_key')),
+                    $queryBuilder->expr()->lt('a.integer_version', $queryBuilder->quoteIdentifier('b.integer_version'))
+                )
+            )
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'a.repository',
+                    $queryBuilder->createNamedParameter($repositoryUid, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->isNull('b.extension_key')
+            )
+            ->orderBy('a.uid')
+            ->execute();
 
         $extensionUids = [];
-        while ($row = $this->databaseConnection->sql_fetch_assoc($queryResult)) {
+        while ($row = $queryResult->fetch()) {
             $extensionUids[] = $row['uid'];
         }
-        $this->databaseConnection->sql_free_result($queryResult);
+
         return $extensionUids;
     }
 
@@ -359,11 +399,18 @@ class ExtensionRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
      */
     protected function getNumberOfCurrentExtensions()
     {
-        return $this->databaseConnection->exec_SELECTcountRows(
-            '*',
-            self::TABLE_NAME,
-            'current_version = 1'
-        );
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable(self::TABLE_NAME);
+
+        return (int)$queryBuilder
+            ->count('*')
+            ->from(self::TABLE_NAME)
+            ->where($queryBuilder->expr()->eq(
+                'current_version',
+                $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT)
+            ))
+            ->execute()
+            ->fetchColumn(0);
     }
 
     /**
