@@ -22,6 +22,7 @@ use TYPO3\CMS\Extensionmanager\Exception;
 
 /**
  * Utility for dealing with dependencies
+ * @internal This class is a specific ExtensionManager implementation and is not part of the Public TYPO3 API.
  */
 class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
 {
@@ -112,7 +113,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
 
     /**
      * @param string $localExtensionStorage
-     * @return void
      */
     public function setLocalExtensionStorage($localExtensionStorage)
     {
@@ -122,8 +122,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
     /**
      * Setter for available extensions
      * gets available extensions from list utility if not already done
-     *
-     * @return void
      */
     protected function setAvailableExtensions()
     {
@@ -132,7 +130,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
 
     /**
      * @param bool $skipDependencyCheck
-     * @return void
      */
     public function setSkipDependencyCheck($skipDependencyCheck)
     {
@@ -143,7 +140,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Checks dependencies for special cases (currently typo3 and php)
      *
      * @param Extension $extension
-     * @return void
      */
     public function checkDependencies(Extension $extension)
     {
@@ -246,7 +242,7 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
             if (!($dependency->getLowestVersion() === '') && version_compare(PHP_VERSION, $dependency->getLowestVersion()) === -1) {
                 throw new Exception\UnresolvedPhpDependencyException(
                     'Your PHP version is lower than necessary. You need at least PHP version ' . $dependency->getLowestVersion(),
-                     1377977857
+                    1377977857
                 );
             }
             if (!($dependency->getHighestVersion() === '') && version_compare($dependency->getHighestVersion(), PHP_VERSION) === -1) {
@@ -335,6 +331,8 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
                                 1430562374
                             );
                         }
+                        // Dependency check is skipped and the local version has to be installed
+                        $this->managementService->markExtensionForInstallation($extensionKey);
                     }
                 }
             } else {
@@ -353,7 +351,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $extensionKey
      * @param Dependency $dependency
-     * @return void
      * @throws Exception\UnresolvedDependencyException
      */
     protected function getExtensionFromRepository($extensionKey, Dependency $dependency)
@@ -389,7 +386,6 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
      * @param string $extensionKey
      * @param Dependency $dependency
      * @throws Exception\UnresolvedDependencyException
-     * @return void
      */
     protected function getExtensionFromTer($extensionKey, Dependency $dependency)
     {
@@ -401,12 +397,11 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
                         'The extension ' . $extensionKey . ' is not available from TER.',
                         1399161266
                     );
-                } else {
-                    throw new Exception\MissingExtensionDependencyException(
-                        'The extension ' . $extensionKey . ' could not be checked. Please update your Extension-List from TYPO3 Extension Repository (TER).',
-                        1430580308
-                    );
                 }
+                throw new Exception\MissingExtensionDependencyException(
+                    'The extension ' . $extensionKey . ' could not be checked. Please update your Extension-List from TYPO3 Extension Repository (TER).',
+                    1430580308
+                );
             }
             return;
         }
@@ -522,7 +517,9 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
     {
         $versions = $this->getLowestAndHighestIntegerVersions($dependency);
         $count = $this->extensionRepository->countByVersionRangeAndExtensionKey(
-            $dependency->getIdentifier(), $versions['lowestIntegerVersion'], $versions['highestIntegerVersion']
+            $dependency->getIdentifier(),
+            $versions['lowestIntegerVersion'],
+            $versions['highestIntegerVersion']
         );
         return !empty($count);
     }
@@ -597,7 +594,7 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
                 if ($dependency->getIdentifier() === 'typo3') {
                     try {
                         if ($this->checkTypo3Dependency($dependency)) {
-                            array_push($suitableExtensions, $extension);
+                            $suitableExtensions[] = $extension;
                         }
                     } catch (Exception\UnresolvedTypo3DependencyException $e) {
                     }
@@ -607,4 +604,35 @@ class DependencyUtility implements \TYPO3\CMS\Core\SingletonInterface
         }
         return $suitableExtensions;
     }
+
+    /**
+     * Gets a list of various extensions in various versions and returns
+     * a filtered list containing the extension-version combination with
+     * the highest version number.
+     *
+     * @param Extension[] $extensions
+     * @param bool $showUnsuitable
+     *
+     * @return \TYPO3\CMS\Extensionmanager\Domain\Model\Extension[]
+     */
+    public function filterYoungestVersionOfExtensionList(array $extensions, $showUnsuitable)
+    {
+        if (!$showUnsuitable) {
+            $extensions = $this->getExtensionsSuitableForTypo3Version($extensions);
+        }
+        $filteredExtensions = [];
+        foreach ($extensions as $extension) {
+            $extensionKey = $extension->getExtensionKey();
+            if (!array_key_exists($extensionKey, $filteredExtensions)) {
+                $filteredExtensions[$extensionKey] = $extension;
+                continue;
+            }
+            $currentVersion = $filteredExtensions[$extensionKey]->getVersion();
+            $newVersion = $extension->getVersion();
+            if (version_compare($newVersion, $currentVersion, '>')) {
+                $filteredExtensions[$extensionKey] = $extension;
+            }
+        }
+        return $filteredExtensions;
+    }
 }
