@@ -13,20 +13,22 @@ namespace TYPO3\CMS\Extensionmanager\Utility\Importer;
  *
  * The TYPO3 project - inspiring people to share!
  */
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Platform\PlatformInformation;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Utility\VersionNumberUtility;
+use TYPO3\CMS\Extensionmanager\Utility\Parser\AbstractExtensionXmlParser;
 
-/**
- * Module: Extension manager - Extension list importer
- */
 /**
  * Importer object for extension list
- * @since 2010-02-10
+ * @internal This class is a specific ExtensionManager implementation and is not part of the Public TYPO3 API.
  */
 class ExtensionListUtility implements \SplObserver
 {
     /**
      * Keeps instance of a XML parser.
      *
-     * @var \TYPO3\CMS\Extensionmanager\Utility\Parser\AbstractExtensionXmlParser
+     * @var AbstractExtensionXmlParser
      */
     protected $parser;
 
@@ -72,6 +74,21 @@ class ExtensionListUtility implements \SplObserver
         'update_comment'
     ];
 
+    /**
+     * Table name to be used to store extension models.
+     *
+     * @var string
+     */
+    protected static $tableName = 'tx_extensionmanager_domain_model_extension';
+
+    /**
+     * Maximum of rows that can be used in a bulk insert for the current
+     * database platform.
+     *
+     * @var int
+     */
+    protected $maxRowsPerChunk = 50;
+
     /**
      * Keeps indexes of fields that should not be quoted.
      *
@@ -103,6 +120,19 @@ class ExtensionListUtility implements \SplObserver
      */
     protected $extensionModel;
 
+    /**
+     * @var \TYPO3\CMS\Extbase\Object\ObjectManager
+     */
+    protected $objectManager;
+
+    /**
+     * Only import extensions newer than this date (timestamp),
+     * see constructor
+     *
+     * @var int
+     */
+    protected $minimumDateToImport;
+
     /**
      * Class constructor.
      *
@@ -112,7 +142,7 @@ class ExtensionListUtility implements \SplObserver
      */
     public function __construct()
     {
-        /** @var $objectManager \TYPO3\CMS\Extbase\Object\ObjectManager */
+        /** @var \TYPO3\CMS\Extbase\Object\ObjectManager $objectManager */
         $this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Extbase\Object\ObjectManager::class);
         $this->repositoryRepository = $this->objectManager->get(\TYPO3\CMS\Extensionmanager\Domain\Repository\RepositoryRepository::class);
         $this->extensionRepository = $this->objectManager->get(\TYPO3\CMS\Extensionmanager\Domain\Repository\ExtensionRepository::class);
@@ -122,8 +152,33 @@ class ExtensionListUtility implements \SplObserver
         if (is_object($this->parser)) {
             $this->parser->attach($this);
         } else {
-            throw new \TYPO3\CMS\Extensionmanager\Exception\ExtensionManagerException(get_class($this) . ': No XML parser available.');
+            throw new \TYPO3\CMS\Extensionmanager\Exception\ExtensionManagerException(
+                static::class . ': No XML parser available.',
+                1476108717
+            );
         }
+
+        $connection = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getConnectionForTable(self::$tableName);
+        $maxBindParameters = PlatformInformation::getMaxBindParameters(
+            $connection->getDatabasePlatform()
+        );
+        $countOfBindParamsPerRow = count(self::$fieldNames);
+        // flush at least chunks of 50 elements - in case the currently used
+        // database platform does not support that, the threshold is lowered
+        $this->maxRowsPerChunk = min(
+            $this->maxRowsPerChunk,
+            floor($maxBindParameters / $countOfBindParamsPerRow)
+        );
+        // Only import extensions that are compatible with 7.6 or higher.
+        // TER only allows to publish extensions with compatibility if the TYPO3 version has been released
+        // And 7.6 was released on 10th of November 2015.
+        // This effectively reduces the number of extensions imported into this TYPO3 installation
+        // by more than 70%. As long as the extensions.xml from TER includes these files, we need to "hack" this
+        // within TYPO3 Core.
+        // For TYPO3 v10.0, this date could be set to 2017-04-04 (8 LTS release).
+        // Also see https://decisions.typo3.org/t/reduce-size-of-extension-manager-db-table/329/
+        $this->minimumDateToImport = strtotime('2015-11-10T00:00:00+00:00');
     }
 
     /**
@@ -135,15 +190,21 @@ class ExtensionListUtility implements \SplObserver
      */
     public function import($localExtensionListFile, $repositoryUid = null)
     {
-        if (!is_null($repositoryUid) && is_int($repositoryUid)) {
+        if ($repositoryUid !== null && is_int($repositoryUid)) {
             $this->repositoryUid = $repositoryUid;
         }
         $zlibStream = 'compress.zlib://';
         $this->sumRecords = 0;
-        $this->parser->parseXML($zlibStream . $localExtensionListFile);
+        $this->parser->parseXml($zlibStream . $localExtensionListFile);
         // flush last rows to database if existing
         if (!empty($this->arrRows)) {
-            $GLOBALS['TYPO3_DB']->exec_INSERTmultipleRows('tx_extensionmanager_domain_model_extension', self::$fieldNames, $this->arrRows, self::$fieldIndicesNoQuote);
+            GeneralUtility::makeInstance(ConnectionPool::class)
+                ->getConnectionForTable('tx_extensionmanager_domain_model_extension')
+                ->bulkInsert(
+                    'tx_extensionmanager_domain_model_extension',
+                    $this->arrRows,
+                    self::$fieldNames
+                );
         }
         $extensions = $this->extensionRepository->insertLastVersion($this->repositoryUid);
         $this->repositoryRepository->updateRepositoryCount($extensions, $this->repositoryUid);
@@ -153,17 +214,21 @@ class ExtensionListUtility implements \SplObserver
     /**
      * Method collects and stores extension version details into the database.
      *
-     * @param \SplSubject|\TYPO3\CMS\Extensionmanager\Utility\Parser\AbstractExtensionXmlParser &$subject a subject notifying this observer
-     * @return void
+     * @param AbstractExtensionXmlParser $subject a subject notifying this observer
      */
-    protected function loadIntoDatabase(\SplSubject &$subject)
+    protected function loadIntoDatabase(AbstractExtensionXmlParser &$subject)
     {
-        // flush every 50 rows to database
-        if ($this->sumRecords !== 0 && $this->sumRecords % 50 === 0) {
-            $GLOBALS['TYPO3_DB']->exec_INSERTmultipleRows('tx_extensionmanager_domain_model_extension', self::$fieldNames, $this->arrRows, self::$fieldIndicesNoQuote);
+        if ($this->sumRecords !== 0 && $this->sumRecords % $this->maxRowsPerChunk === 0) {
+            GeneralUtility::makeInstance(ConnectionPool::class)
+                ->getConnectionForTable(self::$tableName)
+                ->bulkInsert(
+                    self::$tableName,
+                    $this->arrRows,
+                    self::$fieldNames
+                );
             $this->arrRows = [];
         }
-        $versionRepresentations = \TYPO3\CMS\Core\Utility\VersionNumberUtility::convertVersionStringToArray($subject->getVersion());
+        $versionRepresentations = VersionNumberUtility::convertVersionStringToArray($subject->getVersion());
         // order must match that of self::$fieldNames!
         $this->arrRows[] = [
             $subject->getExtkey(),
@@ -173,11 +238,11 @@ class ExtensionListUtility implements \SplObserver
             0,
             (int)$subject->getAlldownloadcounter(),
             (int)$subject->getDownloadcounter(),
-            !is_null($subject->getTitle()) ? $subject->getTitle() : '',
+            $subject->getTitle() !== null ? $subject->getTitle() : '',
             $subject->getOwnerusername(),
-            !is_null($subject->getAuthorname()) ? $subject->getAuthorname() : '',
-            !is_null($subject->getAuthoremail()) ? $subject->getAuthoremail() : '',
-            !is_null($subject->getAuthorcompany()) ? $subject->getAuthorcompany() : '',
+            $subject->getAuthorname() !== null ? $subject->getAuthorname() : '',
+            $subject->getAuthoremail() !== null ? $subject->getAuthoremail() : '',
+            $subject->getAuthorcompany() !== null ? $subject->getAuthorcompany() : '',
             (int)$subject->getLastuploaddate(),
             $subject->getT3xfilemd5(),
             $this->repositoryUid,
@@ -195,12 +260,13 @@ class ExtensionListUtility implements \SplObserver
      * Method receives an update from a subject.
      *
      * @param \SplSubject $subject a subject notifying this observer
-     * @return void
      */
     public function update(\SplSubject $subject)
     {
-        if (is_subclass_of($subject, \TYPO3\CMS\Extensionmanager\Utility\Parser\AbstractExtensionXmlParser::class)) {
-            $this->loadIntoDatabase($subject);
+        if (is_subclass_of($subject, AbstractExtensionXmlParser::class)) {
+            if ((int)$subject->getLastuploaddate() > $this->minimumDateToImport) {
+                $this->loadIntoDatabase($subject);
+            }
         }
     }
 }
