@@ -14,6 +14,10 @@ namespace TYPO3\CMS\Extensionmanager\Utility;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
+use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Core\Database\Schema\SchemaMigrator;
+use TYPO3\CMS\Core\Database\Schema\SqlReader;
 use TYPO3\CMS\Core\Service\OpcodeCacheService;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Extensionmanager\Domain\Model\Extension;
@@ -22,6 +26,7 @@ use TYPO3\CMS\Impexp\Utility\ImportExportUtility;
 
 /**
  * Extension Manager Install Utility
+ * @internal This class is a specific ExtensionManager implementation and is not part of the Public TYPO3 API.
  */
 class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
 {
@@ -30,11 +35,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      */
     public $objectManager;
 
-    /**
-     * @var \TYPO3\CMS\Install\Service\SqlSchemaMigrationService
-     */
-    public $installToolSqlParser;
-
     /**
      * @var \TYPO3\CMS\Extensionmanager\Utility\DependencyUtility
      */
@@ -50,11 +50,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      */
     protected $listUtility;
 
-    /**
-     * @var \TYPO3\CMS\Extensionmanager\Utility\DatabaseUtility
-     */
-    protected $databaseUtility;
-
     /**
      * @var \TYPO3\CMS\Extensionmanager\Domain\Repository\ExtensionRepository
      */
@@ -88,14 +83,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
         $this->objectManager = $objectManager;
     }
 
-    /**
-     * @param \TYPO3\CMS\Install\Service\SqlSchemaMigrationService $installToolSqlParser
-     */
-    public function injectInstallToolSqlParser(\TYPO3\CMS\Install\Service\SqlSchemaMigrationService $installToolSqlParser)
-    {
-        $this->installToolSqlParser = $installToolSqlParser;
-    }
-
     /**
      * @param \TYPO3\CMS\Extensionmanager\Utility\DependencyUtility $dependencyUtility
      */
@@ -120,14 +107,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
         $this->listUtility = $listUtility;
     }
 
-    /**
-     * @param \TYPO3\CMS\Extensionmanager\Utility\DatabaseUtility $databaseUtility
-     */
-    public function injectDatabaseUtility(\TYPO3\CMS\Extensionmanager\Utility\DatabaseUtility $databaseUtility)
-    {
-        $this->databaseUtility = $databaseUtility;
-    }
-
     /**
      * @param \TYPO3\CMS\Extensionmanager\Domain\Repository\ExtensionRepository $extensionRepository
      */
@@ -172,24 +151,33 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Helper function to install an extension
      * also processes db updates and clears the cache if the extension asks for it
      *
-     * @param string $extensionKey
+     * @param array $extensionKeys
      * @throws ExtensionManagerException
-     * @return void
      */
-    public function install($extensionKey)
+    public function install(...$extensionKeys)
     {
-        $extension = $this->enrichExtensionWithDetails($extensionKey);
-        $this->ensureConfiguredDirectoriesExist($extension);
-        $this->loadExtension($extensionKey);
-        if (!empty($extension['clearcacheonload']) || !empty($extension['clearCacheOnLoad'])) {
+        $flushCaches = false;
+        foreach ($extensionKeys as $extensionKey) {
+            $this->loadExtension($extensionKey);
+            $extension = $this->enrichExtensionWithDetails($extensionKey, false);
+            $this->saveDefaultConfiguration($extensionKey);
+            if (!empty($extension['clearcacheonload']) || !empty($extension['clearCacheOnLoad'])) {
+                $flushCaches = true;
+            }
+        }
+
+        if ($flushCaches) {
             $this->cacheManager->flushCaches();
         } else {
             $this->cacheManager->flushCachesInGroup('system');
         }
         $this->reloadCaches();
-        $this->processExtensionSetup($extensionKey);
+        $this->updateDatabase($extensionKeys);
 
-        $this->emitAfterExtensionInstallSignal($extensionKey);
+        foreach ($extensionKeys as $extensionKey) {
+            $this->processExtensionSetup($extensionKey);
+            $this->emitAfterExtensionInstallSignal($extensionKey);
+        }
     }
 
     /**
@@ -197,11 +185,11 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function processExtensionSetup($extensionKey)
     {
-        $extension = $this->getExtensionArray($extensionKey);
-        $this->importInitialFiles($extension['siteRelPath'], $extensionKey);
-        $this->processDatabaseUpdates($extension);
-        $this->processRuntimeDatabaseUpdates($extensionKey);
-        $this->saveDefaultConfiguration($extensionKey);
+        $extension = $this->enrichExtensionWithDetails($extensionKey, false);
+        $this->ensureConfiguredDirectoriesExist($extension);
+        $this->importInitialFiles($extension['siteRelPath'] ?? '', $extensionKey);
+        $this->importStaticSqlFile($extension['siteRelPath']);
+        $this->importT3DFile($extension['siteRelPath']);
     }
 
     /**
@@ -209,7 +197,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $extensionKey
      * @throws ExtensionManagerException
-     * @return void
      */
     public function uninstall($extensionKey)
     {
@@ -223,9 +210,8 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
                 ),
                 1342554622
             );
-        } else {
-            $this->unloadExtension($extensionKey);
         }
+        $this->unloadExtension($extensionKey);
     }
 
     /**
@@ -251,7 +237,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Wrapper function for loading extensions
      *
      * @param string $extensionKey
-     * @return void
      */
     protected function loadExtension($extensionKey)
     {
@@ -262,7 +247,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Wrapper function for unloading extensions
      *
      * @param string $extensionKey
-     * @return void
      */
     protected function unloadExtension($extensionKey)
     {
@@ -323,14 +307,19 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Fetch additional information for an extension key
      *
      * @param string $extensionKey
-     * @access private
+     * @param bool $loadTerInformation
+     * @internal
      * @return array
      * @throws ExtensionManagerException
      */
-    public function enrichExtensionWithDetails($extensionKey)
+    public function enrichExtensionWithDetails($extensionKey, $loadTerInformation = true)
     {
         $extension = $this->getExtensionArray($extensionKey);
-        $availableAndInstalledExtensions = $this->listUtility->enrichExtensionsWithEmConfAndTerInformation([$extensionKey => $extension]);
+        if (!$loadTerInformation) {
+            $availableAndInstalledExtensions = $this->listUtility->enrichExtensionsWithEmConfInformation([$extensionKey => $extension]);
+        } else {
+            $availableAndInstalledExtensions = $this->listUtility->enrichExtensionsWithEmConfAndTerInformation([$extensionKey => $extension]);
+        }
 
         if (!isset($availableAndInstalledExtensions[$extensionKey])) {
             throw new ExtensionManagerException(
@@ -352,9 +341,8 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
         $availableExtensions = $this->listUtility->getAvailableExtensions();
         if (isset($availableExtensions[$extensionKey])) {
             return $availableExtensions[$extensionKey];
-        } else {
-            throw new ExtensionManagerException('Extension ' . $extensionKey . ' is not available', 1342864081);
         }
+        throw new ExtensionManagerException('Extension ' . $extensionKey . ' is not available', 1342864081);
     }
 
     /**
@@ -372,42 +360,39 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Additionally adds the table definitions for the cache tables
      *
      * @param array $extension
+     * @deprecated since TYPO3 v9, will be removed with TYPO3v10
      */
     public function processDatabaseUpdates(array $extension)
     {
-        $extTablesSqlFile = PATH_site . $extension['siteRelPath'] . 'ext_tables.sql';
+        trigger_error('This method will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
+        $extTablesSqlFile = Environment::getPublicPath() . '/' . $extension['siteRelPath'] . 'ext_tables.sql';
         $extTablesSqlContent = '';
         if (file_exists($extTablesSqlFile)) {
-            $extTablesSqlContent .= GeneralUtility::getUrl($extTablesSqlFile);
+            $extTablesSqlContent .= file_get_contents($extTablesSqlFile);
         }
         if ($extTablesSqlContent !== '') {
-            $this->updateDbWithExtTablesSql($extTablesSqlContent);
+            try {
+                $this->updateDbWithExtTablesSql($extTablesSqlContent);
+            } catch (\TYPO3\CMS\Core\Database\Schema\Exception\StatementException $e) {
+                throw new ExtensionManagerException(
+                    $e->getMessage(),
+                    1476340371
+                );
+            }
         }
 
         $this->importStaticSqlFile($extension['siteRelPath']);
         $this->importT3DFile($extension['siteRelPath']);
     }
 
-    /**
-     * Gets all database updates due to runtime configuration, like caching framework or
-     * category api for example
-     *
-     * @param string $extensionKey
-     */
-    protected function processRuntimeDatabaseUpdates($extensionKey)
-    {
-        $sqlString = $this->emitTablesDefinitionIsBeingBuiltSignal($extensionKey);
-        if (!empty($sqlString)) {
-            $this->updateDbWithExtTablesSql(implode(LF . LF . LF . LF, $sqlString));
-        }
-    }
-
     /**
      * Emits a signal to manipulate the tables definitions
      *
      * @param string $extensionKey
-     * @return mixed
      * @throws ExtensionManagerException
+     * @return mixed
+     * @deprecated since TYPO3 v9, will be removed with TYPO3v10
+     * @see \TYPO3\CMS\Core\Database\Schema\SqlReader::emitTablesDefinitionIsBeingBuiltSignal
      */
     protected function emitTablesDefinitionIsBeingBuiltSignal($extensionKey)
     {
@@ -426,19 +411,28 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
                 1382360258
             );
         }
+        if (!empty($sqlString)) {
+            trigger_error(
+                sprintf(
+                    'The signal %s of class %s is deprecated and will be removed in TYPO3 v10.0.',
+                    'tablesDefinitionIsBeingBuilt',
+                    __CLASS__
+                ),
+                E_USER_DEPRECATED
+            );
+        }
         return $sqlString;
     }
 
     /**
      * Reload Cache files and Typo3LoadedExtensions
-     *
-     * @return void
      */
     public function reloadCaches()
     {
         $this->reloadOpcache();
         \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::loadExtLocalconf(false);
-        \TYPO3\CMS\Core\Core\Bootstrap::getInstance()->loadExtensionTables(false);
+        \TYPO3\CMS\Core\Core\Bootstrap::loadBaseTca(false);
+        \TYPO3\CMS\Core\Core\Bootstrap::loadExtTables(false);
     }
 
     /**
@@ -449,42 +443,64 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
         GeneralUtility::makeInstance(OpcodeCacheService::class)->clearAllActive();
     }
 
+    /**
+     * Executes all safe database statements.
+     * Tables and fields are created and altered. Nothing gets deleted or renamed here.
+     *
+     * @param array $extensionKeys
+     */
+    protected function updateDatabase(array $extensionKeys)
+    {
+        $sqlReader = GeneralUtility::makeInstance(SqlReader::class);
+        $schemaMigrator = GeneralUtility::makeInstance(SchemaMigrator::class);
+        $sqlStatements = [];
+        $sqlStatements[] = $sqlReader->getTablesDefinitionString();
+        foreach ($extensionKeys as $extensionKey) {
+            $sqlStatements += $this->emitTablesDefinitionIsBeingBuiltSignal($extensionKey);
+        }
+        $sqlStatements = $sqlReader->getCreateTableStatementArray(implode(LF . LF, array_filter($sqlStatements)));
+        $updateStatements = $schemaMigrator->getUpdateSuggestions($sqlStatements);
+
+        $updateStatements = array_merge_recursive(...array_values($updateStatements));
+        $selectedStatements = [];
+        foreach (['add', 'change', 'create_table', 'change_table'] as $action) {
+            if (empty($updateStatements[$action])) {
+                continue;
+            }
+            $selectedStatements = array_merge(
+                $selectedStatements,
+                array_combine(array_keys($updateStatements[$action]), array_fill(0, count($updateStatements[$action]), true))
+            );
+        }
+
+        $schemaMigrator->migrate($sqlStatements, $selectedStatements);
+    }
+
     /**
      * Save default configuration of an extension
      *
      * @param string $extensionKey
-     * @return void
      */
     protected function saveDefaultConfiguration($extensionKey)
     {
-        /** @var $configUtility \TYPO3\CMS\Extensionmanager\Utility\ConfigurationUtility */
-        $configUtility = $this->objectManager->get(\TYPO3\CMS\Extensionmanager\Utility\ConfigurationUtility::class);
-        $configUtility->saveDefaultConfiguration($extensionKey);
+        $extensionConfiguration = $this->objectManager->get(ExtensionConfiguration::class);
+        $extensionConfiguration->synchronizeExtConfTemplateWithLocalConfiguration($extensionKey);
     }
 
     /**
      * Update database / process db updates from ext_tables
      *
      * @param string $rawDefinitions The raw SQL statements from ext_tables.sql
-     * @return void
+     * @deprecated since TYPO3 v9, will be removed with TYPO3v10
      */
     public function updateDbWithExtTablesSql($rawDefinitions)
     {
-        $fieldDefinitionsFromFile = $this->installToolSqlParser->getFieldDefinitions_fileContent($rawDefinitions);
-        if (!empty($fieldDefinitionsFromFile)) {
-            $fieldDefinitionsFromCurrentDatabase = $this->installToolSqlParser->getFieldDefinitions_database();
-            $diff = $this->installToolSqlParser->getDatabaseExtra($fieldDefinitionsFromFile, $fieldDefinitionsFromCurrentDatabase);
-            $updateStatements = $this->installToolSqlParser->getUpdateSuggestions($diff);
-            $db = $this->getDatabaseConnection();
-            foreach ((array)$updateStatements['add'] as $string) {
-                $db->admin_query($string);
-            }
-            foreach ((array)$updateStatements['change'] as $string) {
-                $db->admin_query($string);
-            }
-            foreach ((array)$updateStatements['create_table'] as $string) {
-                $db->admin_query($string);
-            }
+        trigger_error('This method will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
+        $sqlReader = GeneralUtility::makeInstance(SqlReader::class);
+        $statements = $sqlReader->getCreateTableStatementArray($rawDefinitions);
+        if (count($statements) !== 0) {
+            $schemaMigrationService = GeneralUtility::makeInstance(SchemaMigrator::class);
+            $schemaMigrationService->install($statements);
         }
     }
 
@@ -492,24 +508,14 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Import static SQL data (normally used for ext_tables_static+adt.sql)
      *
      * @param string $rawDefinitions
-     * @return void
      */
     public function importStaticSql($rawDefinitions)
     {
-        $statements = $this->installToolSqlParser->getStatementarray($rawDefinitions, 1);
-        list($statementsPerTable, $insertCount) = $this->installToolSqlParser->getCreateTables($statements, 1);
-        $db = $this->getDatabaseConnection();
-        // Traverse the tables
-        foreach ($statementsPerTable as $table => $query) {
-            $db->admin_query('DROP TABLE IF EXISTS ' . $table);
-            $db->admin_query($query);
-            if ($insertCount[$table]) {
-                $insertStatements = $this->installToolSqlParser->getTableInsertStatements($statements, $table);
-                foreach ($insertStatements as $statement) {
-                    $db->admin_query($statement);
-                }
-            }
-        }
+        $sqlReader = GeneralUtility::makeInstance(SqlReader::class);
+        $statements = $sqlReader->getStatementArray($rawDefinitions);
+
+        $schemaMigrationService = GeneralUtility::makeInstance(SchemaMigrator::class);
+        $schemaMigrationService->importStaticData($statements, true);
     }
 
     /**
@@ -517,7 +523,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $extension
      * @throws ExtensionManagerException
-     * @return void
      */
     public function removeExtension($extension)
     {
@@ -535,38 +540,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
         }
     }
 
-    /**
-     * Get the data dump for an extension
-     *
-     * @param string $extension
-     * @return array
-     */
-    public function getExtensionSqlDataDump($extension)
-    {
-        $extension = $this->enrichExtensionWithDetails($extension);
-        $filePrefix = PATH_site . $extension['siteRelPath'];
-        $sqlData['extTables'] = $this->getSqlDataDumpForFile($filePrefix . 'ext_tables.sql');
-        $sqlData['staticSql'] = $this->getSqlDataDumpForFile($filePrefix . 'ext_tables_static+adt.sql');
-        return $sqlData;
-    }
-
-    /**
-     * Gets the sql data dump for a specific sql file (for example ext_tables.sql)
-     *
-     * @param string $sqlFile
-     * @return string
-     */
-    protected function getSqlDataDumpForFile($sqlFile)
-    {
-        $sqlData = '';
-        if (file_exists($sqlFile)) {
-            $sqlContent = GeneralUtility::getUrl($sqlFile);
-            $fieldDefinitions = $this->installToolSqlParser->getFieldDefinitions_fileContent($sqlContent);
-            $sqlData = $this->databaseUtility->dumpStaticTables($fieldDefinitions);
-        }
-        return $sqlData;
-    }
-
     /**
      * Checks if an update for an extension is available which also resolves dependencies.
      *
@@ -617,7 +590,6 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Execution state is saved in the this->registry, so it only happens once
      *
      * @param string $extensionSiteRelPath
-     * @return void
      */
     protected function importT3DFile($extensionSiteRelPath)
     {
@@ -637,7 +609,7 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
             $extensionSiteRelPath . 'Initialisation/data.xml'
         ];
         foreach ($possibleImportFiles as $possibleImportFile) {
-            if (!file_exists(PATH_site . $possibleImportFile)) {
+            if (!file_exists(Environment::getPublicPath() . '/' . $possibleImportFile)) {
                 continue;
             }
             $importFileToUse = $possibleImportFile;
@@ -646,11 +618,10 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
             /** @var ImportExportUtility $importExportUtility */
             $importExportUtility = $this->objectManager->get(ImportExportUtility::class);
             try {
-                $importResult = $importExportUtility->importT3DFile(PATH_site . $importFileToUse, 0);
+                $importResult = $importExportUtility->importT3DFile(Environment::getPublicPath() . '/' . $importFileToUse, 0);
                 $this->registry->set('extensionDataImport', $extensionSiteRelPath . 'Initialisation/dataImported', 1);
                 $this->emitAfterExtensionT3DImportSignal($importFileToUse, $importResult);
             } catch (\ErrorException $e) {
-                /** @var \TYPO3\CMS\Core\Log\Logger $logger */
                 $logger = $this->objectManager->get(\TYPO3\CMS\Core\Log\LogManager::class)->getLogger(__CLASS__);
                 $logger->log(\TYPO3\CMS\Core\Log\LogLevel::WARNING, $e->getMessage());
             }
@@ -673,18 +644,19 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
      * Execution state is saved in the this->registry, so it only happens once
      *
      * @param string $extensionSiteRelPath
-     * @return void
      */
     protected function importStaticSqlFile($extensionSiteRelPath)
     {
         $extTablesStaticSqlRelFile = $extensionSiteRelPath . 'ext_tables_static+adt.sql';
         if (!$this->registry->get('extensionDataImport', $extTablesStaticSqlRelFile)) {
-            $extTablesStaticSqlFile = PATH_site . $extTablesStaticSqlRelFile;
+            $extTablesStaticSqlFile = Environment::getPublicPath() . '/' . $extTablesStaticSqlRelFile;
+            $shortFileHash = '';
             if (file_exists($extTablesStaticSqlFile)) {
-                $extTablesStaticSqlContent = GeneralUtility::getUrl($extTablesStaticSqlFile);
+                $extTablesStaticSqlContent = file_get_contents($extTablesStaticSqlFile);
+                $shortFileHash = md5($extTablesStaticSqlContent);
                 $this->importStaticSql($extTablesStaticSqlContent);
             }
-            $this->registry->set('extensionDataImport', $extTablesStaticSqlRelFile, 1);
+            $this->registry->set('extensionDataImport', $extTablesStaticSqlRelFile, $shortFileHash);
             $this->emitAfterExtensionStaticSqlImportSignal($extTablesStaticSqlRelFile);
         }
     }
@@ -710,10 +682,10 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
     {
         $importRelFolder = $extensionSiteRelPath . 'Initialisation/Files';
         if (!$this->registry->get('extensionDataImport', $importRelFolder)) {
-            $importFolder = PATH_site . $importRelFolder;
+            $importFolder = Environment::getPublicPath() . '/' . $importRelFolder;
             if (file_exists($importFolder)) {
                 $destinationRelPath = $GLOBALS['TYPO3_CONF_VARS']['BE']['fileadminDir'] . $extensionKey;
-                $destinationAbsolutePath = PATH_site . $destinationRelPath;
+                $destinationAbsolutePath = Environment::getPublicPath() . '/' . $destinationRelPath;
                 if (!file_exists($destinationAbsolutePath) &&
                     GeneralUtility::isAllowedAbsPath($destinationAbsolutePath)
                 ) {
@@ -735,12 +707,4 @@ class InstallUtility implements \TYPO3\CMS\Core\SingletonInterface
     {
         $this->signalSlotDispatcher->dispatch(__CLASS__, 'afterExtensionFileImport', [$destinationAbsolutePath, $this]);
     }
-
-    /**
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
 }
