@@ -7,13 +7,13 @@ $EM_CONF[$_EXTKEY] = [
     'uploadfolder' => 0,
     'createDirs' => '',
     'clearCacheOnLoad' => 0,
-    'author' => '',
-    'author_email' => '',
+    'author' => 'TYPO3 Core Team',
+    'author_email' => 'typo3cms@typo3.org',
     'author_company' => '',
-    'version' => '7.6.0',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'typo3' => '7.6.0-7.6.99',
+            'typo3' => '9.5.14',
         ],
         'conflicts' => [],
         'suggests' => [],
