@@ -3,17 +3,17 @@ $EM_CONF[$_EXTKEY] = [
     'title' => 'Frontend Login for Website Users',
     'description' => 'A template-based plugin to log in Website Users in the Frontend',
     'category' => 'plugin',
-    'author' => 'Steffen Kamper',
-    'author_email' => 'info@sk-typo3.de',
+    'author' => 'TYPO3 Core Team',
+    'author_email' => 'typo3cms@typo3.org',
     'author_company' => '',
     'state' => 'stable',
-    'uploadfolder' => 1,
+    'uploadfolder' => 0,
     'createDirs' => '',
     'clearCacheOnLoad' => 1,
-    'version' => '7.6.0',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'typo3' => '7.6.0-7.6.99',
+            'typo3' => '9.5.14',
         ],
         'conflicts' => [],
         'suggests' => [],
