@@ -3,14 +3,16 @@ $EM_CONF[$_EXTKEY] = [
     'title' => 'Advanced file metadata',
     'description' => 'Add advanced metadata to File.',
     'category' => 'misc',
+    'author' => 'TYPO3 Core Team',
+    'author_email' => 'typo3cms@typo3.org',
     'state' => 'stable',
-    'uploadfolder' => '0',
+    'uploadfolder' => 0,
     'createDirs' => '',
     'clearCacheOnLoad' => 0,
-    'version' => '7.6.0',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'typo3' => '7.6.0-7.6.99',
+            'typo3' => '9.5.14',
         ],
         'conflicts' => [],
         'suggests' => [],
