@@ -1,30 +1,34 @@
 <?php
 namespace TYPO3\CMS\Fluid\ViewHelpers\Form;
 
-/*                                                                        *
- * This script is backported from the TYPO3 Flow package "TYPO3.Fluid".   *
- *                                                                        *
- * It is free software; you can redistribute it and/or modify it under    *
- * the terms of the GNU Lesser General Public License, either version 3   *
- *  of the License, or (at your option) any later version.                *
- *                                                                        *
- * The TYPO3 project - inspiring people to share!                         *
- *                                                                        */
+/*
+ * This file is part of the TYPO3 CMS project.
+ *
+ * It is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, either version 2
+ * of the License, or any later version.
+ *
+ * For the full copyright and license information, please read the
+ * LICENSE.txt file that was distributed with this source code.
+ *
+ * The TYPO3 project - inspiring people to share!
+ */
 
 /**
- * Textarea view helper.
- * The value of the text area needs to be set via the "value" attribute, as with all other form ViewHelpers.
+ * Generates an :html:`<textarea>`.
  *
- * = Examples =
+ * The value of the text area needs to be set via the ``value`` attribute, as with all other form ViewHelpers.
  *
- * <code title="Example">
- * <f:form.textarea name="myTextArea" value="This is shown inside the textarea" />
- * </code>
- * <output>
- * <textarea name="myTextArea">This is shown inside the textarea</textarea>
- * </output>
+ * Examples
+ * ========
  *
- * @api
+ * Example::
+ *
+ *    <f:form.textarea name="myTextArea" value="This is shown inside the textarea" />
+ *
+ * Output::
+ *
+ *    <textarea name="myTextArea">This is shown inside the textarea</textarea>
  */
 class TextareaViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Form\AbstractFormFieldViewHelper
 {
@@ -35,9 +39,6 @@ class TextareaViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Form\AbstractFormF
 
     /**
      * Initialize the arguments.
-     *
-     * @return void
-     * @api
      */
     public function initializeArguments()
     {
@@ -47,7 +48,9 @@ class TextareaViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Form\AbstractFormF
         $this->registerTagAttribute('cols', 'int', 'The number of columns of a text area');
         $this->registerTagAttribute('disabled', 'string', 'Specifies that the input element should be disabled when the page loads');
         $this->registerTagAttribute('placeholder', 'string', 'The placeholder of the textarea');
-        $this->registerArgument('errorClass', 'string', 'CSS class to set if there are errors for this view helper', false, 'f3-form-error');
+        $this->registerArgument('errorClass', 'string', 'CSS class to set if there are errors for this ViewHelper', false, 'f3-form-error');
+        $this->registerTagAttribute('readonly', 'string', 'The readonly attribute of the textarea', false);
+        $this->registerArgument('required', 'bool', 'Specifies whether the textarea is required', false, false);
         $this->registerUniversalTagAttributes();
     }
 
@@ -55,18 +58,20 @@ class TextareaViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Form\AbstractFormF
      * Renders the textarea.
      *
      * @return string
-     * @api
      */
     public function render()
     {
+        $required = $this->arguments['required'] ?? false;
         $name = $this->getName();
         $this->registerFieldNameForFormTokenGeneration($name);
         $this->setRespectSubmittedDataValue(true);
 
         $this->tag->forceClosingTag(true);
         $this->tag->addAttribute('name', $name);
+        if ($required === true) {
+            $this->tag->addAttribute('required', 'required');
+        }
         $this->tag->setContent(htmlspecialchars($this->getValueAttribute()));
-
         $this->addAdditionalIdentityPropertiesIfNeeded();
         $this->setErrorClassAttribute();
 
