@@ -1,25 +1,18 @@
 <?php
 namespace TYPO3\CMS\Fluid\ViewHelpers\Widget\Controller;
 
-/*                                                                        *
- * This script is backported from the TYPO3 Flow package "TYPO3.Fluid".   *
- *                                                                        *
- * It is free software; you can redistribute it and/or modify it under    *
- * the terms of the GNU Lesser General Public License, either version 3   *
- *  of the License, or (at your option) any later version.                *
- *                                                                        *
- *                                                                        *
- * This script is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHAN-    *
- * TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser       *
- * General Public License for more details.                               *
- *                                                                        *
- * You should have received a copy of the GNU Lesser General Public       *
- * License along with the script.                                         *
- * If not, see http://www.gnu.org/licenses/lgpl.html                      *
- *                                                                        *
- * The TYPO3 project - inspiring people to share!                         *
- *                                                                        */
+/*
+ * This file is part of the TYPO3 CMS project.
+ *
+ * It is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, either version 2
+ * of the License, or any later version.
+ *
+ * For the full copyright and license information, please read the
+ * LICENSE.txt file that was distributed with this source code.
+ *
+ * The TYPO3 project - inspiring people to share!
+ */
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
 use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
@@ -65,15 +58,15 @@ class PaginateController extends AbstractWidgetController
     /**
      * @var int
      */
-    protected $displayRangeStart = null;
+    protected $displayRangeStart;
 
     /**
      * @var int
      */
-    protected $displayRangeEnd = null;
+    protected $displayRangeEnd;
 
     /**
-     * @return void
+     * Initializes the current information on which page the visitor is.
      */
     public function initializeAction()
     {
@@ -86,7 +79,6 @@ class PaginateController extends AbstractWidgetController
 
     /**
      * @param int $currentPage
-     * @return void
      */
     public function indexAction($currentPage = 1)
     {
@@ -102,8 +94,11 @@ class PaginateController extends AbstractWidgetController
             // modify query
             $itemsPerPage = (int)$this->configuration['itemsPerPage'];
             $offset = 0;
+            if ($this->objects instanceof QueryResultInterface) {
+                $offset = (int)$this->objects->getQuery()->getOffset();
+            }
             if ($this->currentPage > 1) {
-                $offset = ((int)($itemsPerPage * ($this->currentPage - 1)));
+                $offset = $offset + ((int)($itemsPerPage * ($this->currentPage - 1)));
             }
             $modifiedObjects = $this->prepareObjectsSlice($itemsPerPage, $offset);
         }
@@ -117,8 +112,6 @@ class PaginateController extends AbstractWidgetController
     /**
      * If a certain number of links should be displayed, adjust before and after
      * amounts accordingly.
-     *
-     * @return void
      */
     protected function calculateDisplayRange()
     {
@@ -180,28 +173,38 @@ class PaginateController extends AbstractWidgetController
     protected function prepareObjectsSlice($itemsPerPage, $offset)
     {
         if ($this->objects instanceof QueryResultInterface) {
+            $currentRange = $offset + $itemsPerPage;
+            $endOfRange = min($currentRange, count($this->objects));
             $query = $this->objects->getQuery();
             $query->setLimit($itemsPerPage);
             if ($offset > 0) {
                 $query->setOffset($offset);
+                if ($currentRange > $endOfRange) {
+                    $newLimit = $endOfRange - $offset;
+                    $query->setLimit($newLimit);
+                }
             }
             $modifiedObjects = $query->execute();
             return $modifiedObjects;
-        } elseif ($this->objects instanceof ObjectStorage) {
+        }
+        if ($this->objects instanceof ObjectStorage) {
             $modifiedObjects = [];
-            $endOfRange = $offset + $itemsPerPage;
+            $objectArray = $this->objects->toArray();
+            $endOfRange = min($offset + $itemsPerPage, count($objectArray));
             for ($i = $offset; $i < $endOfRange; $i++) {
-                $modifiedObjects[] = $this->objects->toArray()[$i];
+                $modifiedObjects[] = $objectArray[$i];
             }
             return $modifiedObjects;
-        } elseif (is_array($this->objects)) {
+        }
+        if (is_array($this->objects)) {
             $modifiedObjects = array_slice($this->objects, $offset, $itemsPerPage);
             return $modifiedObjects;
-        } else {
-            throw new \InvalidArgumentException('The view helper "' . get_class($this)
-                . '" accepts as argument "QueryResultInterface", "\SplObjectStorage", "ObjectStorage" or an array. '
-                . 'given: ' . get_class($this->objects), 1385547291
-            );
         }
+        throw new \InvalidArgumentException(
+            'The ViewHelper "' . static::class
+                . '" accepts as argument "QueryResultInterface", "\SplObjectStorage", "ObjectStorage" or an array. '
+                . 'given: ' . get_class($this->objects),
+            1385547291
+        );
     }
 }
