@@ -3,17 +3,18 @@ $EM_CONF[$_EXTKEY] = [
     'title' => 'Fluid Templating Engine',
     'description' => 'Fluid is a next-generation templating engine which makes the life of extension authors a lot easier!',
     'category' => 'fe',
-    'author' => 'Sebastian Kurfürst, Bastian Waidelich',
-    'author_email' => 'sebastian@typo3.org, bastian@typo3.org',
+    'author' => 'TYPO3 Core Team',
+    'author_email' => 'typo3cms@typo3.org',
     'author_company' => '',
     'state' => 'stable',
     'uploadfolder' => 0,
     'createDirs' => '',
     'clearCacheOnLoad' => 0,
-    'version' => '7.6.0',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'extbase' => '7.6.0-7.6.99',
+            'core' => '9.5.14',
+            'extbase' => '9.5.14',
         ],
         'conflicts' => [],
         'suggests' => [],
