@@ -5,11 +5,13 @@ $EM_CONF[$_EXTKEY] = [
     'category' => 'fe',
     'state' => 'stable',
     'author' => 'TYPO3 Core Team',
-    'author_email' => 'info@typo3.org',
-    'version' => '7.6.0',
+    'author_email' => 'typo3cms@typo3.org',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'typo3' => '7.6.0-7.6.99',
+            'typo3' => '9.5.14',
+            'fluid' => '9.5.14',
+            'frontend' => '9.5.14',
         ],
         'conflicts' => [],
         'suggests' => [],
