@@ -14,11 +14,18 @@ namespace TYPO3\CMS\Frontend\Category\Collection;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Context\Context;
+use TYPO3\CMS\Core\Database\Connection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Query\Restriction\FrontendRestrictionContainer;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Frontend\Page\PageRepository;
 
 /**
  * Extend category collection for the frontend, to collect related records
  * while respecting language, enable fields, etc.
+ *
+ * @internal this is a concrete TYPO3 hook implementation and solely used for EXT:frontend and not part of TYPO3's Core API.
  */
 class CategoryCollection extends \TYPO3\CMS\Core\Category\Collection\CategoryCollection
 {
@@ -34,8 +41,9 @@ class CategoryCollection extends \TYPO3\CMS\Core\Category\Collection\CategoryCol
      */
     public static function create(array $collectionRecord, $fillItems = false)
     {
-        /** @var $collection \TYPO3\CMS\Frontend\Category\Collection\CategoryCollection */
-        $collection = GeneralUtility::makeInstance(__CLASS__,
+        /** @var \TYPO3\CMS\Frontend\Category\Collection\CategoryCollection $collection */
+        $collection = GeneralUtility::makeInstance(
+            __CLASS__,
             $collectionRecord['table_name'],
             $collectionRecord['field_name']
         );
@@ -62,13 +70,26 @@ class CategoryCollection extends \TYPO3\CMS\Core\Category\Collection\CategoryCol
      */
     public static function load($id, $fillItems = false, $tableName = '', $fieldName = '')
     {
-        $collectionRecord = self::getDatabaseConnection()->exec_SELECTgetSingleRow(
-            '*',
-            static::$storageTableName,
-            'uid = ' . (int)$id . self::getTypoScriptFrontendController()->sys_page->enableFields(static::$storageTableName)
-        );
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable(static::$storageTableName);
+        $queryBuilder->setRestrictions(GeneralUtility::makeInstance(FrontendRestrictionContainer::class));
+
+        $collectionRecord = $queryBuilder
+            ->select('*')
+            ->from(static::$storageTableName)
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'uid',
+                    $queryBuilder->createNamedParameter($id, \PDO::PARAM_INT)
+                )
+            )
+            ->setMaxResults(1)
+            ->execute()
+            ->fetch();
+
         $collectionRecord['table_name'] = $tableName;
         $collectionRecord['field_name'] = $fieldName;
+
         return self::create($collectionRecord, $fillItems);
     }
 
@@ -84,95 +105,75 @@ class CategoryCollection extends \TYPO3\CMS\Core\Category\Collection\CategoryCol
      */
     protected function getCollectedRecords()
     {
-        $db = self::getDatabaseConnection();
-
         $relatedRecords = [];
-        // Assemble where clause
-        $where = 'AND ' . self::$storageTableName . '.uid = ' . (int)$this->getIdentifier();
-        // Add condition on tablenames fields
-        $where .= ' AND sys_category_record_mm.tablenames = ' . $db->fullQuoteStr(
-            $this->getItemTableName(),
-            'sys_category_record_mm'
-        );
-        // Add condition on fieldname field
-        $where .= ' AND sys_category_record_mm.fieldname = ' . $db->fullQuoteStr(
-            $this->getRelationFieldName(),
-            'sys_category_record_mm'
-        );
-        // Add enable fields for item table
-        $tsfe = self::getTypoScriptFrontendController();
-        $where .= $tsfe->sys_page->enableFields($this->getItemTableName());
+
+        $queryBuilder = $this->getCollectedRecordsQueryBuilder();
+        $queryBuilder->setRestrictions(GeneralUtility::makeInstance(FrontendRestrictionContainer::class));
+        $context = GeneralUtility::makeInstance(Context::class);
+        $pageRepository = GeneralUtility::makeInstance(PageRepository::class, $context);
+        $languageId = $context->getPropertyFromAspect('language', 'contentId', 0);
+
         // If language handling is defined for item table, add language condition
         if (isset($GLOBALS['TCA'][$this->getItemTableName()]['ctrl']['languageField'])) {
             // Consider default or "all" language
-            $languageField = $this->getItemTableName() . '.' . $GLOBALS['TCA'][$this->getItemTableName()]['ctrl']['languageField'];
-            $languageCondition = $languageField . ' IN (0, -1)';
-            // If not in default language, also consider items in current language with no original
-            if ($tsfe->sys_language_content > 0) {
-                $languageCondition .= '
-					OR (' . $languageField . ' = ' . (int)$tsfe->sys_language_content . '
-					AND ' . $this->getItemTableName() . '.' .
-                    $GLOBALS['TCA'][$this->getItemTableName()]['ctrl']['transOrigPointerField'] . ' = 0)
-				';
-            }
-            $where .= ' AND (' . $languageCondition . ')';
-        }
-        // Get the related records from the database
-        $resource = $db->exec_SELECT_mm_query(
-            $this->getItemTableName() . '.*',
-            self::$storageTableName,
-            'sys_category_record_mm',
-            $this->getItemTableName(),
-            $where
-        );
+            $languageField = sprintf(
+                '%s.%s',
+                $this->getItemTableName(),
+                $GLOBALS['TCA'][$this->getItemTableName()]['ctrl']['languageField']
+            );
 
-        if ($resource) {
-            while ($record = $db->sql_fetch_assoc($resource)) {
-                // Overlay the record for workspaces
-                $tsfe->sys_page->versionOL(
+            $languageConstraint = $queryBuilder->expr()->in(
+                $languageField,
+                $queryBuilder->createNamedParameter([0, -1], Connection::PARAM_INT_ARRAY)
+            );
+
+            // If not in default language, also consider items in current language with no original
+            if ($languageId > 0) {
+                $transOrigPointerField = sprintf(
+                    '%s.%s',
                     $this->getItemTableName(),
-                    $record
+                    $GLOBALS['TCA'][$this->getItemTableName()]['ctrl']['transOrigPointerField']
+                );
+
+                $languageConstraint = $queryBuilder->expr()->orX(
+                    $languageConstraint,
+                    $queryBuilder->expr()->andX(
+                        $queryBuilder->expr()->eq(
+                            $languageField,
+                            $queryBuilder->createNamedParameter($languageId, \PDO::PARAM_INT)
+                        ),
+                        $queryBuilder->expr()->eq(
+                            $transOrigPointerField,
+                            $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
+                        )
+                    )
                 );
-                // Overlay the record for translations
-                if (is_array($record) && $tsfe->sys_language_contentOL) {
-                    if ($this->getItemTableName() === 'pages') {
-                        $record = $tsfe->sys_page->getPageOverlay($record);
-                    } else {
-                        $record = $tsfe->sys_page->getRecordOverlay(
-                            $this->getItemTableName(),
-                            $record,
-                            $tsfe->sys_language_content,
-                            $tsfe->sys_language_contentOL
-                        );
-                    }
-                }
-                // Record may have been unset during the overlay process
-                if (is_array($record)) {
-                    $relatedRecords[] = $record;
-                }
             }
-            $db->sql_free_result($resource);
+
+            $queryBuilder->andWhere($languageConstraint);
         }
+
+        // Get the related records from the database
+        $result = $queryBuilder->execute();
+
+        while ($record = $result->fetch()) {
+            // Overlay the record for workspaces
+            $pageRepository->versionOL(
+                $this->getItemTableName(),
+                $record
+            );
+
+            // Overlay the record for translations
+            if (is_array($record)) {
+                $record = $pageRepository->getLanguageOverlay($this->getItemTableName(), $record);
+            }
+
+            // Record may have been unset during the overlay process
+            if (is_array($record)) {
+                $relatedRecords[] = $record;
+            }
+        }
+
         return $relatedRecords;
     }
-
-    /**
-     * Gets the TSFE object.
-     *
-     * @return \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController
-     */
-    protected static function getTypoScriptFrontendController()
-    {
-        return $GLOBALS['TSFE'];
-    }
-
-    /**
-     * Returns the database connection
-     *
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected static function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
 }
