@@ -14,15 +14,17 @@ namespace TYPO3\CMS\Frontend\Imaging;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Imaging\GraphicalFunctions;
 use TYPO3\CMS\Core\Resource\File;
 use TYPO3\CMS\Core\Resource\ProcessedFile;
-use TYPO3\CMS\Core\TypoScript\TemplateService;
-use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
+use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\File\BasicFileUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
+use TYPO3\CMS\Core\Utility\PathUtility;
 use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
+use TYPO3\CMS\Frontend\Resource\FilePathSanitizer;
 
 /**
  * GIFBUILDER
@@ -33,7 +35,7 @@ use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
  * This class allows for advanced rendering of images with various layers of images, text and graphical primitives.
  * The concept is known from TypoScript as "GIFBUILDER" where you can define a "numerical array" (TypoScript term as well) of "GIFBUILDER OBJECTS" (like "TEXT", "IMAGE", etc.) and they will be rendered onto an image one by one.
  * The name "GIFBUILDER" comes from the time where GIF was the only file format supported. PNG is just as well to create today (configured with TYPO3_CONF_VARS[GFX])
- * Not all instances of this class is truely building gif/png files by layers; You may also see the class instantiated for the purpose of using the scaling functions in the parent class.
+ * Not all instances of this class is truly building gif/png files by layers; You may also see the class instantiated for the purpose of using the scaling functions in the parent class.
  *
  * Here is an example of how to use this class (from tslib_content.php, function getImgResource):
  *
@@ -48,46 +50,6 @@ use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
  */
 class GifBuilder extends GraphicalFunctions
 {
-    /**
-     * the main image
-     *
-     * @var string
-     */
-    public $im = '';
-
-    /**
-     * the image-width
-     *
-     * @var int
-     */
-    public $w = 0;
-
-    /**
-     * the image-height
-     *
-     * @var int
-     */
-    public $h = 0;
-
-    /**
-     * map-data
-     *
-     * @var string
-     */
-    public $map;
-
-    /**
-     * @var array
-     */
-    public $workArea;
-
-    /**
-     * This holds the operational setup for gifbuilder. Basically this is a TypoScript array with properties.
-     *
-     * @var array
-     */
-    public $setup = [];
-
     /**
      * Contains all text strings used on this image
      *
@@ -129,11 +91,6 @@ class GifBuilder extends GraphicalFunctions
      */
     public $XY = [];
 
-    /**
-     * @var array
-     */
-    public $OFFSET = [];
-
     /**
      * @var ContentObjectRenderer
      */
@@ -152,7 +109,6 @@ class GifBuilder extends GraphicalFunctions
      *
      * @param array $conf TypoScript properties for the GIFBUILDER session. Stored internally in the variable ->setup
      * @param array $data The current data record from \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer. Stored internally in the variable ->data
-     * @return void
      * @see ContentObjectRenderer::getImgResource(), \TYPO3\CMS\Frontend\ContentObject\Menu\GraphicalMenuContentObject::makeGifs(), \TYPO3\CMS\Frontend\ContentObject\Menu\GraphicalMenuContentObject::findLargestDims()
      */
     public function start($conf, $data)
@@ -168,11 +124,9 @@ class GifBuilder extends GraphicalFunctions
             // Let's you pre-process the gifbuilder configuration. for
             // example you can split a string up into lines and render each
             // line as TEXT obj, see extension julle_gifbconf
-            if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_gifbuilder.php']['gifbuilder-ConfPreProcess'])) {
-                foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_gifbuilder.php']['gifbuilder-ConfPreProcess'] as $_funcRef) {
-                    $_params = $this->setup;
-                    $this->setup = GeneralUtility::callUserFunction($_funcRef, $_params, $this);
-                }
+            foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_gifbuilder.php']['gifbuilder-ConfPreProcess'] ?? [] as $_funcRef) {
+                $_params = $this->setup;
+                $this->setup = GeneralUtility::callUserFunction($_funcRef, $_params, $this);
             }
             // Initializing global Char Range Map
             $this->charRangeMap = [];
@@ -184,13 +138,13 @@ class GifBuilder extends GraphicalFunctions
                         $this->charRangeMap[$cRMkey] = [];
                         $this->charRangeMap[$cRMkey]['charMapConfig'] = $cRMcfg['charMapConfig.'];
                         $this->charRangeMap[$cRMkey]['cfgKey'] = substr($cRMcfgkey, 0, -1);
-                        $this->charRangeMap[$cRMkey]['multiplicator'] = (double) $cRMcfg['fontSizeMultiplicator'];
+                        $this->charRangeMap[$cRMkey]['multiplicator'] = (double)$cRMcfg['fontSizeMultiplicator'];
                         $this->charRangeMap[$cRMkey]['pixelSpace'] = (int)$cRMcfg['pixelSpaceFontSizeRef'];
                     }
                 }
             }
             // Getting sorted list of TypoScript keys from setup.
-            $sKeyArray = TemplateService::sortedKeyList($this->setup);
+            $sKeyArray = ArrayUtility::filterAndSortByNumericKeys($this->setup);
             // Setting the background color, passing it through stdWrap
             if ($conf['backColor.'] || $conf['backColor']) {
                 $this->setup['backColor'] = isset($this->setup['backColor.']) ? trim($this->cObj->stdWrap($this->setup['backColor'], $this->setup['backColor.'])) : $this->setup['backColor'];
@@ -238,21 +192,28 @@ class GifBuilder extends GraphicalFunctions
                         case 'IMAGE':
                             $fileInfo = $this->getResource($conf['file'], $conf['file.']);
                             if ($fileInfo) {
-                                $this->combinedFileNames[] = preg_replace('/\\.[[:alnum:]]+$/', '', basename($fileInfo[3]));
+                                $this->combinedFileNames[] = preg_replace('/\\.[[:alnum:]]+$/', '', PathUtility::basename($fileInfo[3]));
                                 if ($fileInfo['processedFile'] instanceof ProcessedFile) {
                                     // Use processed file, if a FAL file has been processed by GIFBUILDER (e.g. scaled/cropped)
                                     $this->setup[$theKey . '.']['file'] = $fileInfo['processedFile']->getForLocalProcessing(false);
                                 } elseif (!isset($fileInfo['origFile']) && $fileInfo['originalFile'] instanceof File) {
                                     // Use FAL file with getForLocalProcessing to circumvent problems with umlauts, if it is a FAL file (origFile not set)
-                                    /** @var $originalFile File */
+                                    /** @var File $originalFile */
                                     $originalFile = $fileInfo['originalFile'];
                                     $this->setup[$theKey . '.']['file'] = $originalFile->getForLocalProcessing(false);
                                 } else {
                                     // Use normal path from fileInfo if it is a non-FAL file (even non-FAL files have originalFile set, but only non-FAL files have origFile set)
                                     $this->setup[$theKey . '.']['file'] = $fileInfo[3];
                                 }
-                                $this->setup[$theKey . '.']['BBOX'] = $fileInfo;
-                                $this->objBB[$theKey] = $fileInfo;
+
+                                // only pass necessary parts of fileInfo further down, to not incorporate facts as
+                                // CropScaleMask runs in this request, that may not occur in subsequent calls and change
+                                // the md5 of the generated file name
+                                $essentialFileInfo = $fileInfo;
+                                unset($essentialFileInfo['originalFile'], $essentialFileInfo['processedFile']);
+
+                                $this->setup[$theKey . '.']['BBOX'] = $essentialFileInfo;
+                                $this->objBB[$theKey] = $essentialFileInfo;
                                 if ($conf['mask']) {
                                     $maskInfo = $this->getResource($conf['mask'], $conf['mask.']);
                                     if ($maskInfo) {
@@ -260,7 +221,7 @@ class GifBuilder extends GraphicalFunctions
                                         if ($maskInfo['processedFile'] instanceof ProcessedFile) {
                                             $this->setup[$theKey . '.']['mask'] = $maskInfo['processedFile']->getForLocalProcessing(false);
                                         } elseif (!isset($maskInfo['origFile']) && $maskInfo['originalFile'] instanceof File) {
-                                            /** @var $originalFile File */
+                                            /** @var File $originalFile */
                                             $originalFile = $maskInfo['originalFile'];
                                             $this->setup[$theKey . '.']['mask'] = $originalFile->getForLocalProcessing(false);
                                         } else {
@@ -299,7 +260,7 @@ class GifBuilder extends GraphicalFunctions
             $this->setup['workArea'] = $this->calcOffset($this->setup['workArea']);
             foreach ($sKeyArray as $theKey) {
                 $theValue = $this->setup[$theKey];
-                if ((int)$theKey && ($conf = $this->setup[$theKey . '.'])) {
+                if ((int)$theKey && $this->setup[$theKey . '.']) {
                     switch ($theValue) {
                         case 'TEXT':
 
@@ -379,7 +340,7 @@ class GifBuilder extends GraphicalFunctions
 
     /**
      * Initiates the image file generation if ->setup is TRUE and if the file did not exist already.
-     * Gets filename from fileName() and if file exists in typo3temp/ dir it will - of course - not be rendered again.
+     * Gets filename from fileName() and if file exists in typo3temp/assets/images/ dir it will - of course - not be rendered again.
      * Otherwise rendering means calling ->make(), then ->output(), then ->destroy()
      *
      * @return string The filename for the created GIF/PNG file. The filename will be prefixed "GB_
@@ -388,12 +349,12 @@ class GifBuilder extends GraphicalFunctions
     public function gifBuild()
     {
         if ($this->setup) {
-            // Relative to PATH_site
-            $gifFileName = $this->fileName('GB/');
+            // Relative to Environment::getPublicPath()
+            $gifFileName = $this->fileName('assets/images/');
             // File exists
             if (!file_exists($gifFileName)) {
                 // Create temporary directory if not done:
-                $this->createTempSubDir('GB/');
+                GeneralUtility::mkdir_deep(Environment::getPublicPath() . '/typo3temp/assets/images/');
                 // Create file:
                 $this->make();
                 $this->output($gifFileName);
@@ -410,8 +371,7 @@ class GifBuilder extends GraphicalFunctions
      * Creates a GDlib resource in $this->im and works on that
      * Called by gifBuild()
      *
-     * @return void
-     * @access private
+     * @internal
      * @see gifBuild()
      */
     public function make()
@@ -425,7 +385,7 @@ class GifBuilder extends GraphicalFunctions
         $this->w = $XY[0];
         $this->h = $XY[1];
         // Transparent layer as background if set and requirements are met
-        if (!empty($this->setup['backColor']) && $this->setup['backColor'] === 'transparent' && $this->png_truecolor && !$this->setup['reduceColors'] && (empty($this->setup['format']) || $this->setup['format'] === 'png')) {
+        if (!empty($this->setup['backColor']) && $this->setup['backColor'] === 'transparent' && !$this->setup['reduceColors'] && (empty($this->setup['format']) || $this->setup['format'] === 'png')) {
             // Set transparency properties
             imagesavealpha($this->im, true);
             // Fill with a transparent background
@@ -439,12 +399,12 @@ class GifBuilder extends GraphicalFunctions
         } else {
             // Fill the background with the given color
             $BGcols = $this->convertColor($this->setup['backColor']);
-            $Bcolor = ImageColorAllocate($this->im, $BGcols[0], $BGcols[1], $BGcols[2]);
-            ImageFilledRectangle($this->im, 0, 0, $XY[0], $XY[1], $Bcolor);
+            $Bcolor = imagecolorallocate($this->im, $BGcols[0], $BGcols[1], $BGcols[2]);
+            imagefilledrectangle($this->im, 0, 0, $XY[0], $XY[1], $Bcolor);
         }
         // Traverse the GIFBUILDER objects an render each one:
         if (is_array($this->setup)) {
-            $sKeyArray = TemplateService::sortedKeyList($this->setup);
+            $sKeyArray = ArrayUtility::filterAndSortByNumericKeys($this->setup);
             foreach ($sKeyArray as $theKey) {
                 $theValue = $this->setup[$theKey];
                 if ((int)$theKey && ($conf = $this->setup[$theKey . '.'])) {
@@ -510,17 +470,17 @@ class GifBuilder extends GraphicalFunctions
                             }
                             break;
                         case 'OUTLINE':
-                            if ($this->setup[$conf['textObjNum']] == 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
+                            if ($this->setup[$conf['textObjNum']] === 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
                                 $this->makeOutline($this->im, $conf, $this->workArea, $txtConf);
                             }
                             break;
                         case 'EMBOSS':
-                            if ($this->setup[$conf['textObjNum']] == 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
+                            if ($this->setup[$conf['textObjNum']] === 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
                                 $this->makeEmboss($this->im, $conf, $this->workArea, $txtConf);
                             }
                             break;
                         case 'SHADOW':
-                            if ($this->setup[$conf['textObjNum']] == 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
+                            if ($this->setup[$conf['textObjNum']] === 'TEXT' && ($txtConf = $this->checkTextObj($this->setup[$conf['textObjNum'] . '.']))) {
                                 $this->makeShadow($this->im, $conf, $this->workArea, $txtConf);
                             }
                             break;
@@ -560,7 +520,7 @@ class GifBuilder extends GraphicalFunctions
         if (!$this->saveAlphaLayer) {
             if ($this->setup['transparentBackground']) {
                 // Auto transparent background is set
-                $Bcolor = ImageColorClosest($this->im, $BGcols[0], $BGcols[1], $BGcols[2]);
+                $Bcolor = imagecolorclosest($this->im, $BGcols[0], $BGcols[1], $BGcols[2]);
                 imagecolortransparent($this->im, $Bcolor);
             } elseif (is_array($this->setup['transparentColor_array'])) {
                 // Multiple transparent colors are set. This is done via the trick that all transparent colors get
@@ -588,7 +548,7 @@ class GifBuilder extends GraphicalFunctions
      *
      * @param array $conf GIFBUILDER object TypoScript properties
      * @return array Modified $conf array IF the "text" property is not blank
-     * @access private
+     * @internal
      */
     public function checkTextObj($conf)
     {
@@ -602,9 +562,12 @@ class GifBuilder extends GraphicalFunctions
                 $isStdWrapped[$parameter] = 1;
             }
         }
-        $conf['fontFile'] = $this->checkFile($conf['fontFile']);
+
+        if (!is_null($conf['fontFile'])) {
+            $conf['fontFile'] = $this->checkFile($conf['fontFile']);
+        }
         if (!$conf['fontFile']) {
-            $conf['fontFile'] = ExtensionManagementUtility::siteRelPath('core') . 'Resources/Private/Font/nimbus.ttf';
+            $conf['fontFile'] = $this->checkFile('EXT:core/Resources/Private/Font/nimbus.ttf');
         }
         if (!$conf['iterations']) {
             $conf['iterations'] = 1;
@@ -628,15 +591,11 @@ class GifBuilder extends GraphicalFunctions
         // Max length = 100 if automatic line braks are not defined:
         if (!isset($conf['breakWidth']) || !$conf['breakWidth']) {
             $tlen = (int)$conf['textMaxLength'] ?: 100;
-            if ($this->nativeCharset) {
-                $conf['text'] = $this->csConvObj->substr($this->nativeCharset, $conf['text'], 0, $tlen);
-            } else {
-                $conf['text'] = substr($conf['text'], 0, $tlen);
-            }
+            $conf['text'] = mb_substr($conf['text'], 0, $tlen, 'utf-8');
         }
         if ((string)$conf['text'] != '') {
             // Char range map thingie:
-            $fontBaseName = basename($conf['fontFile']);
+            $fontBaseName = PathUtility::basename($conf['fontFile']);
             if (is_array($this->charRangeMap[$fontBaseName])) {
                 // Initialize splitRendering array:
                 if (!is_array($conf['splitRendering.'])) {
@@ -686,7 +645,7 @@ class GifBuilder extends GraphicalFunctions
      *
      * @param string $string The string to resolve/calculate the result of. The string is divided by a comma first and each resulting part is calculated into an integer.
      * @return string The resolved string with each part (separated by comma) returned separated by comma
-     * @access private
+     * @internal
      */
     public function calcOffset($string)
     {
@@ -708,13 +667,13 @@ class GifBuilder extends GraphicalFunctions
      *
      * @param string $file Filename value OR the string "GIFBUILDER", see documentation in TSref for the "datatype" called "imgResource
      * @param array $fileArray TypoScript properties passed to the function. Either GIFBUILDER properties or imgResource properties, depending on the value of $file (whether that is "GIFBUILDER" or a file reference)
-     * @return array|NULL Returns an array with file information from ContentObjectRenderer::getImgResource()
-     * @access private
+     * @return array|null Returns an array with file information from ContentObjectRenderer::getImgResource()
+     * @internal
      * @see ContentObjectRenderer::getImgResource()
      */
     public function getResource($file, $fileArray)
     {
-        if (!GeneralUtility::inList($this->imageFileExt, $fileArray['ext'])) {
+        if (!in_array($fileArray['ext'], $this->imageFileExt, true)) {
             $fileArray['ext'] = $this->gifExtension;
         }
         /** @var ContentObjectRenderer $cObj */
@@ -727,13 +686,17 @@ class GifBuilder extends GraphicalFunctions
      * Returns the reference to a "resource" in TypoScript.
      *
      * @param string $file The resource value.
-     * @return string Returns the relative filepath
-     * @access private
+     * @return string|null Returns the relative filepath or null if it's invalid
+     * @internal
      * @see TemplateService::getFileName()
      */
     public function checkFile($file)
     {
-        return $GLOBALS['TSFE']->tmpl->getFileName($file);
+        try {
+            return GeneralUtility::makeInstance(FilePathSanitizer::class)->sanitize($file);
+        } catch (\TYPO3\CMS\Core\Resource\Exception $e) {
+            return null;
+        }
     }
 
     /**
@@ -743,39 +706,38 @@ class GifBuilder extends GraphicalFunctions
      * something like "GB_MD5HASH_myfilename_is_very_long_and_such.jpg"
      *
      * @param string $pre Filename prefix, eg. "GB_
-     * @return string The relative filepath (relative to PATH_site)
-     * @access private
+     * @return string The filepath, relative to Environment::getPublicPath()
+     * @internal
      */
     public function fileName($pre)
     {
-        /** @var $basicFileFunctions \TYPO3\CMS\Core\Utility\File\BasicFileUtility */
+        /** @var \TYPO3\CMS\Core\Utility\File\BasicFileUtility $basicFileFunctions */
         $basicFileFunctions = GeneralUtility::makeInstance(BasicFileUtility::class);
         $filePrefix = implode('_', array_merge($this->combinedTextStrings, $this->combinedFileNames));
-        $filePrefix = $basicFileFunctions->cleanFileName($filePrefix);
+        $filePrefix = $basicFileFunctions->cleanFileName(ltrim($filePrefix, '.'));
 
-        return $this->tempPath . $pre . $filePrefix . '_' . GeneralUtility::shortMD5(serialize($this->setup)) . '.' . $this->extension();
+        // shorten prefix to avoid overly long file names
+        $filePrefix = substr($filePrefix, 0, 100);
+
+        return 'typo3temp/' . $pre . $filePrefix . '_' . GeneralUtility::shortMD5(serialize($this->setup)) . '.' . $this->extension();
     }
 
     /**
      * Returns the file extension used in the filename
      *
      * @return string Extension; "jpg" or "gif"/"png
-     * @access private
+     * @internal
      */
     public function extension()
     {
         switch (strtolower($this->setup['format'])) {
             case 'jpg':
-
             case 'jpeg':
                 return 'jpg';
-                break;
             case 'png':
                 return 'png';
-                break;
             case 'gif':
                 return 'gif';
-                break;
             default:
                 return $this->gifExtension;
         }
@@ -801,29 +763,29 @@ class GifBuilder extends GraphicalFunctions
                 $objParts = explode('.', substr($theVal, 1, -1));
                 $theVal = 0;
                 if (isset($this->objBB[$objParts[0]])) {
-                    if ($objParts[1] == 'w') {
+                    if ($objParts[1] === 'w') {
                         $theVal = $this->objBB[$objParts[0]][0];
-                    } elseif ($objParts[1] == 'h') {
+                    } elseif ($objParts[1] === 'h') {
                         $theVal = $this->objBB[$objParts[0]][1];
-                    } elseif ($objParts[1] == 'lineHeight') {
+                    } elseif ($objParts[1] === 'lineHeight') {
                         $theVal = $this->objBB[$objParts[0]][2]['lineHeight'];
                     }
                     $theVal = (int)$theVal;
                 }
-            } elseif (floatval($theVal)) {
-                $theVal = floatval($theVal);
+            } elseif ((float)$theVal) {
+                $theVal = (float)$theVal;
             } else {
                 $theVal = 0;
             }
-            if ($sign == '-') {
+            if ($sign === '-') {
                 $calculatedValue -= $theVal;
-            } elseif ($sign == '+') {
+            } elseif ($sign === '+') {
                 $calculatedValue += $theVal;
-            } elseif ($sign == '/' && $theVal) {
+            } elseif ($sign === '/' && $theVal) {
                 $calculatedValue = $calculatedValue / $theVal;
-            } elseif ($sign == '*') {
+            } elseif ($sign === '*') {
                 $calculatedValue = $calculatedValue * $theVal;
-            } elseif ($sign == '%' && $theVal) {
+            } elseif ($sign === '%' && $theVal) {
                 $calculatedValue %= $theVal;
             }
         }
