@@ -14,10 +14,13 @@ namespace TYPO3\CMS\Frontend\Page;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\SingletonInterface;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
+
 /**
  * Logic for cHash calculation
  */
-class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
+class CacheHashCalculator implements SingletonInterface
 {
     /**
      * @var array Parameters that are relevant for cacheHash calculation. Optional.
@@ -44,17 +47,15 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      */
     protected $excludeAllEmptyParameters = false;
 
-    /**
-     * @var bool
-     */
-    protected $includePageId = false;
-
     /**
      * Initialise class properties by using the relevant TYPO3 configuration
+     *
+     * @param array $configuration
      */
-    public function __construct()
+    public function __construct(array $configuration = null)
     {
-        $this->setConfiguration($GLOBALS['TYPO3_CONF_VARS']['FE']['cacheHash']);
+        $configuration = $configuration ?? $GLOBALS['TYPO3_CONF_VARS']['FE']['cacheHash'] ?? [];
+        $this->setConfiguration($configuration);
     }
 
     /**
@@ -73,6 +74,7 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $queryString Query-parameters: "&xxx=yyy&zzz=uuu
      * @return string Hash of all the values
+     * @throws \RuntimeException
      */
     public function generateForParameters($queryString)
     {
@@ -94,8 +96,9 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
         $hasRequiredParameter = false;
         $parameterNames = array_keys($this->splitQueryStringToArray($queryString));
         foreach ($parameterNames as $parameterName) {
-            if (in_array($parameterName, $this->requireCacheHashPresenceParameters)) {
+            if (in_array($parameterName, $this->requireCacheHashPresenceParameters, true)) {
                 $hasRequiredParameter = true;
+                break;
             }
         }
         return $hasRequiredParameter;
@@ -107,7 +110,8 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param string $queryString Query-parameters: "&xxx=yyy&zzz=uuu
      * @return array Array with key/value pairs of query-parameters WITHOUT a certain list of
-     * @see \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController::makeCacheHash(), \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::typoLink()
+     * @throws \RuntimeException
+     * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::typoLink()
      */
     public function getRelevantParameters($queryString)
     {
@@ -120,18 +124,16 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
             if ($this->hasCachedParametersWhiteList() && !$this->isInCachedParametersWhiteList($parameterName)) {
                 continue;
             }
-            if ((is_null($parameterValue) || $parameterValue === '') && !$this->isAllowedWithEmptyValue($parameterName)) {
+            if (($parameterValue === null || $parameterValue === '') && !$this->isAllowedWithEmptyValue($parameterName)) {
                 continue;
             }
             $relevantParameters[$parameterName] = $parameterValue;
         }
         if (!empty($relevantParameters)) {
-            if ($this->includePageId) {
-                if (empty($parameters['id'])) {
-                    throw new \RuntimeException('ID parameter needs to be passed for the cHash calculation! As a temporary not recommended workaround, you can set $GLOBALS[\'TYPO3_CONF_VARS\'][\'FE\'][\'cHashIncludePageId\'] to false to avoid this error.', 1467983513);
-                }
-                $relevantParameters['id'] = $parameters['id'];
+            if (empty($parameters['id'])) {
+                throw new \RuntimeException('ID parameter needs to be passed for the cHash calculation!', 1467983513);
             }
+            $relevantParameters['id'] = $parameters['id'];
             // Finish and sort parameters array by keys:
             $relevantParameters['encryptionKey'] = $GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey'];
             ksort($relevantParameters);
@@ -145,7 +147,7 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      *
      * e.g. foo[bar]=baz will be array('foo[bar]' => 'baz')
      *
-     * @param $queryString
+     * @param string $queryString
      * @return array
      */
     protected function splitQueryStringToArray($queryString)
@@ -153,22 +155,31 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
         $parameters = array_filter(explode('&', ltrim($queryString, '?')));
         $parameterArray = [];
         foreach ($parameters as $parameter) {
-            list($parameterName, $parameterValue) = explode('=', $parameter);
+            // should not remove empty values with trimExplode, otherwise cases like &=value, value is used as parameterName.
+            $parts = GeneralUtility::trimExplode('=', $parameter, false);
+            $parameterName = $parts[0];
+            $parameterValue = $parts[1] ?? '';
+            if (trim($parameterName) === '') {
+                // This parameter cannot appear in $_GET in PHP even if its value is not empty, so it should be ignored!
+                continue;
+            }
             $parameterArray[rawurldecode($parameterName)] = rawurldecode($parameterValue);
         }
         return $parameterArray;
     }
 
     /**
-     * Checks whether the given parameter starts with TSFE_ADMIN_PANEL
-     * stristr check added to avoid bad performance
+     * Checks whether the given parameter is out of a known data-set starting
+     * with ADMCMD or starts with TSFE_ADMIN_PANEL.
      *
      * @param string $key
      * @return bool
      */
     protected function isAdminPanelParameter($key)
     {
-        return stristr($key, 'TSFE_ADMIN_PANEL') !== false && preg_match('/TSFE_ADMIN_PANEL\\[.*?\\]/', $key);
+        return $key === 'ADMCMD_noBeUser' || $key === 'ADMCMD_view' || $key === 'ADMCMD_editIcons'
+            || $key === 'ADMCMD_simUser' || $key === 'ADMCMD_simTime' || $key === 'ADMCMD_prev'
+            || stripos($key, 'TSFE_ADMIN_PANEL') !== false && preg_match('/TSFE_ADMIN_PANEL\\[.*?\\]/', $key);
     }
 
     /**
@@ -183,14 +194,14 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
     }
 
     /**
-     * Checks whether the given parameter should be exluded from cHash calculation
+     * Checks whether the given parameter should be excluded from cHash calculation
      *
      * @param string $key
      * @return bool
      */
     protected function isExcludedParameter($key)
     {
-        return in_array($key, $this->excludedParameters);
+        return in_array($key, $this->excludedParameters, true);
     }
 
     /**
@@ -201,7 +212,7 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      */
     protected function isInCachedParametersWhiteList($key)
     {
-        return in_array($key, $this->cachedParametersWhiteList);
+        return in_array($key, $this->cachedParametersWhiteList, true);
     }
 
     /**
@@ -222,7 +233,7 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
      */
     protected function isAllowedWithEmptyValue($key)
     {
-        return !($this->excludeAllEmptyParameters || in_array($key, $this->excludedParametersIfEmpty));
+        return !($this->excludeAllEmptyParameters || in_array($key, $this->excludedParametersIfEmpty, true));
     }
 
     /**
@@ -249,14 +260,6 @@ class CacheHashCalculator implements \TYPO3\CMS\Core\SingletonInterface
         $this->cachedParametersWhiteList = $cachedParametersWhiteList;
     }
 
-    /**
-     * @param bool $includePageId
-     */
-    protected function setIncludePageId($includePageId)
-    {
-        $this->includePageId = $includePageId;
-    }
-
     /**
      * @param bool $excludeAllEmptyParameters
      */
