@@ -16,8 +16,11 @@ namespace TYPO3\CMS\IndexedSearch\Controller;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Backend\View\BackendTemplateView;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Database\DatabaseConnection;
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
+use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Imaging\Icon;
+use TYPO3\CMS\Core\Localization\LanguageService;
+use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
 use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
@@ -25,10 +28,10 @@ use TYPO3\CMS\Extbase\Mvc\Web\Request as WebRequest;
 use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
 use TYPO3\CMS\IndexedSearch\Domain\Repository\AdministrationRepository;
 use TYPO3\CMS\IndexedSearch\Indexer;
-use TYPO3\CMS\Lang\LanguageService;
 
 /**
  * Administration controller
+ * @internal This class is a specific controller implementation and is not considered part of the Public TYPO3 API.
  */
 class AdministrationController extends ActionController
 {
@@ -88,13 +91,14 @@ class AdministrationController extends ActionController
         if ($view instanceof BackendTemplateView) {
             /** @var BackendTemplateView $view */
             parent::initializeView($view);
-            $permissionClause = $this->getBackendUserAuthentication()->getPagePermsClause(1);
+            $permissionClause = $this->getBackendUserAuthentication()->getPagePermsClause(Permission::PAGE_SHOW);
             $pageRecord = BackendUtility::readPageAccess($this->pageUid, $permissionClause);
             if ($pageRecord) {
                 $view->getModuleTemplate()->getDocHeaderComponent()->setMetaInformation($pageRecord);
             }
             $this->generateMenu();
             $this->view->getModuleTemplate()->setFlashMessageQueue($this->controllerContext->getFlashMessageQueue());
+            $view->assign('extensionConfiguration', $this->indexerConfig);
         }
     }
 
@@ -107,22 +111,22 @@ class AdministrationController extends ActionController
             'index' => [
                 'controller' => 'Administration',
                 'action' => 'index',
-                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xml:administration.menu.general')
+                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xlf:administration.menu.general')
             ],
             'pages' => [
                 'controller' => 'Administration',
                 'action' => 'pages',
-                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xml:administration.menu.pages')
+                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xlf:administration.menu.pages')
             ],
             'externalDocuments' => [
                 'controller' => 'Administration',
                 'action' => 'externalDocuments',
-                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xml:administration.menu.externalDocuments')
+                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xlf:administration.menu.externalDocuments')
             ],
             'statistic' => [
                 'controller' => 'Administration',
                 'action' => 'statistic',
-                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xml:administration.menu.statistic')
+                'label' => $this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xlf:administration.menu.statistic')
             ]
         ];
         $uriBuilder = $this->objectManager->get(UriBuilder::class);
@@ -145,13 +149,11 @@ class AdministrationController extends ActionController
 
     /**
      * Function will be called before every other action
-     *
-     * @return void
      */
     public function initializeAction()
     {
         $this->pageUid = (int)GeneralUtility::_GET('id');
-        $this->indexerConfig = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['indexed_search']);
+        $this->indexerConfig = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('indexed_search');
         $this->enableMetaphoneSearch = (bool)$this->indexerConfig['enableMetaphoneSearch'];
         $this->indexer = GeneralUtility::makeInstance(Indexer::class);
 
@@ -199,7 +201,6 @@ class AdministrationController extends ActionController
 
     /**
      * @param \TYPO3\CMS\IndexedSearch\Domain\Repository\AdministrationRepository $administrationRepository
-     * @return void
      */
     public function injectAdministrationRepository(AdministrationRepository $administrationRepository)
     {
@@ -208,8 +209,6 @@ class AdministrationController extends ActionController
 
     /**
      * Index action contains the most important statistics
-     *
-     * @return void
      */
     public function indexAction()
     {
@@ -219,8 +218,12 @@ class AdministrationController extends ActionController
         ]);
 
         if ($this->pageUid) {
-            $last24hours = ' AND tstamp > ' . ($GLOBALS['EXEC_TIME'] - 24 * 60 * 60);
-            $last30days = ' AND tstamp > ' . ($GLOBALS['EXEC_TIME'] - 30 * 24 * 60 * 60);
+            $expressionBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+                ->getQueryBuilderForTable('index_stat_word')
+                ->expr();
+
+            $last24hours = $expressionBuilder->gt('tstamp', $GLOBALS['EXEC_TIME'] - 86400);
+            $last30days = $expressionBuilder->gt('tstamp', $GLOBALS['EXEC_TIME'] - 30 * 86400);
 
             $this->view->assignMultiple([
                 'pageUid' => $this->pageUid,
@@ -233,8 +236,6 @@ class AdministrationController extends ActionController
 
     /**
      * Statistics for pages
-     *
-     * @return void
      */
     public function pagesAction()
     {
@@ -243,8 +244,6 @@ class AdministrationController extends ActionController
 
     /**
      * Statistics for external documents
-     *
-     * @return void
      */
     public function externalDocumentsAction()
     {
@@ -255,29 +254,49 @@ class AdministrationController extends ActionController
      * Statistics for a given page hash
      *
      * @param int $pageHash
-     * @return void
      */
     public function statisticDetailsAction($pageHash = 0)
     {
+        $pageHash = (int)$pageHash;
         // Set back button
         $icon = $this->view->getModuleTemplate()->getIconFactory()->getIcon('actions-view-go-up', Icon::SIZE_SMALL);
         $backButton = $this->view->getModuleTemplate()->getDocHeaderComponent()
             ->getButtonBar()->makeLinkButton()
-            ->setTitle($this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xml:administration.back'))
+            ->setTitle($this->getLanguageService()->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang.xlf:administration.back'))
             ->setIcon($icon)
             ->setHref($this->getHref('Administration', 'statistic'));
         $this->view->getModuleTemplate()->getDocHeaderComponent()
             ->getButtonBar()->addButton($backButton);
 
-        $pageHash = (int)$pageHash;
-        $db = $this->getDatabaseConnection();
-        $pageHashRow = $db->exec_SELECTgetSingleRow('*', 'index_phash', 'phash = ' . (int)$pageHash);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_phash');
+        $pageHashRow = $queryBuilder
+            ->select('*')
+            ->from('index_phash')
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                )
+            )
+            ->execute()
+            ->fetch();
 
         if (!is_array($pageHashRow)) {
             $this->redirect('statistic');
         }
 
-        $debugRow = $db->exec_SELECTgetRows('*', 'index_debug', 'phash = ' . (int)$pageHash);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_debug');
+        $debugRow = $queryBuilder
+            ->select('*')
+            ->from('index_debug')
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                )
+            )
+            ->execute()
+            ->fetchAll();
         $debugInfo = [];
         $lexer = '';
         if (is_array($debugRow)) {
@@ -287,13 +306,25 @@ class AdministrationController extends ActionController
         }
         $pageRecord = BackendUtility::getRecord('pages', $pageHashRow['data_page_id']);
         $keywords = is_array($pageRecord) ? array_flip(GeneralUtility::trimExplode(',', $pageRecord['keywords'], true)) : [];
-        $wordRecords = $db->exec_SELECTgetRows(
-            'index_words.*, index_rel.*',
-            'index_rel, index_words',
-            'index_rel.phash = ' . (int)$pageHash . ' AND index_words.wid = index_rel.wid',
-            '',
-            'index_words.baseword'
-        );
+
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_words');
+        $wordRecords = $queryBuilder
+            ->select('index_words.*', 'index_rel.*')
+            ->from('index_words')
+            ->from('index_rel')
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'index_rel.phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_words.wid',
+                    $queryBuilder->quoteIdentifier('index_rel.wid')
+                )
+            )
+            ->orderBy('index_words.baseword')
+            ->execute()
+            ->fetchAll();
         foreach ($wordRecords as $id => $row) {
             if (isset($keywords[$row['baseword']])) {
                 $wordRecords[$id]['is_keyword'] = true;
@@ -316,39 +347,84 @@ class AdministrationController extends ActionController
                 }
             }
         }
+
+        // sections
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_section');
+        $sections = $queryBuilder
+            ->select('*')
+            ->from('index_section')
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                )
+            )
+            ->execute()
+            ->fetchAll();
+
+        // top words
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_words');
+        $topCountWords = $queryBuilder
+            ->select('index_words.baseword', 'index_words.metaphone', 'index_rel.*')
+            ->from('index_words')
+            ->from('index_rel')
+            ->setMaxResults(20)
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'index_rel.phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_words.is_stopword',
+                    $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_words.wid',
+                    $queryBuilder->quoteIdentifier('index_rel.wid')
+                )
+            )
+            ->orderBy('index_rel.count', 'DESC')
+            ->execute()
+            ->fetchAll();
+
+        // top frequency
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_words');
+        $topFrequency = $queryBuilder
+            ->select('index_words.baseword', 'index_words.metaphone', 'index_rel.*')
+            ->from('index_words')
+            ->from('index_rel')
+            ->setMaxResults(20)
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'index_rel.phash',
+                    $queryBuilder->createNamedParameter($pageHash, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_words.is_stopword',
+                    $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_words.wid',
+                    $queryBuilder->quoteIdentifier('index_rel.wid')
+                )
+            )
+            ->orderBy('index_rel.freq', 'DESC')
+            ->execute()
+            ->fetchAll();
+
         $this->view->assignMultiple([
-            'phash' => $pageHash,
+            'phash' => (int)$pageHash,
             'phashRow' => $pageHashRow,
             'words' => $wordRecords,
-            'sections' => $db->exec_SELECTgetRows(
-                '*',
-                'index_section',
-                'index_section.phash = ' . (int)$pageHash
-            ),
-            'topCount' => $db->exec_SELECTgetRows(
-                'index_words.baseword, index_words.metaphone, index_rel.*',
-                'index_rel, index_words',
-                'index_rel.phash = ' . (int)$pageHash . ' AND index_words.wid = index_rel.wid
-					 AND index_words.is_stopword=0',
-                '',
-                'index_rel.count DESC',
-                 '20'
-             ),
-            'topFrequency' => $db->exec_SELECTgetRows(
-                'index_words.baseword, index_words.metaphone, index_rel.*',
-                'index_rel, index_words',
-                'index_rel.phash = ' . (int)$pageHash . ' AND index_words.wid = index_rel.wid
-					 AND index_words.is_stopword=0',
-                '',
-                'index_rel.freq DESC',
-                '20'
-            ),
+            'sections' => $sections,
+            'topCount' => $topCountWords,
+            'topFrequency' => $topFrequency,
             'debug' => $debugInfo,
             'lexer' => $lexer,
             'metaphone' => $metaphone,
             'page' => $pageRecord,
             'keywords' => $keywords
-    ]);
+        ]);
     }
 
     /**
@@ -358,7 +434,6 @@ class AdministrationController extends ActionController
      * @param int $pageId
      * @param array $stopwords
      * @param array $keywords
-     * @return void
      */
     public function saveStopwordsKeywordsAction($pageHash, $pageId, $stopwords = [], $keywords = [])
     {
@@ -379,17 +454,32 @@ class AdministrationController extends ActionController
      *
      * @param int $id
      * @param int $pageHash
-     * @return void
      */
     public function wordDetailAction($id = 0, $pageHash = 0)
     {
-        $rows = $this->getDatabaseConnection()->exec_SELECTgetRows(
-            'index_phash.*, index_section.*, index_rel.*',
-            'index_rel, index_section, index_phash',
-            'index_rel.wid = ' . (int)$id . ' AND index_rel.phash = index_section.phash' . ' AND index_section.phash = index_phash.phash',
-            '',
-            'index_rel.freq DESC'
-        );
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('index_phash');
+        $rows = $queryBuilder
+            ->select('index_phash.*', 'index_section.*', 'index_rel.*')
+            ->from('index_rel')
+            ->from('index_section')
+            ->from('index_phash')
+            ->where(
+                $queryBuilder->expr()->eq(
+                    'index_rel.wid',
+                    $queryBuilder->createNamedParameter($id, \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_rel.phash',
+                    $queryBuilder->quoteIdentifier('index_section.phash')
+                ),
+                $queryBuilder->expr()->eq(
+                    'index_section.phash',
+                    $queryBuilder->quoteIdentifier('index_phash.phash')
+                )
+            )
+            ->orderBy('index_rel.freq', 'desc')
+            ->execute()
+            ->fetchAll();
 
         $this->view->assignMultiple([
             'rows' => $rows,
@@ -402,17 +492,14 @@ class AdministrationController extends ActionController
      *
      * @param int $depth
      * @param string $mode
-     * @return void
      */
     public function statisticAction($depth = 1, $mode = 'overview')
     {
-        if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['indexed_search']['external_parsers'])) {
-            foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['indexed_search']['external_parsers'] as $extension => $_objRef) {
-                /** @var \TYPO3\CMS\IndexedSearch\FileContentParser $fileContentParser */
-                $fileContentParser = GeneralUtility::getUserObj($_objRef);
-                if ($fileContentParser->softInit($extension)) {
-                    $this->external_parsers[$extension] = $fileContentParser;
-                }
+        foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['indexed_search']['external_parsers'] ?? [] as $extension => $className) {
+            /** @var \TYPO3\CMS\IndexedSearch\FileContentParser $fileContentParser */
+            $fileContentParser = GeneralUtility::makeInstance($className);
+            if ($fileContentParser->softInit($extension)) {
+                $this->external_parsers[$extension] = $fileContentParser;
             }
         }
         $this->administrationRepository->external_parsers = $this->external_parsers;
@@ -420,7 +507,7 @@ class AdministrationController extends ActionController
         $allLines = $this->administrationRepository->getTree($this->pageUid, $depth, $mode);
 
         $this->view->assignMultiple([
-            'levelTranslations' => explode('|', $this->getLanguageService()->sL('LLL:EXT:lang/locallang_core.xlf:labels.enterSearchLevels')),
+            'levelTranslations' => explode('|', $this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.enterSearchLevels')),
             'tree' => $allLines,
             'pageUid' => $this->pageUid,
             'mode' => $mode,
@@ -434,7 +521,6 @@ class AdministrationController extends ActionController
      * @param string $id
      * @param int $depth
      * @param string $mode
-     * @return void
      */
     public function deleteIndexedItemAction($id, $depth = 1, $mode = 'overview')
     {
@@ -458,14 +544,6 @@ class AdministrationController extends ActionController
         return $uriBuilder->reset()->uriFor($action, $parameters, $controller);
     }
 
-    /**
-     * @return DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
-
     /**
      * @return BackendUserAuthentication
      */
