@@ -14,9 +14,12 @@ namespace TYPO3\CMS\IndexedSearch;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
+use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Utility\CommandUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
+use TYPO3\CMS\Core\Utility\PathUtility;
 
 /**
  * External standard parsers for indexed_search
@@ -54,10 +57,15 @@ class FileContentParser
     public $pObj;
 
     /**
-     * @var \TYPO3\CMS\Lang\LanguageService|\TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController
+     * @var \TYPO3\CMS\Core\Localization\LanguageService|\TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController
      */
     protected $langObject;
 
+    /**
+     * @var string Backup for setLocaleForServerFileSystem()
+     */
+    protected $lastLocale;
+
     /**
      * Constructs this external parsers object
      */
@@ -76,9 +84,9 @@ class FileContentParser
     public function initParser($extension)
     {
         // Then read indexer-config and set if appropriate:
-        $indexerConfig = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['indexed_search']);
+        $indexerConfig = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('indexed_search');
         // If windows, apply extension to tool name:
-        $exe = TYPO3_OS == 'WIN' ? '.exe' : '';
+        $exe = Environment::isWindows() ? '.exe' : '';
         // lg
         $extOK = false;
         $mainExtension = '';
@@ -94,7 +102,7 @@ class FileContentParser
                 // PDF
                 if ($indexerConfig['pdftools']) {
                     $pdfPath = rtrim($indexerConfig['pdftools'], '/') . '/';
-                    if (@is_file(($pdfPath . 'pdftotext' . $exe)) && @is_file(($pdfPath . 'pdfinfo' . $exe))) {
+                    if (@is_file($pdfPath . 'pdftotext' . $exe) && @is_file($pdfPath . 'pdfinfo' . $exe)) {
                         $this->app['pdfinfo'] = $pdfPath . 'pdfinfo' . $exe;
                         $this->app['pdftotext'] = $pdfPath . 'pdftotext' . $exe;
                         // PDF mode:
@@ -111,7 +119,7 @@ class FileContentParser
                 // Catdoc
                 if ($indexerConfig['catdoc']) {
                     $catdocPath = rtrim($indexerConfig['catdoc'], '/') . '/';
-                    if (@is_file(($catdocPath . 'catdoc' . $exe))) {
+                    if (@is_file($catdocPath . 'catdoc' . $exe)) {
                         $this->app['catdoc'] = $catdocPath . 'catdoc' . $exe;
                         $extOK = true;
                     } else {
@@ -127,7 +135,7 @@ class FileContentParser
                 // ppthtml
                 if ($indexerConfig['ppthtml']) {
                     $ppthtmlPath = rtrim($indexerConfig['ppthtml'], '/') . '/';
-                    if (@is_file(($ppthtmlPath . 'ppthtml' . $exe))) {
+                    if (@is_file($ppthtmlPath . 'ppthtml' . $exe)) {
                         $this->app['ppthtml'] = $ppthtmlPath . 'ppthtml' . $exe;
                         $extOK = true;
                     } else {
@@ -142,7 +150,7 @@ class FileContentParser
                 // Xlhtml
                 if ($indexerConfig['xlhtml']) {
                     $xlhtmlPath = rtrim($indexerConfig['xlhtml'], '/') . '/';
-                    if (@is_file(($xlhtmlPath . 'xlhtml' . $exe))) {
+                    if (@is_file($xlhtmlPath . 'xlhtml' . $exe)) {
                         $this->app['xlhtml'] = $xlhtmlPath . 'xlhtml' . $exe;
                         $extOK = true;
                     } else {
@@ -180,7 +188,7 @@ class FileContentParser
                 // Oasis OpenDocument Text
                 if ($indexerConfig['unzip']) {
                     $unzipPath = rtrim($indexerConfig['unzip'], '/') . '/';
-                    if (@is_file(($unzipPath . 'unzip' . $exe))) {
+                    if (@is_file($unzipPath . 'unzip' . $exe)) {
                         $this->app['unzip'] = $unzipPath . 'unzip' . $exe;
                         $extOK = true;
                     } else {
@@ -194,7 +202,7 @@ class FileContentParser
                 // Catdoc
                 if ($indexerConfig['unrtf']) {
                     $unrtfPath = rtrim($indexerConfig['unrtf'], '/') . '/';
-                    if (@is_file(($unrtfPath . 'unrtf' . $exe))) {
+                    if (@is_file($unrtfPath . 'unrtf' . $exe)) {
                         $this->app['unrtf'] = $unrtfPath . 'unrtf' . $exe;
                         $extOK = true;
                     } else {
@@ -274,7 +282,6 @@ class FileContentParser
             case 'tif':
                 // TIF images (EXIF comment)
                 return true;
-                break;
         }
         return false;
     }
@@ -288,7 +295,7 @@ class FileContentParser
     public function searchTypeMediaTitle($extension)
     {
         // Read indexer-config
-        $indexerConfig = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['indexed_search']);
+        $indexerConfig = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('indexed_search');
         // Ignore extensions
         $ignoreExtensions = GeneralUtility::trimExplode(',', strtolower($indexerConfig['ignoreExtensions']), true);
         if (in_array($extension, $ignoreExtensions)) {
@@ -390,24 +397,19 @@ class FileContentParser
             case 'tif':
                 // PHP EXIF
                 return sprintf($this->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang_main.xlf:extension.Images'), $extension);
-                break;
             case 'html':
             case 'htm':
                 // PHP strip-tags()
                 return sprintf($this->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang_main.xlf:extension.HTML'), $extension);
-                break;
             case 'txt':
                 // Raw text
                 return sprintf($this->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang_main.xlf:extension.TXT'), $extension);
-                break;
             case 'csv':
                 // Raw text
                 return sprintf($this->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang_main.xlf:extension.CSV'), $extension);
-                break;
             case 'xml':
                 // PHP strip-tags()
                 return sprintf($this->sL('LLL:EXT:indexed_search/Resources/Private/Language/locallang_main.xlf:extension.XML'), $extension);
-                break;
             default:
                 // Do nothing
         }
@@ -426,7 +428,6 @@ class FileContentParser
         switch ((string)$extension) {
             case 'pdf':
                 return true;
-                break;
         }
         return false;
     }
@@ -435,12 +436,11 @@ class FileContentParser
      * Wraps the "splitLabel function" of the language object.
      *
      * @param string $reference: Reference/key of the label
-     * @param bool $useHtmlSpecialChar: Convert special chars to HTML entities (default: FALSE)
      * @return string The label of the reference/key to be fetched
      */
-    protected function sL($reference, $useHtmlSpecialChar = false)
+    protected function sL($reference)
     {
-        return $this->langObject->sL($reference, $useHtmlSpecialChar);
+        return $this->langObject->sL($reference);
     }
 
     /************************
@@ -483,7 +483,7 @@ class FileContentParser
                         $cmd = $this->app['pdftotext'] . ' -f ' . $low . ' -l ' . $high . ' -enc UTF-8 -q ' . escapeshellarg($absFile) . ' ' . $tempFileName;
                         CommandUtility::exec($cmd);
                         if (@is_file($tempFileName)) {
-                            $content = GeneralUtility::getUrl($tempFileName);
+                            $content = file_get_contents($tempFileName);
                             unlink($tempFileName);
                         } else {
                             $content = '';
@@ -520,7 +520,7 @@ class FileContentParser
                     unset($res);
                     $content = $this->pObj->convertHTMLToUtf8($content);
                     $contentArr = $this->pObj->splitHTMLContent($this->removeEndJunk($content));
-                    $contentArr['title'] = basename($absFile);
+                    $contentArr['title'] = PathUtility::basename($absFile);
                     $this->setLocaleForServerFileSystem(true);
                 }
                 break;
@@ -533,7 +533,7 @@ class FileContentParser
                     unset($res);
                     $content = $this->pObj->convertHTMLToUtf8($content);
                     $contentArr = $this->pObj->splitHTMLContent($this->removeEndJunk($content));
-                    $contentArr['title'] = basename($absFile);
+                    $contentArr['title'] = PathUtility::basename($absFile);
                     $this->setLocaleForServerFileSystem(true);
                 }
                 break;
@@ -570,7 +570,7 @@ class FileContentParser
                     $utf8_content = trim(strip_tags(str_replace('<', ' <', $content_xml)));
                     $contentArr = $this->pObj->splitRegularContent($utf8_content);
                     // Make sure the title doesn't expose the absolute path!
-                    $contentArr['title'] = basename($absFile);
+                    $contentArr['title'] = PathUtility::basename($absFile);
                     // Meta information
                     $cmd = $this->app['unzip'] . ' -p ' . escapeshellarg($absFile) . ' docProps/core.xml';
                     CommandUtility::exec($cmd, $res);
@@ -606,7 +606,7 @@ class FileContentParser
                     unset($res);
                     $utf8_content = trim(strip_tags(str_replace('<', ' <', $content_xml)));
                     $contentArr = $this->pObj->splitRegularContent($utf8_content);
-                    $contentArr['title'] = basename($absFile);
+                    $contentArr['title'] = PathUtility::basename($absFile);
                     // Make sure the title doesn't expose the absolute path!
                     // Meta information
                     $metaContent = GeneralUtility::xml2tree($meta_xml);
@@ -645,7 +645,7 @@ class FileContentParser
                 $contentCharset = 'utf-8';
                 $content = $this->pObj->convertHTMLToUtf8($content, $contentCharset);
                 $contentArr = $this->pObj->splitRegularContent($content);
-                $contentArr['title'] = basename($absFile);
+                $contentArr['title'] = PathUtility::basename($absFile);
                 // Make sure the title doesn't expose the absolute path!
                 $this->setLocaleForServerFileSystem(true);
                 break;
@@ -661,11 +661,11 @@ class FileContentParser
                 $fileContent = GeneralUtility::getUrl($absFile);
                 // Finding charset:
                 preg_match('/^[[:space:]]*<\\?xml[^>]+encoding[[:space:]]*=[[:space:]]*["\'][[:space:]]*([[:alnum:]_-]+)[[:space:]]*["\']/i', substr($fileContent, 0, 200), $reg);
-                $charset = $reg[1] ? $this->pObj->csObj->parse_charset($reg[1]) : 'utf-8';
+                $charset = $reg[1] ? trim(strtolower($reg[1])) : 'utf-8';
                 // Converting content:
                 $fileContent = $this->pObj->convertHTMLToUtf8(strip_tags(str_replace('<', ' <', $fileContent)), $charset);
                 $contentArr = $this->pObj->splitRegularContent($fileContent);
-                $contentArr['title'] = basename($absFile);
+                $contentArr['title'] = PathUtility::basename($absFile);
                 // Make sure the title doesn't expose the absolute path!
                 $this->setLocaleForServerFileSystem(true);
                 break;
@@ -685,7 +685,7 @@ class FileContentParser
                     $comment = '';
                 }
                 $contentArr = $this->pObj->splitRegularContent($comment);
-                $contentArr['title'] = basename($absFile);
+                $contentArr['title'] = PathUtility::basename($absFile);
                 // Make sure the title doesn't expose the absolute path!
                 $this->setLocaleForServerFileSystem(true);
                 break;
@@ -695,7 +695,7 @@ class FileContentParser
         // If no title (and why should there be...) then the file-name is set as title. This will raise the hits considerably if the search matches the document name.
         if (is_array($contentArr) && !$contentArr['title']) {
             // Substituting "_" for " " because many filenames may have this instead of a space char.
-            $contentArr['title'] = str_replace('_', ' ', basename($absFile));
+            $contentArr['title'] = str_replace('_', ' ', PathUtility::basename($absFile));
         }
         return $contentArr;
     }
@@ -708,27 +708,25 @@ class FileContentParser
      *
      * @staticvar string $lastLocale Stores the locale used before it is overridden by this method.
      * @param bool $resetLocale TRUE resets the locale to $lastLocale.
-     * @return void
      * @throws \RuntimeException
      */
     protected function setLocaleForServerFileSystem($resetLocale = false)
     {
-        static $lastLocale = null;
         if (!$GLOBALS['TYPO3_CONF_VARS']['SYS']['UTF8filesystem']) {
             return;
         }
 
         if ($resetLocale) {
-            if ($lastLocale == null) {
+            if ($this->lastLocale == null) {
                 throw new \RuntimeException('Cannot reset locale to NULL.', 1357064326);
             }
-            setlocale(LC_CTYPE, $lastLocale);
-            $lastLocale = null;
+            setlocale(LC_CTYPE, $this->lastLocale);
+            $this->lastLocale = null;
         } else {
-            if ($lastLocale !== null) {
+            if ($this->lastLocale !== null) {
                 throw new \RuntimeException('Cannot set new locale as locale has already been changed before.', 1357064437);
             }
-            $lastLocale = setlocale(LC_CTYPE, 0);
+            $this->lastLocale = setlocale(LC_CTYPE, 0);
             setlocale(LC_CTYPE, $GLOBALS['TYPO3_CONF_VARS']['SYS']['systemLocale']);
         }
     }
@@ -781,7 +779,7 @@ class FileContentParser
      *
      * @param array $pdfInfoArray Array of PDF content, coming from the pdfinfo tool
      * @return array Result array
-     * @access private
+     * @internal
      * @see fileContentParts()
      */
     public function splitPdfInfo($pdfInfoArray)
