@@ -15,35 +15,33 @@ namespace TYPO3\CMS\IndexedSearch\ViewHelpers;
  */
 
 use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
-use TYPO3\CMS\Fluid\Core\Rendering\RenderingContextInterface;
-use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
-use TYPO3\CMS\Fluid\Core\ViewHelper\Facets\CompilableInterface;
+use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;
+use TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper;
+use TYPO3Fluid\Fluid\Core\ViewHelper\Traits\CompileWithRenderStatic;
 
 /**
  * renders the header of the results page
  * @internal
  */
-class PageBrowsingResultsViewHelper extends AbstractViewHelper implements CompilableInterface
+class PageBrowsingResultsViewHelper extends AbstractViewHelper
 {
+    use CompileWithRenderStatic;
+
     /**
-     * main render function
+     * As this ViewHelper renders HTML, the output must not be escaped.
      *
-     * @param int $numberOfResults
-     * @param int $resultsPerPage
-     * @param int $currentPage
-     * @return string the content
+     * @var bool
      */
-    public function render($numberOfResults, $resultsPerPage, $currentPage = 1)
+    protected $escapeOutput = false;
+
+    /**
+     * Initialize arguments
+     */
+    public function initializeArguments()
     {
-        return static::renderStatic(
-            [
-                'numberOfResults' => $numberOfResults,
-                'resultsPerPage' => $resultsPerPage,
-                'currentPage' => $currentPage,
-            ],
-            $this->buildRenderChildrenClosure(),
-            $this->renderingContext
-        );
+        $this->registerArgument('numberOfResults', 'int', '', true);
+        $this->registerArgument('resultsPerPage', 'int', '', true);
+        $this->registerArgument('currentPage', 'int', '', false, 1);
     }
 
     /**
