@@ -14,40 +14,35 @@ namespace TYPO3\CMS\Install\ViewHelpers;
  * The TYPO3 project - inspiring people to share!
  */
 
-use TYPO3\CMS\Fluid\Core\Rendering\RenderingContextInterface;
-use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
-use TYPO3\CMS\Fluid\Core\ViewHelper\Facets\CompilableInterface;
+use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;
+use TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper;
+use TYPO3Fluid\Fluid\Core\ViewHelper\Traits\CompileWithRenderStatic;
 
 /**
  * Render value of a constant
  *
  * @internal
  */
-class ConstantViewHelper extends AbstractViewHelper implements CompilableInterface
+class ConstantViewHelper extends AbstractViewHelper
 {
+    use CompileWithRenderStatic;
+
     /**
-     * Render a constant
-     *
-     * @param string $name Name of the constant
-     * @return string Value of constant
+     * Initialize arguments
      */
-    public function render($name)
+    public function initializeArguments()
     {
-        return static::renderStatic(
-            [
-                'name' => $name,
-            ],
-            $this->buildRenderChildrenClosure(),
-            $this->renderingContext
-        );
+        $this->registerArgument('name', 'string', 'Name of the constant', true);
     }
 
     /**
+     * Render a constant
+     *
      * @param array $arguments
-     * @param callable $renderChildrenClosure
+     * @param \Closure $renderChildrenClosure
      * @param RenderingContextInterface $renderingContext
      *
-     * @return string
+     * @return string Value of constant
      */
     public static function renderStatic(array $arguments, \Closure $renderChildrenClosure, RenderingContextInterface $renderingContext)
     {
