@@ -25,12 +25,16 @@ use TYPO3\CMS\Core\Resource\Filter\FileExtensionFilter;
 use TYPO3\CMS\Core\Resource\Folder;
 use TYPO3\CMS\Core\Resource\ProcessedFile;
 use TYPO3\CMS\Core\Resource\ResourceFactory;
+use TYPO3\CMS\Core\Resource\Search\FileSearchDemand;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Utility\HttpUtility;
+use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Recordlist\Tree\View\LinkParameterProviderInterface;
 use TYPO3\CMS\Recordlist\View\FolderUtilityRenderer;
 
 /**
  * Browser for files
+ * @internal This class is a specific LinkBrowser implementation and is not part of the TYPO3's Core API.
  */
 class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterface, LinkParameterProviderInterface
 {
@@ -40,7 +44,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
      * If the value is NOT set, then it will be restored from the module session data.
      * Example value: "/www/htdocs/typo3/32/3dsplm/fileadmin/css/"
      *
-     * @var string|NULL
+     * @var string|null
      */
     protected $expandFolder;
 
@@ -57,8 +61,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
     protected $elements = [];
 
     /**
-    * @var string
-    */
+     * @var string
+     */
     protected $searchWord;
 
     /**
@@ -67,17 +71,30 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
     protected $fileRepository;
 
     /**
-     * @return void
+     * @var array
+     */
+    protected $thumbnailConfiguration = [];
+
+    /**
+     * Loads additional JavaScript
      */
     protected function initialize()
     {
         parent::initialize();
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Recordlist/BrowseFiles');
         $this->fileRepository = GeneralUtility::makeInstance(FileRepository::class);
+
+        $thumbnailConfig = $this->getBackendUser()->getTSConfig()['options.']['file_list.']['thumbnail.'] ?? [];
+        if (isset($thumbnailConfig['width']) && MathUtility::canBeInterpretedAsInteger($thumbnailConfig['width'])) {
+            $this->thumbnailConfiguration['width'] = (int)$thumbnailConfig['width'];
+        }
+        if (isset($thumbnailConfig['height']) && MathUtility::canBeInterpretedAsInteger($thumbnailConfig['height'])) {
+            $this->thumbnailConfiguration['height'] = (int)$thumbnailConfig['height'];
+        }
     }
 
     /**
-     * @return void
+     * Checks additional GET/POST requests
      */
     protected function initVariables()
     {
@@ -112,7 +129,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         $backendUser = $this->getBackendUser();
 
         // The key number 3 of the bparams contains the "allowed" string. Disallowed is not passed to
-        // the element browser at all but only filtered out in TCEMain afterwards
+        // the element browser at all but only filtered out in DataHandler afterwards
         $allowedFileExtensions = GeneralUtility::trimExplode(',', explode('|', $this->bparams)[3], true);
         if (!empty($allowedFileExtensions) && $allowedFileExtensions[0] !== 'sys_file' && $allowedFileExtensions[0] !== '*') {
             // Create new filter object
@@ -120,7 +137,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             $filterObject->setAllowedFileExtensions($allowedFileExtensions);
             // Set file extension filters on all storages
             $storages = $backendUser->getFileStorages();
-            /** @var $storage \TYPO3\CMS\Core\Resource\ResourceStorage */
+            /** @var \TYPO3\CMS\Core\Resource\ResourceStorage $storage */
             foreach ($storages as $storage) {
                 $storage->addFileAndFolderNameFilter([$filterObject, 'filterFileList']);
             }
@@ -149,7 +166,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         // Or get the user's default upload folder
         if (!$this->selectedFolder) {
             try {
-                $this->selectedFolder = $backendUser->getDefaultUploadFolder();
+                [, $pid, $table,, $field] = explode('-', explode('|', $this->bparams)[4]);
+                $this->selectedFolder = $backendUser->getDefaultUploadFolder($pid, $table, $field);
             } catch (\Exception $e) {
                 // The configured default user folder does not exist
             }
@@ -164,7 +182,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         }
 
         // Getting flag for showing/not showing thumbnails:
-        $noThumbs = $backendUser->getTSConfigVal('options.noThumbsInEB');
+        $noThumbs = $backendUser->getTSConfig()['options.']['noThumbsInEB'] ?? false;
         $_MOD_SETTINGS = [];
         if (!$noThumbs) {
             // MENU-ITEMS, fetching the setting for thumbnails from File>List module:
@@ -172,7 +190,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             $_MCONF['name'] = 'file_list';
             $_MOD_SETTINGS = BackendUtility::getModuleData($_MOD_MENU, GeneralUtility::_GP('SET'), $_MCONF['name']);
         }
-        $noThumbs = $noThumbs ?: !$_MOD_SETTINGS['displayThumbs'];
+        $displayThumbs = $_MOD_SETTINGS['displayThumbs'] ?? false;
+        $noThumbs = $noThumbs ?: !$displayThumbs;
         // Create folder tree:
         /** @var ElementBrowserFolderTreeView $folderTree */
         $folderTree = GeneralUtility::makeInstance(ElementBrowserFolderTreeView::class);
@@ -186,35 +205,30 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
 
         $this->initDocumentTemplate();
         // Starting content:
-        $content = $this->doc->startPage('TBE file selector');
-        $content .= $this->doc->getFlashMessages();
+        $content = $this->doc->startPage(htmlspecialchars($this->getLanguageService()->getLL('fileSelector')));
 
-        // Insert the upload form on top, if so configured
-        if ($backendUser->getTSConfigVal('options.uploadFieldsInTopOfEB')) {
-            $content .= $uploadForm;
-        }
         // Putting the parts together, side by side:
-        $content .= '
+        $markup = [];
+        $markup[] = '<!-- Wrapper table for folder tree / filelist: -->';
+        $markup[] = '<div class="element-browser">';
+        $markup[] = '   <div class="element-browser-panel element-browser-main">';
+        $markup[] = '       <div class="element-browser-main-sidebar">';
+        $markup[] = '           <div class="element-browser-body">';
+        $markup[] = '               ' . $tree;
+        $markup[] = '           </div>';
+        $markup[] = '       </div>';
+        $markup[] = '       <div class="element-browser-main-content">';
+        $markup[] = '           <div class="element-browser-body">';
+        $markup[] = '               ' . $this->doc->getFlashMessages();
+        $markup[] = '               ' . $files;
+        $markup[] = '               ' . $uploadForm;
+        $markup[] = '               ' . $createFolder;
+        $markup[] = '           </div>';
+        $markup[] = '       </div>';
+        $markup[] = '   </div>';
+        $markup[] = '</div>';
+        $content .= implode('', $markup);
 
-			<!--
-				Wrapper table for folder tree / filelist:
-			-->
-			<div class="element-browser-section element-browser-filetree">
-			<table border="0" cellpadding="0" cellspacing="0" id="typo3-EBfiles">
-				<tr>
-					<td class="c-wCell" valign="top"><h3>' . $this->getLanguageService()->getLL('folderTree', true) . ':</h3>' . $tree . '</td>
-					<td class="c-wCell" valign="top">' . $files . '</td>
-				</tr>
-			</table>
-			</div>
-			';
-        // Adding create folder + upload forms if applicable:
-        if (!$backendUser->getTSConfigVal('options.uploadFieldsInTopOfEB')) {
-            $content .= $uploadForm;
-        }
-        $content .= $createFolder;
-        // Add some space
-        $content .= '<br /><br />';
         // Ending page, returning content:
         $content .= $this->doc->endPage();
         return $this->doc->insertStylesAndJS($content);
@@ -237,7 +251,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         $titleLen = (int)$this->getBackendUser()->uc['titleLen'];
 
         if ($this->searchWord !== '') {
-            $files = $this->fileRepository->searchByName($folder, $this->searchWord);
+            $searchDemand = FileSearchDemand::createForSearchTerm($this->searchWord)->withRecursive();
+            $files = $folder->searchFiles($searchDemand);
         } else {
             $extensionList = !empty($extensionList) && $extensionList[0] === '*' ? [] : $extensionList;
             $files = $this->getFilesInFolder($folder, $extensionList);
@@ -250,14 +265,14 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         $folderIcon = $this->iconFactory->getIconForResource($folder, Icon::SIZE_SMALL);
 
         $lines[] = '
-			<tr class="t3-row-header">
-				<th class="col-title" nowrap="nowrap">' . $folderIcon . ' ' . htmlspecialchars(GeneralUtility::fixed_lgd_cs($folder->getIdentifier(), $titleLen)) . '</th>
-				<th class="col-control" nowrap="nowrap"></th>
-				<th class="col-clipboard" nowrap="nowrap">
-					<a href="#" class="btn btn-default" id="t3js-importSelection" title="' . $lang->getLL('importSelection', true) . '">' . $this->iconFactory->getIcon('actions-document-import-t3d', Icon::SIZE_SMALL) . '</a>
-					<a href="#" class="btn btn-default" id="t3js-toggleSelection" title="' . $lang->getLL('toggleSelection', true) . '">' . $this->iconFactory->getIcon('actions-document-select', Icon::SIZE_SMALL) . '</a>
+			<tr>
+				<th class="col-title nowrap">' . $folderIcon . ' ' . htmlspecialchars(GeneralUtility::fixed_lgd_cs($folder->getStorage()->getName() . ':' . $folder->getReadablePath(), $titleLen)) . '</th>
+				<th class="col-control nowrap"></th>
+				<th class="col-clipboard nowrap">
+					<a href="#" class="btn btn-default" id="t3js-importSelection" title="' . htmlspecialchars($lang->getLL('importSelection')) . '">' . $this->iconFactory->getIcon('actions-document-import-t3d', Icon::SIZE_SMALL) . '</a>
+					<a href="#" class="btn btn-default" id="t3js-toggleSelection" title="' . htmlspecialchars($lang->getLL('toggleSelection')) . '">' . $this->iconFactory->getIcon('actions-document-select', Icon::SIZE_SMALL) . '</a>
 				</th>
-				<th nowrap="nowrap">&nbsp;</th>
+				<th class="nowrap">&nbsp;</th>
 			</tr>';
 
         if ($filesCount === 0) {
@@ -274,7 +289,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             if (!$noThumbs && GeneralUtility::inList(strtolower($GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'] . ',' . $GLOBALS['TYPO3_CONF_VARS']['SYS']['mediafile_ext']), strtolower($fileExtension))) {
                 $processedFile = $fileObject->process(
                     ProcessedFile::CONTEXT_IMAGEPREVIEW,
-                    ['width' => 64, 'height' => 64]
+                    $this->thumbnailConfiguration
                 );
                 $imageUrl = $processedFile->getPublicUrl(true);
                 $imgInfo = [
@@ -291,8 +306,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
                 $pDim = '';
             }
             // Create file icon:
-            $size = ' (' . GeneralUtility::formatSize($fileObject->getSize()) . 'bytes' . ($pDim ? ', ' . $pDim : '') . ')';
-            $icon = '<span title="' . htmlspecialchars($fileObject->getName() . $size) . '">' . $this->iconFactory->getIconForResource($fileObject, Icon::SIZE_SMALL) . '</span>';
+            $size = ' (' . GeneralUtility::formatSize($fileObject->getSize(), $this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_common.xlf:byteSizeUnits')) . ($pDim ? ', ' . $pDim : '') . ')';
+            $icon = '<span title="id=' . htmlspecialchars($fileObject->getUid()) . '">' . $this->iconFactory->getIconForResource($fileObject, Icon::SIZE_SMALL) . '</span>';
             // Create links for adding the file:
             $filesIndex = count($this->elements);
             $this->elements['file_' . $filesIndex] = [
@@ -306,7 +321,8 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             ];
             if ($this->fileIsSelectableInFileList($fileObject, $imgInfo)) {
                 $ATag = '<a href="#" class="btn btn-default" title="' . htmlspecialchars($fileObject->getName()) . '" data-file-index="' . htmlspecialchars($filesIndex) . '" data-close="0">';
-                $ATag_alt = '<a href="#" title="' . htmlspecialchars($fileObject->getName()) . '" data-file-index="' . htmlspecialchars($filesIndex) . '" data-close="1">';
+                $ATag .= '<span title="' . htmlspecialchars($lang->getLL('addToList')) . '">' . $this->iconFactory->getIcon('actions-add', Icon::SIZE_SMALL)->render() . '</span>';
+                $ATag_alt = '<a href="#" title="' . htmlspecialchars($fileObject->getName()) . $size . '" data-file-index="' . htmlspecialchars($filesIndex) . '" data-close="1">';
                 $ATag_e = '</a>';
                 $bulkCheckBox = '<label class="btn btn-default btn-checkbox"><input type="checkbox" class="typo3-bulk-item" name="file_' . $filesIndex . '" value="0" /><span class="t3-icon fa"></span></label>';
             } else {
@@ -315,8 +331,10 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
                 $ATag_e = '';
                 $bulkCheckBox = '';
             }
+            /** @var \TYPO3\CMS\Backend\Routing\UriBuilder $uriBuilder */
+            $uriBuilder = GeneralUtility::makeInstance(\TYPO3\CMS\Backend\Routing\UriBuilder::class);
             // Create link to showing details about the file in a window:
-            $Ahref = BackendUtility::getModuleUrl('show_item', [
+            $Ahref = (string)$uriBuilder->buildUriFromRoute('show_item', [
                 'type' => 'file',
                 'table' => '_FILE',
                 'uid' => $fileObject->getCombinedIdentifier(),
@@ -328,13 +346,13 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             // Show element:
             $lines[] = '
 					<tr class="file_list_normal">
-						<td class="col-title" nowrap="nowrap">' . $filenameAndIcon . '&nbsp;</td>
+						<td class="col-title nowrap">' . $filenameAndIcon . '&nbsp;</td>
 						<td class="col-control">
-							<div class="btn-group">' . $ATag . '<span title="' .  $lang->getLL('addToList', true) . '">' . $this->iconFactory->getIcon('actions-edit-add', Icon::SIZE_SMALL)->render() . '</span>' . $ATag_e . '
-							<a href="' . htmlspecialchars($Ahref) . '" class="btn btn-default" title="' . $lang->getLL('info', true) . '">' . $this->iconFactory->getIcon('actions-document-info', Icon::SIZE_SMALL) . '</a>
+							<div class="btn-group">' . $ATag . $ATag_e . '
+							<a href="' . htmlspecialchars($Ahref) . '" class="btn btn-default" title="' . htmlspecialchars($lang->getLL('info')) . '">' . $this->iconFactory->getIcon('actions-document-info', Icon::SIZE_SMALL) . '</a>
 						</td>
 						<td class="col-clipboard" valign="top">' . $bulkCheckBox . '</td>
-						<td nowrap="nowrap">&nbsp;' . $pDim . '</td>
+						<td class="nowrap">&nbsp;' . $pDim . '</td>
 					</tr>';
             if ($pDim) {
                 $lines[] = '
@@ -344,23 +362,21 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
             }
         }
 
-        $out = '<h3>' . $lang->getLL('files', true) . ' ' . $filesCount . ':</h3>';
-        $out .= GeneralUtility::makeInstance(FolderUtilityRenderer::class, $this)->getFileSearchField($this->searchWord);
-        $out .= '<div id="filelist">';
-        $out .= $this->getBulkSelector($filesCount);
+        $markup = [];
+        $markup[] = '<h3>' . htmlspecialchars($lang->getLL('files')) . ' ' . $filesCount . ':</h3>';
+        $markup[] = GeneralUtility::makeInstance(FolderUtilityRenderer::class, $this)->getFileSearchField($this->searchWord);
+        $markup[] = '<div id="filelist">';
+        $markup[] = '   ' . $this->getBulkSelector($filesCount);
+        $markup[] = '   <!-- Filelisting -->';
+        $markup[] = '   <div class="table-fit">';
+        $markup[] = '       <table class="table table-striped table-hover" id="typo3-filelist">';
+        $markup[] = '           ' . implode('', $lines);
+        $markup[] = '       </table>';
+        $markup[] = '   </div>';
+        $markup[] = ' </div>';
+        $content = implode('', $markup);
 
-        // Wrap all the rows in table tags:
-        $out .= '
-
-	<!--
-		Filelisting
-	-->
-			<table class="table table-striped table-hover" id="typo3-filelist">
-				' . implode('', $lines) . '
-			</table>';
-        // Return accumulated content for filelisting:
-        $out .= '</div>';
-        return $out;
+        return $content;
     }
 
     /**
@@ -397,13 +413,13 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
         $out = '';
 
         // Getting flag for showing/not showing thumbnails:
-        $noThumbsInEB = $this->getBackendUser()->getTSConfigVal('options.noThumbsInEB');
+        $noThumbsInEB = $this->getBackendUser()->getTSConfig()['options.']['noThumbsInEB'] ?? false;
         if (!$noThumbsInEB && $this->selectedFolder) {
             // MENU-ITEMS, fetching the setting for thumbnails from File>List module:
             $_MOD_MENU = ['displayThumbs' => ''];
             $_MCONF['name'] = 'file_list';
             $_MOD_SETTINGS = BackendUtility::getModuleData($_MOD_MENU, GeneralUtility::_GP('SET'), $_MCONF['name']);
-            $addParams = GeneralUtility::implodeArrayForUrl('', $this->getUrlParameters(['identifier' => $this->selectedFolder->getCombinedIdentifier()]));
+            $addParams = HttpUtility::buildQueryString($this->getUrlParameters(['identifier' => $this->selectedFolder->getCombinedIdentifier()]), '&');
             $thumbNailCheck = '<div class="checkbox" style="padding:5px 0 15px 0"><label for="checkDisplayThumbs">'
                 . BackendUtility::getFuncCheck(
                     '',
@@ -413,7 +429,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
                     $addParams,
                     'id="checkDisplayThumbs"'
                 )
-                . $lang->sL('LLL:EXT:lang/locallang_mod_file_list.xlf:displayThumbs', true) . '</label></div>';
+                . htmlspecialchars($lang->sL('LLL:EXT:filelist/Resources/Private/Language/locallang_mod_file_list.xlf:displayThumbs')) . '</label></div>';
             $out .= $thumbNailCheck;
         } else {
             $out .= '<div style="padding-top: 15px;"></div>';
@@ -454,7 +470,7 @@ class FileBrowser extends AbstractElementBrowser implements ElementBrowserInterf
     {
         return [
             'mode' => 'file',
-            'expandFolder' => isset($values['identifier']) ? $values['identifier'] : $this->expandFolder,
+            'expandFolder' => $values['identifier'] ?? $this->expandFolder,
             'bparams' => $this->bparams
         ];
     }
