@@ -14,12 +14,24 @@ namespace TYPO3\CMS\Recycler\Domain\Model;
  * The TYPO3 project - inspiring people to share!
  */
 
+use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Cache\CacheManager;
+use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
+use TYPO3\CMS\Core\Database\Connection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Platform\PlatformInformation;
+use TYPO3\CMS\Core\Database\Query\QueryBuilder;
+use TYPO3\CMS\Core\Database\Query\QueryHelper;
+use TYPO3\CMS\Core\Database\Query\Restriction\BackendWorkspaceRestriction;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
+use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Recycler\Utility\RecyclerUtility;
 
 /**
  * Model class for the 'recycler' extension.
+ * @internal This class is a specific domain model implementation and is not part of the Public TYPO3 API.
  */
 class DeletedRecords
 {
@@ -86,22 +98,22 @@ class DeletedRecords
         // set the limit
         $this->limit = trim($limit);
         if ($table) {
-            if (in_array($table, RecyclerUtility::getModifyableTables())) {
+            if (in_array($table, RecyclerUtility::getModifyableTables(), true)) {
                 $this->table[] = $table;
-                $this->setData($id, $table, $depth, $GLOBALS['TCA'][$table]['ctrl'], $filter);
+                $this->setData($id, $table, $depth, $filter);
             }
         } else {
-            foreach ($GLOBALS['TCA'] as $tableKey => $tableValue) {
+            foreach (RecyclerUtility::getModifyableTables() as $tableKey) {
                 // only go into this table if the limit allows it
                 if ($this->limit !== '') {
-                    $parts = GeneralUtility::trimExplode(',', $this->limit);
+                    $parts = GeneralUtility::intExplode(',', $this->limit, true);
                     // abort loop if LIMIT 0,0
-                    if ((int)$parts[0] === 0 && (int)$parts[1] === 0) {
+                    if ($parts[0] === 0 && $parts[1] === 0) {
                         break;
                     }
                 }
                 $this->table[] = $tableKey;
-                $this->setData($id, $tableKey, $depth, $tableValue['ctrl'], $filter);
+                $this->setData($id, $tableKey, $depth, $filter);
             }
         }
         return $this;
@@ -121,7 +133,7 @@ class DeletedRecords
         $deletedRecords = $this->loadData($id, $table, $depth, '', $filter)->getDeletedRows();
         $countTotal = 0;
         foreach ($this->table as $tableName) {
-            $countTotal += count($deletedRecords[$tableName]);
+            $countTotal += count($deletedRecords[$tableName] ?? []);
         }
         return $countTotal;
     }
@@ -132,33 +144,40 @@ class DeletedRecords
      * @param int $id UID from record
      * @param string $table Tablename from record
      * @param int $depth How many levels recursive
-     * @param array $tcaCtrl TCA CTRL array
      * @param string $filter Filter text
-     * @return void
      */
-    protected function setData($id, $table, $depth, $tcaCtrl, $filter)
+    protected function setData($id, $table, $depth, $filter)
     {
-        $id = (int)$id;
-        if (!array_key_exists('delete', $tcaCtrl)) {
+        $deletedField = RecyclerUtility::getDeletedField($table);
+        if (!$deletedField) {
             return;
         }
-        $db = $this->getDatabaseConnection();
-        // find the 'deleted' field for this table
-        $deletedField = RecyclerUtility::getDeletedField($table);
-        // create the filter WHERE-clause
-        $filterWhere = '';
-        if (trim($filter) != '') {
-            $filterWhere = ' AND (' . (\TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($filter) ? 'uid = ' . $filter . ' OR pid = ' . $filter . ' OR ' : '') . $tcaCtrl['label'] . ' LIKE "%' . $this->escapeValueForLike($filter, $table) . '%"' . ')';
-        }
+
+        $id = (int)$id;
+        $tcaCtrl = $GLOBALS['TCA'][$table]['ctrl'];
+        $firstResult = 0;
+        $maxResults = 0;
 
         // get the limit
         if (!empty($this->limit)) {
             // count the number of deleted records for this pid
-            $deletedCount = $db->exec_SELECTcountRows('uid', $table, $deletedField . '<>0 AND pid = ' . $id . $filterWhere);
+            $queryBuilder = $this->getFilteredQueryBuilder($table, $id, $depth, $filter);
+            $queryBuilder->getRestrictions()->removeAll();
+
+            $deletedCount = (int)$queryBuilder
+                ->count('*')
+                ->from($table)
+                ->andWhere(
+                    $queryBuilder->expr()->neq(
+                        $deletedField,
+                        $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
+                    )
+                )
+                ->execute()
+                ->fetchColumn(0);
+
             // split the limit
-            $parts = GeneralUtility::trimExplode(',', $this->limit);
-            $offset = $parts[0];
-            $rowCount = $parts[1];
+            list($offset, $rowCount) = GeneralUtility::intExplode(',', $this->limit, true);
             // subtract the number of deleted records from the limit's offset
             $result = $offset - $deletedCount;
             // if the result is >= 0
@@ -168,15 +187,12 @@ class DeletedRecords
                 $this->limit = implode(',', [$offset, $rowCount]);
                 // do NOT query this depth; limit also does not need to be set, we set it anyways
                 $allowQuery = false;
-                $allowDepth = true;
-                $limit = '';
             } else {
                 // the offset for the temporary limit has to remain like the original offset
                 // in case the original offset was just crossed by the amount of deleted records
+                $tempOffset = 0;
                 if ($offset !== 0) {
                     $tempOffset = $offset;
-                } else {
-                    $tempOffset = 0;
                 }
                 // set the offset in the limit to 0
                 $newOffset = 0;
@@ -185,76 +201,124 @@ class DeletedRecords
                 // if the result now is > limit's row count
                 if ($absResult > $rowCount) {
                     // use the limit's row count as the temporary limit
-                    $limit = implode(',', [$tempOffset, $rowCount]);
+                    $firstResult = $tempOffset;
+                    $maxResults = $rowCount;
                     // set the limit's row count to 0
                     $this->limit = implode(',', [$newOffset, 0]);
-                    // do not go into new depth
-                    $allowDepth = false;
                 } else {
                     // if the result now is <= limit's row count
                     // use the result as the temporary limit
-                    $limit = implode(',', [$tempOffset, $absResult]);
+                    $firstResult = $tempOffset;
+                    $maxResults = $absResult;
                     // subtract the result from the row count
                     $newCount = $rowCount - $absResult;
                     // store the new result in the limit's row count
                     $this->limit = implode(',', [$newOffset, $newCount]);
-                    // if the new row count is > 0
-                    if ($newCount > 0) {
-                        // go into new depth
-                        $allowDepth = true;
-                    } else {
-                        // if the new row count is <= 0 (only =0 makes sense though)
-                        // do not go into new depth
-                        $allowDepth = false;
-                    }
                 }
                 // allow query for this depth
                 $allowQuery = true;
             }
         } else {
-            $limit = '';
-            $allowDepth = true;
             $allowQuery = true;
         }
         // query for actual deleted records
         if ($allowQuery) {
-            $recordsToCheck = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField($table, $deletedField, '1', ' AND pid = ' . $id . $filterWhere, '', '', $limit, false);
-            if ($recordsToCheck) {
-                $this->checkRecordAccess($table, $recordsToCheck);
+            $queryBuilder = $this->getFilteredQueryBuilder($table, $id, $depth, $filter);
+            if ($firstResult) {
+                $queryBuilder->setFirstResult($firstResult);
             }
-        }
-        // go into depth
-        if ($allowDepth && $depth >= 1) {
-            // check recursively for elements beneath this page
-            $resPages = $db->exec_SELECTquery('uid', 'pages', 'pid=' . $id, '', 'sorting');
-            if ($resPages) {
-                while ($rowPages = $db->sql_fetch_assoc($resPages)) {
-                    $this->setData($rowPages['uid'], $table, $depth - 1, $tcaCtrl, $filter);
-                    // some records might have been added, check if we still have the limit for further queries
-                    if (!empty($this->limit)) {
-                        $parts = GeneralUtility::trimExplode(',', $this->limit);
-                        // abort loop if LIMIT 0,0
-                        if ((int)$parts[0] === 0 && (int)$parts[1] === 0) {
-                            break;
-                        }
-                    }
-                }
-                $db->sql_free_result($resPages);
+            if ($maxResults) {
+                $queryBuilder->setMaxResults($maxResults);
+            }
+            $recordsToCheck = $queryBuilder->select('*')
+                ->from($table)
+                ->andWhere(
+                    $queryBuilder->expr()->eq(
+                        $deletedField,
+                        $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT)
+                    )
+                )
+                ->orderBy('uid')
+                ->execute()
+                ->fetchAll();
+
+            if ($recordsToCheck !== false) {
+                $this->checkRecordAccess($table, $recordsToCheck);
             }
         }
         $this->label[$table] = $tcaCtrl['label'];
         $this->title[$table] = $tcaCtrl['title'];
     }
 
+    /**
+     * Helper method for setData() to create a QueryBuilder that filters the records by default.
+     *
+     * @param string $table
+     * @param int $pid
+     * @param int $depth
+     * @param string $filter
+     * @return \TYPO3\CMS\Core\Database\Query\QueryBuilder
+     */
+    protected function getFilteredQueryBuilder(string $table, int $pid, int $depth, string $filter): QueryBuilder
+    {
+        $pidList = $this->getTreeList($pid, $depth);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
+        $queryBuilder->getRestrictions()
+            ->removeAll()
+            ->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
+
+        // create the filter WHERE-clause
+        $filterConstraint = null;
+        if (trim($filter) !== '') {
+            $filterConstraint = $queryBuilder->expr()->like(
+                $GLOBALS['TCA'][$table]['ctrl']['label'],
+                $queryBuilder->createNamedParameter(
+                    '%' . $queryBuilder->escapeLikeWildcards($filter) . '%',
+                    \PDO::PARAM_STR
+                )
+            );
+            if (MathUtility::canBeInterpretedAsInteger($filter)) {
+                $filterConstraint = $queryBuilder->expr()->orX(
+                    $queryBuilder->expr()->eq(
+                        'uid',
+                        $queryBuilder->createNamedParameter($filter, \PDO::PARAM_INT)
+                    ),
+                    $queryBuilder->expr()->eq(
+                        'pid',
+                        $queryBuilder->createNamedParameter($filter, \PDO::PARAM_INT)
+                    ),
+                    $filterConstraint
+                );
+            }
+        }
+
+        $maxBindParameters = PlatformInformation::getMaxBindParameters($queryBuilder->getConnection()->getDatabasePlatform());
+        $pidConstraints = [];
+        foreach (array_chunk($pidList, $maxBindParameters - 10) as $chunk) {
+            $pidConstraints[] = $queryBuilder->expr()->in(
+                'pid',
+                $queryBuilder->createNamedParameter($chunk, Connection::PARAM_INT_ARRAY)
+            );
+        }
+        $queryBuilder->where(
+            $queryBuilder->expr()->andX(
+                $filterConstraint,
+                $queryBuilder->expr()->orX(...$pidConstraints)
+            )
+        );
+
+        return $queryBuilder;
+    }
+
     /**
      * Checks whether the current backend user has access to the given records.
      *
      * @param string $table Name of the table
      * @param array $rows Record row
-     * @return void
      */
     protected function checkRecordAccess($table, array $rows)
     {
+        $deleteField = '';
         if ($table === 'pages') {
             // The "checkAccess" method validates access to the passed table/rows. When access to
             // a page record gets validated it is necessary to disable the "delete" field temporarily
@@ -276,20 +340,6 @@ class DeletedRecords
         }
     }
 
-    /**
-     * Escapes a value to be used for like in a database query.
-     * There is a special handling for the characters '%' and '_'.
-     *
-     * @param string $value The value to be escaped for like conditions
-     * @param string $tableName The name of the table the query should be used for
-     * @return string The escaped value to be used for like conditions
-     */
-    protected function escapeValueForLike($value, $tableName)
-    {
-        $db = $this->getDatabaseConnection();
-        return $db->escapeStrForLike($db->quoteStr($value, $tableName), $tableName);
-    }
-
     /************************************************************
      * DELETE FUNCTIONS
      ************************************************************/
@@ -302,9 +352,9 @@ class DeletedRecords
     public function deleteData($recordsArray)
     {
         if (is_array($recordsArray)) {
-            /** @var $tce DataHandler **/
+            /** @var DataHandler $tce **/
             $tce = GeneralUtility::makeInstance(DataHandler::class);
-            $tce->start('', '');
+            $tce->start([], []);
             $tce->disableDeleteClause();
             foreach ($recordsArray as $record) {
                 list($table, $uid) = explode(':', $record);
@@ -324,11 +374,12 @@ class DeletedRecords
      *
      * @param array $recordsArray Representation of the records
      * @param bool $recursive Whether to recursively undelete
-     * @return bool
+     * @return bool|int
      */
     public function undeleteData($recordsArray, $recursive = false)
     {
         $result = false;
+        $affectedRecords = 0;
         $depth = 999;
         if (is_array($recordsArray)) {
             $this->deletedRows = [];
@@ -351,10 +402,11 @@ class DeletedRecords
                     }
                 }
                 $cmd[$table][$uid]['undelete'] = 1;
+                $affectedRecords++;
                 if ($table === 'pages' && $recursive) {
                     $this->loadData($uid, '', $depth, '');
                     $childRecords = $this->getDeletedRows();
-                    if (count($childRecords) > 0) {
+                    if (!empty($childRecords)) {
                         foreach ($childRecords as $childTable => $childRows) {
                             foreach ($childRows as $childRow) {
                                 $cmd[$childTable][$childRow['uid']]['undelete'] = 1;
@@ -367,7 +419,7 @@ class DeletedRecords
                 $tce = GeneralUtility::makeInstance(DataHandler::class);
                 $tce->start([], $cmd);
                 $tce->process_cmdmap();
-                $result = true;
+                $result = $affectedRecords;
             }
         }
         return $result;
@@ -382,10 +434,18 @@ class DeletedRecords
      */
     protected function getDeletedParentPages($uid, &$pages = [])
     {
-        $db = $this->getDatabaseConnection();
-        $res = $db->exec_SELECTquery('uid, pid', 'pages', 'uid=' . (int)$uid . ' AND ' . $GLOBALS['TCA']['pages']['ctrl']['delete'] . '=1');
-        if ($res !== false && $db->sql_num_rows($res) > 0) {
-            $record = $db->sql_fetch_assoc($res);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
+        $queryBuilder->getRestrictions()->removeAll();
+        $record = $queryBuilder
+            ->select('uid', 'pid')
+            ->from('pages')
+            ->where(
+                $queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->eq($GLOBALS['TCA']['pages']['ctrl']['delete'], 1)
+            )
+            ->execute()
+            ->fetch();
+        if ($record) {
             $pages[] = $record['uid'];
             if ((int)$record['pid'] !== 0) {
                 $this->getDeletedParentPages($record['pid'], $pages);
@@ -403,7 +463,6 @@ class DeletedRecords
      *
      * @param string $table Tablename
      * @param array $row Deleted record row
-     * @return void
      */
     public function setDeletedRows($table, array $row)
     {
@@ -434,12 +493,81 @@ class DeletedRecords
     }
 
     /**
-     * Returns an instance of DatabaseConnection
+     * Get tree list
      *
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
+     * @param int $id
+     * @param int $depth
+     * @param int $begin
+     * @return array
      */
-    protected function getDatabaseConnection()
+    protected function getTreeList(int $id, int $depth, int $begin = 0): array
     {
-        return $GLOBALS['TYPO3_DB'];
+        $cache = $this->getCache();
+        $identifier = md5($id . '_' . $depth . '_' . $begin);
+        $pageTree = $cache->get($identifier);
+        if ($pageTree === false) {
+            $pageTree = $this->resolveTree($id, $depth, $begin, $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW));
+            $cache->set($identifier, $pageTree);
+        }
+
+        return $pageTree;
+    }
+
+    /**
+     * @param $id
+     * @param int $depth
+     * @param int $begin
+     * @param string $permsClause
+     * @return array
+     */
+    protected function resolveTree(int $id, int $depth, int $begin = 0, string $permsClause = ''): array
+    {
+        $depth = (int)$depth;
+        $begin = (int)$begin;
+        $id = abs((int)$id);
+        $theList = [];
+        if ($begin === 0) {
+            $theList[] = $id;
+        }
+        if ($depth > 0) {
+            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
+            $queryBuilder->getRestrictions()->removeAll()->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
+            $statement = $queryBuilder->select('uid')
+                ->from('pages')
+                ->where(
+                    $queryBuilder->expr()->eq('pid', $queryBuilder->createNamedParameter($id, \PDO::PARAM_INT)),
+                    QueryHelper::stripLogicalOperatorPrefix($permsClause)
+                )
+                ->execute();
+            while ($row = $statement->fetch()) {
+                if ($begin <= 0) {
+                    $theList[] = $row['uid'];
+                }
+                if ($depth > 1) {
+                    $theList = array_merge($theList, $this->resolveTree($row['uid'], $depth - 1, $begin - 1, $permsClause));
+                }
+            }
+        }
+        return $theList;
+    }
+
+    /**
+     * Gets an instance of the memory cache.
+     *
+     * @return FrontendInterface
+     */
+    protected function getCache(): FrontendInterface
+    {
+        return GeneralUtility::makeInstance(CacheManager::class)->getCache('cache_runtime');
+    }
+
+    /**
+     * Returns the BackendUser
+     *
+     * @return BackendUserAuthentication
+     */
+    protected function getBackendUser(): BackendUserAuthentication
+    {
+        return $GLOBALS['BE_USER'];
     }
 }
