@@ -3,19 +3,21 @@ $EM_CONF[$_EXTKEY] = [
     'title' => 'System Reports',
     'description' => 'The reports module groups several system reports.',
     'category' => 'module',
-    'author' => 'Ingo Renner',
-    'author_email' => 'ingo@typo3.org',
+    'author' => 'TYPO3 Core Team',
+    'author_email' => 'typo3cms@typo3.org',
     'author_company' => '',
     'state' => 'stable',
     'uploadfolder' => 0,
     'createDirs' => '',
     'clearCacheOnLoad' => 0,
-    'version' => '7.6.0',
+    'version' => '9.5.14',
     'constraints' => [
         'depends' => [
-            'typo3' => '7.6.0-7.6.99',
+            'typo3' => '9.5.14',
         ],
         'conflicts' => [],
-        'suggests' => [],
+        'suggests' => [
+            'scheduler' => ''
+        ],
     ],
 ];
