@@ -1,7 +1,7 @@
 <?php
 namespace TYPO3\CMS\Rsaauth\Storage;
 
-/**
+/*
  * This file is part of the TYPO3 CMS project.
  *
  * It is free software; you can redistribute it and/or modify it under
@@ -13,25 +13,23 @@ namespace TYPO3\CMS\Rsaauth\Storage;
  *
  * The TYPO3 project - inspiring people to share!
  */
+
 /**
  * This class contains the abstract storage for the RSA private keys
- *
- * @author Dmitry Dulepov <dmitry@typo3.org>
  */
-abstract class AbstractStorage {
-
-	/**
-	 * Retrieves the key from the storage
-	 *
-	 * @return string The key or NULL
-	 */
-	abstract public function get();
-
-	/**
-	 * Stores the key in the storage
-	 *
-	 * @param string $key The key
-	 */
-	abstract public function put($key);
+abstract class AbstractStorage
+{
+    /**
+     * Retrieves the key from the storage
+     *
+     * @return string The key or NULL
+     */
+    abstract public function get();
 
+    /**
+     * Stores the key in the storage
+     *
+     * @param string $key The key
+     */
+    abstract public function put($key);
 }
