@@ -14,17 +14,25 @@ namespace TYPO3\CMS\Scheduler;
  * The TYPO3 project - inspiring people to share!
  */
 
+use Psr\Log\LoggerAwareInterface;
+use Psr\Log\LoggerAwareTrait;
+use Psr\Log\LoggerInterface;
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
+use TYPO3\CMS\Core\Database\Connection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Database\Query\QueryHelper;
+use TYPO3\CMS\Core\Log\LogManager;
 use TYPO3\CMS\Core\Registry;
-use TYPO3\CMS\Core\Utility\CommandUtility;
+use TYPO3\CMS\Core\SingletonInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\MathUtility;
 
 /**
  * TYPO3 Scheduler. This class handles scheduling and execution of tasks.
- * Formerly known as "Gabriel TYPO3 arch angel"
  */
-class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
+class Scheduler implements SingletonInterface, LoggerAwareInterface
 {
+    use LoggerAwareTrait;
+
     /**
      * @var array $extConf Settings from the extension manager
      */
@@ -32,19 +40,14 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
 
     /**
      * Constructor, makes sure all derived client classes are included
-     *
-     * @return \TYPO3\CMS\Scheduler\Scheduler
      */
     public function __construct()
     {
         // Get configuration from the extension manager
-        $this->extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['scheduler']);
+        $this->extConf = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('scheduler');
         if (empty($this->extConf['maxLifetime'])) {
             $this->extConf['maxLifetime'] = 1440;
         }
-        if (empty($this->extConf['useAtdaemon'])) {
-            $this->extConf['useAtdaemon'] = 0;
-        }
         // Clean up the serialized execution arrays
         $this->cleanExecutionArrays();
     }
@@ -61,14 +64,21 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
         if (empty($taskUid)) {
             $fields = [
                 'crdate' => $GLOBALS['EXEC_TIME'],
-                'disable' => $task->isDisabled(),
+                'disable' => (int)$task->isDisabled(),
                 'description' => $task->getDescription(),
                 'task_group' => $task->getTaskGroup(),
                 'serialized_task_object' => 'RESERVED'
             ];
-            $result = $this->getDatabaseConnection()->exec_INSERTquery('tx_scheduler_task', $fields);
+            $connection = GeneralUtility::makeInstance(ConnectionPool::class)
+                ->getConnectionForTable('tx_scheduler_task');
+            $result = $connection->insert(
+                'tx_scheduler_task',
+                $fields,
+                ['serialized_task_object' => Connection::PARAM_LOB]
+            );
+
             if ($result) {
-                $task->setTaskUid($this->getDatabaseConnection()->sql_insert_id());
+                $task->setTaskUid($connection->lastInsertId('tx_scheduler_task'));
                 $task->save();
                 $result = true;
             } else {
@@ -83,19 +93,28 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
     /**
      * Cleans the execution lists of the scheduled tasks, executions older than 24h are removed
      * @todo find a way to actually kill the job
-     *
-     * @return void
      */
     protected function cleanExecutionArrays()
     {
         $tstamp = $GLOBALS['EXEC_TIME'];
-        $db = $this->getDatabaseConnection();
+        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
+        $queryBuilder = $connectionPool->getQueryBuilderForTable('tx_scheduler_task');
+
         // Select all tasks with executions
         // NOTE: this cleanup is done for disabled tasks too,
         // to avoid leaving old executions lying around
-        $res = $db->exec_SELECTquery('uid, serialized_executions, serialized_task_object', 'tx_scheduler_task', 'serialized_executions <> \'\'');
+        $result = $queryBuilder->select('uid', 'serialized_executions', 'serialized_task_object')
+            ->from('tx_scheduler_task')
+            ->where(
+                $queryBuilder->expr()->neq(
+                    'serialized_executions',
+                    $queryBuilder->createNamedParameter('', \PDO::PARAM_STR)
+                ),
+                $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            )
+            ->execute();
         $maxDuration = $this->extConf['maxLifetime'] * 60;
-        while ($row = $db->sql_fetch_assoc($res)) {
+        while ($row = $result->fetch()) {
             $executions = [];
             if ($serialized_executions = unserialize($row['serialized_executions'])) {
                 foreach ($serialized_executions as $task) {
@@ -103,8 +122,7 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
                         $executions[] = $task;
                     } else {
                         $task = unserialize($row['serialized_task_object']);
-                        $logMessage = 'Removing logged execution, assuming that the process is dead. Execution of \'' . get_class($task) . '\' (UID: ' . $row['uid'] . ') was started at ' . date('Y-m-d H:i:s', $task->getExecutionTime());
-                        $this->log($logMessage);
+                        $this->log('Removing logged execution, assuming that the process is dead. Execution of \'' . get_class($task) . '\' (UID: ' . $row['uid'] . ') was started at ' . date('Y-m-d H:i:s', $task->getExecutionTime()));
                     }
                 }
             }
@@ -115,10 +133,14 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
                 } else {
                     $value = serialize($executions);
                 }
-                $db->exec_UPDATEquery('tx_scheduler_task', 'uid = ' . (int)$row['uid'], ['serialized_executions' => $value]);
+                $connectionPool->getConnectionForTable('tx_scheduler_task')->update(
+                    'tx_scheduler_task',
+                    ['serialized_executions' => $value],
+                    ['uid' => (int)$row['uid']],
+                    ['serialized_executions' => Connection::PARAM_LOB]
+                );
             }
         }
-        $db->sql_free_result($res);
     }
 
     /**
@@ -127,11 +149,11 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      *
      * @param Task\AbstractTask $task The task to execute
      * @return bool Whether the task was saved successfully to the database or not
-     * @throws FailedExecutionException
-     * @throws \Exception
+     * @throws \Throwable
      */
     public function executeTask(Task\AbstractTask $task)
     {
+        $task->setRunOnNextCronJob(false);
         // Trigger the saving of the task, as this will calculate its next execution time
         // This should be calculated all the time, even if the execution is skipped
         // (in case it is skipped, this pushes back execution to the next possible date)
@@ -144,12 +166,11 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
         if (!$task->areMultipleExecutionsAllowed() && $task->isExecutionRunning()) {
             // Log multiple execution error
             $logMessage = 'Task is already running and multiple executions are not allowed, skipping! Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid();
-            $this->log($logMessage);
+            $this->logger->info($logMessage);
             $result = false;
         } else {
             // Log scheduler invocation
-            $logMessage = 'Start execution. Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid();
-            $this->log($logMessage);
+            $this->logger->info('Start execution. Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid());
             // Register execution
             $executionID = $task->markExecution();
             $failure = null;
@@ -159,21 +180,17 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
                 if (!$successfullyExecuted) {
                     throw new FailedExecutionException('Task failed to execute successfully. Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid(), 1250596541);
                 }
-            } catch (\Exception $e) {
+            } catch (\Throwable $e) {
                 // Store exception, so that it can be saved to database
                 $failure = $e;
             }
-            // make sure database-connection is fine
-            // for long-running tasks the database might meanwhile have disconnected
-            $this->getDatabaseConnection()->isConnected();
             // Un-register execution
             $task->unmarkExecution($executionID, $failure);
             // Log completion of execution
-            $logMessage = 'Task executed. Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid();
-            $this->log($logMessage);
+            $this->logger->info('Task executed. Class: ' . get_class($task) . ', UID: ' . $task->getTaskUid());
             // Now that the result of the task execution has been handled,
             // throw the exception again, if any
-            if ($failure instanceof \Exception) {
+            if ($failure instanceof \Throwable) {
                 throw $failure;
             }
         }
@@ -184,7 +201,6 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      * This method stores information about the last run of the Scheduler into the system registry
      *
      * @param string $type Type of run (manual or command-line (assumed to be cron))
-     * @return void
      */
     public function recordLastRun($type = 'cron')
     {
@@ -210,13 +226,12 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
     {
         $taskUid = $task->getTaskUid();
         if (!empty($taskUid)) {
-            $result = $this->getDatabaseConnection()->exec_DELETEquery('tx_scheduler_task', 'uid = ' . $taskUid);
+            $result = GeneralUtility::makeInstance(ConnectionPool::class)
+                ->getConnectionForTable('tx_scheduler_task')
+                ->update('tx_scheduler_task', ['deleted' => 1], ['uid' => $taskUid]);
         } else {
             $result = false;
         }
-        if ($result) {
-            $this->scheduleNextSchedulerRunUsingAtDaemon();
-        }
         return $result;
     }
 
@@ -228,10 +243,15 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function saveTask(Task\AbstractTask $task)
     {
+        $result = true;
         $taskUid = $task->getTaskUid();
         if (!empty($taskUid)) {
             try {
-                $executionTime = $task->getNextDueExecution();
+                if ($task->getRunOnNextCronJob()) {
+                    $executionTime = time();
+                } else {
+                    $executionTime = $task->getNextDueExecution();
+                }
                 $task->setExecutionTime($executionTime);
             } catch (\Exception $e) {
                 $task->setDisabled(true);
@@ -240,18 +260,26 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
             $task->unsetScheduler();
             $fields = [
                 'nextexecution' => $executionTime,
-                'disable' => $task->isDisabled(),
+                'disable' => (int)$task->isDisabled(),
                 'description' => $task->getDescription(),
                 'task_group' => $task->getTaskGroup(),
                 'serialized_task_object' => serialize($task)
             ];
-            $result = $this->getDatabaseConnection()->exec_UPDATEquery('tx_scheduler_task', 'uid = ' . $taskUid, $fields);
+            try {
+                GeneralUtility::makeInstance(ConnectionPool::class)
+                    ->getConnectionForTable('tx_scheduler_task')
+                    ->update(
+                        'tx_scheduler_task',
+                        $fields,
+                        ['uid' => $taskUid],
+                        ['serialized_task_object' => Connection::PARAM_LOB]
+                    );
+            } catch (\Doctrine\DBAL\DBALException $e) {
+                $result = false;
+            }
         } else {
             $result = false;
         }
-        if ($result) {
-            $this->scheduleNextSchedulerRunUsingAtDaemon();
-        }
         return $result;
     }
 
@@ -267,48 +295,69 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function fetchTask($uid = 0)
     {
+        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
+        $queryBuilder = $connectionPool->getQueryBuilderForTable('tx_scheduler_task');
+
+        $queryBuilder->select('t.uid', 't.serialized_task_object')
+            ->from('tx_scheduler_task', 't')
+            ->setMaxResults(1);
         // Define where clause
         // If no uid is given, take any non-disabled task which has a next execution time in the past
         if (empty($uid)) {
-            $queryArray = [
-                'SELECT' => 'tx_scheduler_task.uid AS uid, serialized_task_object',
-                'FROM' => 'tx_scheduler_task LEFT JOIN tx_scheduler_task_group ON tx_scheduler_task.task_group = tx_scheduler_task_group.uid',
-                'WHERE' => 'disable = 0 AND nextexecution != 0 AND nextexecution <= ' . $GLOBALS['EXEC_TIME'] . ' AND (tx_scheduler_task_group.hidden = 0 OR tx_scheduler_task_group.hidden IS NULL)',
-                'LIMIT' => 1
-            ];
+            $queryBuilder->getRestrictions()->removeAll();
+            $queryBuilder->leftJoin(
+                't',
+                'tx_scheduler_task_group',
+                'g',
+                $queryBuilder->expr()->eq('t.task_group', $queryBuilder->quoteIdentifier('g.uid'))
+            );
+            $queryBuilder->where(
+                $queryBuilder->expr()->eq('t.disable', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->neq('t.nextexecution', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->lte(
+                    't.nextexecution',
+                    $queryBuilder->createNamedParameter($GLOBALS['EXEC_TIME'], \PDO::PARAM_INT)
+                ),
+                $queryBuilder->expr()->orX(
+                    $queryBuilder->expr()->eq('g.hidden', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)),
+                    $queryBuilder->expr()->isNull('g.hidden')
+                ),
+                $queryBuilder->expr()->eq('t.deleted', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            );
+            $queryBuilder->orderBy('t.nextexecution', 'ASC');
         } else {
-            $queryArray = [
-                'SELECT' => 'uid, serialized_task_object',
-                'FROM' => 'tx_scheduler_task',
-                'WHERE' => 'uid = ' . (int)$uid,
-                'LIMIT' => 1
-            ];
+            $queryBuilder->where(
+                $queryBuilder->expr()->eq('t.uid', $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->eq('t.deleted', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            );
         }
 
-        $db = $this->getDatabaseConnection();
-        $res = $db->exec_SELECT_queryArray($queryArray);
-        if ($res === false) {
-            throw new \OutOfBoundsException('Query could not be executed. Possible defect in tables tx_scheduler_task or tx_scheduler_task_group or DB server problems', 1422044826);
-        }
-        // If there are no available tasks, thrown an exception
-        if ($db->sql_num_rows($res) == 0) {
-            throw new \OutOfBoundsException('No task', 1247827244);
-        } else {
-            $row = $db->sql_fetch_assoc($res);
-            /** @var $task Task\AbstractTask */
-            $task = unserialize($row['serialized_task_object']);
-            if ($this->isValidTaskObject($task)) {
-                // The task is valid, return it
-                $task->setScheduler();
-            } else {
-                // Forcibly set the disable flag to 1 in the database,
-                // so that the task does not come up again and again for execution
-                $db->exec_UPDATEquery('tx_scheduler_task', 'uid = ' . $row['uid'], ['disable' => 1]);
-                // Throw an exception to raise the problem
-                throw new \UnexpectedValueException('Could not unserialize task', 1255083671);
+        $row = $queryBuilder->execute()->fetch();
+        if (empty($row)) {
+            if (empty($uid)) {
+                // No uid was passed and no overdue task was found
+                throw new \OutOfBoundsException('No tasks available for execution', 1247827244);
             }
-            $db->sql_free_result($res);
+            // Although a uid was passed, no task with given was found
+            throw new \OutOfBoundsException('No task with id ' . $uid . ' found', 1422044826);
         }
+        /** @var Task\AbstractTask $task */
+        $task = unserialize($row['serialized_task_object']);
+        if ($this->isValidTaskObject($task)) {
+            // The task is valid, return it
+            $task->setScheduler();
+        } else {
+            // Forcibly set the disable flag to 1 in the database,
+            // so that the task does not come up again and again for execution
+            $connectionPool->getConnectionForTable('tx_scheduler_task')->update(
+                'tx_scheduler_task',
+                ['disable' => 1],
+                ['uid' => (int)$row['uid']]
+            );
+            // Throw an exception to raise the problem
+            throw new \UnexpectedValueException('Could not unserialize task', 1255083671);
+        }
+
         return $task;
     }
 
@@ -323,15 +372,22 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function fetchTaskRecord($uid)
     {
-        $db = $this->getDatabaseConnection();
-        $res = $db->exec_SELECTquery('*', 'tx_scheduler_task', 'uid = ' . (int)$uid);
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable('tx_scheduler_task');
+        $row = $queryBuilder->select('*')
+            ->from('tx_scheduler_task')
+            ->where(
+                $queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter((int)$uid, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            )
+            ->execute()
+            ->fetch();
+
         // If the task is not found, throw an exception
-        if ($db->sql_num_rows($res) == 0) {
+        if (empty($row)) {
             throw new \OutOfBoundsException('No task', 1247827245);
-        } else {
-            $row = $db->sql_fetch_assoc($res);
-            $db->sql_free_result($res);
         }
+
         return $row;
     }
 
@@ -345,31 +401,38 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function fetchTasksWithCondition($where, $includeDisabledTasks = false)
     {
-        $whereClause = '';
         $tasks = [];
-        if (!empty($where)) {
-            $whereClause = $where;
-        }
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getQueryBuilderForTable('tx_scheduler_task');
+
+        $queryBuilder
+            ->select('serialized_task_object')
+            ->from('tx_scheduler_task')
+            ->where(
+                $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            );
+
         if (!$includeDisabledTasks) {
-            if (!empty($whereClause)) {
-                $whereClause .= ' AND ';
-            }
-            $whereClause .= 'disable = 0';
+            $queryBuilder->andWhere(
+                $queryBuilder->expr()->eq('disable', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT))
+            );
         }
-        $db = $this->getDatabaseConnection();
-        $res = $db->exec_SELECTquery('serialized_task_object', 'tx_scheduler_task', $whereClause);
-        if ($res) {
-            while ($row = $db->sql_fetch_assoc($res)) {
-                /** @var Task\AbstractTask $task */
-                $task = unserialize($row['serialized_task_object']);
-                // Add the task to the list only if it is valid
-                if ($this->isValidTaskObject($task)) {
-                    $task->setScheduler();
-                    $tasks[] = $task;
-                }
-            }
-            $db->sql_free_result($res);
+
+        if (!empty($where)) {
+            $queryBuilder->andWhere(QueryHelper::stripLogicalOperatorPrefix($where));
         }
+
+        $result = $queryBuilder->execute();
+        while ($row = $result->fetch()) {
+            /** @var Task\AbstractTask $task */
+            $task = unserialize($row['serialized_task_object']);
+            // Add the task to the list only if it is valid
+            if ($this->isValidTaskObject($task)) {
+                $task->setScheduler();
+                $tasks[] = $task;
+            }
+        }
+
         return $tasks;
     }
 
@@ -387,7 +450,7 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function isValidTaskObject($task)
     {
-        return $task instanceof Task\AbstractTask;
+        return $task instanceof Task\AbstractTask && get_class($task->getExecution()) !== '__PHP_Incomplete_Class';
     }
 
     /**
@@ -397,82 +460,31 @@ class Scheduler implements \TYPO3\CMS\Core\SingletonInterface
      * @param string $message The message to write to the log
      * @param int $status Status (0 = message, 1 = error)
      * @param mixed $code Key for the message
-     * @return void
      */
-    public function log($message, $status = 0, $code = 'scheduler')
+    public function log($message, $status = 0, $code = '')
     {
-        // Log only if enabled
-        if (!empty($this->extConf['enableBELog'])) {
-            $GLOBALS['BE_USER']->writelog(4, 0, $status, $code, '[scheduler]: ' . $message, []);
+        // this method could be called from the constructor (via "cleanExecutionArrays") and no logger is instantiated
+        // by then, that's why check if the logger is available
+        if (!($this->logger instanceof LoggerInterface)) {
+            $this->setLogger(GeneralUtility::makeInstance(LogManager::class)->getLogger(__CLASS__));
         }
-    }
-
-    /**
-     * Schedule the next run of scheduler
-     * For the moment only the "at"-daemon is used, and only if it is enabled
-     *
-     * @return bool Successfully scheduled next execution using "at"-daemon
-     * @see tx_scheduler::fetchTask()
-     */
-    public function scheduleNextSchedulerRunUsingAtDaemon()
-    {
-        if ((int)$this->extConf['useAtdaemon'] !== 1) {
-            return false;
+        $message = trim('[scheduler]: ' . $code) . ' - ' . $message;
+        switch ((int)$status) {
+            // error (user problem)
+            case 1:
+                $this->logger->alert($message);
+                break;
+            // System Error (which should not happen)
+            case 2:
+                $this->logger->error($message);
+                break;
+            // security notice (admin)
+            case 3:
+                $this->logger->emergency($message);
+                break;
+            // regular message (= 0)
+            default:
+                $this->logger->info($message);
         }
-        /** @var $registry Registry */
-        $registry = GeneralUtility::makeInstance(Registry::class);
-        // Get at job id from registry and remove at job
-        $atJobId = $registry->get('tx_scheduler', 'atJobId');
-        if (MathUtility::canBeInterpretedAsInteger($atJobId)) {
-            shell_exec('atrm ' . (int)$atJobId . ' 2>&1');
-        }
-        // Can not use fetchTask() here because if tasks have just executed
-        // they are not in the list of next executions
-        $tasks = $this->fetchTasksWithCondition('');
-        $nextExecution = false;
-        foreach ($tasks as $task) {
-            try {
-                /** @var $task Task\AbstractTask */
-                $tempNextExecution = $task->getNextDueExecution();
-                if ($nextExecution === false || $tempNextExecution < $nextExecution) {
-                    $nextExecution = $tempNextExecution;
-                }
-            } catch (\OutOfBoundsException $e) {
-                // The event will not be executed again or has already ended - we don't have to consider it for
-                // scheduling the next "at" run
-            }
-        }
-        if ($nextExecution !== false) {
-            if ($nextExecution > $GLOBALS['EXEC_TIME']) {
-                $startTime = strftime('%H:%M %F', $nextExecution);
-            } else {
-                $startTime = 'now+1minute';
-            }
-            $cliDispatchPath = PATH_site . 'typo3/cli_dispatch.phpsh';
-            list($cliDispatchPathEscaped, $startTimeEscaped) =
-                CommandUtility::escapeShellArguments([$cliDispatchPath, $startTime]);
-            $cmd = 'echo ' . $cliDispatchPathEscaped . ' scheduler | at ' . $startTimeEscaped . ' 2>&1';
-            $output = shell_exec($cmd);
-            $outputParts = '';
-            foreach (explode(LF, $output) as $outputLine) {
-                if (GeneralUtility::isFirstPartOfStr($outputLine, 'job')) {
-                    $outputParts = explode(' ', $outputLine, 3);
-                    break;
-                }
-            }
-            if ($outputParts[0] === 'job' && MathUtility::canBeInterpretedAsInteger($outputParts[1])) {
-                $atJobId = (int)$outputParts[1];
-                $registry->set('tx_scheduler', 'atJobId', $atJobId);
-            }
-        }
-        return true;
-    }
-
-    /**
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
     }
 }
