@@ -15,6 +15,10 @@ namespace TYPO3\CMS\Workspaces\Service;
  */
 
 use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Database\Connection;
+use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\SingletonInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Workspaces\Domain\Record\StageRecord;
@@ -23,7 +27,7 @@ use TYPO3\CMS\Workspaces\Domain\Record\WorkspaceRecord;
 /**
  * Stages service
  */
-class StagesService implements \TYPO3\CMS\Core\SingletonInterface
+class StagesService implements SingletonInterface
 {
     const TABLE_STAGE = 'sys_workspace_stage';
     // if a record is in the "ready to publish" stage STAGE_PUBLISH_ID the nextStage is STAGE_PUBLISH_EXECUTE_ID, this id wont be saved at any time in db
@@ -88,7 +92,7 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function getPreviousStageForElementCollection(
         $workspaceItems,
-        array $byTableName = ['tt_content', 'pages', 'pages_language_overlay']
+        array $byTableName = ['tt_content', 'pages']
     ) {
         $currentStage = [];
         $previousStage = [];
@@ -136,7 +140,7 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function getNextStageForElementCollection(
         $workspaceItems,
-        array $byTableName = ['tt_content', 'pages', 'pages_language_overlay']
+        array $byTableName = ['tt_content', 'pages']
     ) {
         $currentStage = [];
         $usedStages = [];
@@ -200,11 +204,11 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function getStagesForWSUser()
     {
-        if ($GLOBALS['BE_USER']->isAdmin()) {
+        if ($this->getBackendUser()->isAdmin()) {
             return $this->getStagesForWS();
         }
 
-        /** @var $allowedStages StageRecord[] */
+        /** @var StageRecord[] $allowedStages */
         $allowedStages = [];
         $stageRecords = $this->getWorkspaceRecord()->getStages();
 
@@ -227,12 +231,14 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
             }
         }
 
-        uasort($allowedStages, function (StageRecord $first, StageRecord $second) { return $first->determineOrder($second); });
+        uasort($allowedStages, function (StageRecord $first, StageRecord $second) {
+            return $first->determineOrder($second);
+        });
         return $this->prepareStagesArray($allowedStages);
     }
 
     /**
-     * Prepares simplified stages array to be used in ExtJs components.
+     * Prepares simplified stages array
      *
      * @param StageRecord[] $stageRecords
      * @return array
@@ -246,7 +252,7 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
                 'label' => $stageRecord->getTitle(),
             ];
             if (!$stageRecord->isExecuteStage()) {
-                $stage['title'] = $GLOBALS['LANG']->sL(($this->pathToLocallang . ':actionSendToStage')) . ' "' . $stageRecord->getTitle() . '"';
+                $stage['title'] = $GLOBALS['LANG']->sL($this->pathToLocallang . ':actionSendToStage') . ' "' . $stageRecord->getTitle() . '"';
             } else {
                 $stage['title'] = $GLOBALS['LANG']->sL($this->pathToLocallang . ':publish_execute_action_option');
             }
@@ -263,16 +269,15 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
      */
     public function getStageTitle($ver_stage)
     {
-        $stageTitle = '';
         switch ($ver_stage) {
             case self::STAGE_PUBLISH_EXECUTE_ID:
-                $stageTitle = $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_mod_user_ws.xlf:stage_publish');
+                $stageTitle = $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang_mod_user_ws.xlf:stage_publish');
                 break;
             case self::STAGE_PUBLISH_ID:
                 $stageTitle = $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang_mod.xlf:stage_ready_to_publish');
                 break;
             case self::STAGE_EDIT_ID:
-                $stageTitle = $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_mod_user_ws.xlf:stage_editing');
+                $stageTitle = $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang_mod_user_ws.xlf:stage_editing');
                 break;
             default:
                 $stageTitle = $this->getPropertyOfCurrentWorkspaceStage($ver_stage, 'title');
@@ -298,19 +303,22 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
      * Gets next stage in process for given stage id
      *
      * @param int $stageId Id of the stage to fetch the next one for
-     * @return int The next stage Id
+     * @return array The next stage (id + details)
      * @throws \InvalidArgumentException
      */
     public function getNextStage($stageId)
     {
         if (!MathUtility::canBeInterpretedAsInteger($stageId)) {
-            throw new \InvalidArgumentException($GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'), 1291109987);
+            throw new \InvalidArgumentException(
+                $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'),
+                1291109987
+            );
         }
         $nextStage = false;
         $workspaceStageRecs = $this->getStagesForWS();
         if (is_array($workspaceStageRecs) && !empty($workspaceStageRecs)) {
             reset($workspaceStageRecs);
-            while (!is_null(($workspaceStageRecKey = key($workspaceStageRecs)))) {
+            while (key($workspaceStageRecs) !== null) {
                 $workspaceStageRec = current($workspaceStageRecs);
                 if ($workspaceStageRec['uid'] == $stageId) {
                     $nextStage = next($workspaceStageRecs);
@@ -318,13 +326,12 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
                 }
                 next($workspaceStageRecs);
             }
-        } else {
         }
         if ($nextStage === false) {
             $nextStage[] = [
                 'uid' => self::STAGE_EDIT_ID,
-                'title' => $GLOBALS['LANG']->sL(($this->pathToLocallang . ':actionSendToStage')) . ' "'
-                    . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_mod_user_ws.xlf:stage_editing') . '"'
+                'title' => $GLOBALS['LANG']->sL($this->pathToLocallang . ':actionSendToStage') . ' "'
+                    . $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang_mod_user_ws.xlf:stage_editing') . '"'
             ];
         }
         return $nextStage;
@@ -342,43 +349,43 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
         // Current stage is "Ready to publish" - there is no next stage
         if ($stageId == self::STAGE_PUBLISH_ID) {
             return $nextStageArray;
-        } else {
-            $nextStageRecord = $this->getNextStage($stageId);
-            if (empty($nextStageRecord) || !is_array($nextStageRecord)) {
-                // There is no next stage
-                return $nextStageArray;
-            } else {
-                // Check if the user has the permission to for the current stage
-                // If this next stage record is the first next stage after the current the user
-                // has always the needed permission
-                if ($this->isStageAllowedForUser($stageId)) {
-                    $nextStageArray[] = $nextStageRecord;
-                    return $this->getNextStages($nextStageArray, $nextStageRecord['uid']);
-                } else {
-                    // He hasn't - return given next stage array
-                    return $nextStageArray;
-                }
-            }
         }
+        $nextStageRecord = $this->getNextStage($stageId);
+        if (empty($nextStageRecord) || !is_array($nextStageRecord)) {
+            // There is no next stage
+            return $nextStageArray;
+        }
+        // Check if the user has the permission to for the current stage
+        // If this next stage record is the first next stage after the current the user
+        // has always the needed permission
+        if ($this->isStageAllowedForUser($stageId)) {
+            $nextStageArray[] = $nextStageRecord;
+            return $this->getNextStages($nextStageArray, $nextStageRecord['uid']);
+        }
+        // He hasn't - return given next stage array
+        return $nextStageArray;
     }
 
     /**
      * Get next stage in process for given stage id
      *
      * @param int $stageId Id of the stage to fetch the previous one for
-     * @return int The previous stage Id
+     * @return bool|array The previous stage or false
      * @throws \InvalidArgumentException
      */
     public function getPrevStage($stageId)
     {
         if (!MathUtility::canBeInterpretedAsInteger($stageId)) {
-            throw new \InvalidArgumentException($GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'));
+            throw new \InvalidArgumentException(
+                $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'),
+                1476048351
+            );
         }
         $prevStage = false;
         $workspaceStageRecs = $this->getStagesForWS();
         if (is_array($workspaceStageRecs) && !empty($workspaceStageRecs)) {
             end($workspaceStageRecs);
-            while (!is_null(($workspaceStageRecKey = key($workspaceStageRecs)))) {
+            while (key($workspaceStageRecs) !== null) {
                 $workspaceStageRec = current($workspaceStageRecs);
                 if ($workspaceStageRec['uid'] == $stageId) {
                     $prevStage = prev($workspaceStageRecs);
@@ -386,8 +393,8 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
                 }
                 prev($workspaceStageRecs);
             }
-        } else {
         }
+
         return $prevStage;
     }
 
@@ -512,9 +519,9 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
             return [];
         }
 
-        $backendUserList = $this->getDatabaseConnection()->cleanIntList($backendUserList);
+        $backendUserList = implode(',', GeneralUtility::intExplode(',', $backendUserList));
         $backendUsers = BackendUtility::getUserNames(
-            'username, uid, email, realName',
+            'username, uid, email, realName, lang, uc',
             'AND uid IN (' . $backendUserList . ')' . BackendUtility::BEenableFields('be_users')
         );
 
@@ -535,9 +542,8 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
                 $stageRecord->getPreselectedRecipients(),
                 $this->getRecordService()->getCreateUserIds()
             );
-        } else {
-            return $stageRecord->getPreselectedRecipients();
         }
+        return $stageRecord->getPreselectedRecipients();
     }
 
     /**
@@ -556,33 +562,40 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
     private function fetchGroups($grList, $idList = '')
     {
         $cacheKey = md5($grList . $idList);
-        $groupList = [];
         if (isset($this->fetchGroupsCache[$cacheKey])) {
-            $groupList = $this->fetchGroupsCache[$cacheKey];
-        } else {
-            if ($idList === '') {
-                // we're at the beginning of the recursion and therefore we need to reset the userGroups member
-                $this->userGroups = [];
-            }
-            $groupList = $this->fetchGroupsRecursive($grList);
-            $this->fetchGroupsCache[$cacheKey] = $groupList;
+            return $this->fetchGroupsCache[$cacheKey];
         }
+        if ($idList === '') {
+            // we're at the beginning of the recursion and therefore we need to reset the userGroups member
+            $this->userGroups = [];
+        }
+        $groupList = $this->fetchGroupsRecursive($grList);
+        $this->fetchGroupsCache[$cacheKey] = $groupList;
         return $groupList;
     }
 
     /**
      * @param array $groups
-     * @return void
      */
     private function fetchGroupsFromDB(array $groups)
     {
-        $whereSQL = 'deleted=0 AND hidden=0 AND pid=0 AND uid IN (' . implode(',', $GLOBALS['TYPO3_DB']->cleanIntArray($groups)) . ') ';
-        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'be_groups', $whereSQL);
         // The userGroups array is filled
-        while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('be_groups');
+
+        $result = $queryBuilder
+            ->select('*')
+            ->from('be_groups')
+            ->where(
+                $queryBuilder->expr()->in(
+                    'uid',
+                    $queryBuilder->createNamedParameter($groups, Connection::PARAM_INT_ARRAY)
+                )
+            )
+            ->execute();
+
+        while ($row = $result->fetch()) {
             $this->userGroups[$row['uid']] = $row;
         }
-        $GLOBALS['TYPO3_DB']->sql_free_result($res);
     }
 
     /**
@@ -613,11 +626,11 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
                     if (trim($row['subgroup'])) {
                         // Make integer list
                         $theList = implode(',', GeneralUtility::intExplode(',', $row['subgroup']));
-                        // Get the subarray
-                        $subbarray = $this->fetchGroups($theList, $idList . ',' . $uid);
-                        list($subUid, $subArray) = each($subbarray);
-                        // Merge the subarray to the already existing userGroups array
-                        $this->userGroups[$subUid] = $subArray;
+                        // Get the subgroups
+                        $subGroups = $this->fetchGroups($theList, $idList . ',' . $uid);
+                        // Merge the subgroups to the already existing userGroups array
+                        $subUid = key($subGroups);
+                        $this->userGroups[$subUid] = $subGroups[$subUid];
                     }
                 }
             }
@@ -637,7 +650,10 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
     {
         $result = null;
         if (!MathUtility::canBeInterpretedAsInteger($stageId)) {
-            throw new \InvalidArgumentException($GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'));
+            throw new \InvalidArgumentException(
+                $GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'),
+                1476048371
+            );
         }
         $workspaceStage = BackendUtility::getRecord(self::TABLE_STAGE, $stageId);
         if (is_array($workspaceStage) && isset($workspaceStage[$property])) {
@@ -727,13 +743,11 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
     protected function isStageAllowedForUser($stageId)
     {
         $cacheKey = $this->getWorkspaceId() . '_' . $stageId;
-        $isAllowed = false;
         if (isset($this->workspaceStageAllowedCache[$cacheKey])) {
-            $isAllowed = $this->workspaceStageAllowedCache[$cacheKey];
-        } else {
-            $isAllowed = $GLOBALS['BE_USER']->workspaceCheckStageForCurrent($stageId);
-            $this->workspaceStageAllowedCache[$cacheKey] = $isAllowed;
+            return $this->workspaceStageAllowedCache[$cacheKey];
         }
+        $isAllowed = $this->getBackendUser()->workspaceCheckStageForCurrent($stageId);
+        $this->workspaceStageAllowedCache[$cacheKey] = $isAllowed;
         return $isAllowed;
     }
 
@@ -756,44 +770,6 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
         return $isValid;
     }
 
-    /**
-     * Returns the notification mode from stage configuration
-     *
-     * Return values:
-     * 0 = notify someone / old way / default setting
-     * 1 = notify all responsible users (some users checked per default and you're not allowed to uncheck them)
-     * 2 = notify all responsible users (all users are checked and nothing can be changed during stage change)
-     *
-     * @param int $stageId Stage id to return the notification mode for
-     * @return int
-     * @throws \InvalidArgumentException
-     * @deprecated since TYPO3 CMS 7, will be removed in TYPO3 CMS 8. No replacement for this method. Notifications have been adjusted with issue #35245
-     */
-    public function getNotificationMode($stageId)
-    {
-        GeneralUtility::logDeprecatedFunction();
-        if (!MathUtility::canBeInterpretedAsInteger($stageId)) {
-            throw new \InvalidArgumentException($GLOBALS['LANG']->sL('LLL:EXT:workspaces/Resources/Private/Language/locallang.xlf:error.stageId.integer'));
-        }
-        switch ($stageId) {
-            case self::STAGE_PUBLISH_EXECUTE_ID:
-
-            case self::STAGE_PUBLISH_ID:
-                $workspaceRecord = BackendUtility::getRecord('sys_workspace', $this->getWorkspaceId());
-                return $workspaceRecord['publish_notification_mode'];
-                break;
-            case self::STAGE_EDIT_ID:
-                $workspaceRecord = BackendUtility::getRecord('sys_workspace', $this->getWorkspaceId());
-                return $workspaceRecord['edit_notification_mode'];
-                break;
-            default:
-                $workspaceStage = BackendUtility::getRecord(self::TABLE_STAGE, $stageId);
-                if (is_array($workspaceStage) && isset($workspaceStage['notification_mode'])) {
-                    return $workspaceStage['notification_mode'];
-                }
-        }
-    }
-
     /**
      * @return RecordService
      */
@@ -806,18 +782,10 @@ class StagesService implements \TYPO3\CMS\Core\SingletonInterface
     }
 
     /**
-     * @return \TYPO3\CMS\Core\Authentication\BackendUserAuthentication
+     * @return BackendUserAuthentication
      */
     protected function getBackendUser()
     {
         return $GLOBALS['BE_USER'];
     }
-
-    /**
-     * @return \TYPO3\CMS\Core\Database\DatabaseConnection
-     */
-    protected function getDatabaseConnection()
-    {
-        return $GLOBALS['TYPO3_DB'];
-    }
 }
